      program add_atv
c
c  Create a new version of the TV engineering data base with added
c  entries for proposed new ATV channels.  Information on new channels
c  is read from standard input in the usual TVSTA or tvsta form with
c  new channel numbers added in the last column, or in the form of the
c  allotment table evaluation of Appendix B, Sixth Report and Order.
c
c  Run with a command like "add_atv < tvsta.pasted"; or to use this
c  program with a command line argument specifing the binary input file,
c  type "add_atv alternate.unf < tvsta.pasted".
c
      parameter (MAX_ENG_REC = 8000, MAX_NEW = 3200)
c
c  Default file names
c
      character*48
     &   eng_db     /'eng_db.unf'/,
     &   atv_eng_db /'atv_eng_db.unf'/
c
c  Character variables in records of the engineering data base
c
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn*8
c
c  Composite strings with lengths a multiple of 4
c
      character name*32, params*32
c
c  Variables used to eliminate multiple records for same city-channel
c  combination
c
      character current_city*23
      integer current_channel
      logical multiple
c
c  Structure in a form which can be used with qsort
c
      structure /ENG_DB_RECORDS/
         integer ichan
         character*32 name
         integer latsec
         integer lonsec
         real erp
         real haat
         real rcamsl
         real ref_az
         integer itype
         integer istat
         character*32 params
      end structure
      record /ENG_DB_RECORDS/ struct_array(MAX_ENG_REC)
c
c  Arrays storing new channel information
c
      character*1 cty_array(MAX_NEW)
      character*2 state_array(MAX_NEW)
      character*20 city_array(MAX_NEW)
      integer chan(MAX_NEW), newchan(MAX_NEW)
      real atv_pwr(MAX_NEW)
c
c  The name of the input binary data base file can be supplied as a
c  command line argument in place of the default name
c
      numargs = iargc()
      if (numargs .gt. 0) then
         if (numargs .ne. 1) then
            write(0, '(a)') 'Too many command line arguments'
            call exit(257)
         end if
         call getarg(1, eng_db)
         atv_eng_db = 'atv_' // eng_db
      end if
c
c  Read new channel allotments from standard input.  Note elements of
c  city_array will contain only 15 significant characters if input is
c  in the form of the allotment table evaluation of Appendix B, Sixth
c  Report and Order.
c
      call stations(MAX_NEW, numsta, cty_array, state_array,
     &   city_array, chan, newchan, atv_pwr)
c
c  Open the engineering data base binary file
c
      call getlun(imain)
      open(imain, file=eng_db, status='old', access='direct', recl=99,
     &   iostat=ierr)
      if (ierr .ne. 0) then
         call perror(eng_db)
         call exit(ierr)
      end if
c
c  Read engineering data base records
c
      irec = 0
   10 continue
      irec = irec + 1
      read(imain, rec=irec, end=20, err=900)
     &   ichan, cty, city, state, call, latsec, lonsec,
     &   erp, haat, rcamsl, ref_az, itype, istat, svc_cls,
     &   off, pol, da, beamt, zone, antmnf, antmod, arn
      if (irec .gt. MAX_ENG_REC) then
         write(0, '(a)') 'Too many records in engineering data base'
         call exit(257)
      end if
c
c  Compose string of station parameters and identifiers
c
      name = cty//city//state//call
      params = svc_cls//off//pol//da//beamt//zone//antmnf//antmod//arn
c
c  Load structure
c
      struct_array(irec).ichan = ichan
      struct_array(irec).name = name
      struct_array(irec).latsec = latsec
      struct_array(irec).lonsec = lonsec
      struct_array(irec).erp = erp
      struct_array(irec).haat = haat
      struct_array(irec).rcamsl = rcamsl
      struct_array(irec).ref_az = ref_az
      struct_array(irec).itype = itype
      struct_array(irec).istat = istat
      struct_array(irec).params = params
      goto 10
c
c  Finished reading engineering data base.  Now sort the records so
c  that multiples can be eliminated.
c
   20 continue
      numrec = irec - 1
      close(imain)
      write(0, '(a,i4,a)') 'Sorting ', numrec, ' records'
      call sort_struct(struct_array, numrec)
c
c  Retrieve sorted records and write new binary file eliminating
c  multiple entries for same city-channel combination
c
      call getlun(jmain)
      open(jmain, file=atv_eng_db, status='new', access='direct',
     &   recl=99, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(atv_eng_db)
         call exit(ierr)
      end if
      current_channel = 0
      current_city = ' '
c
      newrec = 0
      do i=1, numrec
         ichan = struct_array(i).ichan
         name = struct_array(i).name
c
         cty = name(1:1)
         city = name(2:21)
         state = name(22:23)
         call = name(24:31)
c
         latsec = struct_array(i).latsec
         lonsec = struct_array(i).lonsec
         erp = struct_array(i).erp
         haat = struct_array(i).haat
         rcamsl = struct_array(i).rcamsl
         ref_az = struct_array(i).ref_az
         itype = struct_array(i).itype
         istat = struct_array(i).istat
         params = struct_array(i).params
c
         svc_cls = params(1:2)
         off = params(3:3)
         pol = params(4:4)
         da = params(5:5)
         beamt = params(6:6)
         zone = params(7:7)
         antmnf = params(8:10)
         antmod = params(11:24)
         arn = params(25:32)
c
         multiple = ichan .eq. current_channel .and.
     &      name(1:23) .eq. current_city
         if (.not. multiple) then
            newrec = newrec + 1
            struct_array(newrec) = struct_array(i)
            write(jmain, rec=newrec) ichan, name(1:31), latsec, lonsec,
     &         erp, haat, rcamsl, ref_az, itype, istat, params
            current_channel = ichan
            current_city = name(1:23)
         end if
      end do
c
c  Find new channel by matching station names and channel numbers.
c  Add an ATV station record for every NTSC city-channel combination
c  of the United States, inserting an ATV channel number of 0 if
c  there is no match.
c
      num_added = 0
      do j=1, newrec
         name = struct_array(j).name
         if (name(1:1) .ne. 'A') goto 100
         city = name(2:21)
         state = name(22:23)
         call = name(24:31)
         params = struct_array(j).params
         call find_allotment(struct_array(j), numsta, cty_array,
     &      state_array, city_array, chan, nth_allotment)
         num_added = num_added + 1
         jrec = newrec + num_added
         if (nth_allotment .gt. 0) then
            jchan = newchan(nth_allotment)
            erp = atv_pwr(nth_allotment)
            istat = 0	!initial allotment
            itype = 33	!index of file no. prefix for ATV station
            params(3:3) = ' '	!no offset
            params(5:5) = 'Y'	!directional antenna
            params(8:10) = 'rep'	!anntenna replicates NTSC coverage
            call create_pattern_name(params(11:24), state, city(1:10),
     &         jchan)
            params(25:32) = '00000000'	!application reference number
            name(24:31) = 'D' // call	!new call sign
         else
            jchan = 0
            erp = 0.0
            istat = 5	!no initial allotment
            itype = 0	!no application file number
            params(1:6) = ' '	!no service class or transmitting parameters
            params(8:32) = ' '	!no other particulars except zone
         end if
         ref_az = 0.0
         latsec = struct_array(j).latsec
         lonsec = struct_array(j).lonsec
         haat = struct_array(j).haat
         rcamsl = struct_array(j).rcamsl
         write(jmain, rec=jrec) jchan, name(1:31), latsec, lonsec,
     &      erp, haat, rcamsl, ref_az, itype, istat, params
      end do
c
  100 continue
      write(0, '(3(i5,a))') numrec, ' records in; ', newrec,
     &   ' city-channel combinations; ', num_added, ' ATV records added'
      write(0, '(i5,2a)') newrec + num_added,
     &   ' total records written to file ',
     &   atv_eng_db(1:lnblnk(atv_eng_db))
      stop
c
  900 continue
      write(0, '(2a)') 'error reading file', eng_db(1:lnblnk(eng_db))
      end

      subroutine stations(max_new, numsta, cty, state, city, chan,
     &   newchan, atv_pwr)
      integer chan(max_new), newchan(max_new)
      character*1 cty(max_new)
      character*2 state(max_new)
      character*20 city(max_new)
      character record*128
      real atv_pwr(max_new)
c
      numsta = 0
  100 continue
      read(*, '(a)', end=200) record
      length = lnblnk(record)
      if (length .lt. 100) then
         if (record(1:1) .lt. 'A') goto 100
         if (record(1:1) .gt. 'M') goto 100
      end if
      numsta = numsta + 1
      if (numsta .gt. max_new) then
         write(0, '(a)') 'Too many new assignments'
         call exit(257)
      end if
      if (length .eq. 116) then
         read(record, 1000) state(numsta), city(numsta), chan(numsta),
     &      newchan(numsta), atv_pwr(numsta)
         cty(numsta) = 'A'
      else if (length .eq. 69) then
         read(record, 1001) cty(numsta), state(numsta), city(numsta),
     &      chan(numsta), newchan(numsta)
         atv_pwr(numsta) = -1.0
      else if (length .eq. 46) then
         read(record, 1002) cty(numsta), state(numsta), city(numsta),
     &      chan(numsta), newchan(numsta)
         atv_pwr(numsta) = -1.0
      else if (length .eq. 66) then
         read(record, 1001) cty(numsta), state(numsta), city(numsta),
     &      chan(numsta)
         newchan(numsta) = 0
         atv_pwr(numsta) = 0.0
      else if (length .eq. 43) then
         read(record, 1002) cty(numsta), state(numsta), city(numsta),
     &      chan(numsta)
         newchan(numsta) = 0
         atv_pwr(numsta) = 0.0
      else
         write(0, '(a/a)') 'Bad record:', record(1:length)
         call exit(257)
      end if
      include 'special_cases.inc'
      goto 100
c
c  Finished
c
  200 continue
      return
c
 1000 format(a2, 1x, a15, i3, t24, i3, t29, f7.1)
c12345678901234567890123456789012345
cAL ANNISTON        40   58    253.2   350.0    21331     1124      17127      616        0.2          0.0       99.3
 1001 format(a1, t3, a2, t6, a20, t65, i2, 1x, i2)
 1002 format(a1, t3, a2, t6, a20, t42, i2, 1x, i2)
c123456789012345678901234567890123456789012345678901234567890123456789
cA CA SAN DIEGO             OP      Z2 0000875538 324146 1165607 15 25
cA CT BRIDGEPORT           411048 0731120 43 12
      end

      subroutine sort_struct(struct_array, num_elements)
      integer num_elements
      structure /ENG_DB_RECORDS/
         integer ichan
         character*32 name
         integer latsec
         integer lonsec
         real erp
         real haat
         real rcamsl
         real ref_az
         integer itype
         integer istat
         character*32 params
      end structure
      record /ENG_DB_RECORDS/ struct_array(num_elements)
      external compare_struct
      integer*2 compare_struct
      integer struct_size
      struct_size = 100
      call qsort(struct_array, num_elements, struct_size,
     &   compare_struct)
      return
      end

      integer*2 function compare_struct(x, y)
c
c  Specify order for qsort of engineering data base records
c
      structure /ENG_DB_RECORDS/
         integer ichan
         character*32 name
         integer latsec
         integer lonsec
         real erp
         real haat
         real rcamsl
         real ref_az
         integer itype
         integer istat
         character*32 params
      end structure
      record /ENG_DB_RECORDS/ x, y
      character*1 cty_x, cty_y
      character*20 city_x, city_y
      character*2 state_x, state_y
      character*8 arn_x, arn_y
c
      cty_x = x.name(1:1)
      cty_y = y.name(1:1)
      city_x = x.name(2:21)
      city_y = y.name(2:21)
      state_x = x.name(22:23)
      state_y = y.name(22:23)
      arn_x = x.params(25:32)
      arn_y = y.params(25:32)
      if (cty_y .eq. cty_x) then
         if (state_y .eq. state_x) then
            if (city_y .eq. city_x) then
               if (y.ichan .eq. x.ichan) then
                  if (y.istat .eq. x.istat) then
                     if (arn_y .eq. arn_x) then
                        compare_struct = 0
                     else if (arn_y .gt. arn_x) then
                        compare_struct = -1
                     else
                        compare_struct =  1
                     end if
                  else
                     compare_struct = sign(1, x.istat - y.istat)
                  end if
               else
                  compare_struct = sign(1, x.ichan - y.ichan)
               end if
            else if (city_y .gt. city_x) then
               compare_struct = -1
            else
               compare_struct =  1
            end if
         else if (state_y .gt. state_x) then
            compare_struct = -1
         else
            compare_struct =  1
         end if
      else if (cty_y .gt. cty_x) then
         compare_struct = -1
      else
         compare_struct =  1
      end if
      return
      end

      subroutine find_allotment(db_rec, numsta, cty, state, city, chan,
     &   nth_allotment)
      structure /ENG_DB_RECORDS/
         integer ichan
         character*32 name
         integer latsec
         integer lonsec
         real erp
         real haat
         real rcamsl
         real ref_az
         integer itype
         integer istat
         character*32 params
      end structure
      record /ENG_DB_RECORDS/ db_rec
      character*1 cty(numsta)
      character*2 state(numsta)
      character*20 city(numsta)
      integer chan(numsta)
c
      character db_cty*1, db_state*2, db_city*20, name*32
      logical match
c
      ichan = db_rec.ichan
      name = db_rec.name
      db_cty = name(1:1)
      db_city = name(2:16)
      db_state = name(22:23)
      do n=1, numsta
         match =
     &      cty(n) .eq. db_cty .and.
     &      state(n) .eq. db_state .and.
     &      city(n)(1:15) .eq. db_city .and.
     &      chan(n) .eq. ichan
         if (match) then
            nth_allotment = n
            return
         end if
      end do
      nth_allotment = 0
      return
      end

      subroutine create_pattern_name(string, state, city, jchan)
      character*(*) string, state, city
      if (len(string) .lt. len(state) + len(city) + 2) then
         write(0, 1000) state, city, jchan
         call exit(257)
      end if
      write(string, '(2a,i2.2)') state, city, jchan
      call substitute_underscore(string)
      return
 1000 format('Output string too short for "', 2a, i2, '"')
      end

      subroutine substitute_underscore(string)
      character*(*) string
      l = index(string, ' ')
      do while (l .ne. 0)
         string(l:l) = '_'
         l = index(string, ' ')
      end do
      return
      end
