      program create_tvdb
c
c  Read TV Engineering Data Base from standard input, and write
c  unformatted 99-byte records for use by tvserv program flr.
c
c  Run with the command "zcat $TVdata/tvdb.lis | create_tvdb"
c
      parameter(MAX_ENG_REC = 4800)	!before adding ATV records
      parameter(TOP_CHANNEL = 69)
      character eng_db*48 /'eng_db.unf'/
      character record*360
      character border*1, call_sign*8, comment*62, not_city*1, offset*1
      character country*1, state*2, city*20, zone*1
      character directional_ant*1, polarization*1, tilt*1
      character service_class*2, status*6
      character educ_or_comm*1, prefix*6, refnum*8
      character antmnf*3, antmod*14
c
c  The following file number prefixes may be found in the TV Engineering
c  Data Base.  They are right-justified in a field of 6 so that they
c  are joined with no intervening spaces to the application reference
c  number.  The field is all blank for vacant allotments.  An "L" in the
c  sixth position signifies low power TV.  A prefix ending in "DT"
c  identifies a DTV application.
c
c     BLCT    BLET    BLTT    BLTTB   BLTTL   BLTTV   BLTVL   BLUB
c     BLVB    BMLCT   BMLET   BMLTT   BMLTTV  BMPCT   BMPET   BMPTT
c     BMPTTL  BMPTTV  BMPTVL  BPCT    BPCTI   BPET    BPEX    BPTT
c     BPTTL   BPTTV   BPTVL   BPUB    BPVB    BRCT    BRET    RM
c     BPCDT   BPEDT   BMPCDT  BMPEDT
c
      integer channel
      integer date(3), lat(3), long(3)
      real erp, haat
      real latitude_seconds, longitude_seconds
c
      character status_symbol*2
      logical booster, land_mobile, rule_making, translator, vacant
      logical add_mex, directional, ignore, lptv, used_elsewhere
      logical educational
      logical header
c
      read(*, '(a)') record
      header = record(1:10) .eq. '00000001'
      if (.not. header) then
         write(0, *) record(1:10)
         write(0, *) 'no header record'
      else
         read(record(25:30), '(3i2)') date
         write(0, 1000) date(2), date(3), date(1)
      end if
      if (header .and. record(31:31) .ne. 'M') then
         write(0, *) record(1:31)
         write(0, *) 'No metric indicator found in header record'
      end if
c
      call getlun(imain)
      open(imain, file=eng_db, status='new', access='direct',
     &   recl=99, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(eng_db)
         call exit(257)
      end if
c
      numrec = 0
      irec = 0
      if (.not. header) goto 20
   10 continue
      read(*, '(a)', end=200) record
      if (record(1:8) .eq. '99999999') goto 200
   20 continue
      numrec = numrec + 1
      read(record, 1010, err=300)
     &   channel, service_class, country, state, city,
     &   lat(1), lat(2), latitude_seconds,
     &   long(1), long(2), longitude_seconds,
     &   border, call_sign, prefix, refnum, status, offset, erp, haat,
     &   polarization, directional_ant, tilt, izone, not_city,
     &   educ_or_comm, comment, rcamsl, ref_az, antmnf, antmod
c
      if (channel .gt. TOP_CHANNEL) goto 10
      lat(3) = nint(latitude_seconds)
      if (lat(3) .eq. 60) call carry(lat)
      long(3) = nint(longitude_seconds)
      if (long(3) .eq. 60) call carry(long)
      if (status(1:3) .eq. 'LIC') then
         status_symbol = 'OP'
      else if (service_class .eq. 'TA') then
         status_symbol = 'VA'
      else
         status_symbol = status(1:2)
      end if
      booster = service_class .eq. 'TB'
      land_mobile = service_class .eq. 'LM'
      translator = service_class .eq. 'TX'
      rule_making = service_class .eq. 'TR'
      vacant = service_class .eq. 'TA'
      educational = educ_or_comm .eq. 'E' .or. educ_or_comm .eq. 'e'
      directional = directional_ant .eq. 'Y'
      lptv = translator .and. prefix(6:6) .eq. 'L'
      used_elsewhere = index(comment, ' USED ') .gt. 0
      add_mex = country .eq. 'M' .and. index(status, 'ADD') .gt. 0
      if (add_mex) status_symbol = 'VA'
c
      ignore = booster .or. land_mobile .or. translator
      ignore = ignore .or. (rule_making .and. .not. add_mex)
      ignore = ignore .or. (vacant .and. used_elsewhere)
      ignore = ignore .or. state .eq. 'GU'
      ignore = ignore .or. (country .ne. 'A' .and. country .ne. 'C')
      ignore = ignore .or. (vacant .and. .not. educational .and.
     &   country .eq. 'A')
      if (ignore) goto 10
c
      irec = irec + 1
      latsec = lat(3) + 60*(lat(2) + 60*lat(1))
      lonsec = long(3) + 60*(long(2) + 60*long(1))
      write(zone, '(i1)') izone
      if (status_symbol .eq. 'OP') then
         istat = 2
      else if (status_symbol .eq. 'CP') then
         istat = 1
      else if (status_symbol .eq. 'AP') then
         istat = 3
      else if (status_symbol .eq. 'VA') then
         istat = 4
      else
         write(0, '(a,a2,1x,a,i3)')
     &      'Status symbol undetermined for ', state,
     &      city(1:lnblnk(city)), channel
         istat = 0
      end if
      itype = match(prefix)
      write(imain, rec=irec, err=400)
     &   channel, country, city, state, call_sign, latsec, lonsec,
     &   erp, haat, rcamsl, ref_az, itype, istat, service_class,
     &   offset, polarization, directional_ant, tilt, zone, antmnf,
     &   antmod, refnum
      goto 10
c
  200 continue
      if (header .and. record(1:8) .ne. '99999999') then
         write(0, *) 'Improper trailer.  Expected "999999999", got ',
     &      record(1:8)
      end if
      write(0, *) numrec, ' records found,', irec, ' converted'
      if (irec .gt. MAX_ENG_REC) write(0, '(2a)')
     &   'WARNING: the number of records converted may be too large ',
     &   'for other programs'
      stop
c
  300 continue
      write(0, '(a/a)') 'Error reading the following record:', record
      call exit(257)
c
  400 continue
      write(0, '(2a)') 'Error writing to file ',
     &   eng_db(1:lnblnk(eng_db))
c
 1000 format('TV Engineering Data Base, ', i2, '/', i2.2, '/', i2.2)
c
c  Format of input records
c
 1010 format(i2, t9, a2, t17, a1, a2, a20, i3, i2, f4, i4, i2, f4, a1,
     &   a8, t76, a6, t82, a8, t111, a6, a1, f4, f5, a1, a1, a1, i1,
     &   t133, a1, t142, a1, t199, a62, t285, f5, t290, f5, t295, a3,
     &   a14)
      end

      subroutine carry(lat)
      integer lat(3)
      if (lat(3) .eq. 60) then
         lat(3) = 0
         lat(2) = lat(2) + 1
         if (lat(2) .eq. 60) then
            lat(2) = 0
            lat(1) = lat(1) + 1
         end if
      end if
      return
      end

      integer function match(string)
c
c  Replace the application reference number prefixes found in the
c  TV Engineering Data Base with an identifying number.
c
      character*(*) string
      include 'ENGdb_prefixes.inc'
      do i=1, NUM_PREFIXES
         if (string .eq. prefix(i)) then
            match = i
            return
         end if
      end do
      match = 0
      return
      end
