c
c  File global.inc
c  %W% %G%
c
      integer ATV, A_TO_N, A_TO_A
      parameter(
     &   NTSC = 0,
     &   ATV = 1,
     &   N_TO_N = 1,
     &   N_TO_A = 2,
     &   A_TO_N = 3,
     &   A_TO_A = 4)
c
      integer FCC_CURVES, LONGLEY_RICE
      parameter (
     &   FCC_CURVES = 0,
     &   LONGLEY_RICE = 1)
c
      parameter (
     &   PI = 3.1415927,
     &   DEGKM = 111.15,
     &   DEGREE = 180.0/PI,
     &   DEG_TO_RAD = 1.0/DEGREE,
     &   RADIAN = DEG_TO_RAD,
     &   RAD_TO_DEG = 180.0/PI,
     &   RMILE = DEGKM/1.609344,
     &   RPMI = RADIAN/RMILE,
     &   TWOPI = 2*PI)
c
      parameter (
     &   MAX_ENG_REC = 7400,	!allows for 1800 US plus 1900 Can NTSC
     &   MAX_ATV_STA = 2500,
     &   MAX_ANT_REC = 10000,
     &   MAX_DIR = 4,
     &   MAX_RAD = 360,
     &   MAX_TER_BLK = 600,	!Max 7.5 x 7.5 min blocks in memory
     &   MAX_POP_BLOCKS = 1000000)
c
      parameter (
     &   MAX_ATV = 2500,
     &   NUM_ANTENNA_SECTORS = 24,
     &   DELTA_DEGREES = 360.0/NUM_ANTENNA_SECTORS,
     &   NUM_DB_INTERVALS = 20)
c
      integer UNCALCULATED_SERVICE, UNCALCULATED_IX
      parameter (
     &   FLAGGED_FIELD = 320.0,
     &   IGNORED_CELL = 32100,
     &   UNCALCULATED_SERVICE = 100 * FLAGGED_FIELD,
     &   UNCALCULATED_IX = -UNCALCULATED_SERVICE)
c
      include 'terrain_indices.inc'
c
c  Parameters set in options.inc and built into the executable at the
c  time the program is compiled
c
c  calculate_area   - Count all cells with service, including those with
c                     population zero
c  correlated_fades - Apply correlation considerations to calculations
c                     of interference from ATV stations to the service
c                     provided by nearby ATV stations
c  sideband_filter  - Apply improvements to adjacent-channel A-to-A D/U
c                     ratios
c  cap_by_truncation- Enforce power cap in each direction separately
c                     rather than by scaling (scaling retains shape of
c                     coverage area)
c  per_6th_order    - Bypass program changes made since 6th MO&O (disables
c                     mod1, mod2, mod3 and mod4).  Calculations will be made
c                     as they were for the Appendix B tables of the 6th R&O
c                     and the two reconsideration orders when per_6th_order
c                     is true.
c  analyze_tilt     - Enable shift of vertical pattern for particular
c                     stations
c  adjust_du  - Include effect of low desired SNR on D/U ratios in cases
c               of co-channel interference to ATV
c  dipole_adj - Apply dipole factor to UHF fields for NL coverage
c  pairwise   - Evaluate percent mismatch between existing NTSC service
c               and that which will be provided by paired ATV stations
c  graphs     - Create two-column lists identifying grid cells where
c               the service bit has been turned off
c  new_dadb   - Create directional antenna file in standard FCC format
c               (400-character records) for ATV stations
c  mod1 - Apply the fringe effect to co-channel interference only.  The
c         Appendix B tables of the 6th R&O and reconsideration orders
c         applied the fringe effect to both co- and adjacent channel
c         interference to DTV.
c  mod2 - Set the vertical radiation factor to the last value of the
c         pattern array for points within 1 km of the transmitter.
c         The vertical radiation factor for these points was erroneously
c         set to unity in computations for Appendix B tables of the 6th
c         R&O and the reconsideration orders.
c  mod3 - Extend terrain profiles to a point within half d_inc from the
c         receiver.  Appendix B tables were calculated from profiles that
c         stopped as much as 0.9*d_inc from the receiver, never going as
c         much as 0.1*d_inc beyond.
c  mod4 - Calculate depression angles using antenna heights AMSL, except
c         when determining grade B contours.  Appendix B tables were
c         calulated using heights AGL (HAAT for grade B).
c
      logical calculate_area, correlated_fades
      logical sideband_filter, cap_by_truncation, analyze_tilt
      logical adjust_du, dipole_adj
      logical graphs, new_dadb, pairwise
      common /compile_time_params/ calculate_area, correlated_fades,
     &   sideband_filter, cap_by_truncation, analyze_tilt, adjust_du,
     &   dipole_adj, pairwise, graphs, new_dadb
      logical per_6th_order, mod1, mod2, mod3, mod4
      common /version/ per_6th_order, mod1, mod2, mod3, mod4
c
c  Runtime parameter settings read from input stream
c
c  cntr_nl_n(3)  -  The NTSC noise limited contour values for the three
c                   TV bands low/high VHF UHF (values in dBu)
c  cntr_nl_a(3)  -  The ATV noise limited contour values for the three
c                   TV bands low/high VHF UHF (values in dBu)
c  cntr_md_n(3)  -  The NTSC moderate level contour values for the three
c                   TV bands low/high VHF UHF (values in dBu) (Grade A)
c  cntr_md_a(3)  -  The ATV moderate level contour values for the three
c                   TV bands low/high VHF UHF (values in dBu)
c  cntr_st_n(3)  -  The NTSC strong level contour values for the three
c                   TV bands low/high VHF UHF (values in dBu) (city grade)
c  cntr_st_a(3)  -  The ATV moderate level contour values for the three
c                   TV bands low/high VHF UHF (values in dBu)
c  prop_crv_des_n - Propagation curves used to compute desired signals
c                   from NTSC stations - acceptatble values are 10, 50
c                   and 90 (F50/10, F50/50, F50/90)
c  prop_crv_des_a - Propagation curves used to compute desired signals
c                   from ATV stations - acceptatble values are 10, 50
c                   and 90 (F50/10, F50/50, F50/90)
c  prop_crv_undes_n - Propagation curves used to compute undesired signals
c                     from NTSC stations - acceptatble values are 10, 50
c                     and 90 (F50/10, F50/50, F50/90)
c  prop_crv_undes_a - Propagation curves used to compute undesired signals
c                     from ATV stations - acceptatble values are 10, 50
c                     and 90 (F50/10, F50/50, F50/90)
c  use_xmit_ant_n - Logical variable switch indicates if a vertical antenna
c                   pattern is to be used when computing signals from
c                   NTSC stations
c  use_xmit_ant_a - Logical variable switch indicates if a vertical antenna
c                   pattern is to be used when computing signals from
c                   ATV stations
c  use_rec_ant_n - Logical variable switch indicates if a receive antenna
c                  pattern is to be used when computing IX to NTSC stations
c  use_rec_ant_a - Logical variable switch indicates if a receive antenna
c                  pattern is to be used when computing IX to ATV stations
c  use_ccir_rec_pat - Logical variable switch indicates if the FCC receive
c                     antenna patterns are to be used. (If false the
c                     PS/WP-3 pattern will be applied)  This switch is
c                     ignored for the respective computations if
c                     use_rec_ant_n or use_rec_ant_a is false.
c  use_fcc_xmit_pat - Logical variable switch indicates if the FCC xmit
c                     antenna patterns are to be used. (If false the
c                     PS/WP-3 pattern will be applied)  This switch is
c                     ignored for the respective computations if
c                     use_xmit_ant_n or use_xmit_ant_a is false.
c  use_n_radial_haat - Logical variable switch to indicate if
c                      computations using FCC propagation curves are
c                      to use the n standard radials for haat values
c                      If false then the single values specified in
c                      the TV Engineering data base will be used. The
c                      n standard radials are determined by using the
c                      terrain data file and the rcamsl specified in
c                      the data base.  The maximum number for n is 360
c                      the typical value is 8
c  use_da_n - Logical variable switch indicates if the directional
c             antenna patterns specified in the TV Engineering data
c             base are to be used when making computations of signals
c             from NTSC stations
c  use_da_a - Logical variable switch indicates if the directional
c             antenna patterns specified in the TV Engineering data
c             base are to be used when making computations of signals
c             from ATV stations
c  n_rads_used - Number of radials used to define noise limited
c                service contour of desired stations
c  n_std_rad - Number of evenly spaced radials used to compute haat
c              values when the single value specified in the TV
c              Engineering data base is not used
c  replicate - Logical switch indicates whether ATV stations are to
c              replicate the grade B contour of a paired NTSC station.
c
      logical use_xmit_ant_n, use_xmit_ant_a
      logical use_rec_ant_n, use_rec_ant_a
      logical use_ccir_rec_pat, use_fcc_xmit_pat
      logical use_n_radial_haat, use_da_n, use_da_a
      logical replicate
      common /runtime_menu/
     &   cntr_nl_n(3), cntr_nl_a(3), cntr_md_n(3), cntr_md_a(3),
     &   cntr_st_n(3), cntr_st_a(3), prop_crv_des_n, prop_crv_des_a,
     &   prop_crv_undes_n, prop_crv_undes_a, n_rads_used, n_std_rad,
     &   use_xmit_ant_n, use_xmit_ant_a, use_rec_ant_n, use_rec_ant_a,
     &   use_ccir_rec_pat, use_fcc_xmit_pat, use_n_radial_haat,
     &   use_da_n, use_da_a, replicate
c
c  Additional parameters specified in runtime input stream
c
      common /channels_considered/ ichan_ck(4, 25)
      common /max_analy_dist/ analysis_limits(4, 25)
      common /propagation_model/ model_for_prop
      common /protection_ratios/ n_off_nn(25), pr_nn(3, 25),
     &   n_off_an(25), pr_an(3, 25), n_off_na(25), pr_na(3, 25),
     &   n_off_aa(25), pr_aa(3, 25)
      common /receive_ant/ rgain_ntsc(3), rgain_atv(3),
     &   fb_ntsc(3), fb_atv(3), fac_k
      common /receive_ant_ccir/ ccir_rant_arcs(3, 4),
     &   ccir_rant_gain(3, 4)
      common /va_power_height_limits/ erp_va_limits(2, 3),
     &   haat_va_limits(2, 3)
      common /antenna_height_rec/ rec_ant_hgt
      common /atv_power_height_limits/ erp_atv_limits(2, 3),
     &   haat_atv_limits(2, 3)
      common /ntsc_power_height_limits/ erp_ntsc_limits(2, 3),
     &   haat_ntsc_limits(2, 3)
      common /xmit_ant/ xant_fac(2, 3, -2:13), x_ang(-2:13)
c
      common /dipole_factors/ dipole_adjustment(2:69)
      common /grid_cell_size/ cell_size, cell_diam
c
c  Parameter compute_haat, normally the same as use_n_radial_haat, above
c
      logical compute_haat
      common /comp_haat/ compute_haat
c
      logical abset
      common /fcc_xmit_ant/ tilt(3), theta_1(3), theta_2(3),
     &   volts_1(3), volts_2(3), aa(3, 2), bb(3, 2), abset
c
      integer undetermined(2, MAX_ENG_REC)
      logical ignore_kwx, test_switch, warn
      common /performance/ map_loads, n_lr_call, npath_pts,
     &   ignore_kwx, undetermined, run_start, test_switch, warn
c
      character routine_name*32
      common /current_routine/ routine_name
c
      character*48 tv_db_name, tv_da_db_name, fsc_file
      character*48 haat_db_name, population_db
      character*12 graph_filename
      common /data_base_files/ tv_db_name, tv_da_db_name,
     &   haat_db_name, population_db
      common /output_files/ iresult, graph_filename
c
      character*48 pop_scratch_file, ix_scratch_file, scratch_dir
      common /scratch_files/ ipop_scr, ix_scr, pop_scratch_file,
     &   ix_scratch_file, scratch_dir
c
      character country_r*4, city_r*20, state_r*2, call_r*8
      character antmnf_r*3, antmod_r*14, arn_with_prefix*14, sta_type*1
      logical replicated
      common /eng_db/ n_eng_rec, n_first_atv,
     &   erp_r(MAX_ENG_REC), capped_db(MAX_ENG_REC),
     &   haat_r(MAX_ENG_REC), ichan_r(MAX_ENG_REC),
     &   lat_site(MAX_ENG_REC), lon_site(MAX_ENG_REC),
     &   rcamsl_r(MAX_ENG_REC), ref_az_r(MAX_ENG_REC),
     &   front_to_back_ratio(MAX_ENG_REC), country_r(MAX_ENG_REC),
     &   city_r(MAX_ENG_REC), state_r(MAX_ENG_REC),
     &   call_r(MAX_ENG_REC), antmnf_r(MAX_ENG_REC),
     &   antmod_r(MAX_ENG_REC), arn_with_prefix(MAX_ENG_REC),
     &   sta_type(MAX_ENG_REC), replicated(MAX_ATV_STA)
c
      common /atv_proto_pwr_hght/ erp_atv_prot(3), haat_atv_prot(3)
c
      byte n_ant_az
      character da_string*17
      integer*2 ih_rad, ispecial_az, index_da, locator
      common /antenna_h_pat/ num_ant_pat, locator(MAX_ANT_REC),
     &   ih_rad(MAX_ANT_REC, 46), ispecial_az(MAX_ANT_REC, 10),
     &   index_da(MAX_ANT_REC), da_string(MAX_ANT_REC),
     &   n_ant_az(MAX_ANT_REC)
c
      integer current_antenna
      common /h_pat_data/ nn_az, ddbaz(46), ddbrad(46),
     &   current_antenna
c
      common /field_str_files/ fsc_file(9)
      common /crv_dat/ d50(25), h50(13), d10(30), h10(13),
     &   f55lv(25,13), f51lv(30,13), f55hv(25,13),
     &   f51hv(30,13), f55u(25,13), f51u(30,13),
     &   f59lv(25,13), f59hv(25,13), f59u(25,13)
c
      common /haat_index/ ihaat_file, n_haat_rec, az_inc_haat,
     &   index_haat(MAX_ENG_REC)
      common /haat_values/ n_haat_radials, last_haat, ref_rcamsl,
     &   radial_haat(MAX_RAD)
c
      character desired_file*48
      common /desired_list_file/ desired_file
c
      pointer (ptr_desired_stations, desired_stations)
      structure /STATION_LIST/
         integer id
         integer ichan
         character city*20
         character state*2
         character sta_type*1
         character fill*1
      end structure
      record /STATION_LIST/ desired_stations(*)
      common /desired_list/ n_des_sta, n_ntsc, next_ntsc,
     &   ptr_desired_stations
c
      character dir_list*32
      common /terrain_dir_loc/ n_dir, dir_list(MAX_DIR),
     &   lon_dir(MAX_DIR)
c
      integer*2 i_elev
      integer*2 index_75_ter
      integer*2 terrain_mem
      integer terrain_index
      common /ter_time_stamp/ i_time(MAX_TER_BLK), next_time
      common /terrain_indices/ index_75_ter(NUM_TERRAIN_INDEX_ELEM+4)
      common /terrain_mem_indices/ terrain_mem(LAST_IDX+1)
      common /terrain_7_5/ i_elev(MAX_TER_BLK, 22500), next_pt,
     &   terrain_index(MAX_TER_BLK)
c
      character type_prob_area*10, city_pa*20, state_pa*2, sta_type_pa*1
      common /prob_area_def/ ichan_pa, lat_pa(2, 3), lon_pa(2, 3),
     &   type_prob_area, city_pa, state_pa, sta_type_pa
c
      pointer (pcell_offsets, icell_offsets(*))
      integer*2 icell_offsets, icell_h, icell_w
      logical flagged_offsets
      common /problem_grid/ n_cells, flagged_offsets, pcell_offsets,
     &   lat_cg, lon_cg, lat_lim(2), lon_lim(2), icell_h, icell_w,
     &   num_cell_h, num_cell_w, cell_area
c
      pointer (ptr_census_blks, census_block_array)
      structure /CENSUS_BLOCKS/
         integer latsec
         integer lonsec
         integer ipop
      end structure
      record /CENSUS_BLOCKS/ census_block_array(*)
      common /blocks_of_interest/ num_blocks, ptr_census_blks
c
      pointer (ptr_pop_tot, i_pop_tot(n_cells))
      common /grid_cell_population/ ptr_pop_tot
c
      pointer (ptr_contour, current_contour)
      structure /CONTOUR/
         integer lat
         integer lon
         real distance
      end structure
      record /CONTOUR/ current_contour(*)
      common /standard_coverage/ n_cntr_rads, ptr_contour
c
      byte icell_service, outside_cells
      integer*2 icell_service_fields, icell_service_fades
      pointer (pcell_service, icell_service(*))
      pointer (pcell_service_fields, icell_service_fields(*))
      pointer (pcell_service_fades, icell_service_fades(*))
      pointer (ptr_outside_cells, outside_cells(*))
      common /service_grid/ pcell_service, pcell_service_fields,
     &   pcell_service_fades, ptr_outside_cells
c
      integer*2 sector_erp(MAX_ATV, NUM_ANTENNA_SECTORS)
      integer*2 margins(MAX_ATV, NUM_ANTENNA_SECTORS,
     &   -NUM_DB_INTERVALS:NUM_DB_INTERVALS+1)
      common /maximization_statistics/ margins, sector_erp
c
      pointer (ptr_pop_results, pop_results)
      pointer (ptr_cell_results, cell_results)
      structure /RESULTS/
         integer nl_tot
         integer nl_td_tot
         integer ix_loss_tot
         integer ix_ntsc_loss
         integer ix_atv_loss
         integer ix_atv_only_loss
      end structure
      record /RESULTS/ pop_results(*), cell_results(*)
      common /saved_results/ ptr_pop_results, ptr_cell_results,
     &   percent_area_match, percent_pop_match
c
      pointer (prad_desired, rad_desired)
      pointer (prad_undes, rad_undes)
      structure /RADIAL/
         real azimuth
         real distance
      end structure
      record /RADIAL/ rad_desired(*), rad_undes(*)
      common /cell_radials/ prad_desired, prad_undes
c
      common /undesired_srch_list/ search_dist_list(4, 68)
      common /ix_chans/ n_ixch, ixchans(25)
c
      pointer (pid_undesired, id_undesired(*))
      integer*2 id_undesired
      common /undesired_list/ n_undes_sta, n_unntsc, pid_undesired
c
      pointer (ptr_separations, separations(*))
      real separations
      common /station_separations/ ptr_separations
c
      integer*2 icell_ix_fields, icell_ix_fades
      pointer (pcell_ix_fields, icell_ix_fields(n_cells))
      pointer (pcell_ix_fades, icell_ix_fades(n_cells))
      common /ix_grid/ pcell_ix_fields, pcell_ix_fades
c
      integer matrix_svc(n_ntsc+1, (n_des_sta-n_ntsc)+1)
      pointer (ptr_matrix_svc, matrix_svc)
      common /ss_matrix/ ptr_matrix_svc
c
      common /lr_params_a/ pct_time_a, pct_location_a,
     &   pct_confidence_a
      common /lr_params_aix/ pct_time_aix, pct_location_aix,
     &   pct_confidence_aix
      common /lr_params_n/ pct_time_n, pct_location_n,
     &   pct_confidence_n
      common /lr_params_nix/ pct_time_nix, pct_location_nix,
     &   pct_confidence_nix
      common /confidence_fac/ zt, zl, zc
      common /error_limits/ error_min, error_max
      complex zgnd
      common /prop/ kwx, aref, mdp, dist, hg(2), wn, dh, ens, gme, zgnd, 
     &   he(2), dl(2), the(2)
      common /propa/ dlsa, dx, ael, ak1, ak2, aed, emd, aes, ems,
     &   dls(2), dla, tha
      common /propv/ lvar, sgc, mdvar, klim
      common /save/ dummy(50)
      common /terrain_path/ n_ter_pt, path_elev_pt(5000)
c
c  End of file global.inc
