c***********************************************************************
c
c                         read_haat
c
c  Read direct access file haat_db.dbs containing HAAT data for stations
c  of the TV Engineering Data Base.  The HAAT record for each station
c  contains data for azimuths regularly spaced with an increment given in
c  the first record of the file.  Subsequent records appear in the same
c  order as those of the file tv_main.dbs representing the engineering
c  data base at the time the HAAT file was prepared.
c
c  The tv_main.dbs version of the engineering data base used when
c  haat_db.dbs was created may be used to identify HAAT records.
c  Alternatively, the HAAT record for a specific station can be
c  identified by reference to the index file haat_db.idx.
c
c***********************************************************************
c
      character*48
     &   eng_db /'tv_main.dbs'/,
     &   haat_file  /'haat_db.dbs'/,
     &   haat_index /'haat_db.idx'/,
     &   text_file  /'haat_db.txt'/
c
      include 'global.inc'
      character line*43
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn*8
      common /mainrec/
     &   latsec, lonsec, erp, haat, rcamsl, ref_az, itype, istat,
     &   svc_cls, off, pol, da, beamt, zone, antmnf, antmod,
     &   ichan, cty, city, state, call, arn
      logical indexed, last_station, main
c
c  Open HAAT index file or engineering data base
c
      inquire(file=haat_index, exist=indexed)
      if (indexed) then
         call getlun(index_file)
         open(unit=index_file, file=haat_index, status='old',
     &      form='formatted', access='sequential', iostat=ierr)
         if (ierr .ne. 0) then
            call perror(haat_index)
            call exit(ierr)
         end if
      else
         inquire(file=eng_db, exist=main)
         if (.not. main) then
            write(0, '(5a)') 'Files ', haat_index(1:lnblnk(haat_index)),
     &         ' and ', eng_db(1:lnblnk(eng_db)), ' are both missing'
            call exit(257)
         end if
         call getlun(imain)
         open(unit=imain, file=eng_db, status='old', form='unformatted',
     &      access='direct', recl=99, iostat=ierr)
         if (ierr .ne. 0) then
            call perror(eng_db)
            call exit(ierr)
         end if
      end if
c
c  Find record length of unformatted HAAT file
c
      call getlun(ihaat_file)
      open(unit=ihaat_file, file=haat_file, status='old',
     &   form='unformatted', access='direct', recl=4, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(haat_file)
         call exit(ierr)
      end if
      read(ihaat_file, rec=1) az_inc
      close(ihaat_file)
c
c  Open unformatted HAAT file
c
      numrad = 360.0/az_inc
      if (real(numrad) .ne. 360.0/az_inc) then
         write(0, '(3a,f7.3)') 'Azimuth increment read from file ',
     &      haat_file, ' is', az_inc
         write(0, '(a)') 'Does not divide 360 degrees evenly'
         call exit(257)
      end if
c
      n_size = (numrad * 4) + 4
      call getlun(ihaat_file)
      open(unit=ihaat_file, file=haat_file, status='old',
     &   form='unformatted', access='direct', recl=n_size, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(haat_file)
         call exit(ierr)
      end if
c
c  Open output file
c
      call getlun(itext_file)
      open(unit=itext_file, file=text_file, status='new')
c
c  If the HAAT file is accompanied by an index, loop through that index
c  writing a text output line with HAAT values for each station listed.
c
      if (indexed) then
         irec = 0
   10    continue
         read(index_file, '(a)', end=20) line
         irec = irec + 1
         read(line, 1000) state, city, ichan
         read(ihaat_file, rec=irec+1, end=30) rcamsl,
     &      (radial_haat(k), k=1, numrad)
         if (numrad .eq. 72) then
            inc = numrad / 8
            write(itext_file, 1010) state, city(1:10), ichan,
     &         (radial_haat(1+k*inc), k=0, 7)
         else
            write(0, '(a)') 'No output format statement available'
            call exit(257)
         end if
         goto 10
c
c  End of index file should also be end of HAAT file
c
   20    continue
         irec = irec + 1
         read(ihaat_file, rec=irec+1, end=22) dummy
         write(0, '(a)') 'Index file is incomplete'
         call exit(257)
c
c  Normal exit for reading indexed HAAT records
c
   22    continue
         close(index_file)
         close(ihaat_file)
         close(itext_file)
         nrec = irec - 1
         write(0, '(i5,2a)') nrec, ' indexed records found in file ',
     &      haat_file(1:lnblnk(haat_file))
         stop	!Finished
c
c  More records in index than in data file
c
   30    continue
         write(0, '(3a)') 'Reached end of data file ',
     &      haat_file(1:lnblnk(haat_file)), ' before end of index'
         call exit(257)
      end if
c
c  No index file available to identify HAAT data.  Loop thru records of
c  HAAT file and identify station from the corresponding record of the
c  engineering data base.
c
      irec = 1
   90 continue
      read(ihaat_file, rec=irec+1, end=100) jdum,
     &   (radial_haat(k), k=1, numrad)
      call rd_main(imain, irec)
      last_station = city(1:11) .eq. 'end of file'
      if (last_station) then
         write(0, '(3a)') 'Reached end of file ',
     &      eng_db(1:lnblnk(eng_db)), ' before last HAAT record'
         call exit(257)
      end if
      if (numrad .eq. 72) then
         inc = numrad / 8
         write(itext_file, 1010) state, city(1:10), ichan,
     &      (radial_haat(k*inc), k=1, 8)
      else
         write(0, '(a)') 'No output format statement available'
         call exit(257)
      end if
      irec = irec + 1
      goto 90
c
c  End of HAAT file.  Close it, and save number of records found.
c
  100 continue
      close(ihaat_file)
      nrec = irec - 1
c
c  Finish reading engineering data base if longer than the HAAT file
c
      last_station = city(1:11) .eq. 'end of file'
      do while (.not. last_station)
         call rd_main(imain, irec)
         last_station = city(1:11) .eq. 'end of file'
         irec = irec + 1
      end do
      close(imain)
      close(itext_file)
      write(0, '(i5,2a)') nrec, ' records found in file ',
     &   haat_file(1:lnblnk(haat_file))
      write(0, '(i5,2a)') irec-2, ' records found in file ',
     &   eng_db(1:lnblnk(eng_db))
c
c1234567890123456789012345678901234567890123
cA CT BRIDGEPORT           411048 0731120 43
 1000 format(t3, a2, 1x, a20, t41, i3)
 1010 format(a, 1x, a, i3, 8f8.1)
      end
