      program read_tvdb
c
c  Make a formatted version of the binary engineering data base file.
c
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn_prefix*6, arn*8
      character line*136
      integer lat(3), long(3)
c
c  Default file names
c
      character*48
     &   eng_db      /'tv_main.dbs'/,
     &   text_output /'tv_main.txt'/
c
c  The name of the input binary data base file can be supplied as a
c  command line argument in place of the default name
c
      numargs = iargc()
      if (numargs .gt. 0) then
         if (numargs .ne. 1) stop 'Too many command line arguments'
         call getarg(1, eng_db)
         L = index(eng_db, '.')
         if (L .eq. 0) then
            text_output = eng_db // '.txt'
         else if (L .eq. 1) then
            text_output = 'txt'
         else
            text_output = eng_db(1:L) // 'txt'
         end if
      end if
c
c  Open direct access file
c
      call getlun(imain)
      open(imain, file=eng_db, status='old', access='direct', recl=99,
     &   iostat=ierr)
      if (ierr .ne. 0) then
         call perror(eng_db)
         stop
      end if
c
c  Open output file
c
      call getlun(iout)
      open(iout, file=text_output, status='new', access='sequential',
     &   iostat=ierr)
      if (ierr .ne. 0) then
         call perror(text_output)
         stop
      end if
c
c  Read record
c
      irec = 1
   10 continue
      read(imain, rec=irec, end=20, err=30)
     &   ichan, cty, city, state, call, latsec, lonsec,
     &   erp, haat, rcamsl, ref_az, itype, istat, svc_cls,
     &   off, pol, da, beamt, zone, antmnf, antmod, arn
      call hms(latsec, lat)
      call hms(lonsec, long)
      write(line, 900)
     &   ichan, cty, city, state, call, lat, long,
     &   erp, haat, rcamsl, ref_az, itype, istat, svc_cls,
     &   off, pol, da, beamt, zone, antmnf, antmod,
     &   arn_prefix(itype), arn
      write(iout, '(a)') line(1:lnblnk(line))
      irec = irec + 1
      goto 10
c
   20 continue
      numrec = irec - 1
      write(0, '(i5,2a)') numrec, ' formatted records written to file ',
     &   text_output(1:lnblnk(text_output))
      stop
c
   30 continue
      stop 'error reading input file'
c
  900 format(
     &   i2, 1x, a1, 1x, a20, 1x, a2, 1x, a8, 1x, 3i2.2, 1x,
     &   i3.3, 2i2.2, 3(1x, f7.2), 1x, f6.2, 1x, i2, 1x, i1,
     &   1x, a2, 4(1x, a1), 1x, 'Z', a1, 1x, a3, 1x, a14, 1x,
     &   a6, a8)
c40 A ANNISTON             AL WJSUTV   332441 0861223 5000.00  350.00  523.00   0.00 20 1 TV - H N Y Z2                      BPCT950808KF
c1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456
      end

      subroutine hms(total_seconds, components)
      integer components(3), total_seconds
      components(1) = total_seconds/3600
      components(2) = mod(total_seconds, 3600)/60
      components(3) = mod(total_seconds, 60)
      return
      end

      character*6 function arn_prefix(itype)
c
c  Retrieve the application reference number prefix found in the TV
c  Engineering Data Base from its identifying number.  The present
c  character function reverses the assignment of index number to string
c  made by program create_tvdb or program add_atv, and changes to
c  pertinent parts of the code for those programs must be reflected here.
c
c  File ENGdb_prefixes.inc defines the parameter NUM_PREFIXES and the
c  array prefix(1:NUM_PREFIXES).
c
      include 'ENGdb_prefixes.inc'
c
      if (itype .le. 0 .or. itype .gt. NUM_PREFIXES) then
         arn_prefix = '      '
      else
         arn_prefix = prefix(itype)
      end if
      return
      end
