      program refine_tvdb
c
c  Eliminate multiple records for the same city-channel combination
c  from the TV engineering data base, and optionally delete records for
c  city-channel combinations absent from a control list.  The control
c  list must be a file of the usual TVSTA or tvsta form.
c
c  To exercise the option, supply the control list of stations as a
c  command line argument, e.g. "refine_tvdb tvsta.master".
c
c  The country of entries in the control list is read as a capital letter
c  because player/non-player status has no significance in the TV
c  engineering data base.  Thus the control list will determine which
c  Canadian and Mexican stations are accepted, and both player and
c  non-player U.S. stations will be included in the output TV engineering
c  data base.
c
      parameter(
     &   MAX_ENG_REC = 4800,	!before adding ATV records
     &   MAXSTA = MAX_ENG_REC)	!Allow all to be retained
c
c  File names
c
      character*48
     &   eng_db /'eng_db.unf'/,
     &   refined_eng_db /'refined_eng_db.unf'/,
     &   control_list
c
c  Character variables in records of the engineering data base
c
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn*8
c
c  Composite strings with lengths a multiple of 4
c
      character params*32, name*32
c
c  Variables used to eliminate multiple records for same city-channel
c  combination
c
      character*23 current_city, unmatched_city
      integer current_channel, unmatched_channel
      logical multiple, repeated
c
c  Structure in a form which can be used with qsort
c
      structure /ENG_DB_RECORDS/
         integer ichan
         character*32 name
         integer latsec
         integer lonsec
         real erp
         real haat
         real rcamsl
         real ref_az
         integer itype
         integer istat
         character*32 params
      end structure
      record /ENG_DB_RECORDS/ struct_array(MAX_ENG_REC)
c
c  Arrays stations to be retained
c
      character*1 cty_array(MAXSTA)
      character*2 state_array(MAXSTA)
      character*20 city_array(MAXSTA)
      integer chan(MAXSTA)
c
c  Optional control list of city-channel combinations
c
      logical filter
      logical show_unselected /.false./
c
c  The presence of a command line argument indicates whether the option
c  is to be employed.
c
      numargs = iargc()
      filter = numargs .gt. 0
      if (filter) then
         if (numargs .ne. 1) then
            write(0, '(a)') 'Too many command line arguments'
            call exit(257)
         end if
         call getarg(1, control_list)
         call stations(control_list, MAXSTA, numsta, cty_array,
     &      state_array, city_array, chan)
      end if
c
c  Open direct access file
c
      call getlun(imain)
      open(imain, file=eng_db, status='old', access='direct', recl=99,
     &   iostat=ierr)
      if (ierr .ne. 0) then
         call perror(eng_db)
         call exit(ierr)
      end if
c
c  Read engineering data base records
c
      irec = 0
   10 continue
      irec = irec + 1
      read(imain, rec=irec, end=20, err=900)
     &   ichan, cty, city, state, call, latsec, lonsec,
     &   erp, haat, rcamsl, ref_az, itype, istat, svc_cls,
     &   off, pol, da, beamt, zone, antmnf, antmod, arn
      if (irec .gt. MAX_ENG_REC) then
         write(0, '(a)') 'Too many records in engineering data base'
         call exit(257)
      end if
c
c  Compose string of station parameters and identifiers
c
      name = cty//city//state//call
      params = svc_cls//off//pol//da//beamt//zone//antmnf//antmod//arn
c
c  Load structure
c
      struct_array(irec).ichan = ichan
      struct_array(irec).name = name
      struct_array(irec).latsec = latsec
      struct_array(irec).lonsec = lonsec
      struct_array(irec).erp = erp
      struct_array(irec).haat = haat
      struct_array(irec).rcamsl = rcamsl
      struct_array(irec).ref_az = ref_az
      struct_array(irec).itype = itype
      struct_array(irec).istat = istat
      struct_array(irec).params = params
      goto 10
c
   20 continue
      numrec = irec - 1
      close(imain)
c
c  Sort
c
      write(0, '(a,i4,a)') 'Sorting ', numrec, ' records'
      call sort_struct(struct_array, numrec)
c
c  Retrieve sorted records and re-write binary file
c
      call getlun(jmain)
      open(jmain, file=refined_eng_db, status='new', access='direct',
     &   recl=99, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(refined_eng_db)
         call exit(ierr)
      end if
      current_channel = 0
      current_city = ' '
      unmatched_channel = 0
      unmatched_city = ' '
c
      newrec = 0
      num_unmatched = 0
      do i=1, numrec
         ichan = struct_array(i).ichan
         name = struct_array(i).name
c
         cty = name(1:1)
         city = name(2:21)
         state = name(22:23)
         call = name(24:31)
c
         latsec = struct_array(i).latsec
         lonsec = struct_array(i).lonsec
         erp = struct_array(i).erp
         haat = struct_array(i).haat
         rcamsl = struct_array(i).rcamsl
         ref_az = struct_array(i).ref_az
         itype = struct_array(i).itype
         istat = struct_array(i).istat
         params = struct_array(i).params
c
         svc_cls = params(1:2)
         off = params(3:3)
         pol = params(4:4)
         da = params(5:5)
         beamt = params(6:6)
         zone = params(7:7)
         antmnf = params(8:10)
         antmod = params(11:24)
         arn = params(25:32)
c
         multiple = ichan .eq. current_channel .and.
     &      name(1:23) .eq. current_city
         if (.not. multiple) then
            include = .not. filter .or. match(struct_array(i), numsta,
     &         cty_array, state_array, city_array, chan)
            if (include) then
               newrec = newrec + 1
               struct_array(newrec) = struct_array(i)
               write(jmain, rec=newrec) ichan, name(1:31), latsec,
     &            lonsec, erp, haat, rcamsl, ref_az, itype, istat,
     &            params
               current_channel = ichan
               current_city = name(1:23)
            else
               repeated = ichan .eq. unmatched_channel .and.
     &            name(1:23) .eq. unmatched_city
               if (.not. repeated) then
                  num_unmatched = num_unmatched + 1
                  if (show_unselected) write(*, '(4(a,1x),i2)')
     &               'unmatched:', cty, state, city, ichan
                  unmatched_channel = ichan
                  unmatched_city = name(1:23)
               end if
            end if
         end if
      end do
c
c  Finished
c
      close(jmain)
      write(0, '(a,3(i5,a))') 'refine:', numrec, ' records in; ',
     &   num_unmatched, ' unmatched;', newrec, ' out'
      write(0, '(i5,2a)') newrec, ' records written to file ',
     &   refined_eng_db(1:lnblnk(refined_eng_db))
      stop
c
  900 continue
      write(0, '(2a)') 'error reading file', eng_db(1:lnblnk(eng_db))
c
  903 format(
     &   i2, 1x, a1, 1x, a20, 1x, a2, 1x, a8, 2(1x, i6), 3(1x, f7.2),
     &   1x, f6.3, 1x, i2, 1x, i1, 1x, a2, 5(1x, a1), 1x, a3, 1x, a14,
     &   1x, a8)
      end

      subroutine stations(filename, maxsta, numsta, cty, state,
     &   city, chan)
      character*(*) filename
      character*1 cty(maxsta)
      character*2 state(maxsta)
      character*20 city(maxsta)
      character record*72
      integer chan(maxsta)
c
      call getlun(lun)
      open(lun, file=filename, status='old', access='sequential',
     &   iostat=ierr)
      if (ierr .ne. 0) then
         call perror(filename)
         call exit(ierr)
      end if
c
      numsta = 0
  100 continue
      read(lun, '(a)', end=200) record
      if (record(1:1) .eq. ' ') then
         write(0, '(a)') record(1:lnblnk(record))
         goto 100
      end if
      numsta = numsta + 1
      if (numsta .gt. maxsta) then
         write(0, '(2a)') 'Too many entries in ', filename
         call exit(257)
      end if
      length = lnblnk(record)
      if (length .eq. 66 .or. length .eq. 69) then
         read(record, 1000, err=900) cty(numsta), state(numsta),
     &      city(numsta), chan(numsta)
      else if (length .eq. 43 .or. length .eq. 46) then
         read(record, 1001, err=900) cty(numsta), state(numsta),
     &      city(numsta), chan(numsta)
      else
         goto 900
      end if
      call makcap(cty(numsta))
      goto 100
c
c  Finished
c
  200 continue
      close(lun)
      return
c
  900 continue
      write(0, '(3a/a)') 'Bad record in ', filename, ':',
     &   record(1:length)
      call exit(257)
c
c123456789012345678901234567890123456789012345678901234567890123456789
cA CA SAN DIEGO             OP      Z2 0000875538 324146 1165607 15 25
cA CT BRIDGEPORT           411048 0731120 43 25
 1000 format(a1, t3, a2, t6, a20, t65, i2)
 1001 format(a1, t3, a2, t6, a20, t42, i2)
      end

      subroutine sort_struct(struct_array, num_elements)
      integer num_elements
      structure /ENG_DB_RECORDS/
         integer ichan
         character*32 name
         integer latsec
         integer lonsec
         real erp
         real haat
         real rcamsl
         real ref_az
         integer itype
         integer istat
         character*32 params
      end structure
      record /ENG_DB_RECORDS/ struct_array(num_elements)
      external compare_struct
      integer*2 compare_struct
      integer struct_size
      struct_size = 100
      call qsort(struct_array, num_elements, struct_size,
     &   compare_struct)
      return
      end

      integer*2 function compare_struct(x, y)
c
c  Specify order for qsort of engineering data base records
c
      structure /ENG_DB_RECORDS/
         integer ichan
         character*32 name
         integer latsec
         integer lonsec
         real erp
         real haat
         real rcamsl
         real ref_az
         integer itype
         integer istat
         character*32 params
      end structure
      record /ENG_DB_RECORDS/ x, y
      character*1 cty_x, cty_y
      character*20 city_x, city_y
      character*2 state_x, state_y
      character*8 arn_x, arn_y
c
      cty_x = x.name(1:1)
      cty_y = y.name(1:1)
      city_x = x.name(2:21)
      city_y = y.name(2:21)
      state_x = x.name(22:23)
      state_y = y.name(22:23)
      arn_x = x.params(25:32)
      arn_y = y.params(25:32)
      if (cty_y .eq. cty_x) then
         if (state_y .eq. state_x) then
            if (city_y .eq. city_x) then
               if (y.ichan .eq. x.ichan) then
                  if (y.istat .eq. x.istat) then
                     if (arn_y .eq. arn_x) then
                        compare_struct = 0
                     else if (arn_y .gt. arn_x) then
                        compare_struct = -1
                     else
                        compare_struct =  1
                     end if
                  else
                     compare_struct = sign(1, x.istat - y.istat)
                  end if
               else
                  compare_struct = sign(1, x.ichan - y.ichan)
               end if
            else if (city_y .gt. city_x) then
               compare_struct = -1
            else
               compare_struct =  1
            end if
         else if (state_y .gt. state_x) then
            compare_struct = -1
         else
            compare_struct =  1
         end if
      else if (cty_y .gt. cty_x) then
         compare_struct = -1
      else
         compare_struct =  1
      end if
      return
      end

      logical function match(db_rec, numsta, cty, state, city, chan)
      structure /ENG_DB_RECORDS/
         integer ichan
         character*32 name
         integer latsec
         integer lonsec
         real erp
         real haat
         real rcamsl
         real ref_az
         integer itype
         integer istat
         character*32 params
      end structure
      record /ENG_DB_RECORDS/ db_rec
      character*1 cty(numsta)
      character*2 state(numsta)
      character*20 city(numsta)
      integer chan(numsta)
c
      character db_cty*1, db_state*2, db_city*20, name*32
c
      ichan = db_rec.ichan
      name = db_rec.name
      db_cty = name(1:1)
      db_city = name(2:16)
      db_state = name(22:23)
      do n=1, numsta
         match =
     &      cty(n) .eq. db_cty .and.
     &      state(n) .eq. db_state .and.
     &      city(n)(1:15) .eq. db_city .and.
     &      chan(n) .eq. ichan
         if (match) return
      end do
      return
      end
