      program write_tvdb
c
c  Create a binary engineering data base file from a text version.
c  The text version must be in the format produced by program read_tvdb.
c
c  If the text version is compatible with the replication option of
c  program flr, a list of changes to DTV allotments will be read from
c  stdin.  These changes will then be made to both the text version and
c  the binary file.
c
c  Expected format of the list of changes:
c
c                            6th
c     State and City    NTSC R&O  Change
c     --------------    ---- ---  ------
c     AL ANNISTON        40   58   53
c     CA LOS ANGELES      2   60   42
c     1234567890123456789012345678901
c
c  To make a list of changes, cut the first 29 characters from lines
c  of the Allotment Table of the Sixth R&O (cut -c1-29 tbl) and then
c  add a column of alternate channels.
c
      parameter(MAXCHG = 2000)
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, prefix*6, arn*8
      character command*48
      character line*136		!lines of text form of engineering data base
      character*26 sorted_list(MAXCHG)	!list of changes will be sorted
      character*21 pattern		!state, city and DTV channel (before change)
      integer lat(3), long(3)
      logical dtv_record
c
c  Default file names
c
      character*48
     &   text_input /'tv_main.txt'/,
     &   eng_db     /'tv_main.dbs'/
c
c  Retrieve name under which executed to determine whether to read
c  changes to DTV allotments from stdin.
c
      call getarg(0, command)
      if (command .eq. 'revise_tvdb') then
         call revisions(MAXCHG, num_changes, sorted_list)
         write(0, '(i5,a)') num_changes, ' changes proposed'
      end if
      num_diff = 0
c
c  The name of the input text file can be supplied as a command line
c  argument in place of the default name.  If, for example, the input
c  text file is tv_mods.txt, an output binary file named tv_mods.dbs
c  will be produced.
c
      numargs = iargc()
      if (numargs .gt. 0) then
         if (numargs .ne. 1) stop 'Too many command line arguments'
         call getarg(1, text_input)
         L = index(text_input, '.')
         if (L .eq. 0) then
            eng_db = text_input // '.dbs'
         else if (L .eq. 1) then
            eng_db = 'dbs'
         else
            eng_db = text_input(1:L) // 'dbs'
         end if
      end if
c
c  Open input file
c
      call getlun(in)
      open(in, file=text_input, status='old', access='sequential',
     &   iostat=ierr)
      if (ierr .ne. 0) then
         call perror(text_input)
         stop
      end if
c
c  Open direct access file
c
      call getlun(imain)
      open(imain, file=eng_db, status='new', access='direct', recl=99,
     &   iostat=ierr)
      if (ierr .ne. 0) then
         call perror(eng_db)
         stop
      end if
c
c  Read record
c
      irec = 1
   10 continue
      read(in, '(a)', end=20) line
      read(line, 900, err=30)
     &   ichan, cty, city, state, call, lat, long,
     &   erp, haat, rcamsl, ref_az, itype, istat, svc_cls,
     &   off, pol, da, beamt, zone, antmnf, antmod, prefix, arn
      latsec = lat(3) + 60*(lat(2) + 60*lat(1))
      lonsec = long(3) + 60*(long(2) + 60*long(1))
c
c  Modify DTV records per instructions read from stdin
c
      dtv_record = call(1:1) .eq. 'D'
      if (dtv_record .and. num_changes .gt. 0) then
         write(pattern, 902) state, city, ichan
         item = item_number(pattern, sorted_list, num_changes)
         if (item .ne. 0) then
            if (erp .ne. -1.0) then
               write(0, '(3a)') 'File ',
     &            text_input(1:lnblnk(text_input)),
     &            ' is not in form suitable for change of DTV channels.'
               call system('/bin/rm ' // eng_db(1:lnblnk(eng_db)))
               stop
            end if
            read(sorted_list(item), 904) jchan
            if (jchan .ne. ichan) then
               ichan = jchan
               num_diff = num_diff + 1
            end if
         end if
      end if
c
c  Write to binary file
c
      write(imain, rec=irec)
     &   ichan, cty, city, state, call, latsec, lonsec,
     &   erp, haat, rcamsl, ref_az, itype, istat, svc_cls,
     &   off, pol, da, beamt, zone, antmnf, antmod, arn
      irec = irec + 1
      goto 10
c
   20 continue
      numrec = irec - 1
      write(0, '(i5,2a)') numrec, ' records converted to binary file ',
     &   eng_db(1:lnblnk(eng_db))
      if (num_changes .gt. 0)
     &   write(0, '(i5,a)') num_diff, ' changes made'
      stop
c
   30 continue
      write(0, '(a)') line
      write(0, '(a)') 'error reading input file'
      call exit(257)
c
  900 format(
     &   i2, 1x, a1, 1x, a20, 1x, a2, 1x, a8, 1x, 3i2.2, 1x,
     &   i3.3, 2i2.2, 3(1x, f7.2), 1x, f6.2, 1x, i2, 1x, i1,
     &   1x, a2, 4(1x, a1), 1x, 'Z', a1, 1x, a3, 1x, a14, 1x, a6, a8)
c40 A ANNISTON             AL WJSUTV   332441 0861223 5000.00  350.00  523.00   0.00 20 1 TV - H N Y Z2                      BPCT950808KF
c1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456
c58 A ANNISTON             AL DWJSUTV  332441 0861223  253.20  350.00  523.00   0.00 33 0 TV   H Y Y Z2 rep ALANNISTON__58   BDTV00000000
  902 format(a2, 1x, a15, 1x, i2)
cAL ANNISTON        58   59
c12345678901234567890123456
  904 format(t25, i2)
      end

      integer function item_number(pattern, sorted_list, list_length)
      character*(*) pattern, sorted_list(list_length)
      integer first, pattern_len
      pattern_len = len(pattern)
      if (pattern_len .gt. len(sorted_list(1))) stop
     &   'Pattern longer than items in list'
      first = 1
      last = list_length
      do while (first .le. last)
         i = (last + first) / 2
         if (pattern .eq. sorted_list(i)(1:pattern_len)) then
            item_number = i
            return
         else if (pattern .lt. sorted_list(i)(1:pattern_len)) then
            last = i - 1
         else
            first = i + 1
         end if
      end do
      item_number = 0
      return
      end

      subroutine revisions(maxchg, num_changes, sorted_list)
      character*(*) sorted_list(*)
      character*31 change
      integer*2 compare_strings
      external compare_strings
      data list_width /26/
      if (len(sorted_list(1)) .ne. list_width) then
         write(0, '(a)') 'Revisions: inconsistent string lengths'
         stop
      end if
      num_changes = 0
   10 continue
      read(*, '(a)', end=20) change
      num_changes = num_changes + 1
      if (num_changes .gt. maxchg) stop
     &   'Too many changes'
      sorted_list(num_changes) = change(1:19) // change(25:31)
      goto 10
   20 continue
      call qsort(sorted_list, num_changes, list_width,
     &   compare_strings)
      return
c
c  Input lines are expected to include the NTSC channel and be 31
c  characters long.  The list of changes excludes the NTSC channel
c  and is 26 characters long.
c
cAL ANNISTON        40   58   53
cAL ANNISTON        58   53
c1234567890123456789012345678901
      end

      integer*2 function compare_strings(x, y)
      character*(*) x, y
      if (x .eq. y) then
         compare_strings = 0
      else if (x .gt. y) then
         compare_strings = 1
      else
         compare_strings = -1
      end if
      return
      end
