c  %W% %G%
c
c***********************************************************************
c
c                         main_flr
c
c  This is the main program for multi-station analysis of TV service and
c  interference.
c
c***********************************************************************
c
      program main_flr
      include 'global.inc'
c
c  Default data may be changed by options.inc
c
      integer ianalysis_type /1/
      integer model_for_prop /1/
      data scratch_dir /'~'/
      data adjust_du /.true./
      data calculate_area /.true./
      data cap_by_truncation /.false./
      data correlated_fades /.false./
      data per_6th_order /.true./
      data analyze_tilt /.false./
      data flagged_offsets /.true./
      data pairwise /.true./
      data ignore_kwx /.false./
      data dipole_adj /.true./
      data sideband_filter /.true./
      data test_switch /.false./
      data warn /.false./
      data error_min /0.1/
      data error_max /3.0/
      logical handle /.false./
      logical margins_file /.false./
c
      external handler
c
c  Local variables.  Default names of input and results files, and
c  the default value for d_inc, may be changed by options.inc.
c
      character*48 input_file /'stdin'/, results_file /'results'/
      character fdate*24, host*16
      character time_string*10
      integer hostnm
      logical applied	!sideband filter to reduce A-to-A adj channel IX
      logical conflict	!scratch files already in use
      real d_inc /1.0/	!distance increment along radials in km
      real time_array(2)
c
      include 'options.inc'
c
c  Equate routine_name for error reporting
c
      routine_name = 'main_flr'
c
c  Get name of host machine
c
      istat = hostnm(host)
c
c  Set IEEE floating point exception handlers
c
      if (handle) then
         ieeer1 = ieee_handler('set', 'division', handler)
         ieeer2 = ieee_handler('set', 'overflow', handler)
         ieeer3 = ieee_handler('set', 'invalid', handler)
      end if
c
c  Find a place to write scratch files
c
      call set_scratch_filenames(conflict)
      if (.not. conflict) call lock_scratch_directory
c
c  Set unit numbers to indicate that scratch files have not been opened
c  by this program
c
      ipop_scr = 0
      ix_scr = 0
      graph_filename = ' '	! indicates no previous file to be closed
c
c  Set pointers to indicate no memory allocations
c
      ptr_desired_stations = 0
      ptr_census_blks = 0
      pcell_offsets = 0
      ptr_pop_results = 0
      ptr_cell_results = 0
      ptr_contour = 0
      ptr_separations = 0
      prad_desired = 0
      prad_undes = 0
      pid_undesired = 0
      pcell_ix_fields = 0
      ptr_matrix_svc = 0
c
c  Check propagation model number.  Number 1 is the Longley-Rice model,
c  at present the only model available for this program.

      if (model_for_prop .ne. 1) stop 'Unrecognized propagation model'
      n_lr_call = 0
c
c  Check type of analysis
c
      if (ianalysis_type .lt. 1 .or. ianalysis_type .gt. 2)
     &   stop 'Unrecognized analysis type'
      if (pairwise) ianalysis_type = 1
c
c  Identify program options set at compile time
c
      write(0, 1000) d_inc, flagged_offsets, calculate_area,
     &   correlated_fades, ignore_kwx, sideband_filter, adjust_du,
     &   dipole_adj, cap_by_truncation, per_6th_order, analyze_tilt,
     &   pairwise, graphs, margins_file, new_dadb
      if (.not. per_6th_order) write(0, 1002) mod1, mod2, mod3, mod4
      write(0, '(4a)') 'Scratch files are ',
     &   ix_scratch_file(1:lnblnk(ix_scratch_file)), ' and ',
     &   pop_scratch_file(1:lnblnk(pop_scratch_file))
c
c  Go no farther if scratch files are unavailable
c
      if (conflict) stop 'Conflict with existing scratch files'
c
c  Open output file
c
      if (results_file .eq. 'stdout') then
         iresult = 6
      else
         call getlun(iresult)
         open(iresult, file=results_file, status='new',
     &      access='sequential', iostat=ierr)
         if (ierr .ne. 0) then
            call perror(results_file)
            stop 'Unable to create file for results'
         end if
      end if
c
c  Load input run stream from text file
c
      call read_input(input_file, ierr)
      if (ierr .ne. 0) then
         if (iresult .ne. 6) close(iresult, status='delete')
         if (ierr .eq. -500) stop	!No data supplied
         stop 'Error reading input file'
      end if
c
c  Check compatibility of runtime and compile time options
c
      if (.not. replicate .and. new_dadb) then
         write(0, '(a)')
     &      'Create DTV antenna patterns only when replicating'
         call exit(257)
      end if
c
c  Miscellaneous initializations
c
      call sideband_mask(applied)
      if (applied) write(iresult, 1005) pr_aa(1, 2), pr_aa(1, 9)
      call compute_dipole_adj
      compute_haat = use_n_radial_haat
      last_haat = 0
      cell_diam = sqrt(2.0)*cell_size
      xant_fac(1, 1, -2) = 0.0	!Vertical transmit pattern not yet symmetrized
c
c  Begin timing
c
      write(iresult, 1010) fdate(), host(1:lnblnk(host))
      call flush(iresult)
      run_start = etime(time_array)
c
c  Load engineering data, desired station list, and terrain file index
c
      call main_load
      if (pairwise) last_ntsc = n_ntsc + 2
c
c  Initialize arrays of ATV power and margins
c
      if (margins_file) then
         call initialize_sectors
         call initialize_margins
      end if
c
c  Define problem area grid and determine service conditions
c
  100 continue
      call main_desired(d_inc)
c
c  Calculate undesired fields
c
      call main_undesired(d_inc)
c
c  Store distances between desired and undesired stations
c
      call separation_distances
c
c  Evaluate margins, service and interference
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Evaluating service and interference', hours, 0)
      if (margins_file) call margin_eval
      call service_eval
c
c  Create service matrix
c
      if (ianalysis_type .eq. 2) call create_svc_matrix
c
c  Print results
c
      call print_results
      if (ianalysis_type .eq. 2) call display_matrix(iresult)
c
c  If in pairwise mode, repeat the evaluation procedure for each NTSC
c  station of the input list and its paired ATV
c
      if (pairwise .and. next_ntsc .lt. last_ntsc) then
         next_ntsc = next_ntsc + 1
         call load_next_pair
         call release_offsets_mem
         call release_svc_mem
         call release_results_mem
         call release_unds_mem
         call release_pop_mem(ptr_pop_tot)
         call release_separations
         goto 100
      end if
      if (margins_file) call write_margins_file
c
c  Finished
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Finished ', hours, 0)
      write(time_string, 1100) int(hours),
     &   int(60*mod(hours, 1.0)),
     &   int(60*mod(60*mod(hours, 1.0), 1.0))
      write(iresult, 1102) fdate(), time_string
      write(iresult, 1104) n_lr_call, d_inc
c
 1000 format('Terrain profile distance increment =', f6.3, ' km'/
     &   'Flagged offsets =  ', l2, t25,
     &      'No further evaluation of cells empty of people'/
     &   'Calculate area =   ', l2, t25,
     &      'Include empty cells in coverage area'/
     &   'Correlated fades = ', l2, t25,
     &      'Use statistical formula for A-to-A IX'/
     &   'Ignore kwx =       ', l2, t25,
     &      'Reset Longley-Rice error flag of 3 to 0'/ t25,
     &      '(A non-zero error flag causes cells to be'/ t25,
     &      'treated as served with no interference)'/
     &   'Sideband filters = ', l2, t25,
     &      'Assume transmitter filters reduce out-of-band splatter'/
     &   'Adjust D/U ratios =', l2, t25,
     &      'Include degradation due to noise in ATV reception'/
     &   'Dipole factor =    ', l2, t25,
     &      'Use in computing noise-limited contours'/
     &   'Cap by truncation =', l2, t25,
     &      'Opposite of scaling ERPs around the compass'/
     &   'Per_6th_order =    ', l2, t25,
     &      'Bypass program changes since Sixth MO&O'/
     &   'Analyze_tilt  =    ', l2, t25,
     &      'Enable shift of vertical pattern as function of azimuth'/
     &   'Pairwise =         ', l2, t25,
     &      'Examine NTSC and paired ATV station together'/
     &   'Graphs =           ', l2, t25,
     &      'Create file for plotting service and IX areas'/
     &   'Margins file =     ', l2, t25,
     &      'Create file for exploring maximization potential'/
     &   'New dadb =         ', l2, t25,
     &      'Create directional antenna data base for ATV'/ t25
     &      'based on replication')
 1002 format(
     &   'Mod1 =             ', l2, t25,
     &      'Use fringe effect only for co-channel IX to DTV'/
     &   'Mod2 =             ', l2, t25,
     &      'Set resonable value for field near transmitter'/
     &   'Mod3 =             ', l2, t25,
     &      'Extend terrain profile within half d_inc of receiver'/
     &   'Mod4 =             ', l2, t25,
     &      'Calculate depression angles from heights AMSL')
 1005 format('Sideband masking assumed to improve first-adjacent ',
     &   'A-to-A D/U ratios'/
     &   '                   D/U Ratios in dB'/
     &   '           ------------------------------------'/
     &   '  Channel  Intital     Including   with assumed'/
     &   '  Offset   Testing     Splatter    improvement'/
     &   '  ------   -------     ---------   ------------'/
     &   '   +1      -43.17       -21.15     ', f8.2/
     &   '   -1      -41.98       -22.83     ', f8.2)
 1010 format('Run begins ', a24, ', host ', a)
c
 1100 format(i4, ':', i2.2, ':', i2.2)
 1102 format(/'Finished ', a19, '; run time ', a)
 1104 format(i10, ' calls to Longley-Rice; path distance increment',
     &   f6.2, ' km')
      end
c  %W% %G%
c
c***********************************************************************
c
c                          allocate_cntr_mem
c  
c  This routine sets the number of radials to be used when computing
c  station contours and allocates the appropriate amount of memory to
c  store the geographic coordinates of the end points of these radials
c  and their length.  The number of evenly spaced radials to be used is
c  an input argument, and the pointer to the allocated memory is saved
c  in global common.
c
c  The end-point latitude and longitude and its distance from the
c  station is stored for each radial in a 12-byte structure.
c
c***********************************************************************
c
      subroutine allocate_cntr_mem(n_radials)
      include 'global.inc'
c
      character routine*32 /'allocate_cntr_mem'/
      character pointer_name /'ptr_contour'/
c
c  Store the number of radials to be used in global common.
c
      n_cntr_rads = n_radials
c
c  The pointer should be initialized to zero before the first use of
c  this subprogram and released before subsequent calls.
c
      if (ptr_contour .ne. 0)
     &   call report_mem_error_1(routine, pointer_name)
c
c  Allocate memory
c
      n = n_radials * 12
      ptr_contour = malloc(n)
      if (ptr_contour .eq. 0)
     &   call report_mem_error(routine, pointer_name, n)
      return
c
c  Entry point to free the memory allocated above
c
      entry release_cntr_mem
      if (ptr_contour .eq. 0) then
         pointer_name = 'ptr_contour'
         call report_mem_error_2(routine, pointer_name)
      end if
      call free(ptr_contour)
      ptr_contour = 0
      end
c  %W% %G%
c
c***********************************************************************
c
c                            allocate_ds_mem
c  
c  This routine allocates the appropriate amount of memory to store the
c  city, state, channel and station type (NTSC or ATV) of the desired
c  stations of interest and their record numbers in the engineering data
c  base.  The STATION_LIST structure holds this data for individual
c  stations, and the memory allocation is for an array of these
c  structure.  The pointer is stored in global common.
c
c***********************************************************************
c
      subroutine allocate_ds_mem
      include 'global.inc'
c
      character pointer_name*32 /'ptr_desired_stations'/
      character routine*32 /'allocate_ds_mem'/
c
c  The pointer should have been initialized to zero before its first
c  use and released before subsequent calls to this subprogram.
c
      if (ptr_desired_stations .ne. 0)
     &   call report_mem_error_1(routine, pointer_name)
c
c  Allocate memory
c
      ptr_desired_stations = malloc(n_des_sta * 32)
      if (ptr_desired_stations .eq. 0)
     &   call report_mem_error(routine, pointer_name, n_des_sta)
      return
c
c  Entry for freeing memory holding the list of desired stations.  The
c  associated pointer is kept in global common.
c
      entry release_ds_mem
      if (ptr_desired_stations .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(ptr_desired_stations)
      ptr_desired_stations = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        allocate_matrix_mem
c
c  This routine allocates memory to hold results for the spread sheet
c  output of the the FCC terrain dependent coverage and interference
c  analysis.  The pointer is stored in global common.
c
c***********************************************************************
c
      subroutine allocate_matrix_mem
      include 'global.inc'
c
      character routine*32 /'allocate_matrix_mem'/
      character pointer_name*32 /'ptr_matrix_svc'/
c
c  The pointer should have been initialized to zero before the first
c  call of this subprogran and released before subsequent calls.
c
      if (ptr_matrix_svc .ne. 0)
     &   call report_mem_error_1(routine, pointer_name)
c
c  Allocate memory
c
      n = 4 * ((n_ntsc + 1) * ((n_des_sta - n_ntsc) + 1))
      ptr_matrix_svc = malloc(n)
      if (ptr_matrix_svc .eq. 0)
     &   call report_mem_error(routine, pointer_name, n)
      return
c
c  Entry point for freeing the memory allocated above
c
      entry release_matrix_mem
      if (ptr_matrix_svc .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(ptr_matrix_svc)
      ptr_matrix_svc = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         allocate_offsets_mem
c  
c  Allocate memory for an array of latitude and longitude offsets from
c  grid center.  The offsets (in units of arc seconds) are integer*2
c  values so the total memory requirement in bytes is 4 times the number
c  of cells.
c
c  This routine must find the number of cells in global common and save
c  the pointer there.
c
c***********************************************************************
c
      subroutine allocate_offsets_mem
      include 'global.inc'
c
      character pointer_name*32 /'pcell_offsets'/
      character routine*32 /'allocate_offsets_mem'/
c
c  The pointer should have been initialized to zero before its first
c  use and released before subsequent calls to this subprogram.
c
      if (pcell_offsets .ne. 0)
     &   call report_mem_error_1(routine, pointer_name)
c
c  Allocate memory
c
      n = 4 * n_cells
      pcell_offsets = malloc(n)
      if (pcell_offsets .eq. 0)
     &   call report_mem_error(routine, pointer_name, n)
      return
c
c  Entry point for freeing the memory allocated above
c
      entry release_offsets_mem
      if (pcell_offsets .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(pcell_offsets)
      pcell_offsets = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         allocate_pop_block_mem
c  
c  Allocate memory for an array of structures for storing census block
c  data.  The number of array elements needed is an input argment, and
c  a pointer to the array is returned.
c
c  Census blocks are stored in structures so they can be handled by
c  qsort.
c
c***********************************************************************
c
      subroutine allocate_pop_block_mem(p_struct, num_blocks)
      pointer (p_struct, census_block_array)
      structure /CENSUS_BLOCKS/
         integer latsec
         integer lonsec
         integer ipop
      end structure
      record /CENSUS_BLOCKS/ census_block_array(*)
c
      character pointer_name*32 /'p_struct'/
      character routine*32 /'allocate_pop_block_mem'/
c
c  The pointer should have been initialized to zero before its first
c  use and released before subsequent calls to this subprogram.
c
      if (p_struct .ne. 0)
     &   call report_mem_error_1(routine, pointer_name)
c
c  Allocate memory
c
      n = 12 * num_blocks
      p_struct = malloc(n)
      if (p_struct .eq. 0)
     &   call report_mem_error(routine, pointer_name, n)
      return
c
c  Entry point for freeing the memory allocated above
c
      entry release_pop_block_mem(p_struct)
      if (p_struct .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(p_struct)
      p_struct = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                       allocate_pop_mem
c  
c  This routine allocates memory for cell population data.  The number
c  of array elements needed, num_elem, is an input.  The pointer to the
c  allocated memory is returned in ptr_pop_tot.
c
c***********************************************************************
c
      subroutine allocate_pop_mem(num_elem, ptr_pop_tot)
c
      pointer (ptr_pop_tot, i_pop_tot(*))
      character pointer_name*32 /'ptr_pop_tot'/
      character routine*32 /'allocate_pop_mem'/
c
c  The pointer should have been initialized to zero before its first
c  use and released before subsequent calls to this subprogram.
c
      if (ptr_pop_tot .ne. 0)
     &   call report_mem_error_1(routine, pointer_name)
c
c  Allocate memory
c
      n = num_elem * 4
      ptr_pop_tot = malloc(n)
      if (ptr_pop_tot .eq. 0)
     &   call report_mem_error(routine, pointer_name, n)
      return
c
c  Entry point for freeing the memory allocated above
c
      entry release_pop_mem(ptr_pop_tot)
      if (ptr_pop_tot .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(ptr_pop_tot)
      ptr_pop_tot = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         allocate_radials
c  
c  This routine allocates the appropriate amount of memory to store the
c  the distance and azimuth to the problem area grid cells for a single
c  desired or undesired station.  The input argument indicates which is
c  to be allocated (true means allocate desired).
c
c  The data for each cell are stored in the RADIAL structure with 4
c  bytes for distance and 4 for azimuth, so the number of bytes required
c  is 8*n_cells.  The respective pointers are stored in global common.
c
c***********************************************************************
c
      subroutine allocate_radials(desired)
      logical desired
      include 'global.inc'
c
      character pointer_name*32
      character routine*32 /'allocate_radials'/
c
c  The pointers should have been initialized to zero before the first
c  call of this subprogram and released before subsequent calls
c
      if (desired) then
         pointer_name = 'prad_desired'
         if (prad_desired .ne. 0)
     &      call report_mem_error_1(routine, pointer_name)
         if (prad_undes .ne. 0) write(0, '(a)')
     &      'Warning: Memory for undesired station radials not released'
      else
         pointer_name = 'prad_undes'
         if (prad_undes .ne. 0)
     &      call report_mem_error_1(routine, pointer_name)
         if (prad_desired .ne. 0) write(0, '(a)')
     &      'Warning: Memory for desired station radials not released'
      end if
c
c  Allocate memory for distance and azimuth arrays
c
      n = 8*n_cells
      if (desired) then
         prad_desired = malloc(n)
         if (prad_desired .eq. 0) then
            pointer_name = 'prad_desired'
            call report_mem_error(routine, pointer_name, n)
         end if
      else
         prad_undes = malloc(n)
         if (prad_undes .eq. 0) then
            pointer_name = 'prad_undes'
            call report_mem_error(routine, pointer_name, n)
         end if
      end if
      return
c
c  Entry point for freeing the memory allocated above
c
      entry release_radials(desired)
      if (desired) then
         if (prad_desired .eq. 0) then
            pointer_name = 'prad_desired'
            call report_mem_error_2(routine, pointer_name)
         end if
         call free(prad_desired)
         prad_desired = 0
      else
         if (prad_undes .eq. 0) then
            pointer_name = 'prad_undes'
            call report_mem_error_2(routine, pointer_name)
         end if
         call free(prad_undes)
         prad_undes = 0
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         allocate_results_mem
c  
c  This routine allocates the appropriate amount of memory to store
c  population and cell counts for later use when printing results.
c  The population and cell counts for each desired station are stored
c  in 24-byte structures, and pointers to arrays of these structures
c  are maintained in global common.
c
c  The RESULTS structure has the following 6 members, 4 bytes each:
c
c     nl_tot = total population or cells within grade B or ATV NLC
c     nl_td_tot = total population or cells within grade B or ATV NLC
c                 less losses due to effects of terrain
c     ix_loss_tot = total population or cells lost to IX
c     ix_ntsc_loss = total lost to IX from NTSC stations
c     ix_atv_loss = total lost to IX from ATV stations.  This is new IX
c                   to NTSC stations, or loss to ATV in addition to any
c                   due to NTSC.
c     ix_atv_only_loss = total lost to IX from ATV stations alone.  This
c                        applies only to ATV stations and would be the
c                        IX remaining after the transition period (when
c                        no more NTSC).
c
c***********************************************************************
c
      subroutine allocate_results_mem
      include 'global.inc'
c
c  Local variables
c
      character pointer_name*32
      character routine*32 /'allocate_results_mem'/
c
c  Pointers should be initialized to zero before the first use of this
c  subprogram and released before subsequent calls.
c
      if (ptr_pop_results .ne. 0) then
         pointer_name = 'ptr_pop_results'
         call report_mem_error_1(routine, pointer_name)
      end if
      if (ptr_cell_results .ne. 0) then
         pointer_name = 'ptr_cell_results'
         call report_mem_error_1(routine, pointer_name)
      end if
c
c  Number of bytes in each array
c
      n = n_des_sta * 24
c
c Allocate memory for population
c
      ptr_pop_results = malloc(n)
      if (ptr_pop_results .eq. 0) then
         pointer_name = 'ptr_pop_results'
         call report_mem_error(routine, pointer_name, n)
      end if
c
c  Allocate memory for cell counts
c
      ptr_cell_results = malloc(n)
      if (ptr_cell_results .eq. 0) then
         pointer_name = 'ptr_cell_results'
         call report_mem_error(routine, pointer_name, n)
      end if
      return
c
c  Entry point to free the memory allocated above
c
      entry release_results_mem
      if (ptr_pop_results .eq. 0) then
         pointer_name = 'ptr_pop_results'
         call report_mem_error_2(routine, pointer_name)
      end if
      call free(ptr_pop_results)
      ptr_pop_results = 0
c
      if (ptr_cell_results .eq. 0) then
         call report_mem_error_2(routine, pointer_name)
         pointer_name = 'ptr_cell_results'
      end if
      call free(ptr_cell_results)
      ptr_cell_results = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        allocate_tmp_1
c 
c  This routine allocates memory to store an array of single bytes.  The
c  input argument num_elem specifies the number of array elements needed
c  and the pointer is returned in ptr_tmp_1.
c
c***********************************************************************
c
      subroutine allocate_tmp_1(ptr_tmp_1, num_elem)
      pointer (ptr_tmp_1, i_1_data(*))
      logical*1 i_1_data
c
c  Local variables
c
      character routine*32 /'allocate_tmp_1'/
      character pointer_name*32 /'ptr_tmp_1'/
c
c  The pointer should have been initialized to zero before its first
c  use and released by the routine that calls this subprogram before
c  any further calls are made.
c
      if (ptr_tmp_1 .ne. 0)
     &   call report_mem_error_1(routine, pointer_name)
c
c  Allocate memory
c
      ptr_tmp_1 = malloc(num_elem)
      if (ptr_tmp_1 .eq. 0)
     &   call report_mem_error(routine, pointer_name, num_elem)
      return
c
c  Entry point for freeing the memory allocated above
c
      entry release_tmp_1(ptr_tmp_1)
      if (ptr_tmp_1 .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(ptr_tmp_1)
      ptr_tmp_1 = 0
      return
      end
c
c***********************************************************************
c
c                            allocate_tmp_2
c  
c  This routine allocates memory to store an integer*2 array. The input
c  argument num_elements specifies the number of array elements needed
c  and the pointer is returned in ptr_tmp_2.
c
c***********************************************************************
c
      subroutine allocate_tmp_2(ptr_tmp_2, num_elements)
      pointer (ptr_tmp_2, i_2_data(*))
      integer*2 i_2_data
c
      character routine*32 /'allocate_tmp_2'/
      character pointer_name*32 /'ptr_tmp_2'/
c
c  The pointer should have been initialized to zero before its first
c  use and released by the routine that calls this subprogram before
c  subsequent calls are made.
c
      if (ptr_tmp_2 .ne. 0)
     &   call report_mem_error_1(routine, pointer_name)
c
c  Allocate memory
c
      n = num_elements * 2
      ptr_tmp_2 = malloc(n)
      if (ptr_tmp_2 .eq. 0)
     &   call report_mem_error(routine, pointer_name, n)
      return
c
c  Entry point for freeing the memory allocated above
c
      entry release_tmp_2(ptr_tmp_2)
      if (ptr_tmp_2 .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(ptr_tmp_2)
      ptr_tmp_2 = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         allocate_unds_mem
c
c  This routine allocates the appropriate amount of memory to store the
c  undesired stations id list, and the undesired field and fade values.
c  All three pointers are stored in global common.
c
c  The arrays pointed to by pcell_ix_fields and pcell_fades are only made
c  large enough for one station as they will be written to disk for later
c  retrieval.  This is necessary to keep memory use resonable.
c
c***********************************************************************
c
      subroutine allocate_unds_mem
      include 'global.inc'
c
      character pointer_name*32
      character routine*32 /'allocate_unds_mem'/
c
c  The pointers should be initialized to zero before the first call of
c  this subprogran and released before subsequent calls.
c
      if (pid_undesired .ne. 0) then
         pointer_name = 'pid_undesired'
         call report_mem_error_1(routine, pointer_name)
      end if
      if (pcell_ix_fields .ne. 0) then
         pointer_name = 'pcell_ix_fields'
         call report_mem_error_1(routine, pointer_name)
      end if
      if (pcell_ix_fades .ne. 0) then
         pointer_name = 'pcell_ix_fades'
         call report_mem_error_1(routine, pointer_name)
      end if
c
c  Allocate memory
c
      n = n_undes_sta
      pid_undesired = malloc(n*2)
      if (pid_undesired .eq. 0) then
         pointer_name = 'pid_undesired'
         call report_mem_error(routine, pointer_name, n*2)
      end if
c
      n = n_cells
      pcell_ix_fields = malloc(n*2)
      if (pcell_ix_fields .eq. 0) then
         pointer_name = 'pcell_ix_fields'
         call report_mem_error
     &      (routine, pointer_name, n*2)
      end if
      pcell_ix_fades = malloc(n*2)
      if (pcell_ix_fades .eq. 0) then
         pointer_name = 'pcell_ix_fades'
         call report_mem_error
     &      (routine, pointer_name, n*2)
      end if
      return
c
c  Entry to free the memory allocated above
c
      entry release_unds_mem
      pointer_name = 'pid_undesired'
      if (pid_undesired .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(pid_undesired)
      pid_undesired = 0
c
      pointer_name = 'pcell_ix_fields'
      if (pcell_ix_fields .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(pcell_ix_fields)
      pcell_ix_fields = 0
c
      pointer_name = 'pcell_ix_fades'
      if (pcell_ix_fades .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(pcell_ix_fades)
      pcell_ix_fades = 0
      return
      end
      subroutine antenna_tilt(irec, az, vpat_bias)
      include 'global.inc'
      logical singled_out
      real mech_tilt
c
      vpat_bias = 0.0
      if (.not. analyze_tilt) return
c
c  Replace BLCTyymmddZZ with the Application Reference Number of the
c  station whose tilt parameters are to be forced to special values.
c
      singled_out = arn_with_prefix(irec) .eq. '  BLCTyymmddZZ'
      if (.not. singled_out) return
c
c  Set tilt parameters and az0 to appropriate values.  The default
c  antenna is electrically tilted by 0.75 degrees.
c
      mech_tilt = 0.8
      elect_tilt = 0.8
      az0 = 134.0
      vpat_bias = (elect_tilt-0.75) + mech_tilt*cos(RADIAN*(az-az0))
      write(0, 1000)
     &   city_r(irec)(1:lnblnk(city_r(irec))), state_r(irec),
     &   ichan_r(irec)
      vpat_bias = 360 + vpat_bias
      return
 1000 format('Specific antenna tilt parameters being used for ',
     &   a, 1x, a, ' channel', i3)
      end
c  %W% %G%
c
c***********************************************************************
c
c                          apply_rec_ant
c
c  Find the relative gain of the receive antenna in the direction of the
c  undesired station.
c
c  The arguments to the routine are the desired and undesired station
c  array pointers and the number of the cell at which the computations are
c  being made.
c
c  The relative gain is returned in dB.
c
c***********************************************************************
c
      subroutine apply_rec_ant(irec_d, irec_u, i_cell, relative_gain)
c
      include 'global.inc'
c
c  Initialize relative gain as 0.0 dB
c
      relative_gain = 0.0
c
c  Check whether relative gain is to be considered.
c
      n_d = NTSC
      if (sta_type(irec_d) .eq. 'a') n_d = ATV
c
      if (n_d .eq. NTSC .and. .not. use_rec_ant_n) goto 500
      if (n_d .eq. ATV .and. .not. use_rec_ant_a) goto 500
c
c  Equate variables
c
      ichan_d = ichan_r(irec_d)
      ichan_u = ichan_r(irec_u)
      latsec_d = lat_site(irec_d)
      lonsec_d = lon_site(irec_d)
      latsec_u = lat_site(irec_u)
      lonsec_u = lon_site(irec_u)
c
c  Get coordinates of cell
c
      i = ((i_cell - 1) * 2) + 1
      latsec_p = lat_cg + icell_offsets(i)
      lonsec_p = lon_cg + icell_offsets(i+1)
c
c  Get distance from desired and undesired stations to the cell
c
      call distance(latsec_d, -lonsec_d, latsec_p, -lonsec_p, d_d)
      call distance(latsec_u, -lonsec_u, latsec_p, -lonsec_p, d_u)
c
c  If distance to point from the undesired or desired station is small,
c  assume the angles are the same.  This will avoid error in the
c  azimuth computation of very small distances.
c
      if (d_d .le. cell_diam .or. d_u .le. 0.1) then
         azu = azd
      else
c
c  Find azimuth from point to desired and undesired stations
c
         call azimuth(latsec_p, lonsec_p, latsec_d, lonsec_d, d_d, azd)
         call azimuth(latsec_p, lonsec_p, latsec_u, lonsec_u, d_u, azu)
         azd = azd * RAD_TO_DEG
         azu = azu * RAD_TO_DEG
      end if
c
      ang = abs(azd - azu)
      if (ang .gt. 180.0) ang = 360.0 - ang
      call rant_pat(ichan_d, n_d, ang, relative_gain)
c
  500 continue
      return
      end

      real function azimuth_to_cell(irec, i_cell)
      include 'global.inc'
c
c  Equate variables
c
      latsec_sta = lat_site(irec)
      lonsec_sta = lon_site(irec)
c
c  Get coordinates of cell
c
      i = ((i_cell - 1) * 2) + 1
      latsec_p = lat_cg + icell_offsets(i)
      lonsec_p = lon_cg + icell_offsets(i+1)
c
c  Find distance from station to the cell
c
      call distance(latsec_sta, -lonsec_sta, latsec_p, -lonsec_p, d_sta)
c
c  Return 0.0 if distance from the station is small.  This will avoid
c  error in the azimuth computation of very small distances.
c
      if (d_sta .le. cell_diam) then
         az = 0.0
      else
c
c  Find azimuth in radians
c
         call azimuth(latsec_sta, lonsec_sta, latsec_p, lonsec_p,
     &      d_sta, az)
         az = az * RAD_TO_DEG
         az = mod(az+360.0, 360.0)
      end if
      azimuth_to_cell = az
      return
      end

c***********************************************************************
c
c                            rant_pat
c
c  This routine returns the antenna discrimination at the given angle
c  based on the formula provided by J. J. Gibson and Burnett Sams and
c  recommended by the FCC Advisory Committee.  The angle is assumed to be
c  given in degrees.  Front-to-back ratios for the band and type of
c  antenna (NTSC or ATV) are stored in global common.  The band will be
c  determined from the specified channel, ichn, and the type from the
c  switch itype (0 = NTSC, 1 = ATV).
c
c  The formula recommended by the FCC Advisory Committee is:
c
c          relative_gain = MAX[cos(ang)**4, Vback]
c          where -20log(Vback) = F/B ratio
c
c  Alternatively, the pattern of CCIR Recomendation 419-2 may be used by
c  setting use_ccir_rec_pat.  Computations for this pattern are made by
c  routine rec_ant_ccir.
c
c***********************************************************************
c
      subroutine rant_pat(ichn, ityp, ang, relative_gain)
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'rant_pat'
c
c  Get band
c
      call get_band(ichn, iband)
c
c  Branch to use the CCIR formula
c
      if (use_ccir_rec_pat) then
         call rec_ant_ccir(ang, iband, ityp, relative_gain)
         goto 500
      end if
c
c  Compute cosine portion
c
      fb = fb_ntsc(iband)
      if (ityp .eq. ATV) fb = fb_atv(iband)
      angl = abs(ang)
      if (angl .lt. 90.0) then
         cs = cos(angl * DEG_TO_RAD)
         cs = 20.0 * fac_k * log10(cs)
      else
         cs = -fb
      end if
c
c  Get gain
c
      relative_gain = max(cs, -fb)
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
C*****************************************************************************
C
C                                AZIMUTH
C
C  This routine computes the azimuth from point A to point B. The
C  inputs to the routine are the coordinates in seconds, with North
C  and East being positive, and the distance in km. The output is
C  in radians.
C
C     ********************    NOTICE   **********************
C     SEVERAL VARIABLES IN THIS ROUTINE ARE DOUBLE PRECISION
C
C  FOR WESTERN HEMISPHERE GIVE LONGITUDES IN NEGATIVE NUMBERS THEN
C  MAKE ANY NECESSRY ADJUSTMENTS AFTER RETURN FROM THIS ROUTINE
C
C*****************************************************************************
C
      subroutine AZIMUTH (LATA, LONA, LATB, LONB, DISTI, AZI)
      include 'global.inc'
      DOUBLE PRECISION RLATA, RLONA, RLATB, RLONB, DRAD, X
      DOUBLE PRECISION dDIST, AZ
c
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'azimuth'
c
c  Equate distance (DISTI) to double precision variable dDIST, and
c  also convert it to miles since this routine is designed to work
c  in terms of miles
c
      dDIST = disti / 1.609344
C
C  CHECK FOR LATITUDES OF +/- 90 DEGREES AND ADJUST TO 1 SECOND LESS
C
      IF ( LATA .EQ. 90 * 3600 ) LATA = LATA - 1
      IF ( LATA .EQ. -90 * 3600 ) LATA = LATA + 1
      IF ( LATB .EQ. 90 * 3600 ) LATB = LATB - 1
      IF ( LATB .EQ. -90 * 3600 ) LATB = LATB + 1
C
C  CONVERT THE COORDINATES TO RADIANS
C
      RLATA = FLOAT( LATA ) * RADIAN / 3600.0
      RLONA = (  FLOAT( LONA ) * RADIAN / 3600.0 )
      RLATB = FLOAT( LATB ) * RADIAN / 3600.0
      RLONB = ( FLOAT( LONB ) * RADIAN / 3600.0 )
C
C  CONVERT THE DISTANCE TO RADIANS
C
      DRAD = dDIST * RPMI
C
C  COMPUTE THE AZIMUTH
C
C     AZ = ACOS ( ( SIN ( RLATB ) - COS ( DRAD ) * SIN ( RLATA ) )
C    & / ( SIN ( DRAD ) * COS ( RLATA ) ) )
      X = ( ( DSIN ( RLATB ) - DCOS ( DRAD ) * DSIN ( RLATA ) )
     &   / ( DSIN ( DRAD ) * DCOS ( RLATA ) ) )
      IF ( X .GT. 1.0 ) X = 1.0
      IF ( X .LT. -1.0 ) X = -1.0
      AZ = DACOS ( X )
C
C  CHECK QUADRANT
C
      IF ( SIN ( RLONB - RLONA ) .LT. 0.0 ) AZ = TWOPI - AZ
c
c  Equate azimuth (AZI) to double precision variable AZ
c
      azi = az
      routine_name = saved_routine_name
      RETURN
      END
c  %W% %G%
c
c**************************************************************************
c
c				CALCRD_R2
c
c  This routine calculates the coordiantes of a point using the RIO-81
c  agreement method. The inputs to the routine are the latitude and
c  longitude of the point in decimal degrees, the distance and the
c  azimuth passed as arguments. The distance is in kilometers and the
c  azimuth is in degrees. The routine returns coordinate in degrees.
c
c		  ************ IMPORTANT ***********
c
c  The latitude and longitude are DOUBLE precision
c
c**************************************************************************
c
      subroutine CALCRD_R2(D_LAT1, D_LON1, DISTANCE,
     &   AZ, RLAT2, RLON2)
      DOUBLE PRECISION D_LAT1, D_LON1, F1, F2, F3, F4, F5, F6, f7
      DOUBLE PRECISION RLAT1, RLON1, RLAT2, RLON2, DRADIAN, DD, AZD
      include 'global.inc'
c
      character saved_routine_name*32
c
      saved_routine_name = routine_name
      routine_name = 'calcrd_r2'
c
c  Check for distance approaching 0.  Errors occur if distance is too
c  small.
c
      if (distance .lt. 0.001) then
         rlat2 = d_lat1
         rlon2 = d_lon1
         goto 110
      end if
      C1 = degkm
c
c  Convert coordinates to radians
c
      DRADIAN = RADIAN
      DD = (DISTANCE / C1) * DRADIAN
      AZD = AZ * DRADIAN
      RLAT1 = D_LAT1 * RADIAN
      RLON1 = D_LON1 * RADIAN
      F1 = DSIN(RLAT1) * DCOS(DD)
      F2 = DCOS(RLAT1) * DSIN(DD) * DCOS(AZD)
      F3 = DASIN (F1 + F2)
      IF (AZ .EQ. 0.0 .OR. AZ .EQ. 180.0) THEN
      RLON2 = RLON1 * DEGREE
      GO TO 100
      END IF
      F4 = (DCOS(DD) - (DSIN(RLAT1) * DSIN(F3)))
      F5 = DCOS(RLAT1) * DCOS(F3)
      f7 = f4 / f5
c
c  Trap out of range values
c
      if (f7 .lt. -1.0) f7 = -1.0
      if (f7 .gt. 1.0) f7 = 1.0
      F6 = DACOS(f7)
      IF (AZ .LT. 180.0) F6 = -F6
      RLON2 = (RLON1 + F6) * DEGREE
  100 continue
      RLAT2 = F3 * DEGREE
  110 continue
      routine_name = saved_routine_name
      RETURN
      END
c  %W% %G%
c
c***********************************************************************
c
c                            ck_cell_svc
c
c  This routine checks the service flag bit for the given station
c  (n_sta) at the given problem area grid cell (n_cell) and returns
c  the bit status in the logical variable bit_status (true = service).
c 
c  See routine init_svc_flag for an explanation of how the arrays
c  are set up.
c
c***********************************************************************
c
      subroutine ck_cell_svc(n_cell, n_sta, bit_status)
      include 'global.inc'
c
      character*32 saved_routine_name
      logical bit_status
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'ck_cell_svc'
c
c  Get correct array element.  There is one element for every 8
c  stations at each cell
c
      n_per_cell = int(n_des_sta / 8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n_per_cell = n_per_cell - 1
c
      n_group = int( n_sta / 8 ) + 1
      if (mod(n_sta, 8) .eq. 0) n_group = n_group - 1
c
      n_el = (n_per_cell * (n_cell - 1)) + n_group
c
c  Determine which bit to examine.  Remember the bits are numbered 0
c  thru 7.
c
      n_bit = (n_sta - ((n_group - 1) * 8)) - 1
c
c  Check bit
c
      bit_status = btest(icell_service(n_el), n_bit)
c     
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         ck_for_nl_svc
c
c  This routine determines whether cells of the problem area grid are
c  within the noise limited service contour of a particular station.
c  Cells outside the NLC are flagged as having no service from this
c  station.
c
c  Inputs to the routine are the index of the station of interest (n_sta),
c  its coordinates in seconds (lat_site, lon_site), contour location data
c  in global common and grid cell data also in global common.
c
c***********************************************************************
c
      subroutine ck_for_nl_svc(n_sta, lat_des_sta, lon_des_sta)
      include 'global.inc'
c
      character saved_routine_name*32
      logical outside
c
c  Statement functions to extract values from the CONTOUR structure
c
      ic_cntr_lat(i) = current_contour(i).lat
      ic_cntr_lon(i) = current_contour(i).lon
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'ck_for_nl_svc'
c
c  Construct a coordinate box containing the contour
c
      lat_max = -100 * 3600
      lat_min = 100 * 3600
      lon_max = -200 * 3600
      lon_min = 200 * 3600
c
      do i=1, n_cntr_rads
         if (ic_cntr_lat(i) .gt. lat_max) lat_max = ic_cntr_lat(i)
         if (ic_cntr_lat(i) .lt. lat_min) lat_min = ic_cntr_lat(i)
         if (ic_cntr_lon(i) .gt. lon_max) lon_max = ic_cntr_lon(i)
         if (ic_cntr_lon(i) .lt. lon_min) lon_min = ic_cntr_lon(i)
      end do
c
c  Loop thru grid cells and determine whether they are within contour
c
      az_inc = 360.0 / float(n_cntr_rads)
      n = 0
      do 100 i=1, n_cells
         outside = .false.
         n = n + 1
c
c  Skip cells with flagged offsets indicating no population.  The service
c  bits for all desired stations have already been set to 0 for such cells.
c
         if (icell_offsets(n) .eq. IGNORED_CELL) then
            n = n + 1
            goto 100
         end if
c
         lat_cell = lat_cg + icell_offsets(n)
         n = n + 1
         lon_cell = lon_cg + icell_offsets(n)
c
c  First check if outside the box surrounding contour
c
         call ck_outside_box(lat_min, lat_max, lon_min, lon_max,
     &      lat_cell, lon_cell, outside)
c
c  If inside box surrounding contour, get distance and bearing and
c  check more closely.  The 4 cells next to the transmitter are always
c  considered inside.
c
         if (.not. outside) then
            call distance(lat_des_sta, -lon_des_sta, lat_cell,
     &         -lon_cell, adist)
            if (adist .gt. cell_diam) then
               call azimuth(lat_des_sta, -lon_des_sta, lat_cell,
     &            -lon_cell, adist, az)
               az = az * rad_to_deg
               call get_radials_btwn(az_inc, az, irad_1, irad_2)
               call interp_lin_drad(irad_1, irad_2, az_inc, az, d_az)
               if (d_az .lt. adist) outside = .true.
            end if
         end if
c
c  If outside NLC, set service bit to 0; otherwise set outside_cells
c  flag bit to 0
c
         if (outside) then
            call zero_svc_bit(i, n_sta)
         else
            call zero_outside_cells(i)
         end if
  100 continue
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         ck_if_within
c
c  This routine determines whether a given point is within a rectangular
c  box constructed so that its sides are a given distance from the
c  corresponding sides of a smaller box.  The inputs to the routine are
c  the given distance (adist), the point in seconds (lat_pt, lon_pt) and
c  the se/nw corners of the smaller rectangle in seconds (lat_se,
c  lon_se, lat_nw, lon_nw).  The result is returned in the logical
c  variable outside.
c
c***********************************************************************
c
      subroutine ck_if_within(adist, lat_pt, lon_pt, lat_se,
     &   lon_se, lat_nw, lon_nw, outside)
      logical outside
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'ck_if_within'
c
c  Get distance in seconds of latitude and longitude
c
      lat_dif = int(((adist / DEGKM) * 3600.0) + 0.5)
      rlat_c = ((float(lat_nw + lat_se) / 2.0)
     &   / 3600.0) * DEG_TO_RAD
      degkm_lon = DEGKM * (cos(rlat_c))
      lon_dif = int(((adist / degkm_lon) * 3600.0) + 0.5)
c
c  Get limits of large rectangle
c
      lat_se_l = lat_se - lat_dif
      lon_se_l = lon_se - lon_dif
      lat_nw_l = lat_nw + lat_dif
      lon_nw_l = lon_nw + lon_dif
c
c  Check if outside box
c
      call ck_outside_box(lat_se_l, lat_nw_l,
     &   lon_se_l, lon_nw_l, lat_pt, lon_pt, outside)
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                       ck_in_water
c
c  Determine whether a given point lies in water.  These tests rely upon
c  the preclusion of points for which elevation data is contained in the
c  U.S. terrain data base.  The points in a test block for which in_water
c  is true are either points which truly lie in water or else are points
c  which will never be presented to this routine because terrain data
c  will have been found for them.
c
c***********************************************************************
c
      subroutine ck_in_water(xlat, xlon, in_water)
      logical in_water, error
c
c  Pacific Coast
c
      in_water = xlat .gt. 32.5 .and. xlat. le. 49.0
     &   .and. xlon .ge. 117.125
      if (in_water) return
c
c  Atlantic Coast and Gulf of Mexico
c
      in_water = xlat .ge. 31.0 .and. xlat .lt. 49.0
     &   .and. xlon .lt. 82.0
      in_water = in_water .or. (xlat .lt. 31.0 .and. xlon .lt. 97.375)
      if (in_water) return
c
c  Points which fail the preceding tests are either in Mexico, or in
c  Mexican coastal waters, or north of the U.S. beyond availability of
c  Canadian terrain data.  This conclusion presumes that the routine
c  has been called after first determining that the point in question
c  is not covered by the U.S. terrain data base.
c
c  Test for improper call of this subroutine or disk IO failure
c
      error = xlat .gt. 33.0 .and. xlat .le. 49.0
     &   .and. xlon .lt. 117.125 .and. xlon .ge. 98.0
      error = error .or. (xlat .ge. 31.0 .and. xlat .lt. 49.0
     &   .and. xlon .lt. 97.375)
c
c  The following if-then block should be commented out if the terrain
c  data base is not complete.  Do not expect to obtain correct results
c  in these circumstances.
c
      if (error) then
         write(0, '(a,f7.3,a1,f8.3,a)') 'ck_in_water: point',
     &      xlat, ',', xlon,
     &      ' should have been found in terrain data base'
         call exit(257)
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                      ck_outside_box
c
c  This routine determines if a point is outside a lat/lon box.
c  The inputs to the routine are the box limits in seconds and the
c  location of the point in seconds.
c
c***********************************************************************
c
      subroutine ck_outside_box(lat_min, lat_max,
     &   lon_min, lon_max, lat_pt, lon_pt, outside)
      logical outside
c
c  Local variables
c
      logical right, left, above, below
c
c  Conditions implying the point is outside the box
c
      right = lon_pt .lt. lon_min
      left =  lon_pt .gt. lon_max
      below = lat_pt .lt. lat_min
      above = lat_pt .gt. lat_max
c
c  Inside if none of the above
c
      outside = right .or. left .or. below .or. above
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        comp_dtc
c
c  Compute the distance to the grade B or noise-limited contour of a
c  specific station at a specific bearing.  The station is identified by
c  its position in the arrays of common block eng_db, and the bearing is
c  given in decimal degrees by the argument az.  The latitude and
c  longitude of the contour point are also computed.  The distance and
c  the coordinates are returned as dtc, c_lat, and c_lon (these are in
c  decimal degrees).
c
c***********************************************************************
c
      subroutine comp_dtc(irec, az, vpat_bias, dtc, c_lat, c_lon)
      include 'global.inc'
      character flag*2
      character saved_routine_name*32
      double precision dlat1, dlon1, dlat2, dlon2
      logical replication_instance
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'comp_dtc'
c
c  Get band for channel number
c
      if (ichan_r(irec) .ne. 0) then
         call get_band(ichan_r(irec), iband)
      else
         dtc = 0.0
         goto 500
      end if
c
c  Get site location in double precision degrees
c
      call sec_to_dpdeg(lat_site(irec), dlat1)
      call sec_to_dpdeg(lon_site(irec), dlon1)
c
c  Get minimum HAAT for station type.  A negative ERP for an ATV station
c  means that its power must be found by replication or its parameters
c  set to prototype values.
c
      if (sta_type(irec) .eq. 'n') then
         haat_min = haat_ntsc_limits(1, iband)
      else
         haat_min = haat_atv_limits(1, iband)
         if (erp_r(irec) .lt. 0.0 .and. .not. replicate)
     &      haat_min = haat_atv_prot(iband)
      end if
c
c  Get horizontal antenna factor if directional antennas are to be used
c
      rad_fac_h = 1.0
      if ((use_da_n .and. sta_type(irec) .eq. 'n') .or.
     &   (use_da_a .and. sta_type(irec) .eq. 'a'))
     &   call horiz_ant_fac(az, irec, rad_fac_h)
c
c  Find HAAT for this azimuth
c
      call get_haat(az, rcamsl_r(irec), dlat1, dlon1, irec, rad_haat)
c
c  Check if HAAT is above minimum levels and if not set it to the
c  minimum.   Also check if routine get_haat returned -10000 and if
c  so set HAAT to value in the engineering data base.
c
      if (rad_haat .eq. -10000.0) rad_haat = haat_r(irec)
      if (rad_haat .lt. haat_min) rad_haat = haat_min
c
c  Compute distance to NLC using the antenna radiation factor and HAAT
c  determined above.  If this is an ATV station, use prototype station
c  parameters unless replication is appropriate.  To replicate, compute
c  the distance to the the paired NTSC station's grade B.
c
      isw = 2	!Specify computation of distance to field in subroutine tvfmfs
      replication_instance = sta_type(irec) .eq. 'a' .and.
     &   (replicate .or. erp_r(irec) .eq. -1.0)
      if (sta_type(irec) .eq. 'n' .or. replication_instance) then
         nrec = irec
         if (sta_type(irec) .eq. 'a') nrec = nrec - (n_first_atv - 1)
         call get_band(ichan_r(nrec), iband)
         n_a = NTSC	!Use NTSC transmitting antenna parameters in tvfmfs
         icurve = 0	!F(50,50) in subroutine tvfmfs
         if (prop_crv_des_n .eq. 10) icurve = 1	!F(50,10)
         if (prop_crv_des_n .eq. 90) icurve = 2	!F(50,90)
         field = cntr_nl_n(iband) - dipole_adjustment(ichan_r(nrec))
         erp_az = erp_r(nrec) * (rad_fac_h**2.0)
         call tvfmfs(erp_az, vpat_bias, rad_haat, ichan_r(nrec),
     &      field, dtc, isw, icurve, flag, n_a)
      else
         n_a = ATV	!Use ATV transmitting antenna parameters in tvfmfs
         icurve = 0
         if (prop_crv_des_a .eq. 10) icurve = 1
         if (prop_crv_des_a .eq. 90) icurve = 2
         field = cntr_nl_a(iband) - dipole_adjustment(ichan_r(irec))
         if (erp_r(irec) .gt. 0.0) then
            erp_az = erp_r(irec) * (rad_fac_h**2.0)
         else
            erp_az = erp_atv_prot(iband)
         end if
         call tvfmfs(erp_az, vpat_bias, rad_haat, ichan_r(irec),
     &      field, dtc, isw, icurve, flag, n_a)
      end if
c
c  Report abnormal return from tvfmfs
c
      if (flag .ne. '  ') then
         call flush(iresult)
         write(0, '(a,a,1x,a,i3)')
     &      'Error flag for station: ', state_r(irec),
     &      city_r(irec)(1:lnblnk(city_r(irec))), ichan_r(irec)
         call fs_error(flag, routine_name)
         call flush(0)
      end if
c
c  Get coordinates at contour
c
      call calcrd_r2(dlat1, dlon1, dtc, az, dlat2, dlon2)
      c_lat = dlat2
      c_lon = dlon2
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         comp_lr_field
c
c  This routine computes the Longley-Rice field at the end point of
c  the terrain path provided in the common block terrain_path. The
c  inputs to the routine are the frequency in mhz, the height of the
c  radiation center above mean sea level (rcamsl) and the percent
c  quantiles for time, location and confidence.  The computations
c  are made for a power of 1 kw and therefore the returned value
c  of field will need to be adjusted for the correct power. Likewise
c  no adjustments are made for horizontal or vertical antenna
c  patterns.
c
c***********************************************************************
c
      subroutine comp_lr_field(d_inc, freq, amslrc, pct_time,
     &    pct_location, pct_confidence, field, fade)
      include 'global.inc'
      character*32 saved_routine_name
      real pfl(5000)
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'comp_lr_field'
c
c  Set distance = (number of terrain points) * (distance increment)
c
      d_point = d_inc * float(n_ter_pt - 1)
c
c  Initialize Longley-Rice parameters
c
      call init_lr(pct_time, pct_location, pct_confidence,
     &   freq, amslrc, d_inc, pfl)
c
c  Run Longley-Rice Model in point-to-point mode
c
      if (n_lr_call .eq. 0) then
         call qlrpfl(pfl, klim, mdvar)
      else
         call qlrpfl(pfl, 0, -1)
      end if
c
c  Check for errors.  Consult page 70 of NTIA Report 82-100, April 1982.
c
c     kwx = 1  Caution; parameters are close to limits
c     kwx = 2  Impossible values; default values have been substituted
c     kwx = 3  Internal calculations show parameters out of range
c     kwx = 4  Parameters out of range
c
      if (kwx .ne. 0) then
         if (warn) write(0, '(a,i3)') 'qlrpfl: kwx =', kwx
         if (kwx .eq. 3 .and. ignore_kwx) then
            kwx = 0
         else
            goto 120
         end if
      end if
c
c  Compute attenuation based on quantiles set above
c
c  With confidence QC there will be at least QL of the locations
c  where the attenuation "a" will not be exceeded for at least QT
c  of the time - zt, zl and zt are computed from QT, QL and QC
c
      a_median = avar(0.0, zl, zc)
      a_loss = avar(zt, zl, zc)
      fade = a_loss - a_median	!positive quantity for pct_time 90%, negative for 10%
c
c  Check for errors
c
      if (kwx .ne. 0) then
         if (warn) write(0, '(a,i3)') 'avar: kwx =', kwx
         goto 120
      end if
c
c  Compute free space signal level for 1 kW
c
      erp = 1.0
      freesp = 106.92 + 10.0*log10(erp) - 20.0*log10(d_point)
c
c  Compute free space signal level minus additional attenuation
c
      fs = freesp - a_loss
c
c  Store field or flag points that could not be computed
c
  120 continue
      if (kwx .eq. 0) then
         field = fs
      else
         field = FLAGGED_FIELD
         fade = FLAGGED_FIELD
      end if
c
c  Count calls to Longley-Rice and exit
c
      n_lr_call = n_lr_call + 1
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        comp_td_cell_flds
c
c  This routine computes the field produced by stations of the desired list
c  at cells within the problem area.  Distance and azimuth from the desired
c  stations to each cell are computed (by routine compute_cell_radials)
c  and stored in common (common block cell_radials_desired).  The distance
c  increment to be used when retrieving terrain path profiles is supplied
c  as the subroutine input argument d_inc.
c
c  In addition this routine compares the computed field to the threshold
c  for noise limited service and sets the service bit to 0 if the field is
c  below this value.
c
c***********************************************************************
c
      subroutine comp_td_cell_flds(d_inc)
      include 'global.inc'
c
c  Local variables
c
      character saved_routine_name*32
      double precision dlat1, dlon1, dlat2, dlon2
      logical desired, replication_instance, apply_horiz_factor,
     &   no_assignment
      real equiv_pat_pwr(360)
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'comp_td_cell_flds'
c
c  Loop thru list of desired stations
c
      desired = .true.
      do i=1, n_des_sta
         k = desired_stations(i).id
         undetermined(1, k) = 0
         undetermined(2, k) = 0
         call print_desired_sta(k, 0)
c
c  Get center frequency of TV channel in MHz and index of the TV band
c
         no_assignment = ichan_r(k) .eq. 0
         if (.not. no_assignment) then
            call get_tv_freq(ichan_r(k), freq)
            call get_band(ichan_r(k), iband)
         end if
c
c  When replicating, compute the ERP and equivalent power pattern for
c  ATV stations.  Save the ERP, power cap reduction, front-to-back
c  ratio, and direction of the pattern maximum in the engineering data
c  base (common block eng_db) so that these values can be included in
c  the results file.  Also save the pattern for inclusion in the FCC
c  antenna data base.
c
         replication_instance = sta_type(k) .eq. 'a' .and.
     &      (replicate .or. erp_r(k) .eq. -1.0)
         if (replication_instance) then
            if (no_assignment) then
               write(0, '(a)') 'No channel assignment to replicate'
               call exit(257)
            end if
            call compute_equiv_pattern(k, erp_atv_limits(1, iband),
     &         equiv_pat_pwr, atv_erp, reduction, direction,
     &         front_to_back)
            erp_r(k) = atv_erp
            replicated(k-(n_first_atv-1)) = .true.
            capped_db(k) = reduction
            front_to_back_ratio(k) = front_to_back
            ref_az_r(k) = direction
            if (new_dadb .and. atv_erp .gt. 0.0)
     &         call dtv_da(state_r(k), city_r(k),
     &            ichan_r(k-(n_first_atv-1)), ichan_r(k),
     &            equiv_pat_pwr, atv_erp, direction)
         end if
c
c  Convert site coordinates to decimal degrees
c
         call sec_to_dpdeg(lat_site(k), dlat1)
         call sec_to_dpdeg(lon_site(k), dlon1)
         spt_lat = dlat1
         spt_lon = dlon1
c
c  Get minimum service field
c
         if (sta_type(k) .eq. 'n') then
            field_min = cntr_nl_n(iband)
         else if (sta_type(k) .eq. 'a') then
            field_min = cntr_nl_a(iband)
         else
            write(0, '(2a)') 'Unrecognized station type ',
     &         sta_type(k)
            stop
         end if
         if (dipole_adj) field_min = field_min
     &      - dipole_adjustment(ichan_r(k))
c
c  The decibel value of ERP is the same for all cells except in the case
c  of ATV stations whose parameters are being determined by replication.
c  Under replication, the ERP will be interpolated from the equiv_pat_pwr
c  array.
c
         apply_horiz_factor = (sta_type(k) .eq. 'n' .or.
     &      .not. replication_instance) .and. .not. no_assignment
         if (apply_horiz_factor) decibel_erp = 10.0 * log10(erp_r(k))
c
c  Compute distances and bearings from the i-th desired station to
c  all cells within its NLC
c
         call compute_cell_radials(desired, i)
c
c  We are now ready to compute the field at these cells
c
         lbase = (i-1) * n_cells
         do 100 j=1, n_cells
            az = rad_desired(j).azimuth
c
c  Set location in the array of service fields at which to store results
c  for the i-th station and j-th cell
c
            l = lbase + j
            if (no_assignment) then
               field = min_field - 1.0
               fade = field
               goto 90
            end if
c
c  Examine the distance from this cell to the desired station.  If it
c  less than d_inc, compute the field assuming free space because the
c  prop model needs at least one point.
c
c  A distance of -1.0 indicates that the cell is outside the NLC or has
c  zero population, so no field needs to be computed.
c
            dtc = rad_desired(j).distance
            if (dtc .lt. d_inc) then
               if (dtc .eq. -1.0) then
                  icell_service_fields(l) = UNCALCULATED_SERVICE
                  goto 100
               else
                  call free_space(dtc, field)
                  goto 50
               end if
            end if
c
c  Compute coordinates of path end points
c
            call calcrd_r2(dlat1, dlon1, dtc, az, dlat2, dlon2)
            ept_lat = dlat2
            ept_lon = dlon2
c 
c  Fetch elevations along this path at distance increments d_inc, km 
c
            call get_terrain_profile(spt_lat, spt_lon, ept_lat,
     &         ept_lon, d_inc, n_error_code)
c
c  If error code is not 0, the terrain path data is bad so set flag and
c  skip this cell.  Bad points are flagged with a field value of
c  FLAGGED_FIELD.
c
            if (n_error_code .ne. 0) then
               undetermined(1, k) = undetermined(1, k) + 1
               field = FLAGGED_FIELD
               goto 90
            end if
c
c  Check RCAMSL
c
            rcamsl_tmp = rcamsl_r(k)
            if (rcamsl_tmp .lt. path_elev_pt(1))
     &         rcamsl_tmp = haat_r(k) + path_elev_pt(1)
c
c  Call prop model to compute field.  Compute at 1 kW and then adjust
c  for actual power.
c
            if (model_for_prop .eq. 1) then
c
c  Compute field by Longley-Rice model.  Quantiles are set according to
c  the station type.
c
               if (sta_type(k) .eq. 'n') then
                  call comp_lr_field(d_inc, freq, rcamsl_tmp,
     &               pct_time_n, pct_location_n, pct_confidence_n,
     &               field, fade)
               else
                  call comp_lr_field(d_inc, freq, rcamsl_tmp,
     &               pct_time_a, pct_location_a, pct_confidence_a,
     &               field, fade)
               end if
c
c  Check for flagged field.  If not flagged, proceed to make adjustments
c  for horizontal and vertical radiation patterns.
c
               if (field .eq. FLAGGED_FIELD) then
                  undetermined(2, k) = undetermined(2, k) + 1
                  goto 90
               end if
c
c  End of Longley-Rice.  Other propagation models inserted here must
c  assume an ERP of 1 kW for subsequent adjustment.
c
            else
               write(0, 900)
               stop
            end if
c
c  Get adjustments for horizontal and vertical radiation patterns (if
c  any used), and adjust for power.
c
   50       continue
            if (apply_horiz_factor) then 
               call horiz_ant_fac(az, k, h_fac)
               if (h_fac .lt. 1.0) then
                  h_log = flog10(h_fac)
                  field = field + 20.0*h_log
               end if
            end if
c
c  Vertical radiation factor.  The tables prepared for the 6th R&O and
c  reconsideration orders used antenna heights AGL and forced the
c  transmitter height to be at least 30.0 meters.  Under mod4, these
c  heights are reckoned AMSL and there is no minimum.
c
            n_ant = NTSC
            if (sta_type(k) .eq. 'a') n_ant = ATV
            call antenna_tilt(k, az, vpat_bias)
            if (mod4 .and. .not. per_6th_order) then
               height = rcamsl_tmp
               rec_height = path_elev_pt(n_ter_pts) + rec_ant_hgt
            else
               height = max(30.0, rcamsl_tmp - path_elev_pt(1))
               rec_height = rec_ant_hgt
            end if
            instance = LONGLEY_RICE
            call gt_vert_rad_fac(height, rec_height, dtc, vpat_bias,
     &         instance, iband, n_ant, v_fac)
            if (v_fac .lt. 1.0) then
               v_log = flog10(v_fac)
               field = field + 20.0*v_log
            end if
c
c  Power adjustment.  If this is an ATV station whose parameters are
c  determined by replication, then power must be computed because no
c  horizontal antenna factor has been applied.
c
            if (apply_horiz_factor) then 
               field = field + decibel_erp
            else
               call interpolate_atv_pwr(az, equiv_pat_pwr, rep_erp)
               field = field + (10.0 * log10(rep_erp))
            end if
c
c  Store field and corresponding fade as integers equal to 100 times their
c  respective real*4 values
c
   90       continue
            icell_service_fields(l) = int((field * 100.0) + 0.5)
            icell_service_fades(l) = int((fade * 100.0) + 0.5)
c
c  Compare field to threshold for noise limited service
c
            if (field .lt. field_min) call zero_svc_bit(j, i)
            if (test_switch .and. field .lt. field_min) then
               n_pt = ((j - 1) * 2) + 1
               write(0, 950) j, field, lat_cg + icell_offsets(n_pt),
     &            lon_cg + icell_offsets(n_pt+1)
            end if
  100    continue
c
c  Free memory
c
         call release_radials(desired)
      end do
c
      routine_name = saved_routine_name
      return
c
  900 format('Selected propagation model not available')
  950 format('comp_td_cell_flds:', i8, 1x, f7.2, 1x, i6, 1x, i6)
      end
c  %W% %G%
c
c***********************************************************************
c
c                    comp_td_cell_udflds
c
c  This routine computes the field from stations of the undesired list at
c  cells within the problem area, after calling compute_cell_radials
c  to find the distance and azimuth from the undesired stations to each
c  cell and storing this information in common block cell_radials_undesired.
c
c  Inputs to this routine are the distance increment, d_inc, to be used
c  when retrieving terrain path profiles, and the undesired station list
c  stored in common block undesired_list.
c
c***********************************************************************
c
      subroutine comp_td_cell_udflds(d_inc)
      include 'global.inc'
c
c  Local variables
c
      pointer (ptr_tmp_2, list_affected(*))
      integer*2 list_affected
      character saved_routine_name*32
      double precision dlat1, dlon1, dlat2, dlon2
      logical desired, replication_instance, apply_horiz_factor
      real equiv_pat_pwr(360)
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'comp_td_cell_udflds'
c
c  Allocate memory to store list of desired stations that may be affected
c
      ptr_tmp_2 = 0
      call allocate_tmp_2(ptr_tmp_2, n_des_sta)
c
c  Loop thru list of undesired stations
c
      desired = .false.
      do 200 i=1, n_undes_sta
         k = id_undesired(i)
         call print_undesired_sta(k, 0)
c
c  Get center frequency of TV channel in MHz, index of the TV band, and
c  minimum service field
c
         call get_tv_freq(ichan_r(k), freq)
         call get_band(ichan_r(k), iband)
c
c  If this is an ATV station and replication is on, compute the
c  equivalent power pattern
c
         if (sta_type(k) .eq. 'a') then
            replication_instance = replicate .or. erp_r(k) .eq. -1.0
     &         .or. replicated(k-(n_first_atv-1))
            if (replication_instance) then
               if (country_r(k)(1:1) .ne. 'A')
     &            write(0, 1000) ichan_r(k), state_r(k),
     &              city_r(k)(1:lnblnk(city_r(k)))
               call compute_equiv_pattern(k, erp_atv_limits(1, iband),
     &            equiv_pat_pwr, atv_erp, reduction, direction,
     &            front_to_back)
               call sectors(k, equiv_pat_pwr)
            else if (margins_file) then
               write(0, '(2a)') 'No provisions for analysis of ',
     &            'margins except when replicating'
               call exit(257)
            end if
         end if
c
c  Get site lat/lon in decimal degrees
c
         call sec_to_dpdeg(lat_site(k), dlat1)
         call sec_to_dpdeg(lon_site(k), dlon1)
         spt_lat = dlat1
         spt_lon = dlon1
c
c  Get distance and bearings from this undesired station to cells
c
         call compute_cell_radials(desired, i)
c
c  Determine which desired stations are to be included from the list of
c  desired stations and the channel relationships to be checked.
c
         call get_affected_list(k, n_aff, ptr_tmp_2)
c
c  The decibel value of ERP is the same for all cells except in the case
c  of ATV stations whose parameters are to be determined by replication.
c  Under replication, the ERP will be interpolated from the equiv_pat_pwr
c  array.
c
         apply_horiz_factor = sta_type(k) .eq. 'n' .or.
     &      .not. replication_instance
         if (apply_horiz_factor) decibel_erp = 10.0 * log10(erp_r(k))
c
c  Loop thru cells computing fields at those where the affected stations
c  provide NL service, where population is present and which are within
c  the maximum distance for which analysis is to be made.
c
         do 100 j=1, n_cells
            call get_max_analy_dist(k, j, n_aff, d_max, ptr_tmp_2)
            az = rad_undes(j).azimuth
            dtc = rad_undes(j).distance
c
c  Propagation models using a terrain profile need at least one point,
c  so if the distance is less than d_inc, compute field using free
c  space.  A distance of -1.0 indicates the cell is of no interest
c  because it has no population or has no service from any of the
c  desired stations.
c
            if (dtc .gt. d_max .or. dtc .eq. -1.0) then
               icell_ix_fields(j) = UNCALCULATED_IX
               icell_ix_fades(j) = UNCALCULATED_IX
               goto 100
            else if (dtc .lt. d_inc) then
               call free_space(dtc, field)
               goto 50
            end if
c
c  Compute coordinates of path end point
c
            call calcrd_r2(dlat1, dlon1, dtc, az, dlat2, dlon2)
            ept_lat = dlat2
            ept_lon = dlon2
            call get_terrain_profile(spt_lat, spt_lon, ept_lat,
     &         ept_lon, d_inc, n_error_code)
c
c  Set flag and go on to next cell if a non-zero error code has been
c  returned indicating the terrain data for this path is bad
c
            if (n_error_code .ne. 0) then
               icell_ix_fields(j) = UNCALCULATED_IX
               icell_ix_fades(j) = UNCALCULATED_IX
               goto 100
            end if
c
c  Check RCAMSL
c
            rcamsl_tmp = rcamsl_r(k)
            if (rcamsl_tmp .lt. path_elev_pt(1))
     &         rcamsl_tmp = haat_r(k) + path_elev_pt(1)
c
c  Call prop model to compute field.  Compute at 1 kw then adjust
c  for actual power
c
            if (model_for_prop .eq. 1) then
c
c  Longley-Rice.  Quantiles depend on the station type.
c
               if (sta_type(k) .eq. 'n') then
                  call comp_lr_field(d_inc, freq, rcamsl_tmp,
     &               pct_time_nix, pct_location_nix,
     &               pct_confidence_nix, field, fade)
               else
                  call comp_lr_field(d_inc, freq, rcamsl_tmp,
     &               pct_time_aix, pct_location_aix,
     &               pct_confidence_aix, field, fade)
               end if
c
c  Skip adjustment for horizontal and vertical radiation patterns when
c  field has been flagged.
c
               if (field .eq. FLAGGED_FIELD) then
                  icell_ix_fields(j) = UNCALCULATED_IX
                  icell_ix_fades(j) = UNCALCULATED_IX
                  goto 100
               end if
c
c  Other prop models to be inserted here.  Remember to compute for
c  an ERP of 1 kw and then adjust.
c
            else
               write(0, 900)
               stop
            end if
c
c  Get horizontal factor except in the case of ATV stations whose
c  parameters are determined by replication.
c
   50       continue
            if (apply_horiz_factor) then 
               call horiz_ant_fac(az, k, h_fac)
               if (h_fac .lt. 1.0) then
                  h_log = flog10(h_fac)
                  field = field + 20.0*h_log
                  fade = fade + 20.0*h_log
               end if
            end if
c
c  Vertical radiation factor.  The tables prepared for the 6th R&O and
c  reconsideration orders used antenna heights AGL and forced the
c  transmitter height to be at least 30.0 meters.  Under mod4, these
c  heights are reckoned AMSL and there is no minimum.
c
c
            n_ant = NTSC
            if (sta_type(k) .eq. 'a') n_ant = ATV
            call antenna_tilt(k, az, vpat_bias)
            if (mod4 .and. .not. per_6th_order) then
               height = rcamsl_tmp
               rec_height = path_elev_pt(n_ter_pts) + rec_ant_hgt
            else
               height = max(30.0, rcamsl_tmp - path_elev_pt(1))
               rec_height = rec_ant_hgt
            end if
            instance = LONGLEY_RICE
            call gt_vert_rad_fac(height, rec_height, dtc, vpat_bias,
     &         instance, iband, n_ant, v_fac)
            if (v_fac .lt. 1.0) then
               v_log = flog10(v_fac)
               field = field + 20.0*v_log
               fade = fade + 20.0*v_log
            end if
c
c  Power adjustment.  If this is an ATV station whose parameters are
c  determined by replication, no horizontal antenna factor has been
c  applied and the ERP in this direction must be calculated from the
c  equiv_pat_pwr array.
c
            if (apply_horiz_factor) then 
               field = field + decibel_erp
               fade = fade + decibel_erp
            else
               call interpolate_atv_pwr(az, equiv_pat_pwr, rep_erp)
               field = field + (10.0 * log10(rep_erp))
               fade = fade + (10.0 * log10(rep_erp))
            end if
c
c  Store field and fade as integers equal to 100 times their respective
c  real*4 values
c
            icell_ix_fields(j) = int((field * 100.0) + 0.5)
            icell_ix_fades(j) = int((fade * 100.0) + 0.5)
  100    continue
c
c  Save the IX array for this undesired station to disk
c
         write(ix_scr, rec=i)
     &      (icell_ix_fields(j), icell_ix_fades(j), j=1, n_cells)
c
c  Free memory used to evaluate IX from this undesired station
c
         call release_radials(desired)
  200 continue
c
c  Free memory used to store lists of affected stations and exit
c
      call release_tmp_2(ptr_tmp_2)
      routine_name = saved_routine_name
      return
c
  900 format('Selected propagation model not available')
 1000 format('Warning: Canadian channel', i3, ' at ', a2, 1x, a,
     &   ' is being replicated with U.S. ERP minima')
      end

c***********************************************************************
c
c                     get_max_analy_dist
c
c  This routine determines which of the desired stations identified in
c  the array list_affected provide noise limited service at the target
c  cell (icell).  Then, based on the channel relationships between these
c  stations and the specified undesired station (nud), the routine
c  determines the maximum distance at which interference analysis should
c  be made.
c
c***********************************************************************
c
      subroutine get_max_analy_dist(nud, icell, n_aff, d_max,
     &   ptr_tmp_2)
      pointer (ptr_tmp_2, list_affected(*))
      integer*2 list_affected
      include 'global.inc'
c
      character*32 saved_routine_name
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'get_max_analy_dist'
c
c  Initialize d_max to a negative distance (no computations needed)
c
      d_max = -10000.0
      if (icell_offsets(2*icell-1) .eq. IGNORED_CELL) goto 500
c
c  Set type of undesired station
c
      itype_u = NTSC
      if (sta_type(nud) .eq. 'a') itype_u = ATV
c
c  Set number of bytes per cell used to store service condition for
c  stations of the desired list.  The maximum analysis distance should
c  be determined with respect to the channels of stations which provide
c  service at this cell.
c
      n_per_cell = int(n_des_sta / 8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n_per_cell = n_per_cell - 1
c
c  Examine each desired station which might be affected
c
      do 100 i=1, n_aff
         ids = list_affected(i)
c
c  Find position of station in desired list
c
         n_sta = 1
         do while (desired_stations(n_sta).id .ne. ids)
            n_sta = n_sta + 1
            if (n_sta .gt. n_des_sta) stop 'Affected station not found'
         end do
c
c  Get correct array element.  There is one element for every 8 stations
c  at each cell
c
         n_group = int(n_sta / 8) + 1
         if (mod(n_sta, 8) .eq. 0) n_group = n_group - 1
         n_el = n_per_cell * (icell - 1) + n_group
c
c  Determine which bit, 0 thru 7, represents station n_sta
c
         n_bit = (n_sta - ((n_group - 1) * 8)) - 1
c
c  Check service bit
c
         if (btest(icell_service(n_el), n_bit)) then
c
c  Check value of the service field.  There is no point in computing an
c  IX field where service was not computed.
c
            l = ((n_sta - 1) * n_cells) + icell
            if (icell_service_fields(l) .eq. UNCALCULATED_SERVICE)
     &         goto 100
c
c  Set type of desired station
c
            itype_d = NTSC
            if (sta_type(ids) .eq. 'a') itype_d = ATV
c
c  Set index of analysis limits array for type of conflict
c
            if (itype_u .eq. NTSC) then
               ncase = N_TO_N
               if (itype_d .eq. ATV) ncase = N_TO_A
            else
               ncase = A_TO_N
               if (itype_d .eq. ATV) ncase = A_TO_A
            end if
c
c  Convert channel difference to second index of the array of analysis
c  limits
c
            idif = ichan_r(nud) - ichan_r(ids)
            i_ck = 1
            do while (ichan_ck(ncase, i_ck) .ne. idif)
               i_ck = i_ck + 1
               if (i_ck .gt. 25) stop
     &            'Error finding max analysis distance'
            end do
c
c  Save greatest distance found
c
            d_max = max(d_max, analysis_limits(ncase, i_ck))
         end if
  100 continue
c
  500 continue
      routine_name = saved_routine_name
      return
      end

      subroutine sectors(irec, equiv_pat_pwr)
c
c  Store the average ERP in sectors around ATV transmitters as integers
c  equal to 100 times the value in dBk.  The storage array is sector_erp,
c  declared integer*2 in global.inc.
c
      include 'global.inc'
      real equiv_pat_pwr(*)
      parameter (NOT_CALCULATED = -32000)
      character saved_routine_name*32
c
      saved_routine_name = routine_name
      routine_name = 'sectors'
      if (DELTA_DEGREES - int(DELTA_DEGREES) .ne. 0.0) stop
     &   'Sector width not an integer number of degrees'
      nsta = irec - (n_first_atv - 1)
      numsector = 1
      if (sector_erp(nsta, numsector) .gt. NOT_CALCULATED) goto 500
      do while (numsector .le. NUM_ANTENNA_SECTORS)
         initial = (numsector-1)*DELTA_DEGREES
         last = numsector*DELTA_DEGREES
         if (initial .ne. 0) then
            sum = 0.5*equiv_pat_pwr(initial)
         else
            sum = 0.5*equiv_pat_pwr(360)
         end if
         do i=initial+1, last-1
            sum = sum + equiv_pat_pwr(i)
         end do
         sum = sum + 0.5*equiv_pat_pwr(last)
         sector_erp(nsta, numsector) =
     &      nint(1000.0*log10(sum/DELTA_DEGREES))
         numsector = numsector + 1
      end do
  500 continue
      routine_name = saved_routine_name
      return
c
      entry initialize_sectors
      num_atv_sta = n_eng_rec - (n_first_atv - 1)
      do nsta=1, num_atv_sta
         do isect=1, NUM_ANTENNA_SECTORS
            sector_erp(nsta, isect) = NOT_CALCULATED
         end do
      end do
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                            compare_svc_pop
c
c  This routine checks bit 1 and bit 2 of an array of service bytes and
c  accumulates the population of cells corresponding to these bytes in
c  two ways.  The first total is of cells for which bit 2 indicates
c  service, and the second is of cells for which both bit 1 and bit 2
c  indicate service.
c
c  The routine is intended to be used when service of exactly two
c  stations, an NTSC and its paired ATV, is being analayzed.  ATV
c  conditions are represented by bit1, and NTSC conditions have been
c  saved in bit2.
c
c***********************************************************************
c
      subroutine compare_svc_pop(pop1, pop2, cell_count1, cell_count2)
      integer pop1, pop2, cell_count1, cell_count2
      include 'global.inc'
c
      integer BIT1, BIT2
      parameter (BIT1 = 1, BIT2 = 2)
c
c  Retrieve population of grid cells if not already loaded
c
      if (ptr_pop_tot .eq. 0) then
         call allocate_pop_mem(n_cells, ptr_pop_tot)
         if (ipop_scr .eq. 0) stop 'Population scratch file not found'
         read(ipop_scr, rec=1, err=400) i_pop_tot
      end if
c
c  Initialize population counts to 0
c
      pop1 = 0
      pop2 = 0
      cell_count1 = 0
      cell_count2 = 0
c
c  Get array element indices.  There should be only two desired stations
c  and one byte for each cell.
c
      if (n_des_sta .ne. 2) then
         write(0, '(a)')
     &      'Inappropriate call of subroutine compare_svc_pop ignored'
         return
      end if
c
c  Loop through the cells examining bits 1 and 2
c
      do i=1, n_cells
         if (btest(icell_service(i), BIT2)) then
            pop2 = pop2 + i_pop_tot(i)
            cell_count2 = cell_count2 + 1
            if (btest(icell_service(i), BIT1)) then
               pop1 = pop1 + i_pop_tot(i)
               cell_count1 = cell_count1 + 1
            end if
         end if
      end do
      return
c
  400 continue
      stop 'Error reading poulation scratch file in compare_svc_pop'
      end
c  %W% %G%
c
c***********************************************************************
c
c                     compute_cell_radials
c
c  This routine computes the distance and azimuth to problem area cells
c  from a particular station of the desired or undesired list.  The
c  logical switch desired, an input argument, determines which list to
c  consider (true means desired list).  Only radials from the single
c  station identified by the argument n_sta are computed.
c
c  Cell coordinates are calculated from the array icell_offsets and
c  ignored (radial distance set to -1.0) if the offset latitude has been
c  flagged to indicate no population.  The radial distance to a desired
c  station is also set to -1.0 for cells outside its NLC, and the radial
c  distance to an undesired station is flagged the same way for cells
c  outside the NLC of every station of the desired list.
c
c***********************************************************************
c
      subroutine compute_cell_radials(desired, n_sta)
      logical desired
      include 'global.inc'
c
      character saved_routine_name*32
      logical service
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'compute_cell_radials'
c
c  Allocate memory for desired or undesired radials to n_cells
c
      call allocate_radials(desired)
c
c  Loop thru grid cells checking for service flags, and compute azimuth
c  and distance to those with flag bit set to 1 except when the cell
c  offset latitude has been flagged to indicate no population.
c
      n = 0
      do 100 i=1, n_cells
         n = n + 1
c
c  Check for cells which may have been flagged by setting the lat offset
c  to 32100.  Set distances to such cells equal to -1.0.
c
         if (icell_offsets(n) .eq. IGNORED_CELL) then
            n = n + 1
            if (desired) then
               rad_desired(i).distance = -1.0
            else
               rad_undes(i).distance = -1.0
            end if
            goto 90
         end if
c
c  Get lat/lon of cell
c
         lat_cell = lat_cg + icell_offsets(n)
         n = n + 1
         lon_cell = lon_cg + icell_offsets(n)
c
c  Branch for desired/undesired station
c
         if (desired) then
            j = n_sta
            k = desired_stations(j).id
c
c  For desired station, first check if cell is within NLC
c
            call ck_cell_svc(i, j, service)
            if (.not. service) then
               rad_desired(i).distance = -1.0
            else
               call get_radial_params(lat_site(k), lon_site(k),
     &            lat_cell, lon_cell, rad_desired(i).distance, az)
               rad_desired(i).azimuth = az * rad_to_deg
               if (rad_desired(i).azimuth .lt. 0.0 .or.
     &            rad_desired(i).azimuth .gt. 360.0) then
                  write(0, 900)
                  write(0, *) lat_site(k), -lon_site(k), lat_cell,
     &               -lon_cell, i, j, k, adist, az*rad_to_deg
                  stop 'Azimuth out of range'
               end if
            end if
         else
c
c  For an undesired station, ignore cells outside the NLC of every
c  desired station.
c
            n_el = int(i/8) + 1
            if (mod(i, 8) .eq. 0) n_el = n_el - 1
            n_bit = mod(i, 8) - 1
            if (n_bit .eq. -1) n_bit = 7
            if (btest(outside_cells(n_el), n_bit)) then
               rad_undes(i).distance = -1.0
            else
               j = n_sta
               k = id_undesired(j)
               call get_radial_params(lat_site(k), lon_site(k),
     &            lat_cell, lon_cell, rad_undes(i).distance, az)
               rad_undes(i).azimuth = az * rad_to_deg
               if (rad_undes(i).azimuth .lt. 0.0 .or.
     &            rad_undes(i).azimuth .gt. 360.0) then
                     write(0, 902)
                     write(0, *) lat_site(k), -lon_site(k), lat_cell,
     &                  -lon_cell, i, j, k, adist, az*rad_to_deg
                  stop 'Azimuth out of range'
               end if
            end if
         end if
c
c  Error conditions
c
   90    continue
         if (desired .and. rad_desired(i).distance .lt. 0.0
     &      .and. rad_desired(i).distance .ne. -1.0) then
            write(0, 910)
            write(0, *) i, j, k, n, lat_site(k), lon_site(k),
     &         lat_cell, adist, az*rad_to_deg
            stop
         else if (desired .and. rad_desired(i).distance .gt. 200.0) then
            write(0, 912)
            write(0, *) i, j, k, n, lat_site(k), lon_site(k),
     &         lat_cell, adist, az*rad_to_deg
         else if (.not. desired .and. rad_undes(i).distance .lt. 0.0
     &      .and. rad_undes(i).distance .ne. -1.0) then
            write(0, 914)
            write(0, *) i, j, k, n, lat_site(k), lon_site(k),
     &         lat_cell, adist, az*rad_to_deg
            stop
         else if (.not. desired
     &      .and. rad_undes(i).distance .gt. 2000.0) then
            write(0, 916)
            write(0, *) i, j, k, n, lat_site(k), lon_site(k),
     &         lat_cell, adist, az*rad_to_deg
         end if
  100 continue
      routine_name = saved_routine_name
      return
c
  900 format('Desired station azimuth in compute_cell_radials:')
  902 format('Undesired station azimuth in compute_cell_radials:')
c
  910 format('Bad desired station radial in compute_cell_radials:')
  912 format(
     &   'Long desired station radial in compute_cell_radials:')
c
  914 format('Bad undesired station radial in compute_cell_radials:')
  916 format(
     &   'Long undesired station radial in compute_cell_radials:')
      end
c
c***********************************************************************
c
c                     get_radial_params
c
c***********************************************************************
c
      subroutine get_radial_params(lat_site, lon_site, lat_cell,
     &   lon_cell, dist, az)
c
      call distance(lat_site, -lon_site, lat_cell, -lon_cell, dist)
      if (dist .lt. 0.1) then
         az = 0.0
      else
         call azimuth(lat_site, -lon_site, lat_cell,
     &      -lon_cell, dist, az)
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                            compute_contour
c
c  This routine computes the coordinates of points on the standard grade
c  B contour (noise-limited contour for ATV) and the distance of these
c  points from the station.  The dBu value defining the contour as well
c  as the curves to be used are determined by the type of station (NTSC
c  or ATV) and the user specified program options in common block
c  runtime_menu.  The station is specifed by the argument irec.
c
c  Computations are made using the FCC propagation curves and results
c  are stored in global common.
c
c***********************************************************************
c
      subroutine compute_contour(irec)
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Equate routine name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'compute_contour'
c
c  Memory has been allocated by routine allocate_cntr_mem for a particular
c  number of radials, and that number of radials is stored as n_cntr_rads
c  in global common.
c
      az_inc = 360.0 / float(n_cntr_rads)
      az = -az_inc
c
      do i=1, n_cntr_rads
         az = az + az_inc
         call antenna_tilt(irec, az, vpat_bias)
         call comp_dtc(irec, az, vpat_bias, dtc, c_lat, c_lon)
         if (dtc .gt. 0.0) then
            call deg_to_sec(c_lat, current_contour(i).lat)
            call deg_to_sec(c_lon, current_contour(i).lon)
         end if
         current_contour(i).distance = dtc
      end do
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          compute_dipole_adj
c
c  This routine calculates an adjustment to the field strength required
c  for grade B service.  The adjustment accounts for the dependence of
c  the antenna aperture on wavelength.
c
c***********************************************************************
c
      subroutine compute_dipole_adj
      include 'global.inc'
      character saved_routine_name*32
      data reference_freq /617.0/	!approx geomentric mean for UHF
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'compute_dipole_adj'
c
c  No adjustments in VHF
c
      do ichn=2, 13
         dipole_adjustment(ichn) = 0.0
      end do
c
c  Calculate UHF adjustments from the ratio of the reference frequency
c  to the channel mid-frequency.  The adjustment for channel 14 will
c  be about 2.3 dB, and this is to be subtracted from the field strength
c  of 64 dBu normally required at the grade B contour for UHF.
c
      do ichn=14, 69
         i = ichn - 14
         if (dipole_adj) then
            channel_mid_freq = 473.0 + 6.0*i
            ratio = reference_freq / channel_mid_freq
            dipole_adjustment(ichn) = 20.0*log10(ratio)
         else
            dipole_adjustment(ichn) = 0.0
         end if
      end do
c
      routine_name = saved_routine_name
      return
      end
c %W% %G%
c
c***********************************************************************
c
c                     compute_equiv_pattern
c
c  This routine computes the ATV power needed to match the grade B
c  service of the NTSC station paired with the station given by
c  the argument to this routine.  The equivalent power is computed
c  every 1 degree.
c
c***********************************************************************
c
      subroutine compute_equiv_pattern(irec, limits, equiv_pat_pwr,
     &   erp, reduction, direction, front_to_back)
      real limits(2), equiv_pat_pwr(*)
      include 'global.inc'
      character*32 saved_routine_name
c
      saved_routine_name = routine_name
      routine_name = 'compute_equiv_pattern'
c
c  Find record number of paired NTSC station
c
      if (sta_type(irec) .ne. 'a') then
         write(0, 1000) ichan_r(irec), state_r(irec),
     &      city_r(irec)(1:lnblnk(city_r(irec)))
         call exit(257)
      end if
      nrec = irec - (n_first_atv - 1)
c
c  Get equivalent power every one degree, 0 to 359.
c
      erp = 0.0
      backside_pwr = 1e6
      direction = 0.0
      do i=1, 360
         az = float(i-1)
         call antenna_tilt(nrec, az, vpat_bias)
         call get_replicate_pwr(irec, nrec, az, vpat_bias, pwr)
         if (pwr .gt. erp) then
            erp = pwr
            direction = az
         else if (pwr .lt. backside_pwr) then
            backside_pwr = pwr
         end if
         equiv_pat_pwr(i) = pwr
      end do
      backside_pwr = max(backside_pwr, 0.001*erp)	!30 dB limit
      front_to_back = 10.0*log10(erp/backside_pwr)
c
      adjust = 1.0
      reduction = 0.0
      if (erp .lt. limits(1)) then
         adjust = limits(1)/erp
         erp = limits(1)
      else if (erp .gt. limits(2)) then
         adjust = limits(2)/erp
         erp = limits(2)
         reduction = -10.0*log10(adjust)
      end if
c
      if (adjust .gt. 1.0) then
         do i=1, 360
            equiv_pat_pwr(i) = equiv_pat_pwr(i)*adjust
         end do
      else if (adjust .lt. 1.0) then
         if (cap_by_truncation) then
            do i=1, 360
               equiv_pat_pwr(i) = max(limits(2), equiv_pat_pwr(i))
            end do
         else
            do i=1, 360
               equiv_pat_pwr(i) = equiv_pat_pwr(i)*adjust
            end do
         end if
      end if
c
      routine_name = saved_routine_name
      return
 1000 format('Improper call to subroutine compute_equiv_pattern:'/
     &   'Channel', i3, ' at ', a2, 1x, a, ' is not a DTV station')
      end

c***********************************************************************
c
c                    get_replicate_pwr
c
c  Compute the power needed by a DTV station to provide the minimum
c  field for noise-limited coverage at the same distance as its paired
c  NTSC station's grade B contour at a specified azimuth.  The DTV
c  station is specified by irec, its position in the arrays of common
c  block eng_db; nrec similarly identifies the paired NTSC station.  The
c  azimuth of interest is given in degrees by the argument az.  The
c  power is returned in rep_pwr.
c
c***********************************************************************
c
      subroutine get_replicate_pwr(irec, nrec, az, vpat_bias, rep_pwr)
      include 'global.inc'
      double precision dlat1, dlon1
      character flag*2, saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'get_replicate_pwr'
c
c  Get band for NTSC channel number
c
      call get_band(ichan_r(nrec), iband)
c
c  Get site location in double precision degrees
c
      call sec_to_dpdeg(lat_site(nrec), dlat1)
      call sec_to_dpdeg(lon_site(nrec), dlon1)
c
c  Get minimum HAAT for station type
c
      haat_min = haat_ntsc_limits(1, iband)
c
c  Get horizontal antenna factor if directional antennas are to be used
c
      rad_fac_h = 1.0
      if (use_da_n) call horiz_ant_fac(az, nrec, rad_fac_h)
c
c  Find HAAT for this azimuth
c
      call get_haat(az, rcamsl_r(nrec), dlat1, dlon1, irec, rad_haat)
c
c  Check if HAAT is above minimum levels and if not set it to the
c  minimum.  Also check if routine get_haat returned -10000, and if
c  so set HAAT to value in the engineering data base.
c
      if (rad_haat .eq. -10000.0) rad_haat = haat_r(nrec)
      if (rad_haat .lt. haat_min) rad_haat = haat_min
c
c  Compute distance to NTSC grade B using the antenna radiation factor
c  and HAAT determined above
c
      isw = 2	!Specify computation of distance to field in subroutine tvfmfs
      n_a = NTSC
      icurve = 0	!F(50,50) in subroutine tvfmfs
      if (prop_crv_des_n .eq. 10) icurve = 1	!F(50,10)
      if (prop_crv_des_n .eq. 90) icurve = 2	!F(50,90)
      field = cntr_nl_n(iband) - dipole_adjustment(ichan_r(nrec))
      erp_az = erp_r(nrec) * (rad_fac_h ** 2.0)
      call tvfmfs(erp_az, vpat_bias, rad_haat, ichan_r(nrec),
     &   field, dtc, isw, icurve, flag, n_a)
c
c  Report abnormal return from tvfmfs
c
      call fs_error(flag, routine_name)
c
c  Compute the field of the DTV station at this distance for 1 kW and
c  then adjust the power to get the desired field value.
c  
      isw = 1	!Compute field at given distance
      n_a = ATV
      icurve = 0	!F(50,50)
      if (prop_crv_des_a .eq. 10) icurve = 1	!F(50,10)
      if (prop_crv_des_a .eq. 90) icurve = 2	!F(50,90)
c
      erp_az = 1.0
      vpat_bias_argument = 0.0
      call tvfmfs(erp_az, vpat_bias_argument, rad_haat, ichan_r(irec),
     &   field, dtc, isw, icurve, flag, n_a)
c
c  Report abnormal return from tvfmfs
c
      call fs_error(flag, routine_name)
c
c  Get band for DTV channel number
c
      call get_band(ichan_r(irec), iband)
c
c  Determine desired NLC value
c
      cntr = cntr_nl_a(iband) - dipole_adjustment(ichan_r(irec))
      dif = cntr - field
      rep_pwr = 10.0 ** (dif / 10.0)
c
      routine_name = saved_routine_name
      return
      end

      subroutine dtv_da(state, city, ichan, newchan, equiv_pat_pwr,
     &   atv_erp, direction)
      character state*2, city*20
      real equiv_pat_pwr(360)
      real rel_volts(36), special_az(10), special_volts(10)
      character da_string*17, comment*76, date_string*6
      character pad*15 /' '/
      character*32 filename /'dtv_dadb.lis'/
      character*32 analysis_file /'dtv_dadb.analysis'/
      logical unity
      integer ida /0/, jda /0/
      save ida, jda
      data date_string /'970403'/
c
c  Create directional antenna records for ATV stations.  It is assumed
c  that atv_erp is the maximum of the values in the array equiv_pat_pwr.
c
      if (ida .eq. 0) then
         call getlun(ida)
         open(ida, file=filename, status='new')
      end if
      write(da_string, 1000) state, city(1:10), newchan
      call substitute_underscore(da_string)
      write(comment, 1002) ichan
c
c  Find pattern factors at 10 degree intervals
c
      do i=1, 36
         iaz = 10*i - 9
         rel_volts(i) = sqrt(equiv_pat_pwr(iaz)/atv_erp)
      end do
c
c  Find unity pattern factors between 10-degree azimuths, and include
c  them as special azimuths if the pattern factors on both sides are
c  less than unity.
c
      num_main = 0	!unity pattern factors on 10-degree azimuths
      num_others = 0	!unity factors between 10-degree azimuths
      included = 0	!between main azimuths neither of which is a unity factor
      ignored = 0		!between main azimuths at least one of which is unity
      v2 = int(1000.0*rel_volts(1) + 0.5)/1000.0
      do i=1, 36
         v1 = v2
         unity = v1 .eq. 1.0
         if (unity) num_main = num_main + 1
         i2 = mod(i, 36) + 1
         v2 = int(1000.0*rel_volts(i2) + 0.5)/1000.0
         iaz = 10*(i-1) + 1
         do while (iaz .lt. 10*i)
            v = sqrt(equiv_pat_pwr(iaz)/atv_erp)
            if (int(1000.0*v + 0.5) .eq. 1000) then
               num_others = num_others + 1
               if (.not. unity .and. v2 .lt. 1.0) then
                  included = included + 1
                  if (included .le. 10) then
                     special_az(included) = iaz
                     special_volts(included) = 1.0
                  else
                     write(0, '(2a,i4,a)') da_string,
     &                  ': No room for special azimuth at', iaz,
     &                  ' degrees'
                  end if
               else if (unity .and. v2 .eq. 1.0) then
                  ignored = ignored + 1
               end if
            end if
            iaz = iaz + 1
         end do
      end do
c
c  Repeat examining only every second in-between azimuth in case the
c  number included exceeds 10.
c
      if (included .le. 10) then
         numspecial = included
      else
         numspecial = 0
         v2 = int(1000.0*rel_volts(1) + 0.5)/1000.0
         i = 0
         do while (i .lt. 36 .and. numspecial .lt. 10)
            i = i + 1
            v1 = v2
            unity = v1 .eq. 1.0
            i2 = mod(i, 36) + 1
            v2 = int(1000.0*rel_volts(i2) + 0.5)/1000.0
            iaz = 10*(i-1) + 2
            do while (iaz .lt. 10*i)
               v = sqrt(equiv_pat_pwr(iaz)/atv_erp)
               if (int(1000.0*v + 0.5) .eq. 1000) then
                  if (.not. unity .and. v2 .lt. 1.0) then
                     if (numspecial .lt. 10) then
                        numspecial = numspecial + 1
                        special_az(numspecial) = iaz
                        special_volts(numspecial) = 1.0
                     else
                        write(0, '(2a)') da_string,
     &                     ': incomplete set of special azimuths'
                     end if
                  end if
               end if
               iaz = iaz + 2
            end do
         end do
      end if
c
c  Write directional antenna record
c
      if (numspecial .gt. 0) then
         write(ida, 1010) da_string, rel_volts,
     &      (special_az(i), special_volts(i), i=1, numspecial),
     &      comment, date_string, pad
      else
         write(ida, 1012) da_string, rel_volts, comment, date_string,
     &      pad
      end if
c
c  Report number of unity pattern factors that fall between 10-degree radials
c
      if (jda .eq. 0) then
         call getlun(jda)
         open(jda, file=analysis_file, status='new')
         write(jda, 1100)
      end if
      write(jda, 1102) da_string, num_main, num_others, ignored,
     &   included
c
 1000 format('rep', a2, a10, i2.2)
 1002 format('Paired with NTSC channel', i3,
     &   '.  Reference azimuth is 0.0 degrees T')
 1010 format(a17, t24, 36f5.3, <numspecial>(f5.1, f5.3), t304, a76,
     &   a6, a15)
 1012 format(a17, t24, 36f5.3, t304, a76, a6, a15)
C123456789012345678901234567890123456789012345678901234567890123456789012
CrepWYSHERIDAN__21      0.6360.6320.6270.6210.6170.6210.6360.6500.6620.6730.7140.6990.6690.7670.9940.9570.7970.7150.5390.5390.5390.5390.5390.5390.5390.5390.5390.5390.7460.9260.9850.8910.6570.6770.7190.694                                                                                                    Paired with NTSC channel 12.  Reference azimuth is 0.0 degrees T            970403               
 1100 format(
     &   '                       NUMBER OF UNITY PATTERN FACTORS'/
     &   '                     -----------------------------------'/
     &   '                               between 10-degree radials'/
     &   '                               -------------------------'/
     &   '                     on 10-            unity   unity'/
     &   '                     degree            both    neither'/
     &   ' ANTENNA PATTERN     radials   total   sides   side'/
     &   '-----------------    -------   -----   -----   --------')
c         12345678901234567x23456789x23456789x2345678x23456789
c         repWYSHERIDAN__21        3       54      18        2
 1102 format(a17, 2i9, i8, i9)
      return
      end

      subroutine substitute_underscore(string)
      character*(*) string
      l = index(string, ' ')
      do while (l .ne. 0)
         string(l:l) = '_'
         l = index(string, ' ')
      end do
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          count_atv_svc_pop
c
c  This routine checks the service flag for the given station (n_sta)
c  (n_sta) and counts the total population of the cells where service
c  was found.
c
c  This routine is different from count_svc_pop in that the service
c  array is only for the single station pointed to by ptr_tmp_1.
c 
c***********************************************************************
c
      subroutine count_atv_svc_pop(n_sta, n_pop_total,
     &   n_cell_total, ptr_tmp_1)
      pointer (ptr_tmp_1, icell_service_atv(*))
      logical*1 icell_service_atv
c
      include 'global.inc'
c
c  Retrieve population of grid cells if not already loaded
c
      if (ptr_pop_tot .eq. 0) then
         call allocate_pop_mem(n_cells, ptr_pop_tot)
         if (ipop_scr .eq. 0) stop 'Population scratch file not found'
         read(ipop_scr, rec=1, err=400) i_pop_tot
      end if
c
c  Initialize population count to 0
c
      n_pop_total = 0
      n_cell_total = 0
c
c  Loop through the cells, checking for service
c
      do i=1, n_cells
         if (icell_service_atv(i)) then
            n_pop_total = n_pop_total + i_pop_tot(i)
            n_cell_total = n_cell_total + 1
         end if
      end do
      return
c
  400 continue
      stop 'Error reading poulation scratch file in count_atv_svc_pop'
      end
c  %W% %G%
c
c***********************************************************************
c
c                        count_outside_pop
c
c  This routine examines the cell flag bits in the array outside_cells
c  and counts the total population of the cells identified as outside
c  the NLC of all stations in the desired station list.
c
c***********************************************************************
c
      subroutine count_outside_pop(n_pop_total)
      include 'global.inc'
c
c  Retrieve population of grid cells if not already loaded
c
      if (ptr_pop_tot .eq. 0) then
         call allocate_pop_mem(n_cells, ptr_pop_tot)
         if (ipop_scr .eq. 0) stop 'Population scratch file not found'
         read(ipop_scr, rec=1, err=400) i_pop_tot
      end if
c
c  Initialize population count to 0
c
      n_pop_total = 0
c
c  Loop through the cells
c
      do i=1, n_cells
c
c  Determine correct array element
c
         n_el = int(i / 8) + 1
         if (mod(i, 8) .eq. 0) n_el = n_el - 1
c
c  Determine correct bit to check.  Remember the bits are numbered 0 thru 7.
c
         n_bit = mod(i, 8) - 1
         if (n_bit .eq. -1) n_bit = 7
c
c  Check bit
c
         if (btest(outside_cells(n_el), n_bit)) n_pop_total =
     &       n_pop_total + i_pop_tot(i)
      end do
      return
c
  400 continue
      stop 'Error reading poulation scratch file in count_outside_pop'
      end
c  %W% %G%
c
c***********************************************************************
c
c                            count_svc_pop
c
c  This routine checks the service flag bits for the given station
c  (n_sta) and counts the total population of the cells where service
c  is found.
c
c***********************************************************************
c
      subroutine count_svc_pop(n_sta, n_pop_total, n_cell_total)
      include 'global.inc'
c
c  Allocate memory and load cell population totals
c
      if (ptr_pop_tot .eq. 0) then
         call allocate_pop_mem(n_cells, ptr_pop_tot)
         if (ipop_scr .eq. 0) stop 'Population scratch file not found'
         read(ipop_scr, rec=1, err=400) i_pop_tot
      end if
c
c  Initialize population count to 0
c
      n_pop_total = 0
      n_cell_total = 0
c
c  Calculate indices to identify the array element corresponding to the
c  given station.  There is one element for every 8 stations at each cell.
c
      n_per_cell = int(n_des_sta / 8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n_per_cell = n_per_cell - 1
c
      n_group = int(n_sta / 8) + 1
      if (mod(n_sta, 8) .eq. 0) n_group = n_group - 1
c
c  Loop through the cells
c
      do i=1, n_cells
         n_el = n_per_cell * (i - 1) + n_group
c
c  Determine which bit to check.  Remember the bits are numbered 0 thru 7.
c
         n_bit = (n_sta - ((n_group - 1) * 8)) - 1
c
c  Check bit
c
         if (btest(icell_service(n_el), n_bit)) then
            n_pop_total = n_pop_total + i_pop_tot(i)
            n_cell_total = n_cell_total + 1
         end if
      end do
      return
c
  400 continue
      stop 'Error reading poulation scratch file in count_svc_pop'
      end
c  %W% %G%
c
c***********************************************************************
c
c                          create_svc_matrix
c
c  This routine examines the desired stations service array and creates
c  a matrix showing the population served by n NTSC stations and m ATV
c  stations.
c
c***********************************************************************
c
      subroutine create_svc_matrix
      include 'global.inc'
c
c  Retrieve population of grid cells if not already loaded
c
      if (ptr_pop_tot .eq. 0) then
         call allocate_pop_mem(n_cells, ptr_pop_tot)
         if (ipop_scr .eq. 0) stop 'Population scratch file not found'
         read(ipop_scr, rec=1, err=400) i_pop_tot
      end if
c
c  Allocate memory to store matrix
c
      call allocate_matrix_mem
c
c  Initialize matrix to 0 population
c
      do i=1, n_ntsc + 1
         do j=1, ((n_des_sta - n_ntsc) + 1)
            matrix_svc(i, j) = 0
         end do
      end do
c
c  Get the number of service byte arrays per cell
c
      n_per_cell = int(n_des_sta / 8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n_per_cell = n_per_cell - 1
c
c  Loop thru cells checking which stations (if any) provide service
c
      do i=1, n_cells
         n_n = 0
         n_a = 0
c
c  Loop thru desired stations and determine if they provide service
c  at this cell
c
         do j=1, n_des_sta
c
c  Get correct array element.  There is one element for every 8
c  stations at each cell.
c
            n_group = int(j / 8) + 1
            if (mod(j, 8) .eq. 0) n_group = n_group - 1
            n_el = n_per_cell * (i - 1) + n_group
c
c  Determine which bit to check.  Remember the bits are numbered 0 thru 7.
c
            n_bit = (j - ((n_group - 1) * 8)) - 1
c
c  Check bit and go on to next station if no service.
c
            if (btest(icell_service(n_el), n_bit)) then
               if (j .le. n_ntsc) then
                  n_n = n_n + 1
               else
                  n_a = n_a + 1
               end if
            end if
        end do
c
c  Get cell population and update counter.  n_n and n_a must be
c  adjusted because position 1 is no service (n_n = 0, or n_a = 0).
c
         n_n = n_n + 1
         n_a = n_a + 1
         matrix_svc(n_n, n_a) = matrix_svc(n_n, n_a) + i_pop_tot(i)
      end do
      return
c
  400 continue
      stop 'Error reading poulation scratch file in count_svc_pop'
      end
c  %W% %G%
c
c***********************************************************************
c
c                         def_srch_chans
c
c  This routine determines channels and distance limits for use in
c  identifying undesired stations relative to a list of specific desired
c  stations.  The inputs to the routine are the list of desired
c  stations, the channel relationships of interest, and specified
c  maximum distances from site to analysis point.
c
c  The array ichan_ck(4, 25) identifies the channel relationships of
c  interest.  The first index is 1 for N to N; 2 for N to A; 3 for A to
c  N; and 4 for A to A.  The second index ranges from 1 to 25 to cover
c  the +/- channel separations.  See routine set_ix_ch for a further
c  details.
c
c  The array ixchans(25) holds possible IX channels based on a given
c  desired channel.  It is filled by routine set_ix_ch.
c
c  Analysis_limits(4, 25) holds maximum distances from undesired
c  stations to service points.  No analysis is performed beyond these
c  distances.
c
c  Search_dist_list is the output of this routine.  It gives the maximum
c  distance from desired stations to be considered in searching for
c  undesired stations on each channel.  A distance of -1000.0 indicates
c  no search is necessary.
c
c***********************************************************************
c
      subroutine def_srch_chans
      include 'global.inc'
c
c  Initialize search list to indicate no search necessary on any channel
c
      do i=1, 4
         do j=1, 68
            search_dist_list(i, j) = -10000.0
         end do
      end do
c
c  Loop thru list of desired stations
c
      do i=1, n_des_sta
         if (pairwise) then
            n_a = i
         else
            n_a = 1
            if (i .gt. n_ntsc) n_a = 2
         end if
         ichn = desired_stations(i).ichan
c
c  Get list of channels that are co, adj or taboo in relation to ichn.
c  The result is stored in ixchans.
c
         call set_ix_ch(ichn)
c
c  Check list of channel relationships to be included in the IX
c  analysis.  n_a = 1 indicates an NTSC station; n_a = 2 indicates an
c  ATV station.
c
         do j=1, n_ixch
            j1 = ixchans(j) - 1
            do k=n_a, n_a + 2, 2
               do l=1, 25
                  i_dif = ixchans(j) - ichn
                  if (i_dif .eq. ichan_ck(k, l)) then
                     search_dist_list(k, j1) =
     &                  max(analysis_limits(k, l),
     &                  search_dist_list(k, j1))
                  end if
               end do
            end do
         end do
      end do
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                       define_prob_area
c
c  This routine determines the boundary coordinates of a rectangular box
c  encompassng the noise limited service contour (NLC) of a specified
c  station.  The boundary coordinates are returned in decimal degrees in
c  the arguments pa_bound_lat, pa_bound_lon.  The input arguments are
c  the array record position of the desired station in the common block
c  eng_db.  The NLC value and the parameters for computing it are
c  contained in the common block runtime_menu.
c
c  This routine is called when the user has elected to define the
c  problem area as the NLC of a specific station rather than
c  actually defining the area.
c
c***********************************************************************
c
      subroutine define_prob_area(pa_bound_lat, pa_bound_lon,
     &   irec)
      real pa_bound_lat(2), pa_bound_lon(2)
c
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'define_prob_area'
c
c  Initialize boundary data to station site
c
      call sec_to_deg(lat_site(irec), pa_bound_lat(1))
      call sec_to_deg(lon_site(irec), pa_bound_lon(1))
      pa_bound_lat(2) = pa_bound_lat(1)
      pa_bound_lon(2) = pa_bound_lon(1)
c
c  Compute the distance to the station's grade B or noise-limited
c  contour.  If use_n_radial_haat is false, then compute for only four
c  radials provided the station's antenna is non-directional or
c  directional antennas are not to be used.  The four radials will suffice
c  for boxing a coverage circle.
c
c  The variable use_n_rad_haat refers to whether the binary HAAT data
c  base is to be used for contour calculations in place of the single
c  HAAT value found in the engineering data base.
c
c  n_rads_used (or only 4) is the number of radials used to determine
c  coordinates along the countour at the time it is boxed.  Another input
c  parameter, n_std_rad, determines the number to use in HAAT
c  calculations.  The ERP in any particular direction is interpolated
c  when necessary from the station's directional antenna pattern (values
c  every 10 degrees with up to 10 special azimuths), and the HAAT in that
c  direction is interpolated from n_std_rad values.
c
      n = 4
      if (use_n_radial_haat) n = n_rads_used
      if (sta_type(irec) .eq. 'n' .and. use_da_n .and.
     &   antmod_r(irec)(1:4) .ne. 'none') n = n_rads_used
      if (sta_type(irec) .eq. 'a' .and. use_da_a .and.
     &   antmod_r(irec)(1:4) .ne. 'none') n = n_rads_used
c
      az_inc = 360.0 / float(n)
      az = -az_inc
      do i=1, n
         az = az + az_inc
         call antenna_tilt(irec, az, vpat_bias)
         call comp_dtc(irec, az, vpat_bias, dtc, c_lat, c_lon)
         if (dtc .gt. 0.0) then
            if (c_lat .lt. pa_bound_lat(1)) pa_bound_lat(1) = c_lat
            if (c_lon .lt. pa_bound_lon(1)) pa_bound_lon(1) = c_lon
            if (c_lat .gt. pa_bound_lat(2)) pa_bound_lat(2) = c_lat
            if (c_lon .gt. pa_bound_lon(2)) pa_bound_lon(2) = c_lon
         end if
      end do
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                            deg_to_sec
c
c  Converts geographic coordinate point in decimal degrees to seconds
c
c***********************************************************************
c
      subroutine deg_to_sec(deg, isec)
      x = abs(deg)
      isec = int ((x * 3600.0) + 0.5)
      if (deg .lt. 0) isec = -isec
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          display_matrix
c
c  This routine writes the spreadsheet matrix, showing the population
c  served by N NTSC stations and M ATV stations, to the unit specified
c  by the argument ifile.
c
c***********************************************************************
c
      subroutine display_matrix(ifile)
      include 'global.inc'
c
      character atv_label*12 /'ATV STATIONS'/
c
c  Write header
c
      write(ifile, 990)
      write(ifile, 990)
      write(ifile, 900)
      write(ifile, 990)
      k = n_ntsc + 1
      l = (n_des_sta - n_ntsc) + 1
c
c  Write matrix
c
      k1 = -11
   50 continue
      k1 = k1 + 12
      k2 = k1 + 11
      if (k2 .gt. k) k2 = k
      write(ifile, 910) (i-1, i=k1, k2)
      write(ifile, 990)
c
      do i=1, l
         if (i .le. 12) then
            write(ifile, 920) atv_label(i:i), i - 1,
     &         (matrix_svc(j, i), j=k1, k2)
         else
            write(ifile, 921) i - 1, (matrix_svc(j, i), j=k1, k2)
         end if
      end do
c
      if (l .lt. 12) then
         do i=l+1, 12
            write(ifile, 920) atv_label(i:i)
         end do
      end if
      if (k2 .lt. k) goto 50
c
c  Count population outside all NLCs
c
      if (matrix_svc(1,1) .gt. 0) then
         call count_outside_pop(n_outside_pop)
         write(ifile, 990)
         write(ifile, 930) n_outside_pop
         write(ifile, 990)
      end if
c
c  Free memory used for matrix
c
      call release_matrix_mem
      return
c
  900 format(30x, 'POPULATION SERVED BY')
  910 format(33x, 'NTSC STATIONS' / 8x, 12(6x, i2, 2x))
  920 format(1x, a1, 2x, i2, 2x, 12(i8, 2x))
  921 format(4x i2, 2x, 12(i8, 2x))
  930 format('Please note that of the population that',
     &   ' receives no service from any station'/ i8,
     &   ' are within the defined problem area but outside the noise'
     &   ' limited'/ 'contours of all the selected stations')
  990 format(/)
      end
c  %W% %G%
c
c***********************************************************************
c
c                                DISTANCE
c
c  Compute the great-circle distance between two points. The formula is
c  as that of the Rio Final Acts. Inputs to the routine are in seconds
c  and the output is in km.
c
c  West longitudes are negative.
c
c
c    ********************    NOTICE   *********************
c    SEVERAL VARIABLES IN THIS ROUTINE ARE DOUBLE PRECISION
c
c***********************************************************************
c
      subroutine distance(LATA, LONA, LATB, LONB, aDIST)
      DOUBLE PRECISION RLATA, RLONA, RLATB, RLONB, DRAD
      include 'global.inc'
c
c  Check for zero distance
c
      adist = 0.0
      if ((lata - latb) .eq. 0 .and. (lona - lonb) .eq. 0) goto 500
c
c  Check for latitudes of +/- 90 degrees and adjust to 1 second less
c
      IF (LATA .EQ. 90 * 3600) LATA = LATA - 1
      IF (LATA .EQ. -90 * 3600) LATA = LATA + 1
      IF (LATB .EQ. 90 * 3600) LATB = LATB - 1
      IF (LATB .EQ. -90 * 3600) LATB = LATB + 1
c
c  Convert the coordinates to radians
c
      RLATA = FLOAT(LATA) * RADIAN / 3600.0
      RLONA = (-1.0) * (FLOAT(LONA) * RADIAN / 3600.0)
      RLATB = FLOAT(LATB) * RADIAN / 3600.0
      RLONB = (-1.0) * (FLOAT(LONB) * RADIAN / 3600.0)
c
c  Compute distance in radians
c
      DRAD = DACOS((DSIN(RLATA) * DSIN(RLATB)) +
     &   (DCOS(RLATA) * DCOS(RLATB) * DCOS(RLONB -RLONA)))
c
c  Convert from radians to km
c
      aDIST = DRAD / (radian/degkm)
  500 RETURN
      END
c  %W% %G%
c
c***********************************************************************
c
c                            dms_to_deg
c
c  Converts geographic coordinate point in DMS to decimal degrees
c
c***********************************************************************
c
      subroutine dms_to_deg(deg, icord1, icord2, icord3)
      integer icord(3)
c
      icord(1) = icord1
      icord(2) = icord2
      icord(3) = icord3
c
      deg = float (abs(icord(1))) + (float (abs(icord(2))) / 60) +
     &   (float (abs(icord(3))) / 3600)
      if (icord(1) .lt. 0) deg = -deg
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          du_adjustments
c
c  These routines calculate adjustments to the co-channel D/U ratios for
c  interference into ATV to account for degradation due to increased
c  noise when the signal-to-noise ratio is near the limiting value for
c  ATV reception.
c
c  The adjustments are made by subroutines du_adjustments_aa (for
c  ATV-into-ATV) and du_adjustments_na for (NTSC-into-ATV).  The argument
c  list is the same for both routines.  The first argument is the
c  signal-to-noise ratio relative to its value for noise-limited
c  reception, and the second is the decibels to be added to the D/U ratio
c  established for low noise conditions.
c
c  Note that the first argument, the relative signal-to-noise level,
c  can be found at any reception point as the difference between the
c  desired field in dbu and the field for noise-limited reception.
c
c***********************************************************************
c
      subroutine du_adjustments_aa(relative_snr, adjustment)
      include 'global.inc'
c
c  To determine the relationship between the D/U and C/N for ATV-to-
c  ATV co-channel interference, let
c
c      n = noise power at reference point where appropriate formula
c          is n = kTBF,
c
c     d = RF power of the desired ATV signal, measured at reference
c         point in units corresponding to those of kTBF,
c
c     u = co-channel interference power, also at reference point
c         in same units as kTBF and the desired signal,
c
c     r = threshold d/u ratio for ATV reception in the presence of
c         co-channel interference and no noise, that is, high
c         levels of desired signal.
c
c  Idealize the interfering co-channel ATV signal as noise, since it
c  is certainly noise-like.  Then at conditions other than the limit
c  case of no noise, the undesired signal power will have to satisfy
c  the equation
c
c           d         1
c     r = ----- = ---------.
c         u + n   u/d + n/d
c
c  Thus in terms of ratios expressed in decibels
c
c                                   1
c     r = 10**(R/10) = -------------------------
c                       10**(-D/10) + 10**(-C/10)
c
c  where R is the D/U ratio established by measurement for the limit
c  case of negligible noise, D is the D/U ratio in higher noise
c  conditions, and C is the "carrier" to noise ratio (C/N).
c
c  Solving, we have
c
c     10**(-D/10) = 10**(-R/10) - 10**(-C/10)
c     10**((-D+R)/10) = 1 - 10**((-C+R)/10).
c
c  In terms of the variables x = (C-R)/10 and y = (D-R)/10
c
c     10**(-y) = 1 - 10**(-x)
c     y = log10(1/(1 - 10**(-x)).
c
c  Statement function
c
      f(x) = log10(1.0/(1.0 - 10**(-x)))
c
c  Calculation
c
      x = max(1e-4, relative_snr/10.0)
      adjustment = 10.0*f(x)
      return
      end

      subroutine du_adjustments_na(relative_snr, adjustment)
      include 'global.inc'
c
c  The adjustment for co-channel NTSC-to-ATV interference is determined
c  from measurements made by the Advanced Television Test Center.  The
c  data (first two columns below) come from tests of the Grand Alliance
c  System reported in October, 1995.
c
c  The last two columns are derived from the measured data.  Values for
c  the extremes are chosen to permit extrapolation.
c
c         Measured Data
c       -----------------    Relative  D/U adj-
c        SNR        D/U        SNR     justment
c       -----      -----     -------   --------
c       15.35      very high   0.00     30.00
c       16.35      19.94       1.00     18.75
c       17.35      17.69       2.00     16.50
c       18.35      16.44       3.00     15.25
c       19.35       7.19       4.00      6.00
c       20.35       4.69       5.00      3.50
c       21.35       3.69       6.00      2.50
c       22.35       2.94       7.00      1.75
c       23.35       2.44       8.00      1.25
c       very high   1.19      20.00      0.00
c
      real dB(0:9)
      data (dB(i), i=0, 9) /
     &   30.0, 18.75, 16.50, 15.25, 6.00, 3.50, 2.50, 1.75, 1.25, 0.00/
      x = max(0.0, relative_snr)
      i = int(x)
      if (i .le. 7) then
         adjustment = dB(i) + (x-real(i)) * (dB(i+1)-dB(i))
      else
         y = dB(8) + (dB(9)-dB(8))*(x-8.0)/16.0
         adjustment = max(0.0, y)
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        fill_problem_grid
c
c  This routine finds the total population within each grid cell of the
c  problem area and the coordinates of the centroid of the census blocks
c  within each cell.  The centroid coordinates will be used when
c  computing coverage and IX at the cells.
c
c  The centroid latitude of cells with 0 population may be flagged with
c  an out of range value to indicate they should be skipped during
c  coverage and IX calculations.
c
c  Grid cells are numbered east to west and south to north as in the
c  example below.
c
c  20 19 18 17 16 15 14 13 12 11
c  10  9  8  7  6  5  4  3  2  1
c
c***********************************************************************
c
      subroutine fill_problem_grid
      include 'global.inc'
      parameter (MAX_CENSUS_BLKS = 800)
c
      integer ipops(MAX_CENSUS_BLKS), ic_lat(MAX_CENSUS_BLKS),
     &   ic_lon(MAX_CENSUS_BLKS)
c
c  Allocate memory to store cell population totals
c
      call allocate_pop_mem(n_cells, ptr_pop_tot)
c
c  Determine the southern and eastern latitude and longitude boundary
c  of the problem area in seconds
c
      lat_b = lat_cg - (num_cell_h * icell_h)
      lon_b = lon_cg - (num_cell_w * icell_w)
c
c  Evaluate one cell at a time starting at the SE corner and moving
c  east to west
c
      n_n = 1
      n_p = 0
      n_rec = 0
      do i=1, num_cell_h * 2
         n = n_n
         k1 = lat_b + (icell_h * (i - 1))
         k2 = k1 + icell_h
         if (i .eq. num_cell_h * 2) k2 = k2 + 1
         do j=1, num_cell_w * 2
            n_pop_tot = 0
            l1 = lon_b + (icell_w * (j - 1))
            l2 = l1 + icell_w
            if (j .eq. num_cell_w * 2) l2 = l2 + 1
            n_pts = 0
            do m=n, num_blocks
               if (census_block_array(m).latsec .ge. k2) then
                  n_n = m
                  goto 50
               else
                  if (census_block_array(m).latsec .ge. k1 .and.
     &               census_block_array(m).lonsec .ge. l1 .and.
     &               census_block_array(m).lonsec .lt. l2) then
                     n_pts = n_pts + 1
c
c  Check for too many points
c
                     if (n_pts .gt. MAX_CENSUS_BLKS) then
                        write(0, 900) census_block_array(m).latsec,
     &                     census_block_array(m).lonsec,
     &                     MAX_CENSUS_BLKS
                        stop
                     end if
c
c  Add another census block to this grid cell
c
                     ic_lat(n_pts) = census_block_array(m).latsec
                     ic_lon(n_pts) = census_block_array(m).lonsec
                     ipops(n_pts) = census_block_array(m).ipop
                     n_pop_tot = n_pop_tot + census_block_array(m).ipop
                  end if
               end if
            end do
c
c  Find the population centroid and store as 1st element of the
c  icell_offsets array for this cell.  If there is no population so
c  that the centroid cannot be computed, use the geometrical center or
c  flag the cell by placing the value IGNORED_CELL (32100) in the
c  latitude offset.
c
   50       continue
            if (n_pop_tot .eq. 0) then
               if (flagged_offsets) then
                  n_p = n_p + 1
                  icell_offsets(n_p) = IGNORED_CELL
                  n_p = n_p + 1
                  icell_offsets(n_p) = 0
               else
                  n_p = n_p + 1
                  icell_offsets(n_p) = (i - num_cell_h - 0.5)*icell_h
                  n_p = n_p + 1
                  icell_offsets(n_p) = (j - num_cell_w - 0.5)*icell_w
               end if
            else
               call find_cell_ctrd(n_pts, ic_lat, ic_lon,
     &            ipops, n_pop_tot)
c
c  Save the cell offset from the grid center in memory and the cell
c  population total on disk.
c
               n_p = n_p + 1
               icell_offsets(n_p) = ic_lat(1) - lat_cg
               n_p = n_p + 1
               icell_offsets(n_p) = ic_lon(1) - lon_cg
            end if
c
            n_rec = n_rec + 1
            i_pop_tot(n_rec) = n_pop_tot
         end do
      end do
      return
c
  900 format('Too many census block within grid cell at', i7, ',', i7/
     &   'In routine fill_problem_grid the maximum number is', i5/
     &   'Rerun with a smaller cell size or modify the program')
      end
c  %W% %G%
c
c***********************************************************************
c
c                           find_cell_ctrd
c
c  This routine finds the population-weighted centroid of a group of
c  points.  The coordinates, in seconds, of the individual points are
c  given by the input arrays ic_lat, ic_lon; the input array ipops
c  contains the respective populations, and n_pop_tot is the sum of
c  these populations.
c
c  The coordinates of the centroid are returned as the first elements of
c  the input arrays ic_lat and ic_lon.
c
c***********************************************************************
c
      subroutine find_cell_ctrd(n_pts, ic_lat, ic_lon,
     &   ipops, n_pop_tot)
      integer ipops(*), ic_lat(*), ic_lon(*)
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Equate routine for reporting errors
c
      saved_routine_name = routine_name
      routine_name = 'find_cell_ctrd'
c
c  Get population weighted coordinates
c
      x = 0.0
      y = 0.0
c
      do i=1, n_pts
         y = y + (float(ic_lat(i)) *
     &      (float(ipops(i))/float(n_pop_tot)))
         x = x + (float(ic_lon(i)) *
     &      (float(ipops(i))/float(n_pop_tot)))
      end do
c
      ic_lat(1) = int (y + 0.5)
      ic_lon(1) = int (x + 0.5)
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         find_len_btw_ch
c
c  This routine searches character variable a for the first pair of
c  btw_ch characters and returns the position of the first and last
c  character inside the btw_chs and the total number of the characters
c  inside the btw_chs.
c
c***********************************************************************
c
      subroutine find_len_btw_ch(a, ch1, ch2, n_l, n_r, n_t)
      character a*(*), ch1*1, ch2*1
      character*1 btw_ch(2)
c
c  Fill array btw_ch
c
      btw_ch(1) = ch1
      btw_ch(2) = ch2
c
c  Get total length of a
c
      l = lnblnk(a)
c
c  Initialize n_l, n_r, n_t
c
      n_l = 0
      n_r = 0
      n_t = 0
c
c  Search for character pair
c
      do i=1, l
         if (a(i:i) .eq. btw_ch(1) .and. n_l .eq. 0) n_l = i
         if (a(i:i) .eq. btw_ch(2) .and. n_r .eq. 0 .and.
     &      n_l .ne. 0 .and. i .ne. n_l) then
            n_r = i
            goto 100
         end if
      end do
c
c  Check to make sure both [ and ] were found
c
  100 continue
      if ( n_l .eq. 0 .or. n_r .eq. 0 ) then
         n_l = 0
         n_r = 0
         goto 500
      end if
c
c  Get total characters and adjust n_l and n_r
c
      n_l = n_l + 1
      n_r = n_r - 1
      n_t = (n_r - n_l) + 1
c
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        find_record
c
c  This routine searches the common block eng_db for a record matching
c  the city, state and channel provided in the arguments to this
c  routine.  It returns the array position of the record or zero if the
c  record is not found.
c
c***********************************************************************
c
      subroutine find_record(city_s, state_s, sta_type_s,
     &  ichan_s, irec_pos)
      include 'global.inc'
c
      character city_s*20, state_s*2, sta_type_s*1
c
c  Initialize record position to 0
c
      irec_pos = 0
c
c  Convert city and state to upper case and get length of city
c
      call makcap(city_s)
      call makcap(state_s)
      L = min(16, lnblnk(city_s))
c
c  Loop thru records in common block eng_db searching for a match
c
      do 100 i=1, n_eng_rec
         if (ichan_r(i) .ne. ichan_s) goto 100
         L2 = min(16, lnblnk(city_r(i)))
         if (city_r(i)(1:L2) .ne. city_s(1:L)) goto 100
         if (state_r(i) .ne. state_s) goto 100
         if (sta_type(i) .ne. sta_type_s) go to 100
         irec_pos = i
         goto 500
  100 continue
c
  500 continue
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          free_space
c
c  This routine computes the field at a distance d using the free space
c  formula.  Computations are made at 1 kw - adjustments for actual power
c  are done outside this routine.
c 
c***********************************************************************
c
      subroutine free_space(d, field)
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'free_space'
c
c  Set erp to 1 kw - see reason above
c
      erp = 1.0
c
c  Check for d very near zero and avoid log(0) error
c  Compute free space field at distance d
c
      if (d .gt. 0.0) then
         field = 106.92+10.0*log10(erp)-20.0*log10(d)
      else
         field = 106.92+10.0*log10(erp)
      end if
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        fs_error
c
c  This routine prints the error/warning messages that are indicated
c  by the variable flag returned by the routines tvfmfs_metric, tvfmfs
c  and tvfmfs. The argument routine is the name of the calling
c  routine.
c
c***********************************************************************
c
      subroutine fs_error(flag, routine)
      character flag*2, routine*20
c
c  Write messages associated with the flag
c
      if (flag .eq. '  ') goto 500
      if (flag .eq. 'A1' .or. flag .eq. 'A7' .or.
     &   flag .eq. 'A8') then
         write(0, 910)
         write(0, 900) routine
         IF (FLAG .EQ. 'A1') WRITE (0, 971)
         IF (FLAG .EQ. 'A7') WRITE (0, 977)
         IF (FLAG .EQ. 'A8') WRITE (0, 978)
      else
         write(0, 920)
         write(0, 900) routine
         IF (FLAG .EQ. 'A2') WRITE (0, 972)
         IF (FLAG .EQ. 'A3') WRITE (0, 973)
         IF (FLAG .EQ. 'A4') WRITE (0, 974)
         IF (FLAG .EQ. 'A5') WRITE (0, 975)
         IF (FLAG .EQ. 'A6') WRITE (0, 976)
         IF (FLAG .EQ. 'A9') WRITE (0, 979)
      end if
  500 return
c
  900 format('In routine: ', a20)
  910 format(20x, 'WARNING - WARNING - WARNING - WARNING')
  920 format(20x, 'ERROR - ERROR - ERROR - ERROR - ERROR')
  971 format('FREE SPACE EQUATION USED TO FIND REQUESTED',
     &   ' ARGUMENT'/ )
  972 format('DISTANCE EXCEEDS GREATEST VALUE ON CURVES.'/)
  973 format('INVALID CHANNEL NUMBER'/)
  974 format('INVALID CURVE SELECTED'/)
  975 format('INVALID SWITCH VALUE'/)
  976 format('ERP LESS THAN OR EQUAL TO 0 KILOWATTS'/)
  977 format('HAAT LESS THAN 30 METERS, SET TO 30 METERS'/)
  978 format('HAAT EXCEEDS 1600 METERS, SET TO 1600 METERS'/)
  979 format('INVALID "DISTANCE" VALUE FOR SWITCH = 1'/)
      end
c  %W% %G%
c
c***********************************************************************
c
c                           get_3sec_corners
c
c  This routine determines the coordinates of the four corners of the 3
c  arc-second box containing the input point coordinates.
c
c***********************************************************************
c
      subroutine get_3sec_corners(pt_lat, pt_lon,
     &    pt_corner_lat, pt_corner_lon )
      double precision pt_lat, pt_lon
      double precision pt_corner_lat(4), pt_corner_lon(4)
c
      include 'global.inc'
c
      double precision pts_per_deg_lat, pts_per_deg_lon
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'free_space'
c
c  Set the points per degree to 3 seconds.  Points have been added
c  above 50 degrees latitude where the original terrain data provided
c  points every 6 seconds.
c
      pts_per_deg_lat = 1200.0
      pts_per_deg_lon = 1200.0
c
c  Find corners in degrees
c
      pt_corner_lat(1) =
     &   int(pt_lat * pts_per_deg_lat ) / pts_per_deg_lat
      pt_corner_lat(2) = pt_corner_lat(1) + 1.0 / pts_per_deg_lat
      pt_corner_lat(3) = pt_corner_lat(2)
      pt_corner_lat(4) = pt_corner_lat(1)
c
      pt_corner_lon(1) =
     &   int(pt_lon * pts_per_deg_lon ) / pts_per_deg_lon
      pt_corner_lon(3) = pt_corner_lon(1) + 1.0 / pts_per_deg_lon
      pt_corner_lon(2) = pt_corner_lon(1)
      pt_corner_lon(4) = pt_corner_lon(3)
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                           get_3sec_elevation
c
c  Given a point in NAD83 coordinates, find the elevation at that point
c  using 3 arc sec terrain data.
c
c***********************************************************************
c
      subroutine get_3sec_elevation(d_pt_lat, d_pt_lon, elev)
      double precision d_pt_lat, d_pt_lon
      include 'global.inc'
      double precision pd
c
c  Increment time stamp
c
      next_time = next_time + 1
      if (next_time .lt. 0) then
         write(0, '(a)') 'Reinitializing terrain memory'
         call init_ter_mem
         next_time = 1
      end if
c
c  Find the index of the 7.5' block containing the given point
c
      pt_lat = d_pt_lat
      pt_lon = d_pt_lon
      call get_terrain_index(pt_lat, pt_lon, num_f, index)
c
c  Determine if this 7.5' block is in memory
c
      if (terrain_mem(index) .ne. 0) then
         n_loc = terrain_mem(index)
         call get_se_corner(pt_lat, pt_lon, se_lat, se_lon)
c
c  Terrain block not now in memory so it must be loaded
c
      else
         map_loads = map_loads + 1
         call get_se_corner(pt_lat, pt_lon, se_lat, se_lon)
         call load_75_terrain(se_lat, se_lon)
         n_loc = next_pt
      end if
c
c  Save time stamp
c
      i_time(n_loc) = next_time
c
c  Check for no terrain data available and set elevation to -10000
c  this will be used by other routines to flag points that do not
c  have elevation data availble
c
      if (i_elev(n_loc, 1) .eq. -10000.0) then
         elev = -10000.0
         goto 500
      end if
c
c  Get elevation at point
c
      pd = 1200.0
c
      iy = 1 + int((d_pt_lat - se_lat) * pd)
      ix = int((d_pt_lon - se_lon) * pd)
c
      ip = iy + (ix * 150)
      elev = float(i_elev(n_loc, ip))
c
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          get_75_parms
c
c  This routine gets the north/south bias, east/west bias, elevation
c  bias, and number of bits used to store elevations for a 7.5 minute
c  terrain block at the index given by the argument idx.  The latter
c  ranges from 1 to LAST_IDX+1, and the last block is a dummy terminator
c  with its parameters set to 0.
c
c***********************************************************************
c
      subroutine get_75_parms(idx, ns_bias, ew_bias, elevation_bias,
     &   num_bits)
      integer*2 ns_bias, ew_bias, num_bits, elevation_bias
      include 'global.inc'
c
      if (idx .gt. LAST_IDX+1) stop
     &   'Index of terrain blocks out of bounds'
      j = 1 + ((idx - 1) * 4)
      ns_bias = index_75_ter(j)
      ew_bias = index_75_ter(j+1)
      elevation_bias = index_75_ter(j+2)
      num_bits = index_75_ter(j+3)
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                    get_affected_list
c
c  Examine the list of desired stations and specified channel
c  relationships and determine which of the desired stations may be
c  affected by the station whose id is given by the input n.  The list is
c  placed in array list_affected and the number of affected stations is
c  returned in n_aff.
c
c***********************************************************************
c
      subroutine get_affected_list(n, n_aff, ptr_tmp_2)
      pointer (ptr_tmp_2, list_affected(*))
      integer*2 list_affected
c
      include 'global.inc'
c
c  Initialize number of affected stations to 0
c
      n_aff = 0
c
c  Set type of undesired station
c
      itype_u = NTSC
      if (sta_type(n) .eq. 'a') itype_u = ATV
c
c  Loop thru list of desired stations testing the channel relationships
c
      do 100 i=1, n_des_sta
         id = desired_stations(i).id
         if (id .eq. n) goto 100
c
         itype_d = NTSC
         if (sta_type(id) .eq. 'a') itype_d = ATV
         if (itype_u .eq. NTSC) then
            ncase = N_TO_N
            if (itype_d .eq. ATV) ncase = N_TO_A
         else
            ncase = A_TO_N
            if (itype_d .eq. ATV) ncase = A_TO_A
         end if
c
         idif = ichan_r(n) - ichan_r(id)
         do j=1, 25
            if (ichan_ck(ncase, j) .eq. idif) then
               n_aff = n_aff + 1
               list_affected(n_aff) = id
               goto 100
            end if
         end do
  100 continue
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          get_band
c
c  This routine determines the TV band based in the channel.  Low
c  VHF is band 1, HIGH VHF is band 2 and UHF is band 3.
c
c***********************************************************************
c
      subroutine get_band(ichan, iband)
c
c  Check for proper TV channel
c
      if (ichan .lt. 2 .or. ichan .gt. 69) then
         write(0, 900) ichan
         stop
      end if
c
c  Get band
c
      iband = 1
      if (ichan .gt. 6 .and. ichan .lt. 14) iband = 2
      if (ichan .gt. 13) iband = 3
      return
c
  900 format('Bad channel number = ', i5)
      end
c  %W% %G%
c
c***********************************************************************
c
c                            get_haat
c
c  This routine determines the HAAT for a given station, azimuth and
c  number of standard radials.  The routine first checks to see if the
c  HAAT values for the station have been previously computed (are in file
c  of HAAT values) and if not then computes the HAAT for the azimuth
c  specified.  The computations are made by calling routine get_rad_haat.
c  If for some reason the HAAT cannot be determined then the routine will
c  return a value of -10000 as a flag.
c
c  The inputs to the routine are the azimuth in degrees (az), the rcamsl
c  in meters, the site coordinates in double precision degrees (dlat1,
c  dlon1), and the engineering data base record number idb.  The number
c  of standard radials over which HAAT is calculated is held in global
c  common.  The output is in meters.
c
c  If the variable compute_haat in global common is false, the routine
c  sets the HAAT to -10000 and returns.
c
c  If replicate is true then the HAAT returned for an ATV station
c  will be that of the paired NTSC station.
c
c***********************************************************************
c
      subroutine get_haat(az, amslrc, dlat1, dlon1, idb, rad_haat)
      include 'global.inc'
      double precision dlat1, dlon1
      logical not_found
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'get_haat'
c
c  If HAAT values are not to be computed (or retrieved from previously
c  computed values), set rad_haat to -10000 and return
c
      if (.not. compute_haat) then
         rad_haat = -10000
         goto 500
      end if
c
c  Find standard azimuths on either side of az
c
      az_std = 360.0 / float(n_std_rad)
      iaz_1 = int(az / az_std) + 1
      iaz_2 = iaz_1 + 1
      if (iaz_2 .gt. n_std_rad) iaz_2 = 1
      az_1 = (iaz_1 - 1) * az_std
      az_2 = (iaz_2 - 1) * az_std
c
c  Set an alias for the engineering data base record number
c
      ialt = idb
c
c  If this is an ATV station and we are replicating, get HAAT data for
c  the paired NTSC
c
      if (ialt .ge. n_first_atv .and. replicate)
     &   ialt = ialt - (n_first_atv - 1)
c
c  Retrieve HAAT data for the standard azimuths
c
      call load_haat(ialt, not_found)
c
c  If HAAT values were not found, compute them by calling routine
c  get_rad_haat
c
      if (not_found) then
         call get_rad_haat(az_1, amslrc, dlat1, dlon1, avg_elev,
     &      rad_haat_1)
         call get_rad_haat(az_2, amslrc, dlat1, dlon1, avg_elev,
     &      rad_haat_2)
         if (rad_haat_1 .eq. -10000 .or. rad_haat_2 .eq.-10000) then
            rad_haat = -10000
            goto 500
         end if
      else
         iaz_1 = int(az_1/az_inc_haat) + 1
         iaz_2 = int(az_2/az_inc_haat) + 1
         rad_haat_1 = radial_haat(iaz_1)
         rad_haat_2 = radial_haat(iaz_2)
      end if
c
c  Interpolate to find HAAT at the desired az
c
      rad_dif = rad_haat_2 - rad_haat_1
      az_inc = (az - az_1) / az_std
      rad_haat = rad_haat_1 + (az_inc * rad_dif)
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c
c***********************************************************************
c
c                            get_rad_haat
c
c  This routine computes the haat for a given location and azimuth
c  based on the method specified in the FCC R&R.  If terrain data
c  is not found at a point along the desired path that point is
c  ignored.  If more than half the points have errors the routine
c  will return a value of -10000.0.  Also if switch compute_haat is
c  set to false the routine returns a value of -10000. Terrain
c  elevations are retrieved every 0.1 km between 3.2 and 16.1 km.
c
c  The inputs to the routine are the azimuth in degrees (az), the
c  RCAMSL in meters and the site coordinates in double precision
c  degrees (dlat1, dlon1).  The output is in meters.
c
c***********************************************************************
c
      subroutine get_rad_haat(az, amslrc, dlat1, dlon1, avg_elev,
     &   rad_haat)
      double precision dlat1, dlon1, dlat2, dlon2
      include 'global.inc'
c
c  Local variables
c
      character saved_routine_name*32
      real pt_elevation(150)
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'get_rad_haat'
c
c  If HAATs are not to be computed, return with rad_haat and avg_elev set
c  to -10000
c
      avg_elev = -10000.0
      rad_haat = -10000.0
      if (.not. compute_haat) goto 500
c
c  Get coordinates of points along path.  Terrain data is to be retrieved
c  at 0.1 km intervals between 3.2 and 16.1 km from the site along the
c  specified azimuth.
c
      n_ter_pt = 0
      r_inc = 0.1
      adist = 3.1
c
c  Loop over distance increments
c
   10 continue
      adist = adist + r_inc
      if (adist .gt. 16.11) goto 400
      call calcrd_r2(dlat1, dlon1, adist, az, dlat2, dlon2)
      rlat = dlat2
      rlon = dlon2
c
c  Get NAD83 coordinates of point
c
      call get_nad_83_coord(rlat, rlon, rlat_83, rlon_83, n_error_code)
c
c  Report error and flag point
c
      if (n_error_code .ne. 0) then
         write(0, 900) rlat, rlon, routine_name(1:lnblnk(routine_name))
         elev = -10000.0
         goto 50
      end if
c
c  Get terrain elevation
c
      call get_point_elevation(rlat_83, rlon_83, elev)
c
   50 continue
      n_ter_pt = n_ter_pt + 1
      pt_elevation(n_ter_pt) = elev
      goto 10
c
c  Compute average elevation and subtract from RCAMSL.  If value is
c  less than 30.5 m, set HAAT to 30.5.
c
  400 continue
      n = 0
      sum = 0.0
      do 100 i=1, n_ter_pt
         if (pt_elevation(i) .eq. -10000.0) goto 100
         n = n + 1
         sum = sum + pt_elevation(i)
  100 continue
c
      if (n .lt. n_ter_pt/2) goto 500
      avg_elev = sum/n
      rad_haat = amslrc - avg_elev
      if (rad_haat .lt. 30.5) rad_haat = 30.5
c
  500 continue
      routine_name = saved_routine_name
      return
c
  900 format('Error retrieving terrain data at ', f9.4, 1x, f9.4,
     &   ' in routine ', a)
      end
c
c***********************************************************************
c
c                          load_haat
c
c  Fetch precalculated HAAT values for the requested station from the
c  disk file already opened with unit number ihaat_file.
c
c***********************************************************************
c
      subroutine load_haat(idb, not_found)
      include 'global.inc'
      logical not_found
c
c  Find record number of HAAT file corresponding to record number idb
c  of the engineering data base.
c
      n_rec = index_haat(idb)
      not_found = n_rec .le. 0
      if (not_found) goto 500
c
c  Check if record requested is already in common block
c
      if (n_rec .eq. last_haat) goto 500
      last_haat = 0
c
c  Get radials of current interest
c
      read(ihaat_file, rec=n_rec, end=400) ref_rcamsl,
     &   (radial_haat(i), i=1, n_haat_radials)
      height_increase = rcamsl_r(idb) - ref_rcamsl
      if (height_increase .ne. 0.0) then
         if (warn) write(0, 1000) height_increase, state_r(idb),
     &      city_r(idb)(1:lnblnk(city_r(idb))), ichan_r(idb),
     &      sta_type(idb)
         do i=1, n_haat_radials
            radial_haat(i) = radial_haat(i) + height_increase
         end do
      end if
      last_haat = n_rec
      goto 500
c
c  Requested record is beyond end of file
c
  400 continue
      write(0, 1010) state_r(idb), city_r(idb)(1:lnblnk(city_r(idb))),
     &   ichan_r(idb)
      not_found = .true.
c
c  Normal exit
c
  500 return
c
 1000 format('RCAMSL increase of', f6.1, ' m for ', a2, 1x, a,
     &   ', channel', i3, a1)
 1010 format('No HAAT record for ', a2, 1x, a, ', NTSC channel', i3)
      end
c  %W% %G%
c
c***********************************************************************
c
c                         get_nad_83_coord
c
c  This routine determines the NAD83 coordinates of a point given the
c  NAD27 coordinates.
c
c***********************************************************************
c
      subroutine get_nad_83_coord(pt_lat_27, pt_lon_27,
     &   pt_lat_83, pt_lon_83, n_error_code )
      integer*2 ns_bias, ew_bias, num_bits, elevation_bias
      logical debug /.false./
c
c  Initialize error code to 0
c
      n_error_code = 0
c
c  Initialize NAD83 coords to impossible values to flag lack of data
c
      pt_lat_83 = -100.0
      pt_lon_83 = -200.0
c
c  Estimate NAD83 coords
c
      est_lat = pt_lat_27
      est_lon = pt_lon_27
c
c  Initialize previous index
c
      idx_p = -1
      iterations = 0
c
c  Get southeast corner of 7.5 minute block containing the point
c
      do while (iterations .le. 3)
         call get_se_corner(est_lat, est_lon, se_lat, se_lon)
c
c  Get index
c
         call get_terrain_index(se_lat, se_lon, num_f, idx)
c
c  The correct block has been found if the index has not changed
c
         if (idx .eq. idx_p) goto 500
c
c  Get parameters for this 7.5 minute block.  Error code 3 means no data.
c
         call get_75_parms(idx, ns_bias, ew_bias, elevation_bias,
     &      num_bits)
         if (num_bits .eq. 0) then
            n_error_code = 3
            goto 510
         end if
c
c  Add bias to form new estimate
c
         est_lat = pt_lat_27 + (float(ns_bias) / 1.0e6 )
         est_lon = pt_lon_27 + (float(ew_bias) / 1.0e6 )
         iterations = iterations + 1
         idx_p = idx
      end do
c
c  Too many iterations
c
      if (debug) write(0, 1000) pt_lat_27, pt_lon_27
c
c  Normal exit
c
  500 continue
      pt_lat_83 = est_lat
      pt_lon_83 = est_lon
c
c  Exit when no data was found for one of the trial blocks
c
  510 continue
      return
c
 1000 format('Trouble converting', f8.4, 'N,', f9.4,
     &   'W to NAD83 coordinates')
      end
c  %W% %G%
c
c***********************************************************************
c
c                        get_point_elevation
c
c  Given a point in NAD83 coordinates get the elevation by interpolation
c  of the elevations of the elevations that are at the corner of a
c  3" box containing the point.  Routine get_3sec_elevation does the
c  actual retrieval of the elevations at the 3" corner points.
c
c***********************************************************************
c
      subroutine get_point_elevation(pt_lat, pt_lon, elev)
      include 'global.inc'
c
      double precision d_pt_lat, d_pt_lon, d_cor_lat(4), d_cor_lon(4)
      logical in_water
      real c_elev(4)
c
c  Get corners of 3" box containing point
c
      d_pt_lat = pt_lat
      d_pt_lon = pt_lon
      call get_3sec_corners(d_pt_lat, d_pt_lon, d_cor_lat,
     &   d_cor_lon)
c
c  Get elevations of corner points
c
      do i=1, 4
         call get_3sec_elevation(d_cor_lat(i), d_cor_lon(i),
     &      c_elev(i))
c
c  Return a large negative value for the elevation if there is no
c  terrain data for one of these corners, unless the corner point is
c  in the water.  Set the elevation of water points to 0.0 so that
c  terrain profiles will not be interrupted.
c
         if (c_elev(i) .eq. -10000.0) then
            x_lat = d_cor_lat(i)
            x_lon = d_cor_lon(i)
            call ck_in_water(x_lat, x_lon, in_water)
            if (in_water) then
               c_elev(i) = 0.0
            else 
               elev = -10000.0
               goto 500
            end if
         end if
      end do
c
c  Interpolate to get elevation at actual point
c
      call interpolate_4_elevations(d_pt_lat, d_pt_lon,
     &   d_cor_lat, d_cor_lon, c_elev, elev )
c
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          get_radials_btwn
c
c  This routine determines the radial numbers that a given azimuth
c  is between.  The radials are evenly spaced beginning at 0 degrees
c  (radial 1).  The inputs to the routine are the spacing between
c  the evenly spaced azimiths in degrees and the azimuth to check.
c
c  If the azimuth equals an evenly spaced azimuth then irad_1 = irad_2.
c
c***********************************************************************
c
      subroutine get_radials_btwn(az_inc, az, irad_1, irad_2)
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'get_radials_btwn'
c
c  Determine the total number of radials
c
      num_rad = int (360.0 / az_inc)
c
      irad_1 = int ( az / az_inc ) + 1
      irad_2 = irad_1 + 1
      if (mod (az, az_inc) .eq. 0.0) irad_2 = irad_1
      if (irad_2 .gt. num_rad) irad_2 = 1
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          get_se_corner
c
c  This routine determines the coordinates of the southeast corner of
c  a 7.5 minute block for a given point (pt_lat, pt_lon).  Coordinates
c  are in decimal degrees.
c
c***********************************************************************
c
      subroutine get_se_corner(pt_lat, pt_lon, se_lat, se_lon)
c
c  Determine the southeast corner coordinates of the 7.5 minute block
c  containing the given point
c
      se_lat = int(8.0 * pt_lat) / 8.0
      se_lon = int(8.0 * pt_lon) / 8.0
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                      get_ter_dirs
c
c  This routine reads the file terrain_directory.dbs to determine where
c  terrain data is stored.  The file contains a list of the directories
c  and the longitude of the western-most block stored in the directory.
c  The directory in which files are stored is determined by the
c  longitude of the terrain block.
c
c***********************************************************************
c
      subroutine get_ter_dirs
      include 'global.inc'
      character*32 directory, filename /'terrain_directory.dbs'/
c
c  Open file
c
      call getlun(infl)
      open(infl, file=filename, status='old', form='formatted',
     &   access='sequential', iostat=ierr)
c
c  Check for error opening file
c
      if (ierr .ne. 0) then
         call perror(filename)
         stop
      end if
c
c  Set next entry to read
c
      n_dir = 1
   10 continue
c
c  Read next entry
c
      read(infl, 800, end=400, err=300) directory, longitude_start
c
c  Check for too many entries
c
      if (n_dir .gt. MAX_DIR) then
         write(0, 910) filename(1:lnblnk(filename)), MAX_DIR
         stop
      end if
c
c  Store directory name and recyle
c
      dir_list(n_dir) = directory
      lon_dir(n_dir) = longitude_start
      n_dir = n_dir + 1
      goto 10
c
c  Error reading file
c
  300 continue
      write(0, 920)
      stop
c
c  Adjust counter, close file and return
c
  400 continue
      n_dir = n_dir - 1
      close(infl)
c
  500 continue
      return
c
  800 format(a32, 1x, i3)
  910 format('Too many entries in ', a, '.  Limit is', i3,
     &   ' directories')
  920 format('Error reading file terrain_directory.dbs')
      end
c  %W% %G%
c
c***********************************************************************
c
c                           get_terrain_index
c
c  Given the geographic coordinates of a point in decimal degrees, this
c  routine returns the index to the array where the parameters of the
c  7.5 minute block are stored and the number of the 7.5 minute data
c  file within the 1 degree block of data (files named as follows:
c  ter_latxlon_nr.dbs, where lat and lon are integer lat and lon,
c  and nr ranges from 1 to 64).
c
c  Example: the file for 40.21 and 74.23 would be ter_40x074_10.dbs,
c  where nr=10 is 8*int(0.23/0.125) + int(0.21/0.125) + 1.
c
c***********************************************************************
c
      subroutine get_terrain_index(d_lat, d_lon, num_f, index)
      include 'terrain_indices.inc'	!defines the integer FIRST_BLOCK
c
      i1 = int(d_lon) * 5760
      i2 = int(d_lat) * 64
      i3 = 8 * (int((d_lon - int(d_lon)) / 0.125))
      i4 = int((d_lat - int(d_lat)) / 0.125)
c
c  FIRST_BLOCK is the offset to the first lat/lon box.  When int(d_lat)
c  and int(d_lon) are equal respectively to the starting latitude and
c  longitude, the index will be 1.
c
      num_f = i3 + i4 + 1
      index = i1 + i2 + num_f - FIRST_BLOCK
      index = min(index, LAST_IDX+1)
      return
      end
c
      integer function lat_from_index(index)
      include 'terrain_indices.inc'
      iblock = FIRST_BLOCK + index - 1
      lat_from_index = mod(int(iblock/64), 90)
      return
      end
c
      integer function lon_from_index(index)
      include 'terrain_indices.inc'
      lat = lat_from_index(index)
      iblock = FIRST_BLOCK + index - 1
      lon_from_index = (iblock - 64*lat)/64/90
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        get_terrain_profile
c
c  This routine determines the NAD83 coordinates of points along a path
c  between end points (spt_lat, spt_lon) and (ept_lat, ept_lon) and then
c  finds the elevations at those points along the path.  The distance
c  between points is given by d_inc (km).  If a problem is encountered
c  in determining the elevations, n_error_code is set to a non-zero value.
c
c***********************************************************************
c
      subroutine get_terrain_profile(spt_lat, spt_lon, ept_lat,
     &   ept_lon, d_inc, n_error_code)
      include 'global.inc'
c
      double precision dlat1, dlon1, dlat2, dlon2
      logical in_water
      real path_lat(5000), path_lon(5000)
c
c  Initialize error flag and the number of terrain points
c
      n_error_code = 0
      n_ter_pt = 0
c
c  Get starting and ending points in NAD83.  The starting point must be
c  found, otherwise the program stops.
c
      call get_nad_83_coord(spt_lat, spt_lon, spt_lat_83,
     &   spt_lon_83, n_error_code )
      if (n_error_code .ne. 0) then
         write(0, '(a)') 'Error getting starting point coordinates:'
         write(0, *) spt_lat, spt_lon, spt_lat_83, spt_lon_83
         call exit(257)	!stop with non-zero exit status
      end if
c
c  If the end point is not found, return an error code and no profile
c  points, except when producing data for graphs.  When not producing data
c  for graphs, the path should be ignored imputing neither service nor
c  interference at the terminating cell.  For graphs, check if the path
c  end point is in water and if so use its NAD27 coordinates.
c
      call get_nad_83_coord(ept_lat, ept_lon, ept_lat_83,
     &   ept_lon_83, n_error_code)
      if (n_error_code .ne. 0) then
         if (graphs) call ck_in_water(ept_lat, ept_lon, in_water)
         if (graphs .and. in_water) then
            ept_lat_83 = ept_lat
            ept_lon_83 = ept_lon
         else
            if (warn) write (0, '(a,f7.3,a2,f8.3)')
     &         'Error converting coords of path endpoint:',
     &         ept_lat, ', ', ept_lon
            goto 500
         end if
      end if
c
c  Save starting point
c
      n_ter_pt = n_ter_pt + 1
      path_lat(n_ter_pt) = spt_lat_83
      path_lon(n_ter_pt) = spt_lon_83
      call deg_to_sec(spt_lat_83, lat1)
      call deg_to_sec(spt_lon_83, lon1)
      call deg_to_sec(ept_lat_83, lat2)
      call deg_to_sec(ept_lon_83, lon2)
      call distance(lat1, -lon1, lat2, -lon2, dist_km)
      call azimuth( lat1, -lon1, lat2, -lon2, dist_km, az_rad)
      az_deg = az_rad * rad_to_deg
c
c  Calculate coordinates of points along the path, increment d_inc (km).
c  Per the way the program was run for the 6th R&O and the February 1998
c  reconsideration order, the last point included in the profile may be
c  short of the receiver by nine-tenths of d_inc but never beyond by as
c  much as one-tenth of d_inc.  It makes better sense to accept the
c  nearest point to the receiver from the ladder of points spaced d_inc
c  from the transmitter.  The last point in the profile will then be no
c  farther from the receiver than half of d_inc.
c
      dlat1 = spt_lat_83
      dlon1 = spt_lon_83
      delta = 0.0
      delta_last = dist_km - 0.9*d_inc
      if (mod3 .and. .not. per_6th_order)
     &   delta_last = dist_km - 0.5*d_inc
      do while (delta .le. delta_last)
         delta = delta + d_inc
         call calcrd_r2(dlat1, dlon1, delta, az_deg, dlat2, dlon2)
         n_ter_pt = n_ter_pt + 1
         if (n_ter_pt .gt. 5000) then
            write(0, '(a,i5)') 'Number of points on radial ', n_ter_pt
            stop 'Too many points'
         end if
         path_lat(n_ter_pt) = dlat2
         path_lon(n_ter_pt) = dlon2
      end do
c
c  Get the terrain elevation of these points
c
      n = 0
      do while (n .lt. n_ter_pt)
         n = n + 1
         call get_point_elevation(path_lat(n), path_lon(n), elev)
         npath_pts = npath_pts + 1
         path_elev_pt(n) = elev
         if (elev .lt. -999.0) then
            n_error_code = 3
            if (n .eq. 1) n_error_code = 1
         end if
      end do
c
c  Number of terrain points is 0 in case of any error
c
      if (n_error_code .ne. 0) n_ter_pt = 0
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                               get_tv_freq
c
c  This routine given a TV channel returns the center frequency of the
c  channel in MHz.
c
c***********************************************************************
c
      subroutine get_tv_freq(ichan, freq)
c
      freq = 0.0
      if (ichan .eq. 0) return
      call get_band(ichan, iband)
      goto (20, 40, 60) iband
c
c  Low VHF
c
   20 continue
      freq = 57.0 + (6.0 * (ichan - 2))
      if (ichan .gt. 4) freq = freq + 4.0
      return
c
c  High VHF
c
   40 continue
      freq = 177.0 + (6.0 * (ichan - 7))
      return
c
c  UHF
c
   60 continue
      freq = 473.0 + (6.0 * (ichan - 14))
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        get_undesired_list
c
c  This routine searches the list of stations and identifies those that
c  may be sources of interference based on their distance from the
c  problem area grid and the list of channel-distance relationships in
c  common block undesired_srch_list.  The latter was created by routine
c  def_srch_chans.
c
c***********************************************************************
c
      subroutine get_undesired_list
      include 'global.inc'
c
c  Local variables
c
      logical outside
      pointer (ptr_tmp, i_undes_list(*))
      integer*2 i_undes_list
c
c  Temporary memory for collection of an as yet unknown number of
c  undesired stations
c
      ptr_tmp = malloc(2*n_eng_rec)
c
c  Initialize number of undesired stations to 0
c
      n_undes_sta = 0
      n_unntsc = 0
c
c  Outer loop over station records
c
      do 100 i=1, n_eng_rec
c
c  Loop thru list of channel-distance relationships to be considered.
c  The array of undesired distances is arranged in the following order:
c  N to N, N to A, A to N and A to A with 68 possible channels for
c  each case
c
         n = 1
         if (sta_type(i) .eq. 'a') n = 3
         do j=n, n + 1
            do k=1, 68
               if (ichan_r(i) .eq. (k + 1) .and.
     &            search_dist_list(j, k) .gt. 0.0) then
c
c  Check distance to cell boundary
c
                  call ck_if_within(search_dist_list(j, k),
     &               lat_site(i), lon_site(i), lat_lim(1), lon_lim(1),
     &               lat_lim(2), lon_lim(2), outside)
                  if (.not. outside) then
                     n_undes_sta = n_undes_sta + 1
                     i_undes_list(n_undes_sta) = i
                     goto 100
                  end if
               end if
            end do
         end do
  100 continue
c
c  Allocate memory for list of undesired stations
c
      call allocate_unds_mem
c
c  Copy list of undesired stations to the probably smaller array which
c  will be retained during the process of evaluating undesired fields at
c  cells of the problem area grid
c
      do i=1, n_undes_sta
         id_undesired(i) = i_undes_list(i)
      end do
c
c  Eliminate desired to desired entries from list.  See comments in
c  following routine for explanation.
c
      call purge_ud_list
c
c  Count NTSC stations
c
      do i=1, n_undes_sta
         idu = id_undesired(i)
         if (sta_type(idu) .eq. 'n') n_unntsc = n_unntsc + 1
      end do
c
c  Free memory allocated for i_undes_list
c
      call free(ptr_tmp)
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          gt_d_ang
c
c  This routine computes the depression angle given the heights of the
c  transmitting and receive antennas and the distance to the receive
c  point.  Heights are assumed to be in meters and the distance in km.
c
c***********************************************************************
c
      subroutine gt_d_ang(txhgt, trdist, rec_hgt, d_ang)
      include 'global.inc'
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'gt_d_ang'
c
c  Angle is 90 degrees when distance is 0.0
c
      if (trdist .eq. 0.0) then
         d_ang = 90.0
         goto 500
      end if
c
c  Get effective height of antenna based on receive antenna height, the
c  distance, and 4/3 earth radius
c
      call gt_e_hgt(txhgt, rec_hgt, trdist, e_hgt)
c
c  Compute angle
c
      d_ang = atan(e_hgt / (trdist * 1000.0))
      d_ang = d_ang * RAD_TO_DEG
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                            gt_e_hgt
c
c  This routine computes the effective HAAT that will be used to
c  determine whether to adjust the computed field due to the proximity
c  to the transmitter.  The computation takes into consideration the
c  given HAAT, the height of the receive antenna and the separation
c  distance.  The heights are in meters and the distance is in km.
c
c***********************************************************************
c
      subroutine gt_e_hgt(haat, rec_hgt, adist, e_haat)
c
c  Convert the distance to miles
c
      d_mi = adist / 1.609344
c
c  Compute h based on 4/3 earth radius, then convert h to meters
c
      h = (d_mi**2.0) / 2.0
      h = h / 3.281
c
c  Compute effective height
c
      e_haat = haat - rec_hgt + h
c
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        gt_vert_rad_fac
c
c  This routine determines the factor by which the station radiation
c  needs to be adjusted due to the proximity to the transmitter.
c  Principal inputs to the routine are the heights of the transmitter
c  and receiver and the distance to the receive point.  Other arguments
c  specify the band, whether NTSC or ATV, and whether the interest is
c  in contour determination or Longley-Rice propagation prediction.
c
c  The factors for depression angles for the various bands and antenna
c  types (NTSC & ATV) are given in the common block xmit_ant.  Variable
c  n_ant is a switch to indicate NTSC (n_ant = 0) or ATV (n_ant = 1).
c  This routine uses the formula provided by the FCC ATV advisory
c  committee, unless the switch use_fcc_xmit_pat is set to true in
c  which case the FCC model is used by calling routine xmit_ant_fcc.
c
c  The transmitter height may be supplied AAT, AGL or AMSL.  The tables
c  in Appendix B of the 6th R&O and reconsideration orders were
c  produced using HAAT for grade B computations and height AGL for
c  terrain dependent fields.
c
c  A shift in depression angle may be supplied as vpat_bias, degrees.
c  The vertical pattern is effectively shifted towards the greater
c  elevations by subracting this value from the actual depression angle
c  before retrieving the pattern factor.
c
c***********************************************************************
c
      subroutine gt_vert_rad_fac(txhgt, rec_hgt, trdist, vpat_bias,
     &   instance, iband, n_ant, factor)
      logical flatten, singled_out
      include 'global.inc'
c
      character*32 saved_routine_name
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'gt_vert_rad_fac'
c
c  Initialize vertical pattern data
c
      if (xant_fac(1, 1, -2) .eq. 0.0) call vertical_symmetry
c
c  Set NTSC/ATV index of array xant_fac
c
         n_a = n_ant + 1
c
c  Check for distance of at least 1 km.  Coverage and interference
c  calculations published with the 6th R&O set the vertical pattern
c  factor equal to unity for points within 1 km of the transmitter.
c  This was also true for the tables published in MO&Os following
c  reconsideration, but it is more reasonable to set the factor to
c  a fractional value.
c
      if (trdist .lt. 1.0) then
         if (mod2 .and. .not. per_6th_order) then
            factor = xant_fac(n_a, iband, 13)
         else
            factor = 1.0
         end if
         goto 500
      end if
c
c  Antenna gain above the main beam in the vertical plane is set to unity
c  when using FCC curves to find grade B contours.  This is also done for
c  Longley-Rice calculations except when special cases have been flagged
c  by adding 360 degrees to vpat_bias.
c
      singled_out = vpat_bias .gt. 270.0
      if (singled_out) then
         shift = vpat_bias - 360.0
      else
         shift = vpat_bias
      end if
c
c  Get depression angle
c
      call gt_d_ang(txhgt, trdist, rec_hgt, d_ang)
      d_ang = d_ang - shift
c
c  Call xmit_ant_fcc to use FCC vertical pattern model.  Otherwise
c  calculate factor in present subprogram.
c
      if (use_fcc_xmit_pat) then
         call xmit_ant_fcc(d_ang, iband, factor)
      else
c
c  Extreme factors
c
         flatten = instance .eq. FCC_CURVES .or. .not. singled_out
         if (d_ang .le. x_ang(1) .and. flatten) then
            factor = 1.0
            goto 500
         else if (d_ang .le. x_ang(-2)) then
            factor = xant_fac(n_a, iband, -2)
            goto 500
         end if
c
c  Interpolation
c
         n = -2
         do i=-1, 13
            if (d_ang .lt. x_ang(i)) goto 50
            n = n + 1
         end do
   50    continue
         if (n .ge. 13) then
            factor = xant_fac(n_a, iband, 13)
         else
            ang_dif = d_ang - x_ang(n)
            adj = ang_dif / (x_ang(n+1) - x_ang(n))
            rad_dif = xant_fac(n_a, iband, n+1)-xant_fac(n_a, iband, n)
            factor = xant_fac(n_a, iband, n) + adj*rad_dif
         end if
      end if
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c
      subroutine vertical_symmetry
c
c  Supplement values of vertical transmit pattern.  The center of
c  symmetry is assumed to be at x_ang(1), and xant_fac(1) should be
c  unity.  Angles and factors are added for indices 0, -1, and -2.
c
      include 'global.inc'
      logical show /.false./
      do i=1, 3
         j = 1 - i
         delta = x_ang(i+1) - x_ang(1)
         x_ang(j) = x_ang(1) - delta
         do k=1, 3
            xant_fac(1, k, j) = xant_fac(1, k, i+1)
            xant_fac(2, k, j) = xant_fac(2, k, i+1)
         end do
      end do
      if (show) then
         write(0, 1000)
         do i=-2, 13
            write(0, 1002) i, x_ang(i),
     &         xant_fac(1, 1, i), xant_fac(2, 1, i),
     &         xant_fac(1, 2, i), xant_fac(2, 2, i),
     &         xant_fac(1, 3, i), xant_fac(2, 3, i)
         end do
      end if
 1000 format(
     &   '                 LOW VHF         HIGH VHF          UHF'/
     &   '               ------------    ------------    ------------'/
     &   'ITEM  ANGLE    NTSC    DTV     NTSC    DTV     NTSC    DTV'/
     &   '----  -----    -----  -----    -----  -----    -----  -----')
 1002 format(i3, f8.2, 3(2x, 2f7.3))
      return
      end
      integer function handler(sig, sip, uap)
c
c  %W% %G%
c
      integer sig, address, code
c
      structure /fault/
           integer address
      end structure
c
      structure /siginfo/
           integer si_signo
           integer si_code
           integer si_errno
           record /fault/ fault
      end structure
      record /siginfo/ sip
c
      character routine_name*32
      common /current_routine/ routine_name
c
c  The following definitions of exception codes 1 through 8 are found
c  in /usr/include/sys/machsig.h:
c
      character*32 meaning(8) /
     &   'integer divide by zero',
     &   'integer overflow',
     &   'floating point divide by zero',
     &   'floating point overflow',
     &   'floating point underflow',
     &   'floating point inexact result',
     &   'invalid floating point operation',
     &   'subscript out of range'/
c
      call flush(6)
      address = sip.fault.address
      code = sip.si_code
      write(0, 1000) address, routine_name(1:lnblnk(routine_name))
      if (code .le. 8) then
         write(0, 1002) code, meaning(code)
      else
         write(0, '(a,i6)') 'Unrecognized exception code', code
      end if
      call flush(0)
      call exit(257)	!or return
c
 1000 format('IEEE exception occurred at address X', Z8.8,
     &   ', routine ', a)
 1002 format('Exception code', i3, ', ', a)
      end
c  %W% %G%
c
c***********************************************************************
c
c                          horiz_ant_fac
c
c  This routine returns the horizontal antenna factor for station i_sta
c  at the specified azimuth.  It will return a factor of 1.0 if the
c  factor cannot be computed for some reason (such as no da data) or if
c  the user did not choose to use da patterns.
c 
c***********************************************************************
c
      subroutine horiz_ant_fac(az, i_sta, h_fac)
      include 'global.inc'
c
c  Initialize h_fac to 1.0, and copy azimuth to local variable so that
c  it can be adjusted according to the pattern reference angle
c
      h_fac = 1.0
      az_p = az
c
c  Check whether NTSC or ATV
c
      if (sta_type(i_sta) .eq. 'n' .or.
     &   (replicate .and. sta_type(i_sta) .eq. 'a' )) then
c
c  Check whether directional antenna patterns are to be used
c
         if (.not. use_da_n) goto 500
c
c  Check whether directional antenna data are available for this station
c
         n_sta = i_sta
         if (sta_type(i_sta) .eq. 'a') n_sta = n_sta - (n_first_atv-1)
         idx_da = index_da(n_sta)
         if (idx_da .eq. 0) goto 500
c
c  Adjust azimuth if pattern reference angle not 0 degrees
c
         if (ref_az_r(n_sta) .ne. 0.0) then
            az_p = az_p - ref_az_r(n_sta)
            if (az_p .lt. 0.0) az_p = 360.0 + az_p
         end if
c
c  Identify directional antenna pattern by its index and get radiation
c  factor.  Return a radiation factor of 1.0 if the pattern is not found.
c
         call get_horiz_rad(idx_da, az_p, h_fac)
      else
c
c  ATV checking
c
         if (.not. use_da_a) goto 500
         idx_da = index_da(i_sta)
         if (idx_da .eq. 0) goto 500
         if (ref_az_r(i_sta) .ne. 0.0) then
            az_p = az_p - ref_az_r(i_sta)
            if (az_p .lt. 0.0) az_p = 360.0 + az_p
         end if
         call get_horiz_rad(idx_da, az_p, h_fac)
      end if
c
  500 continue
      return
      end
c
c***********************************************************************
c
c                         get_horiz_rad
c
c  Compute the horizontal radiation factor given the number of the
c  antenna and the azimuth in degrees.  The antenna number is its place
c  in the directional antenna data base as read from an external file
c  by subroutine load_da_pats.
c
c***********************************************************************
c
      subroutine get_horiz_rad(ia, azimuth, rad_fac_h)
      character*3 make*3, model*14
      include 'global.inc'
c
      character*32 saved_routine_name
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'get_horiz_rad'
c
c  Initialize radiation factor
c
      rad_fac_h = 1.0
c
c  Check if this is same as previous antenna.  No need to sort
c  azimuths and radiations if it is.
c
      if (ia .eq. current_antenna) goto 100
c
c  An antenna number of 0 means it is not in data base.  This routine
c  should never be called with this condition.
c
      if (ia .le. 0) then
         write(0, '(3a)') 'Subroutine ',
     &      routine_name(1:lnblnk(routine_name)),
     &      ' called with zero or negative antenna index'
         call exit(257)
      end if
c
c  Sort azimuths and radiations.  The data base has 36 evenly spaced
c  azimuths that are sometimes followed by as many as 10 special
c  azimuths.
c
      current_antenna = ia
      nn_az = n_ant_az(ia)
      k = 0
      if (nn_az .gt. 36) k = nn_az - 36
      az_inc = 10.0
      az = -10.0
      n = 0
      m = 0
      do 60 i=1, 36
         az = az + az_inc
         n = n + 1
         m = m + 1
         ddbaz(m) = az
         ddbrad(m) =  float(ih_rad(ia, n))/1000.0
         az_c = az
         do l=1, k
            sp_az = float(ispecial_az(ia, l) + 1800) / 10.0
            if (sp_az .gt. az_c .and. sp_az .lt. az + az_inc) then
               m = m + 1
               ddbaz(m) = sp_az
               az_c = sp_az
               ddbrad(m) = float(ih_rad(ia, 36 + l))/1000.0
            end if
         end do
   60 continue
c
c  Get radiation factor at azimuth
c
  100 continue
      do i=1, nn_az
         if (i + 1 .gt. nn_az) then
            end_az = 360.0
         else
            end_az = ddbaz(i+1)
         end if
         if (azimuth .ge. ddbaz(i) .and. azimuth .le. end_az) then
            k = i
            goto 150
         end if
      end do
c
c  Azimuth not found
c
      write(0, 900) azimuth
      call exit(257)
c
c  Interpolate
c
  150 continue
      az_dif = end_az - ddbaz(k)
      az_off = azimuth - ddbaz(k)
      az_fac = az_off/az_dif
      n = k + 1
      if (n .gt. nn_az) n = 1
      rad_dif = ddbrad(k) - ddbrad(n)
      rad_fac_h = ddbrad(k) - (rad_dif * az_fac)
c
c  Check for bad radiation factor values
c
      if (rad_fac_h .le. 0.0 .or. rad_fac_h .gt. 1.0) then
         make = da_string(ia)(1:3)
         model = da_string(ia)(4:17)
         write(0, 910) azimuth, rad_fac_h, make,
     &      model(1:lnblnk(model)), nn_az
         do i=1, nn_az, 2
            write(0, 911)
     &         ddbaz(i), ddbrad(i), ddbaz(i+1), ddbrad(i+1)
         end do
         rad_fac_h = 1.0
      end if
  500 continue
      routine_name = saved_routine_name
      return
c
  900 format('Azimuth ', f7.2, ' not found')
  910 format(
     &   'Error in antenna radiation factor calculation'/
     &   'Azimuth = ', f8.3, ', Factor = ', f8.3/
     &   a, 1x, a, ', number of azimuths =', i3/
     &   ' Azimuth    Factor        Next Azim   Factor')
  911 format(2f10.4, 6x, 2f10.4)
      end
c  %W% %G%
c
c***********************************************************************
c
c                           init_grid
c
c  This routine defines the problem area grid as a rectangle bounded
c  approximately by d_lat_min, d_lon_min, d_lat_max, and d_lon_max.
c  These target values, given in decimal degrees, are an input to this
c  routine.  The size of the grid cells is also an input, given in units
c  of km.
c
c  The exact boundaries of the rectangle are adjusted so that it will
c  contain an integral number of cells all of the same size measured in
c  arc seconds, with east-west distance between adjacent cells
c  determined by cell size in km converted to arc seconds at the center
c  of the rectangle.
c
c  An offset from grid center will be stored in arc seconds for each
c  cell.  This offset will be determined later based on the centroid
c  of the census blocks falling within the cell.
c
c  The array of offsets will be passed to other routines by the common
c  block problem_grid which holds a pointer to the array.  This allows
c  the array to be sized and memory allocated at run time.  Its
c  dimensions are 2 x n_cells where the first dimension is the latitude
c  and the second is the longitude offset for the cell.  The first two
c  elements in the array will be the offsets of the SE corner.
c
c  The present routine puts the following information in common block
c  problem_grid: the coordinates of the center of the grid rectangle in
c  seconds (lat_cg, lon_cg), the cell height (N/S) and width (E/W) in
c  seconds (icell_h, icell_w), the number of cells from the center to
c  the adjusted boundaries (num_cell_h, num_cell_w), the precise cell
c  area (cell_area), and the boundaries of the grid area in seconds
c  (lat_lim, lon_lim).
c
c***********************************************************************
c
      subroutine init_grid(d_lat_min, d_lon_min, d_lat_max,
     &   d_lon_max)
      include 'global.inc'
c
      character saved_routine_name*32
      character pointer_name*32 /'pcell_offsets'/
c
c  Equate routine name for reporting errors
c
      saved_routine_name = routine_name
      routine_name = 'init_grid'
c
c  Check size of problem area to make sure it is not too large.  it must
c  be possible to store the maximum offset values in an integer*2
c  variable.  In addition offset values greater than 32000 are reserved
c  for use as flags to indicate cells, such as those with 0 population,
c  that may be skipped when computing coverage and IX.
c
      y_dif = d_lat_max - d_lat_min
      x_dif = d_lon_max - d_lon_min
      x = max (x_dif, y_dif)
      if (x * 0.5 * 3600.0 .gt. 32000) then
         write (0, 900)
         stop
      end if
c
c  Find cell height and width in seconds rounded to the nearest second.
c  Degkm_lon is the number of km/degree at the center of the area.
c
c  Variables icell_h and icell_w are integer*2
c
      icell_h = int(((cell_size / DEGKM) * 3600.0) + 0.5)
      degkm_lon = DEGKM *
     &   (cos((d_lat_min + (y_dif * 0.5)) * DEG_TO_RAD))
      icell_w = int (((cell_size / degkm_lon) * 3600.0) + 0.5)
      cell_area = (icell_h*DEGKM)*(icell_w*degkm_lon)/(3600.0**2)
c
c  Find the center coordinates in seconds
c
      lat_cg = int(3600.0 * (d_lat_min + (y_dif * 0.5)))
      lon_cg = int(3600.0 * (d_lon_min + (x_dif * 0.5)))
c
c  Determine total number of cells in problem area grid.  First find
c  number of cells east/west and north/south of center point (1/2 x_dif
c  and y_dif).  Total cells will be two times the number of cells from
c  center point.
c
c  The centers of boundary cells will lie inside the latitude and
c  longitude limits given as input to this routine.  If boundary cells
c  fall short of the given limits, it will be by less than half the
c  cell width or height.
c
      n_x = int(((x_dif * 1800.0) / float(icell_w)) + 0.5)
      n_y = int(((y_dif * 1800.0) / float(icell_h)) + 0.5)
      n_cells = n_x * n_y * 4
      num_cell_h = n_y
      num_cell_w = n_x
c
c  Determine the limits of the grid
c
      lat_lim(1) = (lat_cg - (icell_h * num_cell_h))
      lat_lim(2) = (lat_cg + (icell_h * num_cell_h))
      lon_lim(1) = (lon_cg - (icell_w * num_cell_w))
      lon_lim(2) = (lon_cg + (icell_w * num_cell_w))
c
c  Allocate memory for cell_offsets array
c
      call allocate_offsets_mem
      routine_name = saved_routine_name
      return
c
  900 format('Problem area is too large'/
     &   'Maximum height or width must be less than',
     &   ' approximately 17.5 degrees')
      end
c  %W% %G%
c
c***********************************************************************
c
c                             init_lr
c
c  This routine initializes parameters needed to perform a Longley-Rice
c  point to point computation.
c
c  The arguments of this routine are percentages of time, location and
c  confidence; frequency in MHz; transmitter power in kW; transmitting
c  antenna height in m above mean sea level; and the distance in km
c  between successive path profile points.  In addition this routine
c  refers to global common where it must find the receiver antenna height
c  above ground and the elevation of points along the path (rec_ant_hgt
c  and array path_elv_pt).
c
c  The Longley-Rice inputs prepared by this routine are the mode of
c  variability, standard normal deviates corresponding to the given
c  percentages, antenna heights above ground, antenna polarization
c  (both transmit and receive must be the same), relative permittivity
c  of the ground, ground conductivity in Siemens/meter, surface
c  refractivity and radio climate, and the average elevation of the
c  system.
c
c***********************************************************************
c
      subroutine init_lr(pct_time, pct_location, pct_confidence,
     &   fmhz, hamsl, d_inc, pfl)
      include 'global.inc'
      real pfl(5000)
c
c  Broadcast mode of variability.  See page 71 in NTIA Report 82-100.
c
      mdvar = 3
c
c  Horizontal polarization for both xmit and receive.  They must be
c  the same.  If vertical is desired, set to 1.
c
      ipol = 0
c
c  Relative permittivity for average ground.  See page 9 of NTIA Report
c  82-100.
c
      eps = 15.0
c
c  Conductivity in Siemens/meter for average ground.  See page 9 of NTIA
c  Report 82-100.
c
      sgm = 0.005
c
c  Surface refractivity N-sub-s for continental temperate climate,
c  page 9 of NTIA report.
c
      en0 = 301.0
c
c  Average elevation of system, page 72 of NTIA report.
c
      zsys = 0.0
c
c  Climate code, page 71 of NTIA report.
c
      klim = 5
c
c  Longley-Rice error switch kwx is set to a non-zero value if
c  parameters are found to be out of range.  See page 70 of NTIA
c  Report 82-100, April 1982.
c
      kwx = 0
c
c  Standard normal deviates of time, location and situation variability
c
      zt = qerfi(pct_time / 100.0)
      zl = qerfi(pct_location / 100.0)
      zc = qerfi(pct_confidence / 100.0)
c
c  Transmit and receive antenna height above ground.  Set the height
c  above ground of the transmit antenna at the nearest limiting value
c  if it is out of range.
c
c  NTIA Report 82-100 says the valid range is 0.5 and 3000 meters, and
c  subroutine lrprop will return error flag kwx=4 if this condition is
c  not met.  Here, we force all transmitter heights to be at least 10 m.

      hg(1) = hamsl - path_elev_pt(1)
      if (hg(1) .lt. 10.0) hg(1) = 10.0
      if (hg(1) .gt. 3000.0) hg(1) = 3000.0
c
      hg(2) = rec_ant_hgt
c
c  Load terrain profile into array pfl.  n_ter_pt is the number of points
c  along path as determined by d_inc (delta distance)
c
c     pfl(1) = # of points along path not including xmit site
c     pfl(2) = distance increment between points in meters
c     pfl(3) = site ground elevation amsl
c     pfl(3+1...3+n) = ground elevations AMSL of path points, including
c        the receive point which is the last point
c
c  All elevations are in meters
c
      pfl(1) = n_ter_pt - 1
      pfl(2) = d_inc * 1000.0
      do k = 1, n_ter_pt
         pfl(k+2) = path_elev_pt(k)
      end do
c
c  Set other system parameters
c
      call qlrps(fmhz, zsys, en0, ipol, eps, sgm)
c
c  Check for errors
c
      if (kwx .ne. 0) then
         write(0, 900) kwx
      end if
      return
c
  900 format('qlrps: kwx = ', i2)
      end
c  %W% %G%
c
c***********************************************************************
c
c                           init_svc_flag
c
c  This routine sets up four arrays, one indicating the presence of
c  service from particular stations at various cells, two more containing
c  the field strength and 90% fade of particular stations at the cells,
c  and a fourth indicating absence of noise-limited coverage by any
c  desired station.
c
c  Array icell_service tracks whether service from a desired station
c  is present at various cells of the problem area grid.  The array has
c  one bit for each station, and the bit will be set to 1 to indicate
c  that service is present or 0 to indicate domination by noise or
c  interference.  Initially all cells are set to service.
c
c  There will be 1 icell_service array element for each cell for every 8
c  desired stations, and the elements are stored sequentially.  Example:
c  If there are 10 desired stations, array element icell_service(1) will
c  contain the service flags for stations 1 to 8 for problem area cell 1
c  and icell_service(2) will contain the service flags for stations 9
c  and 10 for problem area cell 1.  The flags are stored in bits from 0
c  to 7.  In the above example bit 0 in array element 1 is for desired
c  station 1, and bit 0 of array element 2 is for desired station 9.
c
c  Memory is allocated for the second array, icell_service_fields, and
c  the service fields themselves will be computed later.  There will be
c  one array element in icell_service_fields for each station at each
c  cell.  For the example above, array elements 1 to 10 will be used to
c  store the fields from station 1 for grid cells 1 to 10; station 2,
c  cell 1 will be stored at n_cells + 1; and station 3, cell 1 will be
c  stored at 2*n_cells + 1.
c
c  The organization of the second and third arrays is the same.  The
c  third array holds the 90% fade magnitude corresponding to the service
c  field stored in icell_service_fields.
c
c  The fourth array, outside_cells, contains one bit for every cell or
c  1 + (n_cells-1)/8 bytes.  It is initialized to indicate all cells are
c  outside, i.e. none are within the NLC of any station.
c
c***********************************************************************
c
      subroutine init_svc_flag
      include 'global.inc'
c
      character saved_routine_name*32, pointer_name*32
c
c  Equate routine name for reporting errors
c
      saved_routine_name = routine_name
      routine_name = 'init_svc_flag'
c
c  Pointers should have been initialized to 0 before first use of this
c  subprogram and released before subsequent calls
c
      if (pcell_service .ne. 0) then
         pointer_name = 'pcell_service'
         call report_mem_error_1(routine_name, pointer_name)
      end if
c
      if (pcell_service_fields .ne. 0) then
         pointer_name = 'pcell_service_fields'
         call report_mem_error_1(routine_name, pointer_name)
      end if
c
      if (pcell_service_fades .ne. 0) then
         pointer_name = 'pcell_service_fades'
         call report_mem_error_1(routine_name, pointer_name)
      end if
c
      if (ptr_outside_cells .ne. 0) then
         pointer_name = 'ptr_outside_cells'
         call report_mem_error_1(routine_name, pointer_name)
      end if
c
c  Allocate memory for service flag array
c
      n = (n_des_sta / 8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n = n - 1
      n_s = n
      n = n * n_cells
      pcell_service = malloc(n)
      if (pcell_service .eq. 0) then
         pointer_name = 'pcell_service'
         call report_mem_error(routine_name, pointer_name, n)
      end if
c
c  Allocate memory for service field and fading arrays
c
      n = n_des_sta * n_cells * 2
      pcell_service_fields = malloc(n)
      if (pcell_service_fields .eq. 0) then
         pointer_name = 'pcell_service_fields'
         call report_mem_error(routine_name, pointer_name, n)
      end if
      pcell_service_fades = malloc(n)
      if (pcell_service_fades .eq. 0) then
         pointer_name = 'pcell_service_fades'
         call report_mem_error(routine_name, pointer_name, n)
      end if
c
c  Allocate memory for array to identify cells outside the NLC of all
c  desired stations.  1 bit is needed for each cell.  After allocation
c  set all bits to 1 (true, i.e. no service yet found in any cell).
c
      n = (n_cells / 8)
      if (mod(n_cells, 8) .ne. 0) n = n + 1
      ptr_outside_cells = malloc(n)
      if (ptr_outside_cells .eq. 0) then
         pointer_name = 'ptr_outside_cells'
         call report_mem_error(routine_name, pointer_name, n)
      end if
c
      i_set = -1
      do i=1, n
         outside_cells(i) = i_set
      end do
c
c  Loop thru cells initializing service conditions.  Cells with no
c  population may have a latitude offset flag of 32100, and all service
c  bits of flagged cells are now set to zero.  They will be ignored in
c  subsequent cell counts for determination of area.
c
c  Note the counter k below is needed because there are two
c  icell_offsets for each cell, one for latitude and another for
c  longitude.
c
c  The number of service flag array elements for each cell is n_s, one
c  for every 8 desired stations within the problem area.
c
      n_c = 0
      k = 0
      do i=1, n_cells
         k = k + 1
         i_set = -1
         if (icell_offsets(k) .eq. IGNORED_CELL .and.
     &      .not. calculate_area) i_set = 0
         do j=1, n_s
            n_c = n_c + 1
            icell_service(n_c) = i_set
         end do
         k = k + 1
      end do
c
      routine_name = saved_routine_name
      return
c
c  Entry point to release memory allocated above
c
      entry release_svc_mem
      if (pcell_service .eq. 0) then
         pointer_name = 'pcell_service'
         call report_mem_error_2(routine_name, pointer_name)
      end if
      call free(pcell_service)
      pcell_service = 0
c
      if (pcell_service_fields .eq. 0) then
         pointer_name = 'pcell_service_fields'
         call report_mem_error_2(routine_name, pointer_name)
      end if
      call free(pcell_service_fields)
      pcell_service_fields = 0
c
      if (pcell_service_fades .eq. 0) then
         pointer_name = 'pcell_service_fades'
         call report_mem_error_2(routine_name, pointer_name)
      end if
      call free(pcell_service_fades)
      pcell_service_fades = 0
c
      if (ptr_outside_cells .eq. 0) then
         pointer_name = 'ptr_outside_cells'
         call report_mem_error_2(routine_name, pointer_name)
      end if
      call free(ptr_outside_cells)
      ptr_outside_cells = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                            init_ter_mem
c
c  This routine initializes the array terrain_mem and its inverse,
c  terrain_index, to indicate that no terrain data is yet in memory.  The
c  terrain_mem array contains an index to the array i_elev where the
c  terrain elevations for a specific 7.5' block of terrain data are
c  stored.  Terrain_mem(idx) = 0 means that the 7.5' block with index idx
c  is not in memory.  Terrain block indices run from 0 to LAST_IDX+1, and
c  there is an index for blocks all the way up to the north pole at all
c  longitudes except possibly the last and most western.  The terrain
c  block with index LAST_IDX+1 is a dummy with parameters set to indicate
c  no data.
c
c***********************************************************************
c
      subroutine init_ter_mem
c
      include 'global.inc'
c
c  Initialize arrays to 0.  LAST_IDX is last possible index for the 7.5'
c  terrain blocks found in the terrain data directories.  The latter are
c  identified by subroutine get_ter_dirs.
c
      do i=1, LAST_IDX+1
         terrain_mem(i) = 0
      end do
c
      do i=1, MAX_TER_BLK
         terrain_index(i) = 0
      end do
c
c  Set initial time stamp to 0
c
      next_time = 0
c
c  Set time stamps to 0
c
      do i=1, MAX_TER_BLK
         i_time(i) = 0
      end do
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        interp_lin_drad
c
c  This routine performs a linear interpolation of the distance to a
c  contour at a given azimuth (az) given the evenly spaced radials on
c  either side of the desired azimuth (irad_1, irad_2) and the
c  incremental azimuth spacing (az_inc)
c
c***********************************************************************
c
      subroutine interp_lin_drad(irad_1, irad_2, az_inc, az, d_z)
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Statement function for extracting radial distance from the array
c  of CONTOUR structures
c
      c_cntr_d(i) = current_contour(i).distance
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'interp_lin_drad'
c
c  No interpolation is necessary if irad_1 equals irad_2
c
      if (irad_1 .eq. irad_2) then
         d_z = c_cntr_d(irad_1)
         goto 500
      end if
c
c  Determine the difference in degrees from irad_1 to az
c
      az_dif = az - ((irad_1 - 1) * az_inc)
c
c  Determine the ratio az_dif/az_inc
c
      az_ratio = az_dif/az_inc
c
c  Determine the difference in distance to the contour between irad_1
c  and irad_2
c
      d_dif = c_cntr_d(irad_2) - c_cntr_d(irad_1)
c
c  Interpolate
c
      d_z = c_cntr_d(irad_1) + (az_ratio * d_dif)
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                           interpolat_4_elevations
c
c  This routine finds the elevation at point pt_lat, pt_lon by
c  interpolation of the elevations at four corners of a box surrounding
c  the point.
c
c***********************************************************************
c
      subroutine interpolate_4_elevations(pt_lat, pt_lon,
     &   pt_corner_lat, pt_corner_lon, corner_el, elev )
      double precision pt_lat, pt_lon
      double precision pt_corner_lat(4), pt_corner_lon(4)
c
      double precision pts_per_deg_lat, pts_per_deg_lon
      real corner_el(4)
c
c  Check for bad elevations
c
      elev = -10000.0
      do i=1, 4
         if (corner_el(i) .lt. -999.0) goto 500
      end do
c
c  Set points per degree
c
      pts_per_deg_lat = 1200.0
      pts_per_deg_lon = 1200.0
c
c  Find difference in "distance" from corners
c
      y1 = (pt_lat - pt_corner_lat(1)) * pts_per_deg_lat
      x1 = (pt_lon - pt_corner_lon(1)) * pts_per_deg_lon
      y2 = 1.0 - y1
      x2 = 1.0 - x1
c
c  Interpolate
c
      elev = corner_el(1) * x2 * y2
      elev = elev + corner_el(4) * x1 * y2
      elev = elev + corner_el(3) * x1 * y1
      elev = elev + corner_el(2) * x2 * y1
c
  500 return
      end
c %W% %G%
c
c***********************************************************************
c
c                        interpolate_atv_power
c
c  This routine interpolates to find the ATV power needed to match the
c  grade B service of the NTSC station paired with the station given by
c  the agrument to this routine.  Values found in equiv_pat_pwr should
c  have been computed previously every 1 degree by routine
c  compute_equiv_pattern.
c
c***********************************************************************
c
      subroutine interpolate_atv_pwr(az, equiv_pat_pwr, rep_erp)
      real equiv_pat_pwr(*)
c
c  Interpolate power
c
      i_az_1 = int(az) + 1
      if (i_az_1 .eq. 361) i_az_1 = 1
c
      i_az_2 = i_az_1 + 1
      if (i_az_2 .eq. 361) i_az_2 = 1
c
      az_dif = az - int(az)
      pwr_dif = equiv_pat_pwr(i_az_1) - equiv_pat_pwr(i_az_2)
      rep_erp = equiv_pat_pwr(i_az_1) - (pwr_dif * az_dif)
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                               itp_fs
c
c  This routine is used to find the distance to a field strength when
c  the free space equation must be used and a transmit antenna factor is
c  being employed.  It is assumed that the distance is less than 1.5 km
c  (If greater than 1.5 it should have been found without calling this
c  routine)
c
c***********************************************************************
c
      subroutine itp_fs(field, erp, haat, rec_hgt, vpat_bias, iband,
     &   n_a, distance)
c
      character routine_name*32
      common /current_routine/ routine_name
c
      character saved_routine_name*32
      integer FCC_CURVES
      parameter (FCC_CURVES = 0)
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'itp_fs'
c
c  Initialization
c
      distance = 1.5
      dinc = 0.05
      diff_b = 1.5
      d = 1.5
      instance = FCC_CURVES
c
   50 continue
      d = d - dinc
      call gt_vert_rad_fac(haat, rec_hgt, d, vpat_bias, instance,
     &   iband, n_a, factor)
      f = field / factor
c
c  Deal with values that are too large for computation
c
      if (f .ge. 500.0) then
         d = dinc
         goto 500
      end if
      e_volts_meter = 1.0E-6 * 10.0**(f / 20.0)
      dx = 7.014271E-3 * sqrt(erp * 1000.0) / e_volts_meter
      diff = abs(d - dx)
      if (diff .lt. diff_b) then
         diff_b = diff
         distance = d
      end if
      if (d .gt. dinc + 1.0E-6) goto 50
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                           load_75_terrain
c
c  This routine reads a 7.5 minute block of terrain data into the array
c  i_elev.  The inputs to the routine are the coordinates of the
c  southeast corner of the desired block.  The first element of the
c  output array, i_elev(m, 1), is set to -10000 if no terrain data are
c  available for the m-th block.
c 
c  MAX_TER_BLK is the maximum number of 7.5' terrain data blocks that
c  can be in memory at one time.
c
c***********************************************************************
c
      subroutine load_75_terrain(se_lat, se_lon)
      include 'global.inc'
c
c  Local variables
c
      character saved_routine_name*32
      integer*2 ns_bias, ew_bias, num_bits, elevation_bias
      integer*2 c(22500), d(16875), e(11250)
      integer*2 i_t
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'load_75_terrain'
c
c  Update the number of blocks in memory.  If no memory block is
c  available, replace the oldest.
c
      if (i_time(MAX_TER_BLK) .eq. 0) then
         i_next = 1
         do while (i_time(i_next) .ne. 0)
            i_next = i_next + 1
         end do
      else
         i_old = next_time	!Start with largest possible value
         do i=1, MAX_TER_BLK
            if (i_time(i) .lt. i_old) then
               i_old = i_time(i)
               i_next = i
            end if
         end do
      end if
c
c  Save memory location of the terrain block currently of interest
c
      next_pt = i_next
      m = next_pt
c
c  Initialize elevations to -10000
c
      i_elev(m, 1) = -10000.0
c
c  Get index and file offset
c
      call get_terrain_index(se_lat, se_lon, num_f, idx)
c
c  Update the list of 7.5' blocks retained in memory.
c
      if (terrain_index(m) .ne. 0) terrain_mem(terrain_index(m)) = 0
      terrain_index(m) = idx
      terrain_mem(idx) = m
c
c  Get parameters for file
c
      call get_75_parms(idx, ns_bias, ew_bias, elevation_bias,
     &   num_bits)
c
c  If num_bits = 0 then no terrain data is available for this 7.5'
c  block.  However the value of -10000 has been stored in i_elev(m, 1)
c  to flag this, so do not adjust the counter.
c
      if (num_bits .eq. 0) goto 500
c
c  Numbits = 1 is a flag indicating that all points in the 7.5 minute
c  block are 0 and therefore a data file was not created.  The elevations
c  for this block must be filled in.
c
      if (num_bits .eq. 1) then
         do i=1, 22500
            i_elev(m, i) = 0.0
         end do
         goto 500
      end if
c
c  Set record length for retrieval of data not presently in memory
c
      n_recl = (float(num_bits / 8.0)) * 22500
c
c  Open file with data for this 7.5 minute data block
c
      call open_ter_file(se_lat, se_lon, num_f, n_recl, i_ter_fl)
      if (i_ter_fl .eq. -1) then
         i_elev(m, 1) = -10000.0
         goto 500
      end if
c
c  Deal with whether elevations were stored in 8, 12, or 16 bit words.
c
      if (num_bits .eq. 16) read(i_ter_fl, rec=1) c
      if (num_bits .eq. 12) read (i_ter_fl, rec=1) d
      if (num_bits .eq. 8) read (i_ter_fl, rec=1) e
      close(i_ter_fl)
      n_points = 22500
c
c  if 16 bits
c
      if (num_bits .eq. 16) then
         do i=1, n_points
            i_elev(m, i) = c(i) + elevation_bias
         end do
         goto 500
      end if
c
c  if 12 bits
c
      if (num_bits .eq. 12) then
         n_points = n_points * 0.75
         n = 0
         do i=1, n_points, 3
            n = n + 1
            i_elev(m, n) = ishft(d(i), 4)
            i_elev(m, n) = ishft(i_elev(m, n), -4) + elevation_bias
            n = n + 1
            i_elev(m, n) = ishft(d(i), -12)
            i_t = ishft( d(i+1), 8)
            i_t = ishft ( i_t, -4)
            i_elev(m, n) = ior(i_t, i_elev(m, n)) + elevation_bias
            n = n + 1
            i_elev(m, n) = ishft(d(i+1), -8)
            i_t = ishft(d(i+2), +12)
            i_t = ishft(i_t, -4)
            i_elev(m, n) = ior(i_t, i_elev(m, n)) + elevation_bias
            n = n + 1
            if (n*m .gt. MAX_TER_BLK*22500) then
               write (0, '(a,i4,a,i6,a)') 'Cannot store', m,
     &            ' blocks of', n, ' points in array i_elev'
               stop
            end if
            i_elev(m, n) = ishft(d(i+2), -4) + elevation_bias
         end do
         goto 500
      end if
c
c  if 8 bits
c
      if (num_bits .eq. 8) then
         n_points = n_points * 0.5
         n = 0
         do i=1, n_points
            n = n + 1
            i_elev(m, n) = ishft(e(i), 8)
            i_elev(m, n) = ishft(i_elev(m, n), -8) + elevation_bias
            n = n + 1
            i_elev(m, n) = ishft(e(i), -8) + elevation_bias
         end do
         goto 500
      end if
c
  500 continue
      routine_name = saved_routine_name
      return
      end

      subroutine open_ter_file(se_lat, se_lon, num_f, n_recl, i_ter_fl)
      include 'global.inc'
      character file_name*52
c
      istat = -1
      i_dir = 1
      do while (i_dir .le. MAX_DIR .and. istat .ne. 0)
c
c  Get file name for this 7.5 minute data block
c
         if (int(se_lon) .ge. lon_dir(i_dir)) then
            l = lnblnk(dir_list(i_dir))
            write(file_name, 900) dir_list(i_dir)(1:l), int(se_lat),
     &         int(se_lon), num_f
c
c  Open next file in target range of longitudes
c
            call getlun(i_ter_fl)
            open(unit=i_ter_fl, file=file_name, status='old',
     &         iostat=istat, access='direct', form='unformatted',
     &         recl=n_recl)
         end if
         i_dir = i_dir + 1
      end do
c
c  Set logical unit number to -1 in case file has not been found
c
      if (istat .ne. 0) then
         write(0, '(a,1x,a,1x,a)') file_name(1:lnblnk(file_name)),
     &      'not found by', routine_name(1:lnblnk(routine_name))
         i_ter_fl = -1
      end if
      return
c
  900 format(a, 'ter_', i2.2, 'x', i3.3, '_', i2.2, '.dbs')
      end
c  %W% %G%
c
c***********************************************************************
c
c                      load_atv_sta_proto
c
c  Load prototype power and HAAT for every ATV station for which an ERP
c  of zero is found in the engineering data base.  In these cases, also
c  set the antenna model to "none" so that a relative power factor of
c  unity will be used in every direction.
c
c***********************************************************************
c
      subroutine load_atv_sta_proto
      include 'global.inc'
c
      do i=n_first_atv, n_eng_rec
         if (ichan_r(i) .ne. 0 .and. erp_r(i) .eq. 0) then
            call get_band(ichan_r(i), iband)
            erp_r(i) = erp_atv_prot(iband)
            haat_r(i) = haat_atv_prot(iband)
            antmod_r(i) = 'none'
         end if
      end do
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                            load_da_pats
c
c  Load the horizontal antenna patterns of the TV antenna data base into
c  the common block antenna_h_pat.
c
c     num_ant_pat = number of different antenna patterns
c     da_string = 3 character manuf // 14 character model number
c     n_ant_az = number of azimuths for which radiation values provided
c     ih_rad = 1000 times radiation factor
c     ispecial_az  = representation of special azimuths
c
c  Normal azimuths are every 10 degrees.  Special azimuths are stored
c  as integers equal to 10*(special azimuth in degrees)-180.
c
c***********************************************************************
c
      subroutine load_da_pats
      include 'global.inc'
c
c  Local variables
c
      character*3 make
      character*14 model
      character*76 dbcomm	!comment
      character*6 update
      real dbaz(46), dbrad(46)
c
c  Open data base
c
      call getlun(iant_db)
      open(unit=iant_db, file=tv_da_db_name, status='old',
     &   form='formatted', access='sequential')
c
c  Skip over header.  Setting num_ant_pat = -1 causes the file to be
c  rewound and routine rd_tv_ant_db will read the header and return
c  with num_ant_pat = 0.
c
      num_ant_pat = -1
      call rd_tv_ant_db(iant_db, num_ant_pat, make, model, dbrad,
     &   dbaz, dbcomm, update, naz)
c
c  Read data.  Subroutine rd_tv_ant_db automatically increments
c  num_ant_pat.
c
   50 continue
      call rd_tv_ant_db(iant_db, num_ant_pat, make, model, dbrad,
     &   dbaz, dbcomm, update, naz)
c
c  Check for end of file and subtract 1 from number of patterns
c  in file since last record is a trailer record.
c
      if (make .eq. '~~~') then
         num_ant_pat = num_ant_pat - 1
         goto 100
      end if
c
c  Fill in common block
c
      if (num_ant_pat .gt. MAX_ANT_REC) stop
     &   'Too many records in antenna data base'
      n_ant_az(num_ant_pat) = naz
      da_string(num_ant_pat) = make // model
c
      do i=1, naz
         ih_rad(num_ant_pat, i) = int((dbrad(i)*1000.0) + 0.5)
      end do
c
      k = 0
      do i=37, naz
         k = k + 1
         ispecial_az(num_ant_pat, k) = int((dbaz(i)*10) + 0.5) - 1800
      end do
      goto 50
c
  100 continue
      close(iant_db)
      call make_da_index
      return
      end
c
c***********************************************************************
c
c                              rd_tv_ant_db
c
c  Read one record from the TV directional antenna data base and increment
c  the record number.  The file should have already been opened for
c  sequential formatted IO.
c
c***********************************************************************
c
      subroutine rd_tv_ant_db(iant_db, irec, make, model, dbrad,
     &   dbaz, dbcomm, update, naz)
      character*(*) make, model, dbcomm, update
      character a*400
      real dbaz(46), dbrad(46)
c
c  Local variables
c
      character*5 spaz(10), sprad(10)
c
c  Rewind file if irec = -1
c
      if (irec .eq. -1) rewind(iant_db)
c
c  Read 400 character record
c
      read(iant_db, 800, end=450) a
      irec = irec + 1	!number of records read, excluding header
      if (irec .eq. 0) goto 500
c
c  Unpack record
c
      naz = 0
      read(a, 805) make, model, (dbrad(loop), loop=1, 36, 1), 
     &   (spaz(loop), sprad(loop), loop=1, 10, 1), dbcomm, update
      do 10 loop=1, 10, 1
         if (spaz(loop).eq.'     ') goto 20
         read(spaz(loop), 810) dbaz(36+loop)
         read(sprad(loop), 815) dbrad(36+loop)
10    continue
      loop = 11         !set loop to normal do loop exit value
20    continue
      naz = 35 + loop
      goto 500
c
c  The calling program is expected to intercept the trailer record and
c  make no further calls to this subroutine.
c
  450 continue
      write(0, '(2a)') 'rd_tv_ant_db: ',
     &   'Attempt to read beyond end of file, or no trailer found'
      call exit(257)
c
  500 return
c
  800 format(a400)
  805 format(a3, a14, t24, 36f5.3, 20a5, a76, a6)
  810 format(f5.0)
  815 format(f5.3)
      end

      subroutine make_da_index
      include 'global.inc'
      character*17 pattern
      logical missing
c
      call sort_da
      do i=1, n_eng_rec
         pattern = antmnf_r(i) // antmod_r(i)
         missing = pattern(4:7) .eq. 'none'
     &      .or. index(pattern, '*') .gt. 0
         if (missing) then
            index_da(i) = 0
         else
            j = item_number(pattern, da_string, num_ant_pat)
            if (j .gt. 0) then
               index_da(i) = locator(j)
            else
               index_da(i) = 0
               if (warn) write(0, 1000) pattern(1:lnblnk(pattern)),
     &            state_r(i), city_r(i)(1:lnblnk(city_r(i))),
     &            ichan_r(i), sta_type(i)
            end if
         end if
      end do
      return
 1000 format('DA pattern ', a, ' for ', a2, 1x, a, ' channel', i3, a,
     &   ' not found.')
      end

      subroutine sort_da
c
c  Sort the given list of 17-character strings, saving the original
c  order in the locator array.
c
      include 'global.inc'
      structure /SORTED_ANTENNAS/
         integer id
         character da_string*17
         character fill*3
      end structure
      record /SORTED_ANTENNAS/ da_struct(MAX_ANT_REC)
      do i=1, num_ant_pat
         da_struct(i).id = i
         da_struct(i).da_string = da_string(i)
      end do
      call sort_da_struct(da_struct, num_ant_pat)
      do i=1, num_ant_pat
         locator(i) = da_struct(i).id
         da_string(i) = da_struct(i).da_string
      end do
      return
      end

      subroutine sort_da_struct(da_struct, num_ant_pat)
      structure /SORTED_ANTENNAS/
         integer id
         character da_string*17
         character fill*3
      end structure
      record /SORTED_ANTENNAS/ da_struct(num_ant_pat)
      external compare_da_struct
      integer*2 compare_da_struct
      integer struct_size /24/
      call qsort(da_struct, num_ant_pat, struct_size,
     &   compare_da_struct)
      return
      end

      integer*2 function compare_da_struct(x, y)
c
c  Specify order for qsort of directional antenna records
c
      structure /SORTED_ANTENNAS/
         integer id
         character da_string*17
         character fill*3
      end structure
      record /SORTED_ANTENNAS/ x, y
      character*17 xstring, ystring
c
      xstring = x.da_string
      ystring = y.da_string
      if (ystring .eq. xstring) then
         compare_da_struct =  0
      else if (ystring .gt. xstring) then
         compare_da_struct = -1
      else
         compare_da_struct =  1
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         load_desired_list
c  
c  This routine loads the city, state, channel and station type (NTSC
c  or ATV) of the desired stations specified in a formatted input file.
c  The name of the file is in common block desired_list_file, and the
c  first record of the file is the number of stations in the list.
c
c***********************************************************************
c
      subroutine load_desired_list
      include 'global.inc'
c
c  Open file
c
      call getlun(ides_f)
      open(unit=ides_f, file=desired_file, status='old',
     &   form='formatted', access='sequential', iostat=ierr)
c
c  Check for error opening file
c
      if (ierr .ne. 0) then
         write(0, 900) desired_file(1:lnblnk(desired_file))
         stop
      end if
c
c  Initialize counter
c
      n_ntsc = 0
c
c  Read number of stations in list from first line of file
c
      line = 1
      read(ides_f, 800, err=400, end=400) n_des_sta
      last_line = n_des_sta + 1
c
c  Allocate memory to store list of this many desired stations, or
c  this many plus 2 if the evaluation is to be pairwise.
c
      if (pairwise) n_des_sta = n_des_sta + 2
      call allocate_ds_mem
      n = 1
c
c  Read station list, and sort by NTSC and ATV with NTSC first
c
      if (pairwise) n = n + 2
      line = 2
      do while (line .le. last_line .and. n .le. n_des_sta)
         read(ides_f, 810, err=400, end=400)
     &      desired_stations(n).city, desired_stations(n).state,
     &      desired_stations(n).ichan, desired_stations(n).sta_type
         if (desired_stations(n).sta_type .eq. 'n') then
            n_ntsc = n_ntsc + 1
c
c  Find location of desired NTSC station in data base array
c
            call find_record(desired_stations(n).city,
     &         desired_stations(n).state, desired_stations(n).sta_type,
     &         desired_stations(n).ichan, id)
            if (id .ne. 0) then
               desired_stations(n).id = id
            else
               l = lnblnk(desired_stations(n).city)
               write(0, 920) desired_stations(n).state,
     &            desired_stations(n).city(1:l),
     &            desired_stations(n).ichan,
     &            desired_stations(n).sta_type
               stop
            end if
            n = n + 1
         end if
         line = line + 1
      end do
c
c  If the evaluation is to be pairwise, ignore any ATV stations which
c  may be in the input list.  The actual number of desired stations of
c  interest is twice n_ntsc, but n_des_sta is set equal to 2 so that
c  NTSC-ATV pairs can be examined one at a time.
c
      if (pairwise) then
         n_des_sta = 2
         next_ntsc = 3
         call load_next_pair
         goto 500
      else
         next_ntsc = 1
      end if
c
c  Rewind file and load ATV stations
c
      rewind(ides_f)
      line = 1
      read(ides_f, 800, err=400, end=400) n_des_sta
c
c  Sanity check
c
      if (n .ne. n_ntsc + 1) stop
     &   'Unexpected error loading list of desired stations'
      line = 2
      do while (line .le. last_line .and. n .le. n_des_sta)
         read(ides_f, 810, err=400, end=400)
     &      desired_stations(n).city, desired_stations(n).state,
     &      desired_stations(n).ichan, desired_stations(n).sta_type
c
c  Skip NTSC stations
c
         if (desired_stations(n).sta_type .eq. 'a') then
c
c  Find location of desired ATV station in data base array
c
            call find_record(desired_stations(n).city,
     &         desired_stations(n).state, desired_stations(n).sta_type,
     &         desired_stations(n).ichan, id)
            if (id .ne. 0) then
               desired_stations(n).id = id
            else
               l = lnblnk(desired_stations(n).city)
               write(0, 920) desired_stations(n).state,
     &            desired_stations(n).city(1:l),
     &            desired_stations(n).ichan,
     &            desired_stations(n).sta_type
               stop
            end if
            n = n + 1
         end if
         line = line + 1
      end do
      goto 500
c
c  Report error reading file
c
  400 continue
      write(0, 910) desired_file(1:lnblnk(desired_file)), line
      stop
c
  500 return
c
  800 format(i)
  810 format(a20, 1x, a2, 1x, i2, 1x, a1)
  900 format('Error opening input file of desired stations' /
     &   'File name: ', a)
  910 format('Error reading input file of desired stations' /
     &   'File name: ', a, ' at line number: ' i)
  920 format('Station not in data base'/ a2, 1x, a, i3, a1)
      end
c  %W% %G%
c
c***********************************************************************
c
c                         load_eng_db
c
c  Load TV Engineering data base elements contained in the file
c  tv_db_name.  This name is a program input read from stdin.
c
c***********************************************************************
c
      subroutine load_eng_db
      include 'global.inc'
c
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn_prefix*6, arn*8
      common /mainrec/
     &   latsec, lonsec, erp, haat, rcamsl, ref_az, itype, istat,
     &   svc_cls, off, pol, da, beamt, zone, antmnf, antmod,
     &   ichan, cty, city, state, call, arn
c
      logical atv_rec, mismatch, vacant_allotment, replication_instance
c
c  Open main engineering data base file
c
      call getlun(imain)
      open(unit=imain, file=tv_db_name, status='old', access='direct',
     &   form='unformatted', recl=99)
c
c  Initialize counters
c
      n = 0
      n_first_atv = 0
c
c  Read records of TV engineering data base from file
c
   10 continue
      n = n + 1
      call rd_main(imain, n)
      if (city(1:11) .eq. 'end of file') goto 100
c
c  Check for too many records
c
      if (n .gt. MAX_ENG_REC) goto 400
c
c  Store in engineering data base arrays
c
      ichan_r(n) = ichan
      erp_r(n) = erp
      haat_r(n) = haat
      rcamsl_r(n) = rcamsl
      lat_site(n) = latsec
      lon_site(n) = lonsec
      city_r(n) = city
      state_r(n) = state
      call_r(n) = call
      antmnf_r(n) = antmnf
      antmod_r(n) = antmod
      ref_az_r(n) = ref_az
      if (antmod_r(n)(1:4) .eq. '    ') antmod_r(n) = 'none'
      arn_with_prefix(n) = arn_prefix(itype) // arn
      if (n_first_atv .eq. 0 .and. ichan .eq. 0) then
         write(0, '(2a)')
     &      'Station with no channel precedes first ATV station in ',
     &      tv_db_name(1:lnblnk(tv_db_name))
         call exit(257)
      end if
c
c  Stations with no channel may be found among ATV records.  They
c  correspond to NTSC stations which were not eligible for a digital
c  allotment.
c
      atv_rec = ichan .eq. 0 .or.
     &   svc_cls .eq. 'DT' .or.
     &   call(1:1) .eq. 'D' .or. call(7:8) .eq. 'DT' .or.
     &   index(arn, 'atv_rec') .gt. 0 .or.
     &   index(arn, 'BPCDT') .gt. 0
      if (atv_rec) then
         sta_type(n) = 'a'
      else
         sta_type(n) = 'n'
      end if
      if (cty .eq. 'A') then
         country_r(n) = 'AMER'
      else if (cty .eq. 'C') then
         country_r(n) = 'CANA'
      else if (cty .eq. 'M') then
         country_r(n) = 'MEXI'
      else
         write(0, '(3a,i5)') 'Unrecognized country "', cty,
     &      '" in record number', n
         call exit(257)
      end if
c
c  Identify first ATV station
c
      if (n_first_atv .eq. 0 .and. sta_type(n)  .eq. 'a')
     &   n_first_atv = n
c
c  Force the ATV and DTV site coordinates to be the same when replicating
c
      if (n_first_atv .gt. 0 .and. replicate) then
         n_pair = n - (n_first_atv - 1)
         lat_site(n) = lat_site(n_pair)
         lon_site(n) = lon_site(n_pair)
      end if
c
c  If no channel, there is no need to check power and height.
c
      if (ichan .eq. 0) goto 10
c
c  Check power and heights.  Note the following:
c
c     The program input may have specified that heights above average
c     terrain along radials are not to be calculated (use_n_radials = F).
c     In this case, height limits insure that a reasonable value is used
c     for the tower height (for instance, intercepting negative values).
c
c     Except in that special case, minimum HAAT values play a role only
c     if a station's parameters make it appear that the transmitting
c     antenna is underground for Longley-Rice calculations.  If this
c     condition is found, the RCAMSL is set to the ground elevation at
c     the site given by the station's coordinates plus the HAAT value
c     stored for the station.  The latter is adjusted here if it is less
c     than specified minima.
c
c     If the ERP and HAAT of an NTSC station are both 0, the record is
c     assumed to represent a vacant allotment.  The minimum values
c     set for this case cause vacant allotments to be "turned on" so
c     that interference from these potential sources will be considered.
c
c     Under replication, the height limits for ATV stations are the same
c     as those for NTSC, and ERP limits for ATV stations are imposed
c     later, in the routines which determine equivalent power patterns.
c
c     If replication is off, the HAAT and ERP values given in the
c     engineering data base for ATV stations are not modified, except
c     that prototype values are used if the data base values are zero.
c     Station parameters are set to prototype values by subroutine
c     load_atv_sta_proto.
c
      if (sta_type(n) .eq. 'n') then
         call get_band(ichan, iband)
         vacant_allotment = erp_r(n) .eq. 0.0 .and. haat_r(n) .eq. 0.0
         if (vacant_allotment) then
            erp_r(n) = erp_va_limits(1, iband)
            haat_r(n) = haat_va_limits(1, iband)
         else
            erp_r(n) = max(erp_r(n), erp_ntsc_limits(1, iband))
            haat_r(n) = max(haat_r(n), haat_ntsc_limits(1, iband))
         end if
      else
         n_pair = n - (n_first_atv - 1)
         replication_instance = erp_r(n) .eq. -1.0 .and.
     &      lat_site(n) .eq. lat_site(n_pair) .and.
     &      lon_site(n) .eq. lon_site(n_pair)
         replication_instance = replication_instance .or. replicate
         if (replication_instance) then
            call get_band(ichan_r(n_pair), iband)
            if (haat_r(n) .lt. haat_ntsc_limits(1, iband)) then
               if (erp_r(n_pair) .eq. 0.0 .and. haat_r(n) .eq. 0.0) then
                  haat_r(n) = haat_va_limits(1, iband)
               else
                  haat_r(n) = haat_ntsc_limits(1, iband)
               end if
            end if
         end if
      end if
c
c  An ERP of -1.0 may be found in the engineering data base for ATV
c  stations as an indication that their parameters are to be determined
c  by replication (reserving an ERP of 0.0 for prototypes).  This is
c  an error if the ATV and NTSC station coordinates differ.
c
      if (erp_r(n) .lt. 0.0 .and. .not. replication_instance) then
         write(0, '(2a)') 'Negative ERP found in file ',
     &      tv_db_name(1:lnblnk(tv_db_name))
         write(0, '(3a)') 'Specify replication in runtime input file, ',
     &      'or revise ', tv_db_name(1:lnblnk(tv_db_name))
         call exit(257)
      end if
c
c  Get next record
c
      goto 10
c
c  End of data base file found. Set station counter, and make a few
c  final checks.
c
  100 continue
      n_eng_rec = n - 1
c
c  Check for too many records.  Other routines rely on there being
c  no more than 2**15 - 1 records
c
      if (n_eng_rec .gt. 32767) then
         write(0, 910)
         call exit(257)
      end if
c
c  If there are no ATV records, set n_first_atv to one more than the
c  number of NTSC records.
c
      if (n_first_atv .eq. 0) n_first_atv = n_eng_rec + 1
c
c  When replicating, the site coordinates of the n-th ATV record must
c  match those of the n-th NTSC record.
c
      if (replicate) then
         do n=n_first_atv, n_eng_rec
            call paired_station(n, ntsc_rec)
            mismatch = lat_site(n) .ne. lat_site(ntsc_rec) .or.
     &         lon_site(n) .ne. lon_site(ntsc_rec)
            if (mismatch) then
               write(0, '(2a)') 'Engineering data base ',
     &            'improperly organized for replication'
               call exit(257)
            end if
         end do
      end if
c
c  In pairwise mode the cities must match.
c
      if (pairwise) then
         do n=n_first_atv, n_eng_rec
            call paired_station(n, ntsc_rec)
            mismatch = state_r(n) .ne. state_r(ntsc_rec) .or.
     &         city_r(n) .ne. city_r(ntsc_rec)
            if (mismatch) then
               write(0, '(2a)') 'Engineering data base ',
     &            'improperly organized for pairwise evaluation'
               call exit(257)
            end if
         end do
      end if
c
c  All records loaded successfully.  Close data base and return.
c
      close(imain)
      goto 500
c
c  Too many data base records for array size.  Write message and
c  terminate program.
c
  400 continue
      write(0, '(a,1x,a,i3)') state, city(1:lnblnk(city)), ichan
      write(0, 900) n-1, MAX_ENG_REC
      close(imain)
      call exit(257)
c
  500 continue
      return
c
  900 format('The number of records read from the TV Engineering',
     &   ' data base is > ', i/ 'This exceeds the array size set',
     &   ' by MAX_ENG_REC of ', i/ 'Need to change the parameter',
     &   ' statement in global.inc and recompile')
  910 format('The number of records read from the TV Engineering',
     &   ' data base is > 32767'/ 'This exceeds the maximum allowed'
     &  ' by this program.'/ 'The number of records must be reduced.')
      end
c
c************************************************************************
c
c                           rd_main
c
c  This routine reads one record from the main TV data base
c
c************************************************************************
c
      subroutine rd_main(imain, irec)
      include 'global.inc'
c
      character svc_cls*2, cty*1, state*2, city*20, call*8
      character pol*1, da*1, beamt*1, zone*1, antmnf*3
      character antmod*14, off*1, arn*8
      common /mainrec/
     &   latsec, lonsec, erp, haat, rcamsl, ref_az, itype, istat,
     &   svc_cls, off, pol, da, beamt, zone, antmnf, antmod,
     &   ichan, cty, city, state, call, arn
c
c  Read record
c
      read(imain, rec=irec, end=450, err=400)
     &   ichan, cty, city, state, call, latsec, lonsec,
     &   erp, haat, rcamsl, ref_az, itype, istat, svc_cls,
     &   off, pol, da, beamt, zone, antmnf, antmod, arn
c
c  Adjust call sign for 3 letter US
c
      if (cty .eq. 'A') then
        if (call(4:5) .eq. 'TV') call(4:8) = '     '
        if (call(4:5) .eq. '-T') call(4:8) = '     '
      end if
      goto 500
c
c  Error reading data base
c
  400 continue
      write(0, '(a,i5)') 'Error reading main data base at record',
     &   irec
      call exit(257)
c
c  End of file while reading data base
c
  450 continue
      city = 'end of file'
  500 return
      end

      character*6 function arn_prefix(itype)
c
c  Retrieve the application reference number prefix found in the TV
c  Engineering Data Base from its identifying number.  The present
c  character function reverses the assignment of index number to string
c  made by program create_tvdb or program add_atv, and changes to
c  pertinent parts of the code for those programs must be reflected here.
c
c  File ENGdb_prefixes.inc defines the parameter NUM_PREFIXES and the
c  array prefix(1:NUM_PREFIXES).
c
      include 'ENGdb_prefixes.inc'
c
      if (itype .le. 0 .or. itype .gt. NUM_PREFIXES) then
         arn_prefix = '      '
      else
         arn_prefix = prefix(itype)
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                              load_fsc
c
c  This routine loads the field strength curve data points into the
c  common block CRV_DAT.
c
c***********************************************************************
c
      subroutine load_fsc
      include 'global.inc'
c
c  Open field strength curve files
c
      call getlun(IF55LV)
      OPEN(UNIT=IF55LV, FILE=fsc_file(1), STATUS='old',
     &   FORM='FORMATTED', ACCESS='SEQUENTIAL')
c
      call getlun(IF55HV)
      OPEN(UNIT=IF55HV, FILE=fsc_file(2), STATUS='old',
     &   FORM='FORMATTED', ACCESS='SEQUENTIAL')
c
      call getlun(IF55U)
      OPEN(UNIT=IF55U, FILE=fsc_file(3), STATUS='old', FORM='FORMATTED',
     &   ACCESS='SEQUENTIAL')
c
      call getlun(IF51LV)
      OPEN(UNIT=IF51LV, FILE=fsc_file(4), STATUS='old',
     &   FORM='FORMATTED', ACCESS='SEQUENTIAL')
c
      call getlun(IF51HV)
      OPEN(UNIT=IF51HV, FILE=fsc_file(5), STATUS='old',
     &   FORM='FORMATTED', ACCESS='SEQUENTIAL')
c
      call getlun(IF51U)
      OPEN(UNIT=IF51U, FILE=fsc_file(6), STATUS='old', FORM='FORMATTED',
     &   ACCESS='SEQUENTIAL')
c
      call getlun(IF59LV)
      OPEN(UNIT=IF59LV, FILE=fsc_file(7), STATUS='old',
     &   FORM='FORMATTED', ACCESS='SEQUENTIAL')
c
      call getlun(IF59HV)
      OPEN(UNIT=IF59HV, FILE=fsc_file(8), STATUS='old',
     &   FORM='FORMATTED', ACCESS='SEQUENTIAL')
c
      call getlun(IF59U)
      OPEN(UNIT=IF59U, FILE=fsc_file(9), STATUS='old', FORM='FORMATTED',
     &   ACCESS='SEQUENTIAL')
c
c  Read files and load data into appropriate array.  Note that each file
c  has the distance and antenna heights, however since they are the same
c  for all three F50-50 curves and also the same for all three F50-10
c  curves, they are read into the same variable each time.  The same
c  applies to the antenna heights.  In addition, the F50-90 curves use
c  the same distance and height as the F50-50 curves.
c
c  F50,50 low VHF
c
      do i=1, 25
         read(if55lv, *) d50(i)
      end do
      do i = 1, 13
         read(if55lv, *) h50(i)
      end do
      do i = 1, 25
         do j = 1, 13
         read(if55lv, *) f55lv(i, j)
         end do
      end do
      close(if55lv)
c
c  F50-50 high VHF
c
      do i=1, 25
         read(if55hv, *) d50(i)
      end do
      do i=1, 13
         read(if55hv, *) h50(i)
      end do
      do i=1, 25
         do j=1, 13
            read(if55hv, *) f55hv(i, j)
         end do
      end do
      close(if55hv)
c
c  F50-50 UHF
c
      do i=1, 25
         read(if55u, *) d50(i)
      end do
      do i=1, 13
         read(if55u, *) h50(i)
      end do
      do i=1, 25
         do j=1, 13
            read(if55u, *) f55u(i, j)
         end do
      end do
      close(if55u)
c
c  F50-10 low VHF
c
      do i=1, 30
         read(if51lv, *) d10(i)
      end do
      do i=1, 13
         read(if51lv, *) h10(i)
      end do
      do i=1, 30
         do j=1, 13
            read(if51lv, *) f51lv(i, j)
         end do
      end do
      close(if51lv)
c
c  F50-10 high VHF
c
      do i=1, 30
         read(if51hv, *) d10(i)
      end do
      do i=1, 13
         read(if51hv, *) h10(i)
      end do
      do i=1, 30
         do j=1, 13
            read(if51hv, *) f51hv(i, j)
         end do
      end do
      close(if51hv)
c
c  F50-10 UHF
c
      do i=1, 30
         read(if51u, *) d10(i)
      end do
      do i=1, 13
         read(if51u, *) h10(i)
      end do
      do i=1, 30
         do j=1, 13
            read(if51u, *) f51u(i, j)
         end do
      end do
      close(if51u)
c
c  F50-90 low VHF
c
      do i=1, 25
         read(if59lv, *) d50(i)
      end do
      do i=1, 13
         read(if59lv, *) h50(i)
      end do
      do i=1, 25
         do j=1, 13
            read(if59lv, *) f59lv(i, j)
         end do
      end do
      close(if59lv)
c
c  F50-90 high VHF
c
      do i=1, 25
         read(if59hv, *) d50(i)
      end do
      do i=1, 13
         read(if59hv, *) h50(i)
      end do
      do i=1, 25
         do j=1, 13
            read(if59hv, *) f59hv(i, j)
         end do
      end do
      close(if59hv)
c
c  F50-90 UHF
c
      do i=1, 25
         read(if59u, *) d50(i)
      end do
      do i=1, 13
         read(if59u, *) h50(i)
      end do
      do i=1, 25
         do j=1, 13
            read(if59u, *) f59u(i, j)
         end do
      end do
      close(if59u)
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         load_next_pair
c  
c  This routine loads the city, state, channel and station type of an
c  NTSC-ATV pair of stations into the desired list as stations number 1
c  and 2.  The input argument is the number of an NTSC station in the
c  desired list as initially read by routine load_desired_list.  It must
c  never be less than 3.
c
c***********************************************************************
c
      subroutine load_next_pair
      include 'global.inc'
c
c  Local variable
c
      integer irec(2)
c
c  Find respective record numbers in engineering data base
c
      if (next_ntsc .lt. 3 .or. next_ntsc .gt. n_ntsc+2) stop
     &   'Argument out of bounds in call to load_next_pair'
      irec(1) = desired_stations(next_ntsc).id
      call paired_station(irec(1), irec(2))
c
c  Copy the identifiers of the next NTSC station and its paired ATV
c  into place as those of desired stations numbers 1 and 2.
c
      do i=1, 2 
         desired_stations(i).id = irec(i)
         desired_stations(i).city = city_r(irec(i))
         desired_stations(i).state = state_r(irec(i))
         desired_stations(i).ichan = ichan_r(irec(i))
         desired_stations(i).sta_type = sta_type(irec(i))
      end do  
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        load_ter_index
c
c  This routine loads the index to the 7.5 minute terrain blocks.
c
c  The 4 variables for each block are stored sequentially in the
c  array index_75_ter.  These variables are north/south bias,
c  east/west bias, elevation bias, and number of bits used to store
c  data.  A dummy block with these variables set to 0 is added at
c  the end.
c
c***********************************************************************
c
      subroutine load_ter_index
      include 'global.inc'
c
      parameter(NUM_BYTES = 2*NUM_TERRAIN_INDEX_ELEM)
      character index_file*64
      integer*2 idummy(NUM_TERRAIN_INDEX_ELEM)
      equivalence(idummy, index_75_ter)
c
c  Determine location of index file, and set path and name
c
      index_file = dir_list(HOME_PARTITION)
      l = lnblnk(index_file)
      index_file = index_file(1:l) // 'terrain_index.dbs'
c
c  Recl is the size in bytes of the entire index file
c
      call getlun(idx)
      open(unit=idx, file=index_file, access='direct',
     &   form='unformatted', recl=NUM_BYTES, iostat=ierr)
      if (ierr .ne. 0) then
         call perror(index_file)
         stop 'Unable to open terrain index file'
      end if
      read(idx, rec=1, err=900) idummy
c
c  Set the last four 2-byte integers to 0 to indicate that the last 7.5
c  minute terrain block contains no data.
c
      i = NUM_TERRAIN_INDEX_ELEM + 1
      do while (i .le. NUM_TERRAIN_INDEX_ELEM + 4)
         index_75_ter(i) = 0
         i = i + 1
      end do
      return
c
  900 continue
      call perror(index_file)
      stop 'Unable to read index of terrain data files'
      end
c  %W% %G%
c
c***********************************************************************
c
c                         main_desired
c
c  This is a major component of the program for multi-station analysis
c  of TV service and interference.  Called after engineering data has
c  been loaded, it defines a problem area grid and sets a service bit
c  for each station at each grid cell to indicate whether there is
c  noise-limited coverage (cell within the grade B contour for NTSC
c  stations, or ATV service field strong enough to overcome noise).
c  Desired signal strengths are calculated for the cells which are
c  covered in this sense.
c
c***********************************************************************
c
      subroutine main_desired(d_inc)
      include 'global.inc'
c
c  Local variables
c
      character saved_routine_name*32
      character*8 coord_string1, coord_string3  !latitudes
      character*9 coord_string2, coord_string4  !longitudes
      integer latdms(3), londms(3), presumed
      real pa_bound_lat(2), pa_bound_lon(2)
      real time_array(2)
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'main_desired'
c
c  For pairwise analysis, define the problem area by the grade B contour
c  of the NTSC station currently in number 1 position on the list of
c  desired stations.
c
      if (pairwise) then
         irec_pos = desired_stations(1).id
         ichan_pa = desired_stations(1).ichan
         city_pa = desired_stations(1).city
         state_pa = desired_stations(1).state
         sta_type_pa = desired_stations(1).sta_type
      end if
c
c  If the problem area of interest is that covered by a particular
c  station, find record position in station data arrays and define the
c  rectangle bounding the station's NLC.
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce(
     &   'Setting latitude and longitude bounds of problem area',
     &   hours, 0)
      if (type_prob_area .eq. 'Station' .or. pairwise) then
         if (.not. pairwise) then
            call find_record(city_pa, state_pa, sta_type_pa,
     &         ichan_pa, irec_pos)
            if (irec_pos .eq. 0) then
               write(0, 1010) city_pa, state_pa, ichan_pa
               stop
            end if
         end if
         call define_prob_area(pa_bound_lat, pa_bound_lon,
     &      irec_pos)
         write(0, '(a,$)') 'Problem area defined by grade B contour of '
         call print_station(irec_pos, 0)
      else
c
c  If a rectangular problem area was specified, convert rectangle
c  coordinates from dms to decimal degrees.
c
         call dms_to_deg(pa_bound_lat(1),
     &      lat_pa(1,1), lat_pa(1,2), lat_pa(1,3))
         call dms_to_deg(pa_bound_lon(1),
     &      lon_pa(1,1), lon_pa(1,2), lon_pa(1,3))
         call dms_to_deg(pa_bound_lat(2),
     &      lat_pa(2,1), lat_pa(2,2), lat_pa(2,3))
         call dms_to_deg(pa_bound_lon(2),
     &      lon_pa(2,1), lon_pa(2,2), lon_pa(2,3))
      end if
c
c  Define problem area grid
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Defining problem area grid', hours, 0)
      call init_grid(pa_bound_lat(1), pa_bound_lon(1),
     &   pa_bound_lat(2), pa_bound_lon(2))
c
c  The latitude and longitude grid boundaries determined by subroutine
c  init_grid may differ slightly from pa_bound_lat/lon because of
c  cell_size and the number of cells needed to cover the rectangle.
c  Cells are included if and only if their geometrical centers lie
c  inside the pa_bound limits.
c
c  These boundaries, lat_lim and lon_lim, are in seconds and must be
c  converted to decimal degrees for routine read_pop_data.
c
      call sec_to_deg(lat_lim(1), x_lat1)
      call sec_to_deg(lon_lim(1), x_lon1)
      call sec_to_deg(lat_lim(2), x_lat2)
      call sec_to_deg(lon_lim(2), x_lon2)
c
      write(coord_string1, 1012) int(x_lat1), int(60*mod(x_lat1, 1.0)),
     &   nint(60*mod(60*mod(x_lat1, 1.0), 1.0))
      write(coord_string2, 1014) int(x_lon1), int(60*mod(x_lon1, 1.0)),
     &   nint(60*mod(60*mod(x_lon1, 1.0), 1.0))
      write(coord_string3, 1012) int(x_lat2), int(60*mod(x_lat2, 1.0)),
     &   nint(60*mod(60*mod(x_lat2, 1.0), 1.0))
      write(coord_string4, 1014) int(x_lon2), int(60*mod(x_lon2, 1.0)),
     &   nint(60*mod(60*mod(x_lon2, 1.0), 1.0))
      call hms(lat_cg, latdms)
      call hms(lon_cg, londms)
      write(0, 1016) coord_string1, coord_string2, coord_string3,
     &   coord_string4, n_cells, latdms, londms, icell_h, icell_w,
     &   2*num_cell_h, 2*num_cell_w
      write(0, 1018) cell_size, cell_area
c
c  Fetch census blocks within the problem area, allowing an extra 0.01
c  degrees in each direction to avoid round-off errors.
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Fetching census blocks for problem area', hours, 0)
      call allocate_pop_block_mem(ptr_census_blks, MAX_POP_BLOCKS)
      call read_pop_data(x_lat1-0.01, x_lon1-0.01,
     &   x_lat2+0.1, x_lon2+0.1)
c
c  Sort population census blocks by latitude
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Sorting census blocks by latitude', hours, 0)
      call sort_pop_data
c
c  Find the population of each grid cell and set its coordinates equal
c  to the population-weighted centroid of the census blocks it contains
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce(
     &   'Loading problem area grid with population data', hours, 0)
      call fill_problem_grid
c
c  Free memory used to store individual census blocks
c
      call release_pop_block_mem(ptr_census_blks)
c
c  Initialize problem area grid to show service from desired stations at
c  all cells (calculate_area option), or all where population was found
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Initializing problem area grid', hours, 0)
      call init_svc_flag
c
c  Allocate space to store results
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Allocating memory for results', hours, 0)
      call allocate_results_mem
c
c  Find standard noise-limited contours
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce(
     &   'Determining noise-limited contours using FCC curves',
     &   hours, 0)
c
c  Allocate memory to store NLC points for desired stations based on
c  the number of radials (typically 72) the user has selected
c
      call allocate_cntr_mem(n_rads_used)
c
c  Loop on the number of desired stations.
c
      do i=1, n_des_sta
         irec_pos = desired_stations(i).id
c 
c  Compute noise-limited contour using FCC curves.  Along each of a
c  specified number of radials (typically 72), find the distance to the
c  point at which the field falls below the value needed to overcome
c  noise, and find the coordinates of that point.
c
         if (i .eq. 1 .or. .not. pairwise)
     &      call compute_contour(irec_pos)
c
c  Flag cells outside the contour
c
         call ck_for_nl_svc(i, lat_site(irec_pos), lon_site(irec_pos))
c
c  Save total population within NLC of this desired station
c
         call count_svc_pop(i, n_pop_total, n_cell_total)
         call print_grid(i, 'Grade B coverage')
         pop_results(i).nl_tot = n_pop_total
         cell_results(i).nl_tot = n_cell_total
      end do
c
c  Free memory used for NLC data
c
      call release_cntr_mem
c
c  Open a direct access scratch file and save the array of grid cell
c  populations on disk because it will not be needed in memory while
c  computing fields.  Routines which need these data are expected to
c  restore the array automatically.
c
      call open_pop_scr
      call save_on_disk(ipop_scr, ptr_pop_tot, n_cells)
      call release_pop_mem(ptr_pop_tot)
c
c  Compute field at cell points for desired stations using the selected
c  terrain dependent propagation model.
c
c  The distance increment to be used for terrain retrieval is d_inc (km).
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Calculating service fields', hours, 0)
      map_loads = 0
      npath_pts = 0
      call comp_td_cell_flds(d_inc)
c
c  Get total population of cells within the NLC where the desired field is
c  adequate to overcome noise.  Note the call to count_svc_pop retrieves
c  the array of grid cell populations.
c
      hours = (etime(time_array) - run_start)/3600.0
      irec_pos = desired_stations(1).id
      no_profile = undetermined(1, irec_pos)
      lr_failure = undetermined(2, irec_pos)
      presumed = no_profile + lr_failure
      if (presumed .gt. 0) then
         write(0, 1020) no_profile, no_profile*cell_area
         write(0, 1022) lr_failure, lr_failure*cell_area
         write(0, 1024) presumed, presumed*cell_area
      end if
      call announce('Counting population covered', hours, 0)
      do i=1, n_des_sta
         call count_svc_pop(i, n_td_pop_total, n_td_cell_total)
         pop_results(i).nl_td_tot = n_td_pop_total
         cell_results(i).nl_td_tot = n_td_cell_total
      end do
c
      routine_name = saved_routine_name
      return
c
 1010 format('No record found for ', a, 1x, a2, i3)
 1012 format(i2, '-', i2.2, '-', i2.2)	!latitude
 1014 format(i3, '-', i2.2, '-', i2.2)	!longitude
 1016 format(/
     &   22x, 'LAYOUT OF PROBLEM AREA'//
     &   5x, 'SE corner ', a, ', ', a, '; NW corner ', a, ', ', a//
     &   3x, '           Center     Center     Cell    Cell ',
     &   '    Area      Area'/
     &   3x, '  Total     Lat        Long     Height   Width',
     &   '    Height    Width'/
     &   3x, '  Cells   (DMS, N)   (DMS, W)   (sec)    (sec)',
     &   '   (cells)   (cells)'/ i10, 3x, i2.2, 2('-', i2.2), 2x,
     &   i3.3, 2('-', i2.2), i7, i9, 2i10/)
 1018 format('Specified cell size =', f6.3, ' km each side'/
     &   'Precise cell size = ', f9.5, ' sq km')
 1020 format('No terrain profile for  ', i6, ' cells,', f8.1,
     &   ' sq km')
 1022 format('Longley-Rice failure for', i6, ' cells,', f8.1,
     &   ' sq km')
 1024 format('Undetermined service for', i6, ' cells,', f8.1,
     &   ' sq km')
      end
c  %W% %G%
c
c***********************************************************************
c
c                         main_load
c
c  First major component of the program for evaluation of TV service
c  with multi-station interference.  Loads the engineering data base,
c  the desired stations list, and an index to the files containing
c  terrain elevation data.  Also initializes memory blocks for storage
c  of terrain data.
c
c***********************************************************************
c
      subroutine main_load
      include 'global.inc'
c
      logical incompatible
      real time_array(2)
c
c  Load the engineering data base and initialize all ATV stations as
c  not replicated
c
      write(0, '(a)') 'Loading engineering data'
      hours = (etime(time_array) - run_start)/3600.0
      call announce('   - TV Engineering Data Base', hours, 0)
      call load_eng_db
      num_atv_sta = n_eng_rec - (n_first_atv-1)
      if (num_atv_sta .gt. MAX_ATV_STA) then
         write(0, '(a)') 'Too many ATV stations'
         call exit(257)
      end if
      do nsta=1, num_atv_sta
         replicated(nsta) = .false.
      end do
c
c  Load the directional antenna data base
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('   - data base of directional antennas', hours, 0)
      call load_da_pats
c
c  Wherever the given ERP is zero, replace ATV station parameters with
c  prototype values, except when replication is in force.
c
      if (.not. replicate) then
         hours = (etime(time_array) - run_start)/3600.0
         call announce('   - ATV prototype station parameters',
     &      hours, 0)
         call load_atv_sta_proto
      end if
c
c  Load the FCC field strength curves
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('   - FCC field strength curves', hours, 0)
      call load_fsc
c
c  Open file of computed HAAT values and create index
c
      if (compute_haat) then
         hours = (etime(time_array) - run_start)/3600.0
         call announce('   - precomputed HAAT data', hours, 0)
         call open_haat_file
         incompatible = n_haat_radials .lt. n_std_rad .or.
     &      mod(n_haat_radials, n_std_rad) .ne. 0
         if (incompatible) then
            write(0, 900) haat_db_name(1:lnblnk(haat_db_name))
            call exit(257)
         end if
         call load_haat_index
      end if
c
c  Read list of desired stations
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce( 'Reading list of desired stations', hours, 0)
      call load_desired_list
c
c  Load terrain data index and initialize memory blocks
c
      hours = (etime(time_array) - run_start)/3600.0
      call announce('Initializing terrain elevation arrays', hours, 0)
      call announce('   - directory list', hours, 0)
      call get_ter_dirs
      hours = (etime(time_array) - run_start)/3600.0
      call announce('   - terrain file indices', hours, 0)
      call load_ter_index
      hours = (etime(time_array) - run_start)/3600.0
      call announce('   - array pointers', hours, 0)
      call init_ter_mem
c
      return
  900 format('Azimuth increment in file ', a,
     &   ' incompatible with HAAT interpolation routines')
      end
c
c***********************************************************************
c
c                         open_haat_file
c
c  This routine opens the file of HAATs which have been precalculated
c  using the specific azimuth increment given in the first record.
c  This increment and the number of radials per station are stored in
c  global common.
c
c***********************************************************************
c
      subroutine open_haat_file
      include 'global.inc'
c
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'open_haat_file'
c
c  Open file and read first record to determine actual record size used
c  to create file.  The first record in the file is the azimuth
c  increment at which the HAATs were determined, and the record length
c  is 4 + the number of azimuths (360/increment).  The first four bytes
c  in each record hold the engineering data base record number.
c
      n_size = 4
      call getlun(ihaat_file)
      open(unit=ihaat_file, file=haat_db_name,
     &   status='old', form='unformatted', access='direct',
     &   recl=n_size)
c
c  Find the azimuth increment used in creating the file and the number
c  of HAAT values per station
c
      read(ihaat_file, rec=1) az_inc_haat
      n_haat_radials = 360.0 / az_inc_haat
      if (n_haat_radials .gt. MAX_RAD) stop 'Too many HAAT radials'
c
c  Close and reopen with correct record length
c
      close(ihaat_file)
      n_size = (n_haat_radials * 4) + 4
      call getlun(ihaat_file)
      open(unit=ihaat_file, file=haat_db_name,
     &   status='old', form='unformatted', access='direct',
     &   recl=n_size )
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c
c***********************************************************************
c
c                          load_haat_index
c
c  Build an index cross-referencing record numbers of the HAAT file to
c  records of the main engineering data base.  HAAT data for the i-th
c  station of the engineering data base will be found as record number
c  index_haat(i) in the HAAT data file.
c
c***********************************************************************
c
      subroutine load_haat_index
      include 'global.inc'
c
      character haat_index_file*32
      character*43 record
      character*38 pattern, sorted_list(MAX_ENG_REC)
      character city*20, country*1, state*2
      integer rindex, lat(3), long(3)
      logical indexed_haat_file
c
c  Structure in a form which can be used with qsort
c
      structure /HAAT_INDEX_RECORDS/
         integer id
         integer ichan
         integer latsec
         integer lonsec
         character country*1
         character state*2
         character city*20
         character fill*1
      end structure
      record /HAAT_INDEX_RECORDS/ haat_id(MAX_ENG_REC)
c
c  Identify HAAT index file
c
      l = rindex(haat_db_name, '.')
      if (l .eq. 0) goto 900
      haat_index_file = haat_db_name(1:l) // 'idx'
      inquire(file=haat_index_file, exist=indexed_haat_file)
c
c  Retrieval will be made through the haat_index array which must be
c  built from the haat_index_file unless the HAAT data file was
c  constructed in coordination with the engineering data base.
c
c  Absence of the haat_index_file is taken as indicating that the files
c  are coordinated, that is, successive records of the HAAT file are
c  one-for-one with those of the engineering data base.
c
      if (.not. indexed_haat_file) then
         do 10 i=1, n_first_atv-1
            index_haat(i) = i + 1
   10    continue
      goto 100
      end if
c
c  Open HAAT index file and read contents into haat_id arrays
c
      call getlun(lun)
      open(lun, file=haat_index_file, status='old')
      n_haat_rec = 0
   20 continue
      read(lun, '(a)', end=30) record
      n_haat_rec = n_haat_rec + 1
      read(record, 1000) country, state, city, lat, long, ichan
      haat_id(n_haat_rec).id = n_haat_rec
      haat_id(n_haat_rec).country = country
      haat_id(n_haat_rec).state = state
      haat_id(n_haat_rec).city = city
      haat_id(n_haat_rec).latsec = lat(3) + 60*(lat(2) + 60*lat(1))
      haat_id(n_haat_rec).lonsec = long(3) + 60*(long(2) + 60*long(1))
      haat_id(n_haat_rec).ichan = ichan
      goto 20
c
   30 continue
      close(lun)
c
c  Sort records of HAAT index file to enable binary search for specific
c  stations by city name, channel, and coordinates
c
      call sort_struct(haat_id, n_haat_rec)
c
c  Save the list of sorted index records
c
      do 40 i=1, n_haat_rec
         call hms(haat_id(i).latsec, lat)
         call hms(haat_id(i).lonsec, long)
         write(record, 1002) haat_id(i).country, haat_id(i).state,
     &      haat_id(i).city, haat_id(i).ichan, lat, long
         sorted_list(i) = record
   40 continue
c
c  Match stations of engineering data base with entries in HAAT index as
c  represented by the sorted list.  The record number for locating a
c  station in the HAAT data file is the position of that station in the
c  HAAT index file plus one.  This is because the first record of the HAAT
c  data file contains the azimuth increment rather than elevation data.
c
      do 50 i=1, n_first_atv-1
         call hms(lat_site(i), lat)
         call hms(lon_site(i), long)
         write(pattern, 1002) country_r(i)(1:1), state_r(i), city_r(i),
     &      ichan_r(i), lat, long
         j = item_number(pattern, sorted_list, n_haat_rec)
         if (j .eq. 0) then
            if (country_r(i)(1:1) .eq. 'A')
     &         write(0, 1004) country_r(i)(1:1), state_r(i), city_r(i),
     &         lat, long, ichan_r(i),
     &         haat_index_file(1:lnblnk(haat_index_file))
            index_haat(i) = 0
         else
            index_haat(i) = haat_id(j).id + 1
         end if
   50 continue
c
c  Indices for locating DTV stations
c
  100 continue
      ioffset = n_first_atv - 1
      do 110 i=n_first_atv, n_eng_rec
         index_haat(i) = index_haat(i-ioffset)
  110 continue
      return
c
c  Error exit
c
  900 continue
      write(0, '(3a)') 'HAAT file (',
     &   haat_db_name(1:lnblnk(haat_db_name)), ') improperly named'
      call exit(257)
c
c1234567890123456789012345678901234567890123
cA CT BRIDGEPORT           411048 0731120 43
cACTBRIDGEPORT          434110480731120
 1000 format(a1, 1x, a2, 1x, a20, t27, 3i2.2, 1x, i3.3, 2i2.2, i3)
 1002 format(a1, a2, a20, i2, 3i2.2, i3.3, 2i2.2)
 1004 format(a1, 1x, a2, 1x, a20, t27, 3i2.2, 1x, i3.3, 2i2.2, i3,
     &   ' not found in ', a)
      end

      integer function item_number(pattern, sorted_list, list_length)
      character*(*) pattern, sorted_list(list_length)
      integer first, pattern_len
c
      pattern_len = len(pattern)
      if (pattern_len .gt. len(sorted_list(1))) stop
     &   'Pattern longer than items in list'
c
      first = 1
      last = list_length
      do while (first .le. last)
         i = (last + first) / 2
         if (pattern .eq. sorted_list(i)(1:pattern_len)) then
            item_number = i
            return
         else if (pattern .lt. sorted_list(i)(1:pattern_len)) then
            last = i - 1
         else
            first = i + 1
         end if
      end do
      item_number = 0
      return
      end

      subroutine sort_struct(struct_array, num_elements)
      integer num_elements
      structure /HAAT_INDEX_RECORDS/
         integer id
         integer ichan
         integer latsec
         integer lonsec
         character country*1
         character state*2
         character city*20
         character fill*1
      end structure
      record /HAAT_INDEX_RECORDS/ struct_array(num_elements)
      external compare_haat_id
      integer*2 compare_haat_id
      integer struct_size
      struct_size = 40
      call qsort(struct_array, num_elements, struct_size,
     &   compare_haat_id)
      return
      end

      integer*2 function compare_haat_id(x, y)
c
c  Specify order for qsort of HAAT records
c
      structure /HAAT_INDEX_RECORDS/
         integer id
         integer ichan
         integer latsec
         integer lonsec
         character country*1
         character state*2
         character city*20
         character fill*1
      end structure
      record /HAAT_INDEX_RECORDS/ x, y
      character*20 city_x, city_y
      character*2 state_x, state_y
      character*1 country_x, country_y
c
      country_x = x.country
      country_y = y.country
      city_x = x.city
      city_y = y.city
      state_x = x.state
      state_y = y.state
      if (country_y .eq. country_x) then
         if (state_y .eq. state_x) then
            if (city_y .eq. city_x) then
               if (y.ichan .eq. x.ichan) then
                  compare_haat_id = 0
                else
                  compare_haat_id = sign(1, x.ichan - y.ichan)
               end if
            else if (city_y .gt. city_x) then
               compare_haat_id = -1
            else
               compare_haat_id =  1
            end if
         else if (state_y .gt. state_x) then
            compare_haat_id = -1
         else
            compare_haat_id =  1
         end if
      else if (country_y .gt. country_x) then
         compare_haat_id = -1
      else
         compare_haat_id =  1
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         main_undesired
c
c  This is the third major routine for multi-station analysis of TV
c  service and interference.  It calculates undesired field strengths
c  at the grid cells previously found to be within the NLC of specified
c  desired stations.
c
c***********************************************************************
c
      subroutine main_undesired(d_inc)
      include 'global.inc'
c
c  Local variables
c
      character announcement*64
      real time_array(2)
c
c  Free memory holding grid cell population.  These data have been saved
c  on disk and are not needed while computing grid cell fields.
c
      if (ptr_pop_tot .ne. 0) call release_pop_mem(ptr_pop_tot)
c
c  Determine channels and max distances to check for undesired stations
c
      call def_srch_chans
c
c  Get list of undesired stations
c
      call get_undesired_list
c
c  Open scratch file to store IX fields at cell points
c
      call open_ix_scr
c
c  Compute field at cell points for undesired stations
c
      hours = (etime(time_array) - run_start)/3600.0
      write(announcement, 1020) n_undes_sta
      call announce(announcement(1:lnblnk(announcement)), hours, 0)
      write(0, 1022) 4 * n_cells * n_undes_sta / 1.0e6
      call comp_td_cell_udflds(d_inc)
c
c  Finished computing undesired fields
c
      write(0, 1024) npath_pts
      num_replacements = map_loads - MAX_TER_BLK
      if (num_replacements .gt. 0) then
         write(0, 1026) max(0, num_replacements)
      else
         write(0, 1028) MAX_TER_BLK, map_loads
      end if
      return
c
 1020 format('Computing IX fields of', i4, ' undesired stations')
 1022 format('Scratch file of IX fields will require', f8.3,
     &   ' megabytes')
 1024 format('Total terrain elevations retrieved = ', i10)
 1026 format('Terrain elevation data in memory replaced', i5, ' times')
 1028 format('Of', i4, ' memory blocks available for terrain data,', i4,
     &   ' were used')
      end
c  %W% %G%
c
c***********************************************************************
c
c                          margin_eval
c
c  This routine finds how many dB can be added to the ERP of undesired
c  ATV stations without increasing interference to the NTSC stations
c  currently being examined.
c
c***********************************************************************
c
      subroutine margin_eval
      include 'global.inc'
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'margin_eval'
c
c  Loop thru desired NTSC stations, check the desired/undesired ratios
c  at each cell, and set service bit to no service when an unacceptable
c  ratio is found.
c
c  The IX fields have been stored on disk and are now loaded into
c  memory as needed.
c
c  A cell without service is skipped, and its present state is
c  determined by testing the appropriate bit in the icell_service array
c  which has n_per_cell bytes.
c
      n_per_cell = int(n_des_sta / 8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n_per_cell = n_per_cell - 1
c
c  Outer loop over desired NTSC stations
c
      do 300 i=1, n_des_sta
         id = desired_stations(i).id
         if (sta_type(id) .eq. 'a') goto 300
         i0 = i - 1	!station number starting from 0
         call set_ix_ch(ichan_r(id))
c
c  Set n_group constant for determining the service array element
c  corresponding to this desired station.  There is one element for
c  every 8 stations at each cell.
c
         n_group = int(i0/8) + 1
c
c  Set index of the service bit for this desired station.  Remember the
c  bits are numbered 0 thru 7.
c
         n_bit = mod(i0, 8)
c
c  Set base location for retrieval of the service fields of this
c  desired station.
c
         lbase = i0 * n_cells
c
c  Loop on undesired NTSC stations
c
         ncase = N_TO_N
         do 110 k=1, n_unntsc
            idu = id_undesired(k)
            if (sta_type(idu) .eq. 'a') stop
     &         'Expected an undesired NTSC; found an ATV station'
c
c  Skip the desired station.  It does not interfere with itself.
c
            if (id .eq. idu) goto 110
c
c  Test channel relationship for possible conflict
c
            ixchn = 1
            ichn = ichan_r(idu)
            do while (ixchans(ixchn) .ne. ichn)
               ixchn = ixchn + 1
               if (ixchn .gt. n_ixch) goto 110
            end do
c
            idif = ichan_r(idu) - ichan_r(id)
            m = 1
            do while (ichan_ck(ncase, m) .ne. idif)
               m = m + 1
               if (m .gt. 25) goto 110
            end do
c
c  Load undesired cell fields for this undesired station
c
            read(ix_scr, rec=k)
     &         (icell_ix_fields(j), icell_ix_fades(j), j=1, n_cells)
c
c  Loop on cells
c
            do 100 j=1, n_cells
c
c  Retrieve service condition at this cell for current station, and if
c  no service go on to next cell
c
               n_el = n_per_cell * (j - 1) + n_group
               if (.not. btest(icell_service(n_el), n_bit)) goto 100
c
c  The current service condition for this cell is retained if either
c  the desired or undesired field was not computed
c
               l = lbase + j
               if (icell_service_fields(l) .eq. UNCALCULATED_SERVICE)
     &            goto 100
               if (icell_ix_fields(j) .eq. UNCALCULATED_IX) goto 100
c
c  Retrieve field strengths
c
               field_ds = float(icell_service_fields(l)) / 100.0
               field_uds = float(icell_ix_fields(j)) / 100.0
c
c  Apply receive antenna discrimination
c
               call apply_rec_ant(id, idu, j, relative_gain)
               field_uds = field_uds + relative_gain
c
c  Find index of limiting power ratio
c
               m = 1
               do while (m .le. 25 .and. n_off_nn(m) .ne. idif)
                  m = m + 1
               end do
c
c  Make sure the channel offset is in range
c
               if (m .gt. 25) then
                  write(0, '(a)') 'Error calculating IX from NTSC'
                  write(0, '(a,i3)') 'Invalid channel offset:', idif
                  stop
               end if
c
c  Retrieve D/U ratio for this channel offset
c
               f_dif = field_ds - field_uds
               pr = pr_nn(1, m)
c
c  Set service bit to 0 if D/U ratio is inadequate
c          
               if (f_dif .lt. pr) call zero_svc_bit(j, i)
  100       continue
  110    continue
c
c  Loop on undesired ATV stations
c
         ncase = A_TO_N
         do 210 k=n_unntsc + 1, n_undes_sta
            idu = id_undesired(k)
c
c  Skip the desired station
c
            if (id .eq. idu) goto 210
            if (sta_type(idu) .eq. 'n') then
               write(0, '(2a)') 'In margin_eval: ',
     &            'Expected an ATV undesired station; found an NTSC'
               stop
            end if
c
c  Test channel relationship for possible conflict
c
            ixchn = 1
            ichn = ichan_r(idu)
            do while (ixchans(ixchn) .ne. ichn)
               ixchn = ixchn + 1
               if (ixchn .gt. n_ixch) goto 210
            end do
c
            idif = ichan_r(idu) - ichan_r(id)
            m = 1
            do while (ichan_ck(ncase, m) .ne. idif)
               m = m + 1
               if (m .gt. 25) goto 210
            end do
c
c  Load undesired cell fields for this undesired station
c
            read(ix_scr, rec=k)
     &         (icell_ix_fields(j), icell_ix_fades(j), j=1, n_cells)
c
c  Loop on cells
c
            do 200 j=1, n_cells
c
c  Retrieve service condition at this cell for current desired station,
c  and if no service go on to next cell.
c
               n_el = n_per_cell * (j - 1) + n_group
               if (.not. btest(icell_service(n_el), n_bit)) goto 200
c
c  Check for uncomputable fields
c
               l = lbase + j
               if (icell_service_fields(l) .eq. UNCALCULATED_SERVICE)
     &            goto 200
               if (icell_ix_fields(j) .eq. UNCALCULATED_IX) goto 200
c
c  Retrieve desired and undesired field strengths
c
               field_ds = float(icell_service_fields(l)) / 100.0
               field_uds = float(icell_ix_fields(j)) / 100.0
c
c  Apply receive antenna discrimination
c
               call apply_rec_ant(id, idu, j, relative_gain)
               field_uds = field_uds + relative_gain
c
c  Find index of limiting power ratio
c
               m = 1
               do while (m .le. 25 .and. n_off_an(m) .ne. idif)
                  m = m + 1
               end do
c
c  Make sure the channel offset is in range
c
               if (m .gt. 25) then
                  write(0, '(a)') 'Error calculating IX from ATV'
                  write(0, '(a,i3)') 'Invalid channel offset:', idif
                  stop
               end if
c
c  Retrieve D/U ratio for this channel offset, and save margin at this cell
c
               f_dif = field_ds - field_uds
               pr = pr_an(1, m)
               az = azimuth_to_cell(idu, j)
               isect = int(az/DELTA_DEGREES) + 1
               call save_margins(idu, isect, f_dif, pr)
  200       continue
  210    continue
  300 continue
      routine_name = saved_routine_name
      return
      end

      subroutine save_margins(idu, isect, f_dif, pr)
      include 'global.inc'
      parameter (LARGEST = 128*256 - 1)
      character*1 tag
      nsta = (idu - n_first_atv) + 1
      margin = nint(f_dif - pr)
      i = max(-NUM_DB_INTERVALS, min(NUM_DB_INTERVALS, margin))
      if (margins(nsta, isect, i) .lt. LARGEST) then
         margins(nsta, isect, i) = margins(nsta, isect, i) + 1
      else
         tag = sta_type(idu)
         call makcap(tag)
         write(0, 1000) ichan_r(idu), tag, state_r(idu),
     &      city_r(idu)(1:lnblnk(city_r(idu)))
      end if
 1000 format('Warning: Hit count out of range for ', i2, a1, 1x,
     &   a2, 1x, a)
      return
      end

      subroutine initialize_margins
      include 'global.inc'
      num_atv_sta = n_eng_rec - (n_first_atv - 1)
      do nsta=1, num_atv_sta
         do isect=1, NUM_ANTENNA_SECTORS
            i = -NUM_DB_INTERVALS
            do while (i .lt. NUM_DB_INTERVALS)
               i = i + 1
               margins(nsta, isect, i) = 0
            end do
         end do
      end do
      return
      end

      subroutine write_margins_file
      include 'global.inc'
      parameter (
     &   NUMSLOTS = 2*NUM_DB_INTERVALS + 1,
     &   NUMBYTES = 2*(NUMSLOTS + 3))
      character*16 filename /'margins.unf'/
      integer*2 n2sta, i2sect, idum
      data idum /0/
      call accumulate_margins
      call getlun(lun)
      open(lun, file=filename, status='unknown', access='direct',
     &   recl=NUMBYTES)
      nrec = 0
      do 120 irec=n_first_atv, n_eng_rec
         nsta = irec - (n_first_atv - 1)
         n2sta = nsta
         do 110 isect=1, NUM_ANTENNA_SECTORS
            if (margins(nsta, isect, NUM_DB_INTERVALS) .eq. 0)
     &         goto 110
            nrec = nrec + 1
            i2sect = isect
            write(lun, rec=nrec) n2sta, i2sect, sector_erp(nsta, isect),
     &         (margins(nsta, isect, i),
     &         i=-NUM_DB_INTERVALS, NUM_DB_INTERVALS)
c-------
c           write(*, '(i4,i3,f6.2/2(10i8/),i8/10i8/10i8)') nsta, isect,
c    &         sector_erp(nsta, isect)/100.0, (margins(nsta, isect, i),
c    &         i=-NUM_DB_INTERVALS, NUM_DB_INTERVALS)
c-------
  110    continue
  120 continue
      close(lun)
      return
      end

      subroutine accumulate_margins
      include 'global.inc'
      parameter (LARGEST = 128*256 - 1)
      character*1 tag
      integer accumulated
      logical warned
      num_atv_sta = n_eng_rec - (n_first_atv - 1)
      do nsta=1, num_atv_sta
         do isect=1, NUM_ANTENNA_SECTORS
            warned = .false.
            i = -NUM_DB_INTERVALS
            do while (i .lt. NUM_DB_INTERVALS)
               i = i + 1
               accumulated = margins(nsta, isect, i) +
     &            margins(nsta, isect, i-1)
               if (accumulated .le. LARGEST) then
                  margins(nsta, isect, i) = accumulated
               else
                  if (.not. warned) then
                     tag = sta_type(idu)
                     call makcap(tag)
                     write(0, 1000) ichan_r(idu), tag, state_r(idu),
     &                  city_r(idu)(1:lnblnk(city_r(idu)))
                     warned = .true.
                  end if
                  margins(nsta, isect, i) = margins(nsta, isect, i-1)
               end if
            end do
         end do
      end do
      return
 1000 format('Warning: Accumulated hits out of range for ', i2, a1, 1x,
     &   a2, 1x, a)
      end
c  %W% %G%
c
c***********************************************************************
c
c                         open_scratch_files
c
c  This routine opens a scratch file to store interference fields or
c  population of the grid cells depending on the entry point.
c
c***********************************************************************
c
      subroutine open_scratch_files
      include 'global.inc'
      logical test
c
c  Open scratch file to store the interference field and fade values
c  within every grid cell.  Field strengths and fades are both represented
c  by integer*2 values, so this information requires 4 bytes per cell.
c  Each record will contain the field and fade arrays for a single
c  interfering station, and the size of this scratch file will eventually
c  become 4*n_cells*n_undes_sta.
c
      entry open_ix_scr
c
c  Delete old file if still open and not in use by another running
c  program.  If in use by present program, the unit numbers will be
c  non-zero.
c
      inquire(file=ix_scratch_file, opened=test)
      if (test) then
         if (ix_scr .eq. 0) then
            write(0, 1000) ix_scratch_file(1:lnblnk(ix_scratch_file))
            stop
         end if
         close(ix_scr, status='delete')
      end if
c
c  Open file
c
      call getlun(ix_scr)
      open(unit=ix_scr, file=ix_scratch_file, status='scratch',
     &   form='unformatted', access='direct', iostat=ierr,
     &   recl=n_cells*4)
      if (ierr .ne. 0) then
         call perror(ix_scratch_file)
         stop
      end if
      return
c
c  Open scratch file to store the population of every grid cell
c
      entry open_pop_scr
      inquire(file=pop_scratch_file, opened=test)
      if (test) then
         if (ipop_scr .eq. 0) then
            write(0, 1000) pop_scratch_file(1:lnblnk(pop_scratch_file))
            stop
         end if
         close(ipop_scr, status='delete')
      end if
c
      call getlun(ipop_scr)
      open(unit=ipop_scr, file=pop_scratch_file, status='scratch',
     &   form='unformatted', access='direct', iostat=ierr,
     &   recl=n_cells*4)
      if (ierr .ne. 0) then
         call perror(pop_scratch_file)
         stop
      end if
      return
c
 1000 format(a, ' is in use by another running program')
      end

      subroutine set_scratch_filenames(conflict)
      include 'global.inc'
      character*80 line /' '/
      character*32 filename /'scratch_directory.dbs'/
      character*24 ix_dat  /'/ix_scratch.dat'/
      character*24 pop_dat /'/pop_scratch.dat'/
      logical conflict, exists, too_long
      inquire(file=filename, exist=exists)
      if (exists) then
         call getlun(infl)
         open(infl, file=filename, status='old', form='formatted',
     &      access='sequential', iostat=ierr)
         if (ierr .ne. 0) then
            call perror(filename)
            stop
         end if
         do while (line .eq. ' ' .or. line(1:1) .eq. '#')
            read(infl, '(a)', err=300, end=300) line
         end do
         scratch_dir = line
         close(infl)
      end if
      length = lnblnk(scratch_dir)
      too_long = length + lnblnk(ix_dat) .gt. len(ix_scratch_file)
      too_long = too_long .or.
     &   length + lnblnk(pop_dat) .gt. len(pop_scratch_file)
      if (too_long) stop 'Scratch file pathname is too long'
c
      ix_scratch_file = scratch_dir(1:length) // ix_dat
      inquire(file=ix_scratch_file, exist=exists)
      conflict = exists
c
      pop_scratch_file = scratch_dir(1:length) // pop_dat
      inquire(file=pop_scratch_file, exist=exists)
      conflict = conflict .or. exists
      return
c
  300 continue
      write(0, '(a)') 'Error reading file ',
     &   filename(1:lnblnk(filename))
      stop
      end

      subroutine lock_scratch_directory
      include 'global.inc'
      character flr_lock_file*48
      integer getpid, pid
      logical locked_directory
      pid = getpid()
      flr_lock_file = scratch_dir(1:lnblnk(scratch_dir)) // '/flr_LOCK'
      inquire(file=flr_lock_file, exist=locked_directory)
      call getlun(lock)
      if (locked_directory) then
         open(unit=lock, file=flr_lock_file, status='old',
     &      access='sequential', iostat=ierr)
         if (ierr .ne. 0) then
            call perror(flr_lock_file)
            stop
         end if
         read(lock, '(i8)') pid
         write(0, 1000) pid
         stop
      else
         open(unit=lock, file=flr_lock_file, status='scratch',
     &      access='sequential', iostat=ierr)
         if (ierr .ne. 0) then
            call perror(flr_lock_file)
            stop
         end if
         write(lock, '(i8.8)') pid
         call flush(lock)
      end if
      return
 1000 format('Scratch directory locked for process ', i6.6)
      end
c  %W% %G%
c
c***********************************************************************
c
c                            paired_station
c
c  This routine determines the record number in the engineering data base
c  of the ATV station paired with a given NTSC station, or vice-versa.
c
c  A record number of zero is returned if an error is found in the
c  organization of the data base.
c
c***********************************************************************
c
      subroutine paired_station(irec1, irec2)
      include 'global.inc'
c
      irec2 = 0
      if (irec1 .le. n_eng_rec) then
         if (sta_type(irec1) .eq. 'n' .and. irec1 .lt. n_first_atv) then
            irec2 = n_first_atv + (irec1 - 1)
         else
            if (sta_type(irec1) .eq. 'a' .and. irec1 .ge. n_first_atv)
     &         irec2 = irec1 - (n_first_atv - 1)
         end if
      end if
      if (irec2 .eq. 0) write(0, '(a)')
     &   'Engineering data base improperly organized'
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c  Create a two-column list identifying the grid cells where the service
c  bit representing station ista has been turned off
c
c***********************************************************************
c
      subroutine print_grid(ista, header)
      character header*(*)
      include 'global.inc'
      character city*20, filename*12, type*1
      character state*2, chan*2, station*40
      logical first_graph
c
c  Logical parameter graphs is set in options.inc
c
      if (.not. graphs) return
c
c  Construct an output filename for this desired station.  The filename
c  consists of the community name, possibly truncated, followed by three
c  characters giving the channel number with an "a" or "n" appended to
c  indicate ATV or NTSC service.  The total length of the filename
c  should match that of the global variable graph_filename because the
c  names are compared to determine whether this is a new station.
c
      if (len(graph_filename) .ne. len(filename)) stop
     &   'Graph filenames not of equal length' 
      city = desired_stations(ista).city
      ichn = desired_stations(ista).ichan
      if (ichn .eq. 0) return
      type = desired_stations(ista).sta_type
      ll = len(filename)-3
      l = lnblnk(city(1:ll))
      i = index(city(1:l), ' ')
      do while (i .gt. 0 .and. i .lt. l)
         city(i:i) = '_'
         i = index(city, ' ')
      end do
      write(filename, '(a,i2.2,a1)') city(1:l), ichn, type
c
c  Close the previous graph file if this is a new station
c
      first_graph = graph_filename .eq. ' '
      if (.not. first_graph .and. filename .ne. graph_filename) then
         iunit = 0
         inquire(file=graph_filename, number=iunit)
         if (iunit .ne. 0) then
            close(iunit)
         else
            stop 'Unexpected error in subroutine print_grid'
         end if
      end if
      graph_filename = filename
c
c  Open graph file and write header
c
      call open_graphfile(igraph, filename)
      write(igraph, '(a)') header
c
c  Construct full station name
c
      state = desired_stations(ista).state
      city = desired_stations(ista).city
      if (ichn .lt. 10) then
         write(chan, '(i1)') ichn
      else
         write(chan, '(i2)') ichn
      end if
      if (type .eq. 'n') then
         write(station, 1000) city(1:lnblnk(city)), state,
     &      chan(1:lnblnk(chan))
      else
         write(station, 1002) city(1:lnblnk(city)), state,
     &      chan(1:lnblnk(chan))
      end if
c
c  Write coordinate indices of the cell containing this desired station
c  on a line followed by the station name in quotes.  Data is appended
c  to this line to aid in plotting service and interference conditions.
c  The appended data consists of the coordinates in seconds of the
c  southwest corner of the problem area and the width and height of the
c  cells in seconds.
c
      ix = (lon_lim(2) - lon_site(desired_stations(ista).id))/icell_w
      iy = (lat_site(desired_stations(ista).id) - lat_lim(1))/icell_h
      write(igraph, 1004) ix, iy, station(1:lnblnk(station)),
     &   lon_lim(2), lat_lim(1), icell_w, icell_h
c
c  Set parameters for determining the service array element corresponding
c  to this desired station.  There is one element for every 8 stations at
c  each cell.
c
      n_per_cell = int(n_des_sta / 8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n_per_cell = n_per_cell - 1
c
      ista0 = ista - 1	!station number starting from 0
      n_group = int(ista0/8) + 1
c
c  Set index of the service bit for this desired station.  Remember the
c  bits are numbered 0 thru 7.
c
      n_bit = mod(ista0, 8)
c
c  Loop on cells
c
      igrid_width = 2 * num_cell_w
      do 100 i=1, n_cells
         i0 = i - 1
c
c  Retrieve service condition at this cell
c
         n_el = n_per_cell*i0 + n_group
         if (.not. btest(icell_service(n_el), n_bit)) then
c
c  Write coordinate indices of cells without service.  The indices begin
c  with (0, 0) representing the cell in the southwest corner.
c
            ix = igrid_width - mod(i0, igrid_width) - 1
            iy = i0/igrid_width
            write(igraph, 1006) ix, iy
         end if
  100 continue
      call flush(igraph)
      return
 1000 format(a, 1x, a2, ', NTSC channel ', a)
 1002 format(a, 1x, a2, ', DTV channel ', a)
 1004 format(2i5, 1x, '"', a, '"', t55, 2i7, 2i5)
 1006 format(2i5)
      end

      subroutine open_graphfile(igraph, filename)
      character filename*(*)
      logical exists, opn
c
      inquire(file=filename, exist=exists, opened=opn, number=iunit)
      if (.not. exists) then
         call getlun(igraph)
         open(igraph, file=filename, status='new',
     &      access='sequential')
      else if (.not. opn) then
         call getlun(igraph)
         open(igraph, file=filename, status='old',
     &      access='append')
      else
         igraph = iunit
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          print_results
c
c  This routine prints the results of a coverage and IX analysis.
c
c***********************************************************************
c
      subroutine print_results
      include 'global.inc'
c
c  Statement functions to extract results from the structures
c  pop_results and cell_results
c
      nl_pop_tot(i) = pop_results(i).nl_tot
      nl_td_pop_tot(i) = pop_results(i).nl_td_tot
      ix_ntsc_pop_loss(i) = pop_results(i).ix_ntsc_loss
      ix_atv_pop_loss(i) = pop_results(i).ix_atv_loss
      ix_pop_loss_tot(i) = pop_results(i).ix_loss_tot
      ix_atv_only_pop_loss(i) = pop_results(i).ix_atv_only_loss
c
      nl_cell_tot(i) = cell_results(i).nl_tot
      nl_td_cell_tot(i) = cell_results(i).nl_td_tot
      ix_ntsc_cell_loss(i) = cell_results(i).ix_ntsc_loss
      ix_atv_cell_loss(i) = cell_results(i).ix_atv_loss
      ix_cell_loss_tot(i) = cell_results(i).ix_loss_tot
      ix_atv_only_cell_loss(i) = cell_results(i).ix_atv_only_loss
c
c  Summarize results for each desired station
c
      do i=1, n_des_sta
         irec_pos = desired_stations(i).id
         write(iresult, 900)
         call print_station(irec_pos, iresult)
         if (sta_type(irec_pos) .eq. 'a') then
            if (.not. replicated(irec_pos-(n_first_atv-1))) then
                  write(iresult, 910) haat_r(irec_pos), erp_r(irec_pos)
            else
               if (capped_db(irec_pos) .eq. 0.0) then
                  if (front_to_back_ratio(irec_pos) .lt. 0.1) then
                     write(iresult, 910) haat_r(irec_pos),
     &                  erp_r(irec_pos)
                  else
                     write(iresult, 912) haat_r(irec_pos),
     &                  erp_r(irec_pos), ref_az_r(irec_pos),
     &                  front_to_back_ratio(irec_pos)
                  end if
               else
                  if (front_to_back_ratio(irec_pos) .lt. 0.1) then
                     write(iresult, 914) haat_r(irec_pos),
     &                  erp_r(irec_pos), capped_db(irec_pos)
                  else
                     write(iresult, 916) haat_r(irec_pos),
     &                  erp_r(irec_pos), capped_db(irec_pos),
     &                  ref_az_r(irec_pos),
     &                  front_to_back_ratio(irec_pos)
                  end if
               end if
            end if
         end if
         write(iresult, 920)
         write(iresult, 921) nl_pop_tot(i), cell_area*nl_cell_tot(i)
         write(iresult, 922) nl_td_pop_tot(i),
     &      cell_area*nl_td_cell_tot(i)
         write(iresult, 923) ix_ntsc_pop_loss(i),
     &      cell_area*ix_ntsc_cell_loss(i)
         write(iresult, 924) ix_atv_pop_loss(i),
     &      cell_area*ix_atv_cell_loss(i)
         if (sta_type(irec_pos) .eq. 'a') write(iresult, 925)
     &      ix_atv_only_pop_loss(i), cell_area*ix_atv_only_cell_loss(i)
         write(iresult, 926) ix_pop_loss_tot(i),
     &      cell_area*ix_cell_loss_tot(i)
         if (pairwise .and. sta_type(irec_pos) .eq. 'a')
     &      write(iresult, 927) percent_pop_match, percent_area_match
      end do
      call flush(iresult)
      return
c
  900 format('Analysis of: ', $)
  910 format(3x, 'HAAT', f7.1, ' m, ATV ERP', f7.1, ' kW')
  912 format(3x, 'HAAT', f7.1, ' m, ATV ERP', f7.1, ' kW, direction',
     &   f6.1, ' degrees T, F/B =', f5.1, ' dB')
c  HAAT  161.0 m, ATV ERP   50.0 kW, direction 225.0 degrees T, F/B =  0.9 dB
  914 format(3x, 'HAAT', f7.1, ' m, ATV ERP', f7.1, ' kW, Cap Adj',
     &   f4.1, ' dB')
  916 format(3x, 'HAAT', f7.1, ' m, ATV ERP', f7.1, ' kW, Cap Adj',
     &   f4.1, ' dB', f6.1, ' deg T, F/B =', f5.1, ' dB')
c  HAAT  161.0 m, ATV ERP   50.0 kW, Cap Adj 5.2 dB, 225.0 deg T, F/B =  0.9 dB
  920 format(34x, 'POPULATION   AREA (sq km)')
  921 format(3x, 'within Noise Limited Contour', 4x, i8, f13.1)
  922 format(3x, 'not affected by terrain losses', 2x, i8, f13.1)
  923 format(3x, 'lost to NTSC IX', 17x, i8, f13.1)
  924 format(3x, 'lost to additional IX by ATV', 4x, i8, f13.1)
  925 format(3x, 'lost to ATV IX only', 13x, i8, f13.1)
  926 format(3x, 'lost to all IX', 18x, i8, f13.1)
  927 format(3x, 'percent match ATV/NTSC', t39, f5.1, t52, f5.1)
c2345678901234567890123456789012345678901234567890123456789
c                                 POPULATION   AREA (sq km)
c  within Noise Limited Contour     1727504      40149.4
c  not affected by terrain losses   1112278      39470.1
c  lost to NTSC IX                        0          0.0
c  lost to additional IX by ATV      414000        271.4
c  lost to ATV IX only               414000        271.4
c  lost to all IX                    414000        271.4
c  percent ATV/NTSC match              99.2         94.7
      end
c  %W% %G%
c
c***********************************************************************
c
c                      print_station
c
c  This routine writes information identifying the station in record
c  number irec to the logical unit numbered iunit.  An additional entry
c  point is provided which announces the time in addition to station
c  information.
c
c***********************************************************************
c
      subroutine print_station(irec, iunit)
      include 'global.inc'
c
      character tag*1
      character announcement*64
      logical alt_form /.true./
      real time_array(2)
c
c  Announce the station's identity
c
      if (.not. alt_form) then
         write(iunit, 900) irec, city_r(irec), state_r(irec),
     &      ichan_r(irec), sta_type(irec)
      else
         tag = sta_type(irec)
         call makcap(tag)
         write(iunit, 902) ichan_r(irec), tag, state_r(irec),
     &      city_r(irec)(1:lnblnk(city_r(irec)))
      end if
      return
c
c  Announce desired station identity and time
c
      entry print_desired_sta(irec, iunit)
      tag = sta_type(irec)
      call makcap(tag)
      write(announcement, 910) ichan_r(irec), tag, state_r(irec),
     &   city_r(irec), call_r(irec)
      hours = (etime(time_array) - run_start)/3600.0
      call announce(announcement(1:lnblnk(announcement)), hours, iunit)
      return
c
c  Undesired station identity and time
c
      entry print_undesired_sta(irec, iunit)
      tag = sta_type(irec)
      call makcap(tag)
      write(announcement, 912) ichan_r(irec), tag, state_r(irec),
     &   city_r(irec), call_r(irec)
      hours = (etime(time_array) - run_start)/3600.0
      call announce(announcement(1:lnblnk(announcement)), hours, 0)
      return
c
  900 format(i5, 1x, a20, 1x, a2, 1x, i2, 1x, a1)
  902 format(i2, a1, 1x, a2, 1x, a)
c
  910 format(3x, 'Desired station', i3, a1, 1x, a2, 1x, a20, 1x, a)
  912 format(3x, 'Undesired station', i3, a1, 1x, a2, 1x, a18, 1x, a)
      end
c  %W% %G%
c
c***********************************************************************
c
c                        purge_ud_list
c
c  This routine purges the list of undesired stations of any of the
c  desired stations that are not needed to be considered as undesired.
c  Since the list of undesired stations previously generated was based
c  only on channels and distance the list will also contain the desired
c  stations and these may or may not need to also be considered as
c  undesired stations depending on which stations are in the list of
c  desired stations.
c
c***********************************************************************
c
      subroutine purge_ud_list
      include 'global.inc'
c
      integer match(2), ichan_t(2)
      logical delete
      pointer (ptr, iud(*))
      integer*2 iud
c
c  Loop thru desired stations to see if they will need to be
c  considered as undesired stations
c
      n_d = 0
      do 100 i=1, n_des_sta
c
c  Get ids of station and its paired station and then put them in order
c
         id_1 = desired_stations(i).id
         id_2 = id_1 + (n_first_atv-1)
         if (id_1 .gt. (n_first_atv-1)) id_2 = id_1 - (n_first_atv-1)
         if (id_1 .gt. id_2) then
            id_t = id_1
            id_1 = id_2
            id_2 = id_t
         end if
c
         ichan_t(1) = ichan_r(id_1)
         ichan_t(2) = ichan_r(id_2)
         match(1) = 1
         match(2) = 1
c
         n_a_1 = 0
         do 60 ii=1, 2
            if (ii .eq. 0) n_a_1 = 2
            do 50 j=1, n_des_sta
               if (i .eq. j) goto 50
               n_a_2 = 1
               if (desired_stations(j).sta_type .eq. 'a') n_a_2 = 2
               l = n_a_1 + n_a_2
c
c  Construct array ixchans, a list of channels that are co, adj or taboo
c  from ichan.
c
               jchn = desired_stations(j).ichan
               call set_ix_ch(jchn)
               do k=1, n_ixch
                  if (ixchans(k) .eq. ichan_t(ii)) then
                     idif = ichan_t(ii) - jchn
c
c  Check whether an analysis of this channel relationship was requested
c
                     do m=1, 25
                        if (idif .eq. ichan_ck(l, m)) goto 60
                     end do
                  end if
               end do
   50       continue
            match(ii) = 0
   60    continue
c
c  If no match, station should be deleted from list
c
         if (match(1) + match(2) .eq. 2) goto 100
         do n=1, n_undes_sta
            delete = id_1 .eq. id_undesired(n) .and. match(1) .eq. 0
            delete = delete .or.
     &         (id_2 .eq. id_undesired(n) .and. match(2) .eq. 0)
            if (delete) then
               id_undesired(n) = 0
               n_d = n_d + 1
            end if
         end do
c
  100 continue
c
c  Delete undesired stations with id of 0 from list and reset counter
c  of undesired stations
c
      nonnull = n_undes_sta - n_d
      ptr = malloc(2 * nonnull)
      i = 0
      j = 0
      do while (j .lt. nonnull)
         i = i + 1
         if (i .gt. n_undes_sta) stop
     &      'Unexpected error purging list of undesired stations'
         if (id_undesired(i) .gt. 0) then
            j = j + 1
            iud(j) = id_undesired(i)
         end if
      end do
      call free(pid_undesired)
      pid_undesired = loc(iud)
      n_undes_sta = nonnull
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                      read_des_sig_level
c
c  This routine reads the Desired Signal Levels category of input data
c  from the input file unit given by the argument infile.  This routine
c  is called by routine read_input that opens the file and positions
c  the file for reading at the line following the category name.
c
c***********************************************************************
c
      subroutine read_des_sig_level(infile, file_name, line,
     &   a, ierr)
      character file_name*48, a*300
      include 'global.inc'
c
      character*80 sub_categories(5), sub_sub_categories(5)
      data sub_categories /
     &   'NTSC',
     &   'ATV',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined' /
c
      data sub_sub_categories /
     &   'Moderate',
     &   'Strong',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined' /
c
   10 line = line + 1
      read(infile, 800, err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, 990) a(1:l)
         goto 10
      end if
c
c  if a(1:1) not = [ then error in file
c
      if (a(1:1) .ne. '[') goto 400
c
c  from sub_category inside [] determine what data is
c  to be read - in this case there are multiple sub_categories
c
      call find_len_btw_ch(a, '[', ']', n_l, n_r, n_t)
c
      if (a(n_l:n_r) .eq. sub_categories(1)(1:n_t)) then
         write(iresult, 990) a(1:l)
         goto 20
      else if (a(n_l:n_r) .eq. sub_categories(2)(1:n_t)) then
         write(iresult, 990) a(1:l)
         goto 30
      end if
c
c  from sub_sub_category inside () determine what data is
c  to be read - in this case there are multiple sub_categories
c
c  read ntsc desired signal levels
c
   20 continue
      j = 0
   21 continue
      line = line + 1
      read(infile, 800, err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, 990) a(1:l)
         goto 21
      end if
c
c  if a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
       j = j + 1
       m = 0
       if (a(n_l:n_r) .eq. sub_sub_categories(1)(1:n_t)) m = 1
       if (a(n_l:n_r) .eq. sub_sub_categories(2)(1:n_t)) m = 2
       if (m .eq. 0) goto 400
       if (m .eq. 1) then
          read(a, 810) cntr_md_n(1), cntr_md_n(2), cntr_md_n(3)
          write(iresult, 900) a(1:10), cntr_md_n(1), cntr_md_n(2),
     &       cntr_md_n(3), a(36:l)
       else if (m .eq. 2) then
          read(a, 810) cntr_st_n(1), cntr_st_n(2), cntr_st_n(3)
          write(iresult, 900) a(1:10), cntr_st_n(1), cntr_st_n(2),
     &       cntr_st_n(3), a(36:l)
       end if
       if (j .lt. 2) goto 21
       goto 10
c
c  Read atv desired signal levels
c
   30 continue
      j = 0
   31 continue
      line = line + 1
      read(infile, 800, err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, 990) a(1:l)
         goto 31
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
      j = j + 1
      m = 0
      if (a(n_l:n_r) .eq. sub_sub_categories(1)(1:n_t)) m = 1
      if (a(n_l:n_r) .eq. sub_sub_categories(2)(1:n_t)) m = 2
      if (m .eq. 0) goto 400
      if (m .eq. 1) then
         read(a, 810) cntr_md_a(1), cntr_md_a(2), cntr_md_a(3)
         write(iresult, 900) a(1:10), cntr_md_a(1), cntr_md_a(2),
     &      cntr_md_a(3), a(36:l)
      else if (m .eq. 2) then
         read(a, 810) cntr_st_a(1), cntr_st_a(2), cntr_st_a(3)
         write(iresult, 900) a(1:10), cntr_st_a(1), cntr_st_a(2),
     &      cntr_st_a(3), a(36:l)
      end if
      if (j .lt. 2) goto 31
      goto 10
c
c  Error reading file - write message and return a non-zero error code
c
  400 ierr = -1000
c
  500 return
c
  800 format(a)
  810 format(11x, f6.2, 4x, f6.2, 3x, f6.2)
  900 format(a10, 1x, f6.2, 4x, f6.2, 3x, f6.2, a)
  990 format(a)
      end
c  %W% %G%
c
c***********************************************************************
c
c                      read_du_ratios
c
c  This routine reads the D/U Ratios category of input data from
c  the input file unit given by the argument infile.  This routine
c  is called by routine read_input that opens the file and positions
c  the file for reading at the line following the category name.
c
c***********************************************************************
c
      subroutine read_du_ratios(infile, file_name, line,
     &   a, ierr)
      character file_name*(*), a*(*)
      include 'global.inc'
c
      character sub_sub_categories*8
      character*80 sub_categories(4)
      character saved_routine_name*32
c
      data sub_categories /
     &   'N-to-N',
     &   'A-to-N',
     &   'N-to-A',
     &   'A-to-A' /
c
      data sub_sub_categories /'Ratios'/
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'read_du_ratios'
c
   10 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a )
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 10
      end if
c
c  If a(1:1) not = [ then error in file
c
      if (a(1:1) .ne. '[') goto 400
c
c  from sub_category inside [] determine what data is
c  to be read - in this case there are multiple sub_categories
c
      call find_len_btw_ch(a, '[', ']', n_l, n_r, n_t)
c
      if (a(n_l:n_r) .eq. sub_categories(1)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 20
      else if (a(n_l:n_r) .eq. sub_categories(2)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 30
      else if (a(n_l:n_r) .eq. sub_categories(3)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 40
      else if (a(n_l:n_r) .eq. sub_categories(4)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 50
      end if
c
c  Read NTSC to NTSC ratios
c
   20 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a )
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 20
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
      if (a(n_l:n_r) .ne. sub_sub_categories(1:n_t)) goto 400
      write(iresult, '(a)') a(1:l)
c
      do i = 1, 17
         read(infile, 810) n_off_nn(i), pr_nn(1, i),
     &      pr_nn(2, i), pr_nn(3, i)
         write(iresult, 900) n_off_nn(i), pr_nn(1, i),
     &      pr_nn(2, i), pr_nn(3, i)
      end do
      goto 10
c
c  Read ATV to NTSC ratios
c
   30 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a )
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 30
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
      if (a(n_l:n_r) .ne. sub_sub_categories(1:n_t)) goto 400
      write(iresult, '(a)') a(1:l)
      do i = 1, 17
         read(infile, 810) n_off_an(i), pr_an(1, i),
     &      pr_an(2, i), pr_an(3, i)
         write(iresult, 900) n_off_an(i), pr_an(1, i),
     &      pr_an(2, i), pr_an(3, i)
      end do
      goto 10
c
c  Read NTSC to ATV ratios
c
   40 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a )
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 40
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
      if (a(n_l:n_r) .ne. sub_sub_categories(1:n_t)) goto 400
      write(iresult, '(a)') a(1:l)
      do i = 1, 17
         read(infile, 810) n_off_na(i), pr_na(1, i),
     &      pr_na(2, i), pr_na(3, i)
         write(iresult, 900) n_off_na(i), pr_na(1, i),
     &      pr_na(2, i), pr_na(3, i)
      end do
      goto 10
c
c  Read ATV to ATV ratios
c
   50 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a )
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 50
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
      if (a(n_l:n_r) .ne. sub_sub_categories(1:n_t)) goto 400
      write(iresult, '(a)') a(1:l)
      do i = 1, 17
         read(infile, 810) n_off_aa(i), pr_aa(1, i),
     &      pr_aa(2, i), pr_aa(3, i)
         write(iresult, 900) n_off_aa(i), pr_aa(1, i),
     &      pr_aa(2, i), pr_aa(3, i)
      end do
      goto 10
c
c  Error reading file.  Write message and return a non-zero error code.
c
  400 continue
      ierr = -1000
c
  500 continue
      routine_name = saved_routine_name
      return
c
  810 format(3x, i3, 4x, f8.2, 4x, f8.2, 3x, f8.2)
  900 format(3x, i3, 4x, f8.2, 4x, f8.2, 3x, f8.2)
      end
c  %W% %G%
c
c***********************************************************************
c
c                          read_input
c
c  This routine reads the model input file and loads the user supplied
c  information into the appropriate variables.  The input to the routine
c  is name of the input file.  The system error code is returned if an
c  error is encountered opening the file.  An error code of -500 is
c  returned to indicate no data was found (input file possibly empty).
c  Otherwise the error code returned is -1000.
c
c***********************************************************************
c
      subroutine read_input(file_name, ierr)
      character file_name*(*)
      include 'global.inc'
c
      character a*300
      character*80 categories(10)
      data categories /
     &   'Macros',
     &   'Program Options', 
     &   'TV Station Parameters',
     &   'Receiving Antenna',
     &   'Noise Threshold',
     &   'Desired Signal Levels',
     &   'D/U Ratios',
     &   'Maximum Analysis Distance',
     &   'None defined', 
     &   'END OF INPUT FILE'/
c
c  Open file
c
      if (file_name .ne. 'stdin') then
         call getlun(infile)
         open(unit=infile, file=file_name, status='old',
     &      form='formatted', access='sequential', iostat=ierr)
c
c  If error opening file then return
c
         if (ierr .ne. 0) goto 500
      else
         infile = 5
      end if
c
c  Begin reading file
c
      line = 0
   10 continue
      line = line + 1
      read(infile, '(a)', err=300, end=300) a
      l = lnblnk(a)
         if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 10
      end if
c
c  Check for content of line non-comment/non-blank line.  It must start
c  with { to define category
c
   15 continue
      if (a(1:1) .ne. '{') goto 400
      l = lnblnk(a)
      write(iresult, '(a)') a(1:l)
c
c  Determine category
c
      call find_len_btw_ch(a, '{', '}', n_l, n_r, n_t)
      if (categories(1)(1:l) .eq. a(n_l:n_r)) then
         goto 20
      else if (categories(2)(1:n_t) .eq. a(n_l:n_r)) then
         goto 30
      else if (categories(3)(1:n_t) .eq. a(n_l:n_r)) then
         goto 40
      else if (categories(4)(1:n_t) .eq. a(n_l:n_r)) then
         goto 50
      else if (categories(5)(1:n_t) .eq. a(n_l:n_r)) then
         goto 60
      else if (categories(6)(1:n_t) .eq. a(n_l:n_r)) then
         goto 70
      else if (categories(7)(1:n_t) .eq. a(n_l:n_r)) then
         goto 80
      else if (categories(8)(1:n_t) .eq. a(n_l:n_r)) then
         goto 90
      else if (categories(10)(1:n_t) .eq. a(n_l:n_r)) then
         goto 500
      end if
      goto 400
c
   20 continue
      call read_macros(infile, file_name, line, a, ierr)
      if (ierr .ne. 0) goto 400
      goto 15
c
   30 continue
      call read_prgm_options(infile, file_name, line, a, ierr)
      if (ierr .ne. 0) goto 400
      goto 15
c
   40 continue
      call read_xmit_params(infile, file_name, line, a, ierr)
      if (ierr .ne. 0) goto 400
      goto 15
c
   50 continue
      call read_rec_antenna(infile, file_name, line, a, ierr)
      if (ierr .ne. 0) goto 400
      goto 15
c
   60 continue
      call read_noise_thres(infile, file_name, line, a, ierr)
      if (ierr .ne. 0) goto 400
      goto 15
c
   70 continue
      call read_des_sig_level(infile, file_name, line, a, ierr)
      if (ierr .ne. 0) goto 400
      goto 15
c
   80 continue
      call read_du_ratios(infile, file_name, line, a, ierr)
      if (ierr .ne. 0) goto 400
      goto 15
c
   90 continue
      call read_max_dist(infile, file_name, line, a, ierr)
      if (ierr .ne. 0) goto 400
      goto 15
c
c  Empty file or no data lines
c
  300 continue
      if (infile .ne. 5) close(infile)
      ierr = -500
      return
c
c  Error reading file.  Write message and return non-zero error code.
c
  400 continue
      write (0, 900) file_name, line
      ierr = -1000
c
  500 continue
      if (infile .ne. 5) close(infile)
      return
c
  900 format('Error reading file ', a48 / 'Error at',
     &   ' line ', i5)
      end
c  %W% %G%
c
c***********************************************************************
c
c                      read_macros
c
c  This routine reads the Macros category of input data from the
c  input file unit given by the argument infile.  This routine is
c  called by routine read_input that opens the file and positions
c  the file for reading at the line following the category name.
c
c***********************************************************************
c
      subroutine read_macros(infile, file_name, line, a, ierr)
      character file_name*(*), a*(*)
      include 'global.inc'
c
      character b*300
      character dummy_name*1
      character*80 sub_categories(10)
c
      data sub_categories /
     &   'TVDB',
     &   'DADB',
     &   'HTDB',
     &   'PCDB',
     &   'POPDB',
     &   'Analysis List File',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined' /
c
   10 line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. a(1:1) .eq. '['
     &   .or. l .eq. 0) then
      write(iresult, '(a)') a(1:l)
      goto 10
      end if
c
c  if a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
c
c  from sub_category inside () determine what data is
c  to be read
c
      if (a(2:5) .eq. sub_categories(1)(1:4)) then
      goto 20
      else if (a(2:5) .eq. sub_categories(2)(1:4)) then
      goto 30
      else if (a(2:5) .eq. sub_categories(3)(1:4)) then
      goto 40
      else if (a(2:5) .eq. sub_categories(4)(1:4)) then
      goto 50
      else if (a(2:5) .eq. sub_categories(5)(1:4)) then
      goto 60
      else if (a(2:5) .eq. sub_categories(6)(1:4)) then
      goto 70
      else if (a(2:5) .eq. sub_categories(7)(1:4)) then
      goto 80
      else if (a(2:5) .eq. sub_categories(8)(1:4)) then
      goto 90
      else if (a(2:5) .eq. sub_categories(9)(1:4)) then
      goto 100
      else if (a(2:5) .eq. sub_categories(10)(1:4)) then
      goto 110
      end if
c
c  read path and name of tv engineering data base file
c
   20 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') tv_db_name
      n1 = n1 - 1
      b = a(1:n1) // tv_db_name(1:(n2-n1)) // a(n2+1:l)
      l = lnblnk(b)
      write(iresult, '(a)') b(1:l)
      goto 10
c
c  read path and name of directional antenna data base file
c
   30 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') tv_da_db_name
      n1 = n1 - 1
      b = a(1:n1) // tv_da_db_name(1:(n2-n1)) // a(n2+1:l)
      l = lnblnk(b)
      write(iresult, '(a)') b(1:l)
      goto 10
c
c  read path and name of haat data base
c
   40 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') haat_db_name
      n1 = n1 - 1
      b = a(1:n1) // haat_db_name(1:(n2-n1)) // a(n2+1:l)
      l = lnblnk(b)
      write(iresult, '(a)') b(1:l)
      goto 10
c
c  read path and name of propagation curve
c  data point files - there are nine files
c  one for each band for F50/50, F50/10 an F50/90
c
   50 l = lnblnk(a)
      write(iresult, '(a)') a(1:l)
c
      do i = 1, 9
      read(infile, '(a)', err=400, end=400) a
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      read(a(n_l:n_r), '(a)') fsc_file(i)
      write(iresult, '(a)') fsc_file(i)(1:n_t)
      end do
      goto 10
c
c  read path for population data files
c
   60 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') population_db
      n1 = n1 - 1
      b = a(1:n1) // population_db(1:(n2-n1)) // a(n2+1:l)
      l = lnblnk(b)
      write(iresult, '(a)') b(1:l)
      goto 10
c
c  read path and name of
c
   70 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') desired_file
      n1 = n1 - 1
      b = a(1:n1) // desired_file(1:(n2-n1)) // a(n2+1:l)
      l = lnblnk(b)
      write(iresult, '(a)') b(1:l)
      goto 10
c
c  read path and name of
c
   80 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') dummy_name
      goto 10
c
c  read path and name of
c
   90 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') dummy_name
      goto 10
c
c  read path and name of
c
  100 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') dummy_name
      goto 10
c
c  read path and name of
c
  110 n1 = 0
      n2 = 0
      do i = 1,l
      if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
      if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)')  dummy_name
      goto 10
c
c  error reading file - write message and return
c  a non-zero error code
c
  400 ierr = -1000
c
  500 return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          read_max_dist
c
c  This routine reads maximum IX analysis distances from the text input
c  stream.
c
c***********************************************************************
c
      subroutine read_max_dist(infile, file_name, line,
     &   a, ierr)
      character file_name*(*), a*(*)
      include 'global.inc'
c
c  Initialize array
c
      do l=1, 4
         do m=1, 25
            analysis_limits(l, m) = -10000.0
         end do
      end do
c
   92 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 92
      end if
c
      if (a(1:1) .ne. '(') goto 400
      write(iresult, '(a)') a(1:l)
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
c  Read system relationship defined in ()
c
      k = 0
      if (a(n_l:n_r) .eq. 'N-to-N') k = 1
      if (a(n_l:n_r) .eq. 'N-to-A') k = 2
      if (a(n_l:n_r) .eq. 'A-to-N') k = 3
      if (a(n_l:n_r) .eq. 'A-to-A') k = 4
      if (k .eq. 0) goto 400
c
c  Read up to 25 channel relationships and distances
c
      n = 0
   93 continue
      n = n + 1
      if (n .gt. 25) stop 'Too many channel relationships'
   94 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a )
      if (l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 92
      end if
      read(a, 825) n_relation, adist
c
c  Skip if not a selected relationship
c
      if (n_relation .ne. ichan_ck(k, n)) goto 94
c
c  Store distance in array of analysis limits
c
      analysis_limits(k, n) = adist
      write(iresult, 825) n_relation, analysis_limits(k, n)
      goto 93
c
c  Error reading file
c
  400 continue
      ierr = -1000
c
  500 return
c
  825 format(3x, i3, 16x, f7.1)
      end
c  %W% %G%
c
c***********************************************************************
c
c                      read_noise_thres
c
c  This routine reads the Noise Threshold category of input data from
c  the input file unit given by the argument infile.  This routine is
c  called by routine read_input that opens the file and positions
c  the file for reading at the line following the category name.
c
c***********************************************************************
c
      subroutine read_noise_thres(infile, file_name, line,
     &    a, ierr)
      character file_name*(*), a*(*)
      include 'global.inc'
c
      character*80 sub_categories(5), sub_sub_categories(5)
      data sub_categories /
     &   'NTSC',
     &   'ATV',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined' /
c
      data sub_sub_categories /
     &   'Low VHF',
     &   'High VHF',
     &   'UHF',
     &   'None Defined',
     &   'None Defined' /
c
   10 line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
      write(iresult, '(a)') a(1:l)
      goto 10
      end if
c
c  if a(1:1) not = [ then error in file
c
      if (a(1:1) .ne. '[') goto 400
c
c  from sub_category inside [] determine what data is
c  to be read - in this case there are multiple sub_categories
c
      call find_len_btw_ch(a, '[', ']', n_l, n_r, n_t)
c
      if (a(n_l:n_r) .eq. sub_categories(1)(1:n_t)) then
      write(iresult, '(a)') a(1:l)
      goto 20
      else if (a(n_l:n_r) .eq. sub_categories(2)(1:n_t)) then
      write(iresult, '(a)') a(1:l)
      goto 30
      end if
c
c  from sub_sub_category inside () determine what data is
c  to be read - in this case there are multiple sub_categories
c
c  read ntsc noise limited contour values
c
   20 j = 0
   21 line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
      write(iresult, '(a)') a(1:l)
      goto 21
      end if
c
c  if a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
        j = j + 1
        m = 0
        if (a(n_l:n_r) .eq. sub_sub_categories(1)(1:n_t)) m = 1
        if (a(n_l:n_r) .eq. sub_sub_categories(2)(1:n_t)) m = 2
        if (a(n_l:n_r) .eq. sub_sub_categories(3)(1:n_t)) m = 3
        if (m .eq. 0) goto 400
        read(a, 810) cntr_nl_n(m)
        write(iresult, 900) a(1:10), cntr_nl_n(m)
        if (j .lt. 3) goto 21
        goto 10
c
c  read atv noise limited contour values
c
   30 j = 0
   31 line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
      write(iresult, '(a)') a(1:l)
      goto 31
      end if
c
c  if a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
        j = j + 1
        m = 0
        if (a(n_l:n_r) .eq. sub_sub_categories(1)(1:n_t)) m = 1
        if (a(n_l:n_r) .eq. sub_sub_categories(2)(1:n_t)) m = 2
        if (a(n_l:n_r) .eq. sub_sub_categories(3)(1:n_t)) m = 3
        if (m .eq. 0) goto 400
        read(a, 810) cntr_nl_a(m)
        write(iresult, 900) a(1:10), cntr_nl_a(m)
        if (j .lt. 3) goto 31
        goto 10
c
c  error reading file - write message and return
c  a non-zero error code
c
  400 ierr = -1000
c
  500 return
c
  810 format(11x, f6.2)
  900 format(a10, 1x, f6.2)
      end
c  %W% %G%
c
c***********************************************************************
c
c                        read_pop_data
c
c  This routine loads census block data from files for areas one degree
c  on a side named by the latitude and longitude of their southeast
c  corners.  The inputs to the routine are the latitude and longitude
c  boundaries of the rectangle for which data is to be retrieved.
c
c***********************************************************************
c
      subroutine read_pop_data(
     &   dlat_min, dlon_min, dlat_max, dlon_max)
      include 'global.inc'
c
c  Local variables
c
      character flat*2, flon*3, fname*64, dir_name*48
      integer*2 ndummy
      integer*2 in_lat(80000), in_lon(80000), in_pop(80000)
c
c  Identify directory where one-degree by one-degree files of
c  population data reside
c
      dir_name = population_db
c
c  Initialize census block counter and latitude index
c
      num_blocks = 0
      lat_f = int(dlat_min) - 1
c
c  Identify next file overlapping the area of interest
c
   10 continue
      lat_f = lat_f + 1
      if (lat_f .gt. int(dlat_max)) goto 100
      lon_f = int(dlon_min) - 1
   20 continue
      lon_f = lon_f + 1
      if (lon_f .gt. int(dlon_max)) goto 10
c
      write(flat, 800) lat_f
      write(flon, 810) lon_f
      l = lnblnk(dir_name )
      fname = dir_name(1:l) // '/pop' // flat // 'x' // flon // '.dat'
c
c  Open next one-degree by one-degree file
c
      call getlun(infl)
      open(unit=infl, file=fname, status='old', form='unformatted',
     &   iostat=istat, access='direct', recl=5000000)
      if (istat .ne. 0) then
         if (warn) call perror(fname)
         goto 20
      end if
c
c  Read data
c
      read(infl, rec=1) n_rec, ndummy,
     &   (in_lat(i), in_lon(i), in_pop(i), i=1, n_rec-1)
c
c  Check for too many blocks in file
c
      if (n_rec .gt. 80001) then
         write(0, '(2a)') 'Error - too many census blocks in file ',
     &      fname
         write(0, '(a,i6)') 'Blocks = ', n_rec - 1
         stop
      end if
c
c  Search for blocks inside the area of interest
c
      do i=1, n_rec - 1
         rlat = float(lat_f) + (float(in_lat(i)) / 10000.0)
         rlon = float(lon_f) + (float(in_lon(i)) / 10000.0)
c
c  The blocks in each one-degree file are arranged in ascending order
c  of latitude, so we are finished with the current file when the first
c  out-of-range value is found.
c
         if (rlat .gt. dlat_max) goto 50
c
c  Save census blocks within the area of interest
c
         if (rlat .ge. dlat_min .and. rlon .ge. dlon_min .and.
     &      rlon .le. dlon_max) then
            num_blocks = num_blocks + 1
c
c  Check for too many blocks
c
            if (num_blocks .gt. MAX_POP_BLOCKS) then
               write(0, *) 'Error - too many census boxes'
               write(0, *) 'dlat_min, dlon_min, dlat_max, dlon_max ='
               write(0, *) dlat_min, dlon_min, dlat_max, dlon_max
               stop
            end if
c
c  Save data
c
            census_block_array(num_blocks).latsec = nint(rlat*3600.0)
            census_block_array(num_blocks).lonsec = nint(rlon*3600.0)
            census_block_array(num_blocks).ipop = in_pop(i)
         end if
      end do
   50 continue
      close(infl)
      goto 20
c
  100 continue
      return
c
  800 format(i2.2)
  810 format(i3.3)
      end
c  %W% %G%
c
c***********************************************************************
c
c                      read_prgm_options
c
c  This routine reads the Program Options category of input data from
c  the input file unit given by the argument infile.  This routine is
c  called by routine read_input that opens the file and positions
c  the file for reading at the line following the category name.
c
c***********************************************************************
c
      subroutine read_prgm_options(infile, file_name, line,
     &   a, ierr)
      character file_name*(*), a*(*)
      include 'global.inc'
c
      character b*300, c*300, d*20, e*20, f*20, g*20
      character rep*3
c
      character*80 sub_categories(15)
      data sub_categories /
     &   'Replicate',
     &   'Propagation Curves',
     &   'Longley-Rice Percentiles',
     &   'Receive Antenna Use',
     &   'Apply Xmit Vertical Pattern',
     &   'Apply Xmit Horizontal Pattern',
     &   'Analysis Radials',
     &   'Channel Relationships Considered',
     &   'Problem Area Definition',
     &   'Problem Area Station',
     &   'Problem Area Rectangle',
     &   'Problem Area Cell Size',
     &   'None Defined', 
     &   'None Defined', 
     &   'None Defined' /
c
   10 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. a(1:1) .eq. '['
     &   .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 10
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
c
c  From sub_category inside () determine what data is to be read
c
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
      if (a(n_l:n_r) .eq. sub_categories(1)(1:n_t)) then
         goto 20
      else if (a(n_l:n_r) .eq. sub_categories(2)(1:n_t)) then
         goto 30
      else if (a(n_l:n_r) .eq. sub_categories(3)(1:n_t)) then
         goto 40
      else if (a(n_l:n_r) .eq. sub_categories(4)(1:n_t)) then
         goto 50
      else if (a(n_l:n_r) .eq. sub_categories(5)(1:n_t)) then
         goto 60
      else if (a(n_l:n_r) .eq. sub_categories(6)(1:n_t)) then
         goto 70
      else if (a(n_l:n_r) .eq. sub_categories(7)(1:n_t)) then
         goto 80
      else if (a(n_l:n_r) .eq. sub_categories(8)(1:n_t)) then
         goto 90
      else if (a(n_l:n_r) .eq. sub_categories(9)(1:n_t)) then
         goto 100
      else if (a(n_l:n_r) .eq. sub_categories(12)(1:n_t)) then
         goto 110
      end if
c
c  Read replicate switch setting.  It determines whether ATV stations
c  replicate the NLC of a paired NTSC station answer can be yes or no
c  (lower case only)
c
   20 continue
      n1 = 0
      n2 = 0
      do i = 1,l
         if (a(i:i) .eq. '"' .and. n1 .eq. 0) n1 = i
         if (a(i:i) .eq. '"' .and. n1 .ne. 0) n2 = i
      end do
      n1 = n1 + 1
      n2 = n2 - 1
      read(a(n1:n2), '(a)') rep
      replicate = .false.
      if (rep(1:3) .eq. 'yes') replicate = .true.
      n1 = n1 - 1
      b = a(1:n1) // rep(1:(n2-n1)) // a(n2+1:l)
      l = lnblnk(b)
      write(iresult, '(a)') b(1:l)
      goto 10
c
c  Read which FCC propagation curves are to be used
c
   30 continue
      j = 0
      write(iresult, '(a)') a(1:l)
   31 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 31
      end if
c
      j = j + 1
      if (j .eq. 1) then
         read(a, 810) prop_crv_des_n, prop_crv_undes_n
         write(iresult, 810) prop_crv_des_n, prop_crv_undes_n
      else
         read(a, 810) prop_crv_des_a, prop_crv_undes_a
         write(iresult, 810) prop_crv_des_a, prop_crv_undes_a
      end if
      if (j .lt. 2) goto 31
      goto 10
c
c  Read Longley-Rice percentiles
c
   40 continue
      j = 0
      write(iresult, '(a)') a(1:l)
   41 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 41
      end if
c
      j = j + 1
      if (j .eq. 1) then
         read(a, 815) pct_time_n, pct_location_n, pct_confidence_n,
     &      pct_time_nix, pct_location_nix, pct_confidence_nix
         write(iresult, 815) pct_time_n, pct_location_n,
     &      pct_confidence_n, pct_time_nix, pct_location_nix,
     &      pct_confidence_nix
      else
         read(a, 815) pct_time_a, pct_location_a, pct_confidence_a,
     &      pct_time_aix, pct_location_aix, pct_confidence_aix
         write(iresult, 815) pct_time_a, pct_location_a,
     &      pct_confidence_a, pct_time_aix, pct_location_aix,
     &      pct_confidence_aix
      end if
      if (j .lt. 2) goto 41
      goto 10
c
c  Read whether receive antenna patters are to be used when computing
c  undesired signals
c
   50 continue
      write(iresult, '(a)') a(1:l)
   51 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 51
      end if
c
      use_rec_ant_n = .false.
      use_rec_ant_a = .false.
c
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      read(a(n_l:n_r), '(a)') b
      c = a(1:n_l-1) // b(1:n_t) // '"'
      l_c = lnblnk(c)
      if (b(1:n_t) .eq. 'yes') use_rec_ant_n = .true.
      b = a(n_r+2:l)
      call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
      if (b(n_l:n_r) .eq. 'yes') use_rec_ant_a = .true.
      c = c(1:l_c) // b(1:n_r+1)
      l_c = lnblnk(c)
      write(iresult, '(a)') c(1:l_c)
      goto 10
c
c  Read whether xmit antenna vertical patters are to be applied
c
   60 continue
      write(iresult, '(a)') a(1:l)
   61 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 61
      end if
c
      use_xmit_ant_n = .false.
      use_xmit_ant_a = .false.
c
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      read(a(n_l:n_r), '(a)') b
      c = a(1:n_l-1) // b(1:n_t) // '"'
      l_c = lnblnk(c)
      if (b(1:n_t) .eq. 'yes') use_xmit_ant_n = .true.
      b = a(n_r+2:l)
      call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
      if (b(n_l:n_r) .eq. 'yes') use_xmit_ant_a = .true.
      c = c(1:l_c) // b(1:n_r+1)
      l_c = lnblnk(c)
      write(iresult, '(a)') c(1:l_c)
      goto 10
c
c  Read whether directional antenna patterns are to be used.  If
c  replicate is false, computations using ATV stations will be made
c  on a non-directional basis using the prototype parameters.
c
   70 continue
      write(iresult, '(a)') a(1:l)
   71 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 71
      end if
c
      use_da_n = .false.
      use_da_a = .false.
c
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      read(a(n_l:n_r), '(a)') b
      c = a(1:n_l-1) // b(1:n_t) // '"'
      l_c = lnblnk(c)
      if (b(1:n_t) .eq. 'yes') use_da_n = .true.
      b = a(n_r+2:l)
      call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
      if (b(n_l:n_r) .eq. 'yes') use_da_a = .true.
      c = c(1:l_c) // b(1:n_r+1)
      l_c = lnblnk(c)
      write(iresult, '(a)') c(1:l_c)
      goto 10
c
c  Read number of evenly spaced radials to be used in defining extent
c  of NLCs
c
   80 continue
      write(iresult, '(a)') a(1:l)
   81 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 81
      end if
      read(a, 820) n_rads_used
      write(iresult, 820) n_rads_used
      goto 10
c
c  Read the channel relationships to be considered
c
   90 continue
      j = 0
c
c  Initialize array
c
      do l=1, 4
         do m=1, 25
            ichan_ck(l, m) = 99
         end do
      end do
c
   91 continue
      l = lnblnk(a)
      write(iresult, '(a)') a(1:l)
   92 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 92
      end if
c
      if (a(1:1) .ne. '(') goto 400
      write(iresult, '(a)') a(1:l)
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
c  Read relationships on/off (no/yes) for the system relationship
c  defined in (), such as N-to-N (NTSC to NTSC)
c
      k = 0
      if (a(n_l:n_r) .eq. 'N-to-N') k = 1
      if (a(n_l:n_r) .eq. 'N-to-A') k = 2
      if (a(n_l:n_r) .eq. 'A-to-N') k = 3
      if (a(n_l:n_r) .eq. 'A-to-A') k = 4
      if (k .eq. 0) goto 400
      j = j + 1
      n = 0
   93 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (l .eq. 0) then
         if (j .lt. 4) goto 91
         goto 10
      end if
c
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      if (a(n_l:n_r) .eq. 'yes') then
         read(a, 825) n_relation
         n = n + 1
         ichan_ck(k, n) = n_relation
         write(iresult, 900) n_relation, a(n_l-1:n_r+1)
      else
         write(iresult, '(a)') a(1:l)
      end if
      goto 93
c
c  Determine how user wishes to define problem area and read the
c  definitions.  Both station and rectangle definitions are read so
c  that the input file remains consistent
c
  100 continue
      l = lnblnk(a)
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      if (a(n_l:n_r) .ne. 'Station' .and.
     &   a(n_l:n_r) .ne. 'Rectangle') goto 400
      type_prob_area = a(n_l:n_r)
c
      write(iresult, '(a)') a(1:l)
c
c  Read definitions
c
      j = 0
      m = 0
  101 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 101
      end if
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
      j = j + 1
      k = 0
      if (a(n_l:n_r) .eq. sub_categories(10)(1:n_t)) k = 1
      if (a(n_l:n_r) .eq. sub_categories(11)(1:n_t)) k = 2
      if (j .eq. 1) m = k
      if (k .eq. 0 .or. (j .gt. 1 .and. k .eq. m)) goto 400
      write(iresult, '(a)') a(1:l)
c
  102 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 102
      end if
c
      if (k .eq. 1) then
         call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
         city_pa = a(n_l:n_r)
         c = a(1:n_l-1) // city_pa(1:n_t) // '"'
         l_c = lnblnk(c)
         b = a(n_r+2:l)
         call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
         state_pa = b(n_l:n_r)
         c = c(1:l_c) // b(1:n_l-1) // state_pa(1:n_t) // '"'
         l_c = lnblnk(c)
         l_b = lnblnk(b)
         b = b(n_r+2:l_b)
         call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
         read(b(n_l:n_r), 830) ichan_pa
         c = c(1:l_c) // b(1:n_r) // '"'
         l_c = lnblnk(c)
         l_b = lnblnk(b)
         b = b(n_r+2:l_b)
         call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
         sta_type_pa = b(n_l:n_l)
         d = 'ntsc'
         if (sta_type_pa .eq. 'a') d = 'atv'
         l_d = lnblnk(d)
         c = c(1:l_c) // b(1:n_l-1) // d(1:l_d) // '"'
         l_c = lnblnk(c)
         write(iresult, '(a)') c(1:l_c)
      else
         call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
         d = a(1:n_l-1)
         read(a(n_l:n_r), 835, err=400) lat_pa(1,1),
     &      lat_pa(1,2), lat_pa(1,3)
         b = a(n_r+2:l)
         call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
         e = b(1:n_l-1)
         n_1 = n_t
         if (n_t .eq. 8) read(b(n_l:n_r), 835, err = 400)
     &      lon_pa(1,1), lon_pa(1,2), lon_pa(1,3)
         if (n_t .eq. 9) read(b(n_l:n_r), 840, err = 400)
     &      lon_pa(1,1), lon_pa(1,2), lon_pa(1,3)
         l_b = lnblnk(b)
         b = b(n_r+2:l_b)
         call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
         f = b(1:n_l-1)
         read(b(n_l:n_r), 835, err=400) lat_pa(2,1),
     &      lat_pa(2,2), lat_pa(2,3)
         b = b(n_r+2:l_b)
         call find_len_btw_ch(b, '"', '"', n_l, n_r, n_t)
         g = b(1:n_l-1)
         n_2 = n_t
         if (n_t .eq. 8) read(b(n_l:n_r), 835, err = 400)
     &      lon_pa(2,1), lon_pa(2,2), lon_pa(2,3)
         if (n_t .eq. 9) read(b(n_l:n_r), 840, err = 400)
     &      lon_pa(2,1), lon_pa(2,2), lon_pa(2,3)
c
         l_d = lnblnk(d)
         l_e = lnblnk(e)
         l_f = lnblnk(f)
         l_g = lnblnk(g)
         if (n_1 .eq. 8 .and. n_2 .eq. 8) 
     &      write(iresult, 905) d(1:l_d), lat_pa(1,1), lat_pa(1,2),
     &      lat_pa(1,3), e(1:l_e), lon_pa(1,1), lon_pa(1,2),
     &      lon_pa(1,3), f(1:l_f), lat_pa(2,1), lat_pa(2,2),
     &      lat_pa(2,3), g(1:l_g), lon_pa(2,1), lon_pa(2,2),
     &      lon_pa(2,3)
         if (n_1 .eq. 9 .and. n_2 .eq. 8) 
     &      write(iresult, 906) d(1:l_d), lat_pa(1,1), lat_pa(1,2),
     &      lat_pa(1,3), e(1:l_e), lon_pa(1,1), lon_pa(1,2),
     &      lon_pa(1,3), f(1:l_f), lat_pa(2,1), lat_pa(2,2),
     &      lat_pa(2,3), g(1:l_g), lon_pa(2,1), lon_pa(2,2),
     &      lon_pa(2,3)
         if (n_1 .eq. 8 .and. n_2 .eq. 9) 
     &      write(iresult, 907) d(1:l_d), lat_pa(1,1), lat_pa(1,2),
     &      lat_pa(1,3), e(1:l_e), lon_pa(1,1), lon_pa(1,2),
     &      lon_pa(1,3), f(1:l_f), lat_pa(2,1), lat_pa(2,2),
     &      lat_pa(2,3), g(1:l_g), lon_pa(2,1), lon_pa(2,2),
     &      lon_pa(2,3)
         if (n_1 .eq. 9 .and. n_2 .eq. 9) 
     &      write(iresult, 908) d(1:l_d), lat_pa(1,1), lat_pa(1,2),
     &      lat_pa(1,3), e(1:l_e), lon_pa(1,1), lon_pa(1,2),
     &      lon_pa(1,3), f(1:l_f), lat_pa(2,1), lat_pa(2,2),
     &      lat_pa(2,3), g(1:l_g), lon_pa(2,1), lon_pa(2,2),
     &      lon_pa(2,3)
      end if
      if (j .lt. 2) goto 101
      goto 10
c
c  Read size of cells within the problem area grid.  The size is the
c  length in km of the sides of the square grid cells.
c
  110 continue
      read(a, 845) cell_size 
      l = lnblnk(a)
      write(iresult, '(a)') a(1:l)
      goto 10
c
c  Error reading file.  Write message and return non-zero error code
c
  400 continue
      ierr = -1000
c
  500 return
c
  810 format(4x, f4.1, 6x, f4.1)
  815 format(4x, f4.1, 4x, f4.1, 7x, f4.1, 8x, f4.1, 4x,
     &   f4.1, 7x, f4.1)
  820 format(7x, i3)
  825 format(3x, i3)
  830 format(i2)
  835 format(i2, 1x, i2, 1x, i2)
  840 format(i3, 1x, i2, 1x, i2)
  845 format(25x, f9.5)
  900 format(3x, sp, i3, 16x, a)
  905 format(a, i2, '-', i2.2, '-', i2.2, '"', a,  i2, '-',
     &   i2.2, '-', i2.2, '"', a,  i2, '-', i2.2, '-', i2.2, '"',
     &   a,  i2, '-', i2.2, '-', i2.2, '"')
  906 format(a, i2, '-', i2.2, '-', i2.2, '"', a,  i3, '-',
     &   i2.2, '-', i2.2, '"', a,  i2, '-', i2.2, '-', i2.2, '"',
     &   a,  i2, '-', i2.2, '-', i2.2, '"')
  907 format(a, i2, '-', i2.2, '-', i2.2, '"', a,  i2, '-',
     &   i2.2, '-', i2.2, '"', a,  i2, '-', i2.2, '-', i2.2, '"',
     &   a,  i3, '-', i2.2, '-', i2.2, '"')
  908 format(a, i2, '-', i2.2, '-', i2.2, '"', a,  i3, '-',
     &   i2.2, '-', i2.2, '"', a,  i2, '-', i2.2, '-', i2.2, '"',
     &   a,  i3, '-', i2.2, '-', i2.2, '"')
      end
c  %W% %G%
c
c***********************************************************************
c
c                      read_rec_antenna
c
c  This routine reads the Receiving Antenna category of input data from
c  the input file unit given by the argument infile.  This routine is
c  called by routine read_input which opens the file and positions
c  the file for reading at the line following the category name.
c
c***********************************************************************
c
      subroutine read_rec_antenna(infile, file_name, line, a, ierr)
      character file_name*(*), a*(*)
      include 'global.inc'
c
      character b*300, c*300
      character*80 sub_categories(5), sub_sub_categories(15)
      data sub_categories /
     &   'Patterns',
     &   'Height',
     &   'Pattern to be Used',
     &   'None Defined',
     &   'None Defined' /
c
      data sub_sub_categories /
     &   'Pattern Type',
     &   'Low VHF',
     &   'High VHF',
     &   'UHF',
     &   'NTSC',
     &   'ATV',
     &   'Exponent',
     &   'Rooftop',
     &   'Pattern Selection',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined' /
c
c  Initialize number of patterns to 0
c
      n_pat = 0
c
   10 line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 10
      end if
c
c  Error in file if a(1:1) not = [
c
      if (a(1:1) .ne. '[') goto 400
c
c  From sub_category inside [] determine what data is to be read.
c
      call find_len_btw_ch(a, '[', ']', n_l, n_r, n_t)
c
      if (a(n_l:n_r) .eq. sub_categories(1)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 20
      else if (a(n_l:n_r) .eq. sub_categories(2)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 30
      else if (a(n_l:n_r) .eq. sub_categories(3)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 40
      end if
c
c  From sub_sub_category inside () determine what data is to be read.
c  In this case there are multiple subcategories with the subcategory
c  Pattern Type defining how the data that follows it should be read.
c
   20 n_pat = n_pat + 1
   21 if (n_pat .gt. 2) goto 10
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 21
      end if
c
c  Error in file if a(1:1) not = (
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
c  First () must contain Pattern Type
c
      if (a(n_l:n_r) .ne. sub_sub_categories(1)(1:n_t)) goto 400
c
c  Get type of pattern
c
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
c
      read(a(n_l:n_r), '(a)') b
      c = a(1:n_l-1) // b(1:n_t) // a(n_r+1:l)
      l_c = lnblnk(c)
      write(iresult, '(a)') c(1:l_c)
      if (b(1:n_t) .eq. 'CCIR') then
c
c  Read CCIR receive antenna parameters
c
         j = 0
   22    line = line + 1
         read(infile, '(a)', err=400, end=400) a
         l = lnblnk(a)
         if (a(1:1) .eq. '{') goto 500
         if (a(1:1) .eq. '#' .or. l .eq. 0) then
            write(iresult, '(a)') a(1:l)
            goto 22
         end if
c
         if (a(1:1) .ne. '(') goto 400
         call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
         read(a(n_l:n_r), '(a)') b
         j = j + 1
         m = 0
         if (b(1:n_t) .eq. sub_sub_categories(2)(1:n_t)) m = 1
         if (b(1:n_t) .eq. sub_sub_categories(3)(1:n_t)) m = 2
         if (b(1:n_t) .eq. sub_sub_categories(4)(1:n_t)) m = 3
         if (m .eq. 0) goto 400
         read(a, 810) ccir_rant_arcs(m,1), ccir_rant_arcs(m,2),
     &      ccir_rant_arcs(m,3), ccir_rant_arcs(m,4),
     &      ccir_rant_gain(m,1), ccir_rant_gain(m,2),
     &      ccir_rant_gain(m,3), ccir_rant_gain(m,4)
         write(iresult, 900) a(1:10),
     &      ccir_rant_arcs(m,1), ccir_rant_arcs(m,2),
     &      ccir_rant_arcs(m,3), ccir_rant_arcs(m,4),
     &      ccir_rant_gain(m,1), ccir_rant_gain(m,2),
     &      ccir_rant_gain(m,3), ccir_rant_gain(m,4)
         if (j .lt. 3) goto 22
      else
c
c  Get PSWP3 antenna pattern
c
         j = 0
   23    line = line + 1
         read(infile, '(a)', err=400, end=400) a
         l = lnblnk(a)
         if (a(1:1) .eq. '{') goto 500
         if (a(1:1) .eq. '#' .or. l .eq. 0) then
            write(iresult, '(a)') a(1:l)
            goto 23
         end if
c
         if (a(1:1) .ne. '(') goto 400
         call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
         read(a(n_l:n_r), '(a)') b
         j = j + 1
         m = 0
         if (b(1:n_t) .eq. sub_sub_categories(5)(1:n_t)) m = 1
         if (b(1:n_t) .eq. sub_sub_categories(6)(1:n_t)) m = 2
         if (b(1:n_t) .eq. sub_sub_categories(7)(1:n_t)) m = 3
         if (m .eq. 0) goto 400
c
         if (m .eq. 1) then
            read(a, 810) fb_ntsc(1), fb_ntsc(2), fb_ntsc(3)
            write(iresult, 905) a(1:10), fb_ntsc(1), fb_ntsc(2),
     &         fb_ntsc(3)
         else if (m .eq. 2) then
            read(a, 810) fb_atv(1), fb_atv(2), fb_atv(3)
            write(iresult, 905) a(1:10), fb_atv(1), fb_atv(2), fb_atv(3)
         else if (m .eq. 3) then
            read(a, 810) fac_k
            write(iresult, 905) a(1:10), fac_k
         end if
         if (j .lt. 3) goto 23
c
         j = 0
   24    line = line + 1
         read(infile, '(a)', err=400, end=400) a
         l = lnblnk(a)
         if (a(1:1) .eq. '{') goto 500
         if (a(1:1) .eq. '#' .or. l .eq. 0) then
            write(iresult, '(a)') a(1:l)
            goto 24
         end if
c
         if (a(1:1) .ne. '(') goto 400
         call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
         read(a(n_l:n_r), '(a)') b
         j = j + 1
         m = 0
         if (b(1:n_t) .eq. sub_sub_categories(5)(1:n_t)) m = 1
         if (b(1:n_t) .eq. sub_sub_categories(6)(1:n_t)) m = 2
         if (m .eq. 0) goto 400
c
         if (m .eq. 1) then
            read(a, 810) rgain_ntsc(1), rgain_ntsc(2), rgain_ntsc(3)
            write(iresult, 905) a(1:10), rgain_ntsc(1), rgain_ntsc(2),
     &         rgain_ntsc(3)
         else if (m .eq. 2) then
            read(a, 810) rgain_atv(1), rgain_atv(2), rgain_atv(3)
            write(iresult, 905) a(1:10), rgain_atv(1), rgain_atv(2),
     &         rgain_atv(3)
         end if
         if (j .lt. 2) goto 24
      end if
c
c  Get next sub_category
c
      goto 20
c
c  Get receive antenna height
c
   30 line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 30
      end if
c
c  Error in file if a(1:1) not = (
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
c  First () must contain Rooftop
c
      if (a(n_l:n_r) .ne. sub_sub_categories(8)(1:n_t)) goto 400
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      read(a, 810) rec_ant_hgt
      b = a(n_r+1:l)
      l_b = lnblnk(b)
      write(iresult, 910) a(1:10), rec_ant_hgt, b(1:l_b)
      goto 10
c
c  Get users selection of receive antenna pattern
c
   40 line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 40
      end if
c
c  Error in file if a(1:1) not = (
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
c  First () must contain Pattern Selection
c
      if (a(n_l:n_r) .ne. sub_sub_categories(9)(1:n_t)) goto 400
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      read(a(n_l:n_r), '(a)') c
      l_c = lnblnk(c)
      use_ccir_rec_pat = .false.
      if (c(1:l_c) .eq. 'CCIR') use_ccir_rec_pat = .true.
      if (use_ccir_rec_pat) then
         b = a(1:n_l-1) // 'CCIR' // a(n_r+1:l)
      else
         b = a(1:n_l-1) // 'PSWP3' // a(n_r+1:l)
      end if
      l = lnblnk(b)
      write(iresult, '(a)') b(1:l)
      goto 10
c
c  Error reading file
c
  400 continue
      ierr = -1000
c
  500 return
c
  810 format(11x, f5.1, 3x, f5.1, 3x, f5.1, 3x, f5.1, 3x, f5.1,
     &   3x, f6.1, 4x, f6.1, 3x, f6.1) 
  900 format(a10, 1x, f5.1, 3x, f5.1, 3x, f5.1, 3x, f5.1, 3x, f5.1,
     &   3x, f6.1, 4x, f6.1, 3x, f6.1) 
  905 format(a10, 1x, f5.1, 3x, f5.1, 3x, f5.1)
  910 format(a10, 1x, f5.1, 1x, a)
      end
c  %W% %G%
c
c***********************************************************************
c
c                      read_xmit_params
c
c  This routine reads the TV Station parameters category of input data from
c  the input file unit given by the argument infile.  This routine is
c  called by routine read_input that opens the file and positions
c  the file for reading at the line following the category name.
c
c***********************************************************************
c
      subroutine read_xmit_params(infile, file_name, line,
     &   a, ierr)
      character file_name*(*), a*(*)
      include 'global.inc'
c
      character b*300, c*300
      character saved_routine_name*32
c
      character*80 sub_categories(10), sub_sub_categories(15)
      data sub_categories /
     &   'Vertical Pattern',
     &   'Horizontal Pattern',
     &   'HAAT',
     &   'ERP',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined' /
c
      data sub_sub_categories /
     &   'Type Vertical Pattern',
     &   'Low VHF',
     &   'High VHF',
     &   'UHF',
     &   'Source File',
     &   'NTSC Minima',
     &   'Prototype ATV',
     &   'ATV Minima',
     &   'ATV Maxima',
     &   'Vacant Allotments',
     &   'Pattern Selection',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined',
     &   'None Defined' /
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'read_xmit_params'
c
   10 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 10
      end if
c
c  If a(1:1) not = [ then error in file
c
      if (a(1:1) .ne. '[') goto 400
c
c  From sub_category inside [] determine what data is
c  to be read - in this case there are multiple sub_categories
c  with the sub-category Type Pattern defining how the data
c  that follows it will be read
c
      call find_len_btw_ch(a, '[', ']', n_l, n_r, n_t)
c
      if (a(n_l:n_r) .eq. sub_categories(1)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 20
      else if (a(n_l:n_r) .eq. sub_categories(2)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 30
      else if (a(n_l:n_r) .eq. sub_categories(3)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 40
      else if (a(n_l:n_r) .eq. sub_categories(4)(1:n_t)) then
         write(iresult, '(a)') a(1:l)
         goto 50
      end if
c
c  From sub_sub_category inside () determine what data is
c  to be read - in this case there are multiple sub_categories
c  with the sub-category Type Pattern defining how the data
c  that follows it will be read
c
c  Read vertical pattern data
c
   20 continue
      k = 0
   21 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 21
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
c  First () must contain Type Vertical Pattern
c
      if (a(n_l:n_r) .eq. sub_sub_categories(11)(1:n_t)) goto 24
      if (a(n_l:n_r) .ne. sub_sub_categories(1)(1:n_t)) goto 400
      write(iresult, '(a)') a(1:l)
      k = k + 1
c
c  Get type of pattern
c
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
c
      read(a(n_l:n_r), '(a)') b
c
c  Read xmit antenna parameters depending on the
c  type of pattern
c
      if (b(1:3) .eq. 'FCC') then
         j = 0
   22    continue
         line = line + 1
         read(infile, '(a)', err=400, end=400) a
         l = lnblnk(a)
         if (a(1:1) .eq. '{') goto 500
         if (a(1:1) .eq. '#' .or. l .eq. 0) then
            write(iresult, '(a)') a(1:l)
            goto 22
         end if
         call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
         read(a(n_l:n_r), '(a)') b
         j = j + 1
         m = 0
         if (b(1:n_t) .eq. sub_sub_categories(2)(1:n_t)) m = 1
         if (b(1:n_t) .eq. sub_sub_categories(3)(1:n_t)) m = 2
         if (b(1:n_t) .eq. sub_sub_categories(4)(1:n_t)) m = 3
         if (m .eq. 0) goto 400
         read(a, 810) tilt(m), theta_1(m), theta_2(m),
     &      volts_1(m), volts_2(m)
         write(iresult, 900) a(1:10), tilt(m), theta_1(m), theta_2(m),
     &      volts_1(m), volts_2(m)
         if (j .lt. 3) goto 22
      else
c
c  Get ps/wp3 antenna pattern
c
         j = 0
   23    continue
         line = line + 1
         read(infile, '(a)', err=400, end=400) a
         l = lnblnk(a)
         if (a(1:1) .eq. '{') goto 500
         if (a(1:1) .eq. '#' .or. l .eq. 0) then
            write(iresult, '(a)') a(1:l)
            goto 23
         end if
c
         j = j + 1
         read(a, 830) x_ang(j), xant_fac(1, 1, j),
     &      xant_fac(2, 1, j), xant_fac(1, 2, j), xant_fac(2, 2, j),
     &      xant_fac(1, 3, j), xant_fac(2, 3, j)
         write(iresult, 830) x_ang(j), xant_fac(1, 1, j),
     &      xant_fac(2, 1, j), xant_fac(1, 2, j), xant_fac(2, 2, j),
     &      xant_fac(1, 3, j), xant_fac(2, 3, j)
         if (j .lt. 13) goto 23
      end if
      goto 21
c
c  Get pattern selected FCC or PSWP3
c
   24 continue
      call find_len_btw_ch ( a, '"', '"', n_l, n_r, n_t)
      use_fcc_xmit_pat = .false.
      if (a(n_l:n_r) .eq. 'FCC') use_fcc_xmit_pat = .true.
      if (use_fcc_xmit_pat) then
         b = a(1:n_l-1) // 'FCC' // a(n_r+1:l)
      else
         b = a(1:n_l-1) // 'PSWP3' // a(n_r+1:l)
      end if
      l = lnblnk(b)
      write(iresult, '(a)') b(1:l)
c
c  Get next sub_category
c
      goto 10
c
c  Read info on use of hoizontal pattern - no actual
c  data is read here - it has been moved to the program
c  options section of the input file
c
   30 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0 .or.  a(1:1) .eq. '(') then
         write(iresult, '(a)') a(1:l)
         if (a(1:1) .eq. '(') goto 10
         goto 30
      end if
c
c  Determine how HAATs are to be used and minimum
c  acceptable values
c
   40 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 40
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
c  First () must contain Source File
c
      if (a(n_l:n_r) .ne. sub_sub_categories(5)(1:n_t)) goto 400
      write(iresult, '(a)') a(1:l)
      call find_len_btw_ch(a, '"', '"', n_l, n_r, n_t)
      read(a(n_l:n_r), '(a)') c
      l_c = lnblnk(c)
   41 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 41
      end if
c
c  Get # of radials used when computing HAAT - this line is
c  ignored if the source file is TVDB but it must be present
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      write(iresult, '(a)') a(1:l)
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
      b = a(n_r+3:l)
      read(b, 815) n_std_rad
      if (c(1:l_c) .eq. 'TVDB')  n_std_rad = 1
      use_n_radial_haat = .false.
      if (n_std_rad .gt. 1) use_n_radial_haat = .true.
c
c  Get mimimum HAAT values
c
      j = 0
   42 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 42
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
      m = 0
      if (a(n_l:n_r) .eq. sub_sub_categories(6)(1:n_t)) m = 1
      if (a(n_l:n_r) .eq. sub_sub_categories(7)(1:n_t)) m = 2
      if (m .eq. 0) goto 400
      j = j + 1
      if (m .eq. 1) then
         read(a, 820) haat_ntsc_limits(1,1), haat_ntsc_limits(1,2),
     &      haat_ntsc_limits(1,3)
         write(iresult, 910) a(1:21), haat_ntsc_limits(1,1),
     &      haat_ntsc_limits(1,2), haat_ntsc_limits(1,3)
c
c  Set vacant allotment and ATV minima to same as NTSC
c
         do 45 i=1, 3
            haat_va_limits(1, i) = haat_ntsc_limits(1, i)
            haat_atv_limits(1, i)= haat_ntsc_limits(1, i)
   45    continue
c
      else
         read(a, 820) haat_atv_prot(1), haat_atv_prot(2),
     &      haat_atv_prot(3)
         write(iresult, 910) a(1:21), haat_atv_prot(1),
     &      haat_atv_prot(2), haat_atv_prot(3)
      end if
      if (j .lt. 2) goto 42
      goto 10
c
c  Get ERP limits
c
   50 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 50
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
c
c  First () must contain Source File
c
      if (a(n_l:n_r) .ne. sub_sub_categories(5)(1:n_t)) goto 400
      write(iresult, '(a)') a(1:l)
c
      j = 0
   52 continue
      line = line + 1
      read(infile, '(a)', err=400, end=400) a
      l = lnblnk(a)
      if (a(1:1) .eq. '{') goto 500
      if (a(1:1) .eq. '#' .or. l .eq. 0) then
         write(iresult, '(a)') a(1:l)
         goto 52
      end if
c
c  If a(1:1) not = ( then error in file
c
      if (a(1:1) .ne. '(') goto 400
      call find_len_btw_ch(a, '(', ')', n_l, n_r, n_t)
      m = 0
      if (a(n_l:n_r) .eq. sub_sub_categories(6)(1:n_t)) m = 1
      if (a(n_l:n_r) .eq. sub_sub_categories(8)(1:n_t)) m = 2
      if (a(n_l:n_r) .eq. sub_sub_categories(7)(1:n_t)) m = 3
      if (a(n_l:n_r) .eq. sub_sub_categories(9)(1:n_t)) m = 4
      if (a(n_l:n_r) .eq. sub_sub_categories(10)(1:n_t)) m = 5
      if (m .eq. 0) goto 400
      j = j + 1
c
      if (m .eq. 1) then
         read(a, 820) erp_ntsc_limits(1,1), erp_ntsc_limits(1,2),
     &      erp_ntsc_limits(1,3)
         write(iresult, 910) a(1:21), erp_ntsc_limits(1,1),
     &      erp_ntsc_limits(1,2), erp_ntsc_limits(1,3)
      else if (m .eq. 2) then
         read(a, 820) erp_atv_limits(1,1), erp_atv_limits(1,2),
     &      erp_atv_limits(1,3)
         write(iresult, 910) a(1:21), erp_atv_limits(1,1),
     &      erp_atv_limits(1,2), erp_atv_limits(1,3)
      else if (m .eq. 3) then
         read(a, 820) erp_atv_prot(1), erp_atv_prot(2),
     &      erp_atv_prot(3)
         write(iresult, 910) a(1:21), erp_atv_prot(1), erp_atv_prot(2),
     &      erp_atv_prot(3)
      else if (m .eq. 4) then
         read(a, 820) erp_atv_limits(2,1), erp_atv_limits(2,2),
     &      erp_atv_limits(2,3)
         write(iresult, 910) a(1:21), erp_atv_limits(2,1),
     &      erp_atv_limits(2,2), erp_atv_limits(2,3)
      else if (m .eq. 5) then
         read(a, 820) erp_va_limits(1,1), erp_va_limits(1,2),
     &      erp_va_limits(1,3)
         write(iresult, 910) a(1:21), erp_va_limits(1,1),
     &      erp_va_limits(1,2), erp_va_limits(1,3)
c
c  Set vacant allot maximum to same as minimum.  The value must be
c  positive to avoid floating point errors.
c
         do iband=1, 3
            erp_va_limits(1,iband) = max(1e-4, erp_va_limits(1,iband))
            erp_va_limits(2,iband) = erp_va_limits(1,iband)
         end do
      end if
      if (j .lt. 5) goto 52
      goto 10
c
c  Error reading file.  Write message and return a non-zero error code.
c
  400 continue
      ierr = -1000
c
  500 continue
      routine_name = saved_routine_name
      return
c
  810 format(11x, f5.2, 5x, f5.2, 8x, f5.2, 6x, f4.2, 6x, f4.2)
  815 format(i3)
  820 format(21x, f6.1, 3x, f6.1, 4x, f6.1)
  830 format(2x, f5.2, 6(5x, f5.3))
  900 format(a10, 1x, f5.2, 5x, f5.2, 8x, f5.2, 6x, f4.2, 6x, f4.2)
  910 format(a21, 1x, f6.1, 3x, f6.1, 4x, f6.1)
      end
c  %W% %G%
c
c***********************************************************************
c
c                          rec_ant_ccir
c
c  Recommendation 419-2, Directivity of Antennas in the Reception
c  of Television Broadcasting.  Published in: Recommendations of the
c  CCIR, 1990, Volume XI - Part 1, Broadcasting Service (Television).
c
c  The azimuth is assumed to be in floating point degrees.  The
c  relative gain is returned in dB.
c
c  There is no actual dependence on service type (NTSC, ATV) though
c  it is formally an argument.
c
c  Indices identifying band:
c
c     Low VHF iband = 1
c     High VHF iband = 2
c     UHF iband = 3
c
c  The code performs a linear interpolation in the middle of three ranges
c  of azimuths.  Values to implement the CCIR recommendation (below)
c  should have been read as input.
c
c        Low VHF                  High VHF                  UHF
c
c  Azimuth     Relative      Azimuth     Relative     Azimuth   Relative
c Range, Deg     Gain       Range, Deg     Gain      Range, Deg   Gain
c
c   0- 50         0.0          0- 25        0.0        0- 20       0.0
c  50- 70      0.0  -6.0      25- 60     0.0 -12.0    20- 60    0.0 -16.0
c  70-180        -6.0         60-180      -12.0       60-180     -16.0
c
c***********************************************************************
c
      subroutine rec_ant_ccir(ang_in, iband, isystem, relative_gain)
      include 'global.inc'
c
      real arcs(2)
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'rec_ant_ccir'
c
c  Set beginning and end of range for linear variation.
c
      arcs(1) = ccir_rant_arcs(iband, 2)
      arcs(2) = ccir_rant_arcs(iband, 3)
c
c  Make angle a positive number
c
      ang = abs(ang_in)
      if (ang .gt. 180.0) then
         write(0, '(2a)') 'Azimuth out of range in subroutine ',
     &      routine_name
         call exit(257)
      end if
c
c  Find gain for given angle
c
      if (ang .le. arcs(1)) then
         relative_gain = ccir_rant_gain(iband, 1)
      else if (ang .gt. arcs(2)) then
         relative_gain = ccir_rant_gain(iband, 4)
      else
         ang_d1 = ang - arcs(1)
         ang_d2 = arcs(2) - arcs(1)
         relative_gain = ccir_rant_gain(iband, 3) * (ang_d1 / ang_d2)
      end if
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                       report_mem_error
c
c  Call this routine to report memory management problems.  It reports
c  the condition, the name of the pointer involved, and the name of the
c  routine where the problem occured.
c
c***********************************************************************
c
      subroutine report_mem_error(routine, pointer_name,
     &   memory_requested)
      character routine*(*), pointer_name*(*)
c
      write(0, 900) memory_requested,
     &   pointer_name(1:lnblnk(pointer_name)),
     &   routine(1:lnblnk(routine))
      stop
c
      entry report_mem_error_1(routine, pointer_name)
      write(0, 902)
     &   pointer_name(1:lnblnk(pointer_name)),
     &   routine(1:lnblnk(routine))
      stop
c
      entry report_mem_error_2(routine, pointer_name)
      write(0, 904)
     &   pointer_name(1:lnblnk(pointer_name)),
     &   routine(1:lnblnk(routine))
      stop
c
  900 format('Not enough memory available'/
     &   'Bytes requested: ', i /
     &   'Requested for:   ', a /
     &   'In routine:      ', a )
  902 format('Attempt to allocate memory at a non-zero pointer'/
     &   'Pointer name: ', a/
     &   'In routine:   ', a)
  904 format('Attempt to free unallocated memory'/
     &   'Pointer name: ', a/
     &   'In routine:   ', a)
      end
c  %W% %G%
c
c***********************************************************************
c
c  Copy array of integers to scratch file.  The array contains num_int
c  integer*4 values starting at ptr.
c
c***********************************************************************
c
      subroutine save_on_disk(iscr, ptr, num_int)
      pointer (ptr, in_mem(num_int))
c
      character access*3
      logical opn
c
c  Test whether scratch file has been opened properly
c
      inquire(iscr, opened=opn, direct=access, recl=length_of_rec)
      if (.not. opn) stop 'Scratch file not found'
      if (access .ne. 'YES') stop 'Not a direct access scratch file'
      if (length_of_rec .ne. 4*num_int) stop
     &   'Direct access scratch file has inappropriate record length'
c
c  Write in-memory array to disk
c
      write(iscr, rec=1) in_mem
      return
      end
c  %W% %G%
c
c**************************************************************
c
c                      sec_to_deg
c
c  Convert seconds to decimal degrees, or seconds to degrees
c  in double precision.
c
c**************************************************************
c
      subroutine sec_to_deg(isec, deg)
      ix = abs(isec)
      deg = float(ix) / 3600.0
      if (isec .lt. 0) deg = -deg
      return
      end

      subroutine sec_to_dpdeg(isec, dpdeg)
      double precision dpdeg
      ix = abs(isec)
      dpdeg = dble(ix) / dble(3600)
      if (isec .lt. 0) dpdeg = -dpdeg
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        separation_distances
c
c  Create a matrix storing the distance between desired and undesired
c  stations.  There is a distance for each desired-undesired pair, so
c  the size of the matrix is n_undes_sta * n_des_sta.
c
c***********************************************************************
c
      subroutine separation_distances
      include 'global.inc'
c
c  Allocate storage
c
      call allocate_separations_mem
c
c  Calculate and store distances
c
      do i=1, n_des_sta
         lata = lat_site(desired_stations(i).id)
         lona = lon_site(desired_stations(i).id)
         do j=1, n_undes_sta
            latb = lat_site(id_undesired(j))
            lonb = lon_site(id_undesired(j))
            call distance(lata, lona, latb, lonb, adist)
            k = j + n_undes_sta*(i-1)
            separations(k) = adist
         end do
      end do
      return
      end

      subroutine allocate_separations_mem
      include 'global.inc'
      character pointer_name*32
      character routine*32 /'allocate_separations_mem'/
c
c  The pointers should be initialized to zero before the first call of
c  this subprogran and released before subsequent calls.
c
      if (ptr_separations .ne. 0) then
         pointer_name = 'ptr_separations'
         call report_mem_error_1(routine, pointer_name)
      end if
c
c  Allocate memory
c
      numpairs = n_des_sta * n_undes_sta
      ptr_separations = malloc(4 * numpairs)
      if (ptr_separations .eq. 0) then
         pointer_name = 'ptr_separations'
         call report_mem_error(routine, pointer_name, 4*numpairs)
      end if
      return
c
c  Entry to free the memory allocated above
c
      entry release_separations
      pointer_name = 'ptr_separations'
      if (ptr_separations .eq. 0)
     &   call report_mem_error_2(routine, pointer_name)
      call free(ptr_separations)
      ptr_separations = 0
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          service_eval
c
c  This routine performs a final evaluation of the service provided
c  by the desired stations within the defined problem area taking into
c  consideration the service lost due to terrain and to interference.
c
c***********************************************************************
c
      subroutine service_eval
      include 'global.inc'
c
c  Local variables
c
      logical adjust
      pointer (ptr_tmp_1, icell_service_atv(*))
      logical*1 icell_service_atv
      character saved_routine_name*32
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'service_eval'
c
c  Loop thru desired stations, check the desired/undesired ratios
c  at each cell, and set service bit to no service when an unacceptable
c  ratio is found.
c
c  The IX fields are stored on disk and loaded into memory as needed.
c
c  A cell without service is skipped, and its present state is
c  determined by testing the appropriate bit in the icell_service array
c  which has n_per_cell bytes.
c
      n_per_cell = int(n_des_sta / 8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n_per_cell = n_per_cell - 1
c
c  Set pointer to indicate no memory yet allocated for ATV service
c
      ptr_tmp_1 = 0
c
c  Outer loop over desired stations
c
c  The value of the noise-limited ATV service field is needed for
c  calculations accounting for the effects of noise in addition to
c  undesired co-channel signals.
c
      do i=1, n_des_sta
         call print_grid(i, 'Noise-limited service')
         i0 = i - 1	!station number starting from 0
         id = desired_stations(i).id
         call set_ix_ch(ichan_r(id))
         if (sta_type(id) .eq. 'a') then
            itype_d = ATV
            if (adjust_du) then
               call get_band(ichan_r(id), iband)
               reference_field = cntr_nl_a(iband)
     &            - dipole_adjustment(ichan_r(id))
            end if
         else
            itype_d = NTSC
         end if
c
c  Set n_group constant for determining the service array element
c  corresponding to this desired station.  There is one element for
c  every 8 stations at each cell.
c
         n_group = int(i0/8) + 1
c
c  Set index of the service bit for this desired station.  Remember the
c  bits are numbered 0 thru 7.
c
         n_bit = mod(i0, 8)
c
c  Set base location for retrieval of the service fields of this
c  desired station
c
         lbase = i0 * n_cells
c
c  If station type is ATV, allocate memory for a second service array
c  to be used when determining ATV-only service loss.
c
         if (itype_d .eq. ATV) then
            if (ptr_tmp_1 .eq. 0)
     &         call allocate_tmp_1(ptr_tmp_1, n_cells)
c
c  Initialize ATV service condition
c
            do 10 j=1, n_cells
               n_el = n_per_cell * (j - 1) + n_group
               icell_service_atv(j) = btest(icell_service(n_el), n_bit)
   10       continue
         end if
c
c  All cells of a station with channel 0 should have been marked as
c  without service
c
         if (ichan_r(id) .eq. 0) then
            pop_results(i).ix_ntsc_loss = 0
            cell_results(i).ix_ntsc_loss = 0
            goto 300
         end if
c
c  Loop on undesired NTSC stations
c
         do 110 k=1, n_unntsc
            idu = id_undesired(k)
            if (sta_type(idu) .eq. 'a') stop
     &         'Expected an undesired NTSC; found an ATV station'
c
c  Skip the desired station.  It does not interfere with itself.
c
            if (id .eq. idu) goto 110
c
c  Set type of potential conflict
c
            if (itype_d .eq. NTSC) then
               ncase = N_TO_N
            else
               ncase = N_TO_A
            end if
c
c  Test channel relationship for possible conflict
c
            ixchn = 1
            ichn = ichan_r(idu)
            do while (ixchans(ixchn) .ne. ichn)
               ixchn = ixchn + 1
               if (ixchn .gt. n_ixch) goto 110
            end do
            idif = ichan_r(idu) - ichan_r(id)
c
            m = 1
            do while (ichan_ck(ncase, m) .ne. idif)
               m = m + 1
               if (m .gt. 25) goto 110
            end do
c
c  Load undesired cell fields for this undesired station
c
            read(ix_scr, rec=k)
     &         (icell_ix_fields(j), icell_ix_fades(j), j=1, n_cells)
c
c  Loop on cells
c
            do 100 j=1, n_cells
c
c  Retrieve service condition at this cell for current station, and if
c  no service go on to next cell
c
               n_el = n_per_cell * (j - 1) + n_group
               if (.not. btest(icell_service(n_el), n_bit)) goto 100
c
c  The current service condition for this cell is retained if either
c  the desired or undesired field was not computed
c
               l = lbase + j
               if (icell_service_fields(l) .eq. UNCALCULATED_SERVICE)
     &            goto 100
               if (icell_ix_fields(j) .eq. UNCALCULATED_IX) goto 100
c
c  Retrieve field strengths
c
               field_ds = float(icell_service_fields(l)) / 100.0
               field_uds = float(icell_ix_fields(j)) / 100.0
c
c  Reduce the undesired field by applying the receive antenna
c  discrimination.
c
               call apply_rec_ant(id, idu, j, relative_gain)
               field_uds = field_uds + relative_gain
c
c  Find index of limiting power ratio
c
               m = 1
               if (ncase .eq. N_TO_N) then
                  do while (m .le. 25 .and. n_off_nn(m) .ne. idif)
                     m = m + 1
                  end do
               else if (ncase .eq. N_TO_A) then
                  do while (m .le. 25 .and. n_off_na(m) .ne. idif)
                     m = m + 1
                  end do
               else
                  write(0, '(a,i3,2(a,i2))') 'Variable ncase =', ncase,
     &              '; should be', N_TO_N, ' or', N_TO_A
                  stop
               end if
c
c  Make sure the channel offset is in range
c
               if (m .gt. 25) then
                  write(0, '(a)') 'Error calculating IX from NTSC'
                  write(0, '(a,i3)') 'Invalid channel offset:', idif
                  stop
               end if
c
c  Retrieve D/U ratio for this channel offset
c
               f_dif = field_ds - field_uds
               if (ncase .eq. N_TO_N) then
                  pr = pr_nn(1, m)
               else
                  if (mod1 .and. .not. per_6th_order) then
                     adjust = adjust_du .and. idif .eq. 0
                  else
                     adjust = adjust_du
                  end if
                  if (adjust) then
                     relative_snr = field_ds - reference_field
                     call du_adjustments_na(relative_snr, adjustment)
                     pr = pr_na(1, m) + adjustment
                  else
                     pr = pr_na(1, m)
                  end if
               end if
c
c  Set service bit to 0 if D/U ratio is inadequate
c          
               if (f_dif .lt. pr) call zero_svc_bit(j, i)
  100       continue
  110    continue
c
c  Count population and cells where service is limited by IX from NTSC
c
         call count_svc_pop(i, n_pop_total, n_cell_total)
         pop_results(i).ix_ntsc_loss = pop_results(i).nl_td_tot
     &      - n_pop_total
         cell_results(i).ix_ntsc_loss = cell_results(i).nl_td_tot
     &      - n_cell_total
         call print_grid(i, 'Service limited by IX from NTSC')
c
c  Save existing service condition of NTSC stations when doing pairwise
c  evaluation
c
         if (pairwise .and. itype_d .eq. NTSC) then
            if (n_des_sta .ne. 2 .or. n_group .ne. 1 .or. lbase .ne. 0)
     &         stop 'Pairwise evaluation improperly initialized'
            do 120 j=1, n_cells
               if (.not. btest(icell_service(j), 0))
     &            icell_service(j) = ibclr(icell_service(j), 2)
  120       continue
         end if
c
c  Loop on undesired ATV stations
c
         do 210 k=n_unntsc + 1, n_undes_sta
            idu = id_undesired(k)
c
c  Skip the desired station
c
            if (id .eq. idu) goto 210
            if (sta_type(idu) .eq. 'n') stop
     &         'Expected an ATV undesired station; found an NTSC'
c
c  Set type of potential conflict, A-to-N or A-to-A
c
            if (itype_d .eq. NTSC) then
               ncase = A_TO_N
            else
               ncase = A_TO_A
            end if
c
c  Test channel relationship for possible conflict
c
            ixchn = 1
            ichn = ichan_r(idu)
            do while (ixchans(ixchn) .ne. ichn)
               ixchn = ixchn + 1
               if (ixchn .gt. n_ixch) goto 210
            end do
            idif = ichan_r(idu) - ichan_r(id)
c
            m = 1
            do while (ichan_ck(ncase, m) .ne. idif)
               m = m + 1
               if (m .gt. 25) goto 210
            end do
c
c  Load undesired cell fields for this undesired station
c
            read(ix_scr, rec=k)
     &         (icell_ix_fields(j), icell_ix_fades(j), j=1, n_cells)
c
c  Loop on cells
c
            do 200 j=1, n_cells
c
c  Retrieve service condition at this cell for current desired station,
c  and go on to next cell if no service to a desired NTSC.  All cells
c  of an ATV station must still be tested in order to establish their
c  condition in the future after NTSC has been retired.
c
               n_el = n_per_cell * (j - 1) + n_group
               if (itype_d .eq. NTSC) then
                  if (.not. btest(icell_service(n_el), n_bit)) goto 200
               else
                  if (.not. icell_service_atv(j)) goto 200
               end if
c
c  Check for uncomputable fields
c
               l = lbase + j
               if (icell_service_fields(l) .eq. UNCALCULATED_SERVICE)
     &            goto 200
               if (icell_ix_fields(j) .eq. UNCALCULATED_IX) goto 200
c
c  Retrieve desired and undesired field strengths
c
               field_ds = float(icell_service_fields(l)) / 100.0
               field_uds = float(icell_ix_fields(j)) / 100.0
c
c  Reduce the undesired field by the receive antenna discrimination.
c
               call apply_rec_ant(id, idu, j, relative_gain)
               field_uds = field_uds + relative_gain
c
c  Find index of limiting power ratio
c
               m = 1
               if (ncase .eq. A_TO_N) then
                  do while (m .le. 25 .and. n_off_an(m) .ne. idif)
                     m = m + 1
                  end do
               else if (ncase .eq. A_TO_A) then
                  do while (m .le. 25 .and. n_off_aa(m) .ne. idif)
                     m = m + 1
                  end do
               else
                  write(0, '(a,i3,2(ai2))') 'Variable ncase =', ncase,
     &              '; should be', A_TO_N, ' or', A_TO_A
                  stop
               end if
               if (m .gt. 25) then
                  write(0, '(a)') 'Error calculating IX from ATV'
                  write(0, '(a,i3)') 'Invalid channel offset:', idif
                  stop
               end if
c
c  Find D/U ratio for this channel offset.  Correlated time-fading
c  calculations may be made in case of ATV interference to ATV.
c
               if (ncase .eq. A_TO_A .and. correlated_fades) then
                  ik = k + (i-1)*n_undes_sta
                  sep = separations(ik)
                  if (sep .lt. 0.1) then
                     rho = 0.9
                  else if (sep .lt. 8.0) then
                     rho = 0.7
                  else if (sep .lt. 48.0) then
                     rho = 0.3
                  else
                     rho = 0.0
                  end if
                  fade_ds = float(icell_service_fades(l)) / 100.0
                  fade_uds = float(icell_ix_fades(j)) / 100.0
                  field_ds_median = field_ds + fade_ds
                  field_uds_median = field_uds + fade_uds
                  variance = fade_ds*fade_ds + fade_uds*fade_uds
     &               + 2*rho*fade_ds*fade_uds	!Note fade_uds is negative
                  combined_fade = sqrt(variance)
                  f_dif = field_ds_median - field_uds_median
     &               - combined_fade
               else
                  f_dif = field_ds - field_uds
               end if
c
c  Set limiting D/U value
c
               if (ncase .eq. A_TO_N) then
                  pr = pr_an(1, m)
               else
                  if (mod1 .and. .not. per_6th_order) then
                     adjust = adjust_du .and. idif .eq. 0
                  else
                     adjust = adjust_du
                  end if
                  if (adjust) then
                     relative_snr = field_ds - reference_field
                     call du_adjustments_aa(relative_snr, adjustment)
                     pr = pr_aa(1, m) + adjustment
                  else
                     pr = pr_aa(1, m)
                  end if
               end if
c
c  Set service bit to 0 if D/U ratio is inadequate
c
               if (f_dif .lt. pr) then
                  call zero_svc_bit(j, i)
                  if (itype_d .eq. ATV) icell_service_atv(j) = .false.
               end if
  200       continue
  210    continue
c
c  Count population and cells with service limited by interference
c  from either NTSC or ATV
c
  300    continue
         call count_svc_pop(i, n_pop_total, n_cell_total)
         pop_results(i).ix_loss_tot = pop_results(i).nl_td_tot
     &      - n_pop_total
         cell_results(i).ix_loss_tot = cell_results(i).nl_td_tot
     &      - n_cell_total
         pop_results(i).ix_atv_loss = pop_results(i).ix_loss_tot
     &      - pop_results(i).ix_ntsc_loss
         cell_results(i).ix_atv_loss = cell_results(i).ix_loss_tot
     &      - cell_results(i).ix_ntsc_loss
         call print_grid(i, 'Service during transition')
c
c  If this desired station is ATV, count ATV-to-ATV interference
c
         if (itype_d .eq. ATV) then
            call count_atv_svc_pop(i, n_pop_total, n_cell_total,
     &          ptr_tmp_1)
            pop_results(i).ix_atv_only_loss = pop_results(i).nl_td_tot
     &         - n_pop_total
            cell_results(i).ix_atv_only_loss = cell_results(i).nl_td_tot
     &         - n_cell_total
         end if
c
c  Proceed to evaluate service of next desired station
c
      end do
c
c  Compare prospective ATV service with that of the paired NTSC station
c
      if (pairwise) then
         if (n_des_sta .ne. 2 .or. itype_d .ne. ATV) then
            write(0, '(a)') 'No ATV station for pairwise comparison'
         else
            call compare_svc_pop(ipop_atv, ipop_ntsc, num_atv, num_ntsc)
            percent_pop_match = (100.0*ipop_atv) / ipop_ntsc
            percent_area_match = (100.0*num_atv) / num_ntsc
         end if
      end if
c
c  Free temporary memory allocated for ATV service evaluation, if any
c
      if (ptr_tmp_1 .ne. 0) call free(ptr_tmp_1)
c
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                             set_ix_ch
c
c  This routine determines the TV channel numbers that need to be
c  considered in an interference analysis.  It provides the complete
c  list in common block ix_chans along with the total number of non-null
c  entries.
c
c  The input argument to the routine is the channel number of the
c  station to be protected.  The maximum number of channels to be
c  considered in the current NTSC system is 17, and allowance is made
c  for as many as 25.  Common block elements are set to 0 where no
c  channel is to be considered.
c
c***********************************************************************
c
      subroutine set_ix_ch(ichn)
      include 'global.inc'
c
c  Initialize number of channels to 0
c
      n_ixch = 0
      do i = 1, 25
         ixchans(i) = 0
      end do
c
c  Ignore out of bounds channels
c
      if (ichn .lt. 2 .or. ichn .gt. 69) return
c
c  Determine which channels need to be considered, co-channel first
c
      n_ixch = n_ixch + 1
      ixchans(n_ixch) = ichn
c
c  Upper adjacent
c
      if (ichn .ne. 4 .and. ichn .ne. 6 .and. ichn .ne. 13
     &   .and. ichn .ne. 69) then
         n_ixch = n_ixch + 1
         ixchans(n_ixch) = ichn + 1
      end if
c
c  Lower adjacent
c
      if (ichn .ne. 2 .and. ichn .ne. 5 .and. ichn .ne. 7
     &   .and. ichn .ne. 14) then
         n_ixch = n_ixch + 1
         ixchans(n_ixch) = ichn - 1
      end if
c
c  UHF taboos
c
      if (ichn .lt. 14) return
      do 50 i=1, 7
         if (i .eq. 5) goto 50
         ic = i + 14
         if (ichn .gt. ic) then
            n_ixch = n_ixch + 1
            ixchans(n_ixch) = ichn - (i+1)
         end if
   50 continue
      do 60 i=1, 7
         if (i .eq. 5) goto 60
         ic = 69 - i
         if (ichn .lt. ic) then
            n_ixch = n_ixch + 1
            ixchans(n_ixch) = ichn + (i+1)
         end if
   60 continue
c
      if (ichn .lt. 55) then
         n_ixch = n_ixch + 1
         ixchans(n_ixch) = ichn + 14
         end if
      if (ichn .lt. 54) then
         n_ixch = n_ixch + 1
         ixchans(n_ixch) = ichn + 15
      end if
      return
      end
c  %W% %G%
c
c************************************************************************
c
c                          sideband_mask
c
c  Examine the D/U ratios for first-adjacent channel A-to-A interference,
c  and apply an improvement if it appears that the D/U ratios do not
c  include the effects of transmitter filters.
c
c************************************************************************
c
      subroutine sideband_mask(applied)
      logical applied
      include 'global.inc'
c
c              D/U Ratios in dB
c           ---------------------
c  Channel  Intital     Including
c  Offset   Testing     Splatter
c  ------   -------     ---------
c   +1      -43.17       -21.15
c   -1      -41.98       -22.83
c
      data filter_effect /5.0/	!Assumed improvement, dB
      data threshold /-25.0/
c
      applied = .false.
      if (max(pr_aa(1, 2), pr_aa(1, 9)) .gt. threshold) then
         pr_aa(1, 2) = pr_aa(1, 2) - filter_effect	!Upper adjacent, offset +1
         pr_aa(1, 9) = pr_aa(1, 9) - filter_effect	!Lower
         applied = .true.
      end if
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                         sort_pop_data
c
c  Sort the array of census blocks found in the problem area.  A pointer
c  to this array is stored in global common.
c
c***********************************************************************
c
      subroutine sort_pop_data
      include 'global.inc'
      external compare_struct
      integer*2 compare_struct
      integer struct_size /12/
      call qsort(census_block_array, num_blocks, struct_size,
     &   compare_struct)
      return
      end

      integer*2 function compare_struct(x, y)
c
c  Specify order for qsort of census blocks
c
      structure /CENSUS_BLOCKS/
         integer latsec
         integer lonsec
         integer*2 ipop
      end structure
      record /CENSUS_BLOCKS/ x, y
c
      lat_x = x.latsec
      lon_x = x.lonsec
      lat_y = y.latsec
      lon_y = y.lonsec
      if (lat_y .eq. lat_x) then
         if (lon_y .eq. lon_x) then
            compare_struct = 0
         else
            compare_struct = sign(1, lon_x - lon_y)
         end if
      else
         compare_struct = sign(1, lat_x - lat_y)
      end if
      return
      end
c  %W% %G%
c
c************************************************************************
c
c                              tvfmfs
c
c  Based on "Field Strength Calculations for TV and FM Broadcasting",
c  by Gary S. Kalagian, Report No. RS 76-01, January 1976.
c
c  Modified 12/23/91 to use field strength curve data read from files
c  rather than from data statements in the program.
c
c  Further modified 5/16/92 to include the use of transmit antenna
c  vertical pattern.
c
c  Modified 7/19/98 to include additional argument, vpat_bias, supporting
c  mecahnical tilt of transmitting antennas.
c
c  Modified on 10/22/92 to trap inconsistent values.  If request is to
c  compute field at a distance and the distance is beyond the curve
c  limits, it is set to the limit (flag A2 is set but the computation will
c  continue using the maximum distance for the curve).  Likewise if the
c  distance is less than or equal to 0.0 then it is set to 0.01 so that
c  compuation can continue.  If the request is to compute the distance and
c  the routine determines the distance is beyond the curve limits, the A2
c  flag is set and the distance is set to the maximum distance.
c
c  Argument             Description
c
c       erp    effective radiated power, in kilowatts.  This is the
c              peak value before application of horizontal and vertical
c              antenna pattern factors.
c
c vpat_bias    shift in depression angle, degrees.  The vertical pattern
c              is effectively shifted towards the greater elevations by
c              subracting this value from the actual depression angle
c              before retrieving the pattern factor.
c
c      haat    antenna height above average terrain, in meters
c
c   channel    FM or TV channel number
c
c     field    field strength dbu, input or output, depends on
c              value of switch
c
c  distance    distance to argument "field", in kilometers, input
c              or output, depends on value of switch
c
c    switch    1 means "field" is computed at the input value
c                for "distance".
c
c              2 means "distance" to the input value of "field"
c                is computed.
c
c     curve    0 means F(50,50) field strength curves are used.
c
c              1 means F(50,10) field strength curves are used.
c
c              2 means F(50,90) field strength curves are used.
c
c      flag    a two character error code, values are:
c
c              'A1' = free space equation used to find requested
c                     argument.
c
c              'A2' = distance exceeds greatest value on curves.
c
c              'A3' = invalid channel number.
c
c              'A4' = invalid curve selected.
c
c              'A5' = invalid switch value.
c
c              'A6' = erp less than or equal to 0 kilowatts.
c
c              'A7' = haat less than 30 meters, set to 30 meters.
c
c              'A8' = haat exceeds 1600 meters, set to 1600 meters.
c
c              'A9' = invalid "distance" value for switch = 1.
c
c              '  ' = normal return.
c
c***********************************************************************
c
      subroutine tvfmfs(erpc, vpat_bias, ahaat, channel, field,
     &   distance, switch, curve, flag, n_a)
      character flag*2
      integer channel, switch, curve, curve_2
      include 'global.inc'
c
      character saved_routine_name*32
      data instance /FCC_CURVES/
      logical iant, need_fac
      real d(201), h(201), f(201)
c
      saved_routine_name = routine_name
      routine_name = 'tvfmfs'
      range = 100.
      delta = 0.5
      n_points = 201
      id50 = 25
      ih50 = 13
      id10 = 30
      ih10 = 13
      flag = '  '
c
c  Set switch for determining whether to apply vertical pattern to
c  transmitting antenna.
c
      iant = .false.
      if (n_a .eq. NTSC .and. use_xmit_ant_n) iant = .true.
      if (n_a .eq. ATV .and. use_xmit_ant_a) iant = .true.
c
c  Error checks for argument values
c
      if (erpc .le. 0.0) then
         flag = 'A6'
         goto 500
      else if (channel .lt. 2 .or. channel .gt. 300  .or.
     &   (channel .gt. 69 .and. channel .lt. 200)) then
         flag = 'A3'
         goto 500
      else if (curve .lt. 0 .or. curve .gt. 2) then
         flag = 'A4'
         goto 500
      else if (switch .lt. 1 .or. switch .gt. 2) then
         flag = 'A5'
         goto 500
      else if (switch .eq. 1 .and.  distance .le. 0.0) then
         flag = 'A9'
         distance = 0.01
      end if
c
      erp_db = 10.0 * log10(erpc)
      height = ahaat
      if (ahaat .lt. 30.0) then
         height = 30.0
         flag   = 'A7'
      else if (ahaat .gt. 1600.0) then
         height = 1600.0
         flag   = 'A8'
      end if
c
c  For switch = 1, find the field at the specified distance.  If the
c  distance is less than 1.5 km, use the free space equation.
c
      if (switch .eq. 1) then
         if (distance .lt. 1.5) then
            field = 106.92 - 20.0 * log10(distance) + erp_db
c
c  Adjust for vertical pattern of transmitting antenna.
c
            if (iant) then
               ichan = channel
               if (ichan .gt. 69) ichan = 2	!FM
               call get_band(ichan, iband)
               call gt_vert_rad_fac(max(30.0, ahaat), rec_ant_hgt,
     &            distance, vpat_bias, instance, iband, n_a, factor)
               if (factor .lt. 1.0) then
                  db_factor = 20.0 * flog10(factor)
                  field = field + db_factor
               end if
            end if
            flag = 'A1'
            goto 500
         else if ((curve .eq. 0 .and. distance .gt. 300.0) .or.
     &      (curve .eq. 2 .and. distance .gt. 300.0) .or.
     &      (curve .eq. 1 .and. distance .gt. 500.0)) then
            flag = 'A2'
            distance = 300.0
            if (curve .eq. 1) distance = 500.0
         end if
c
         h(1) = height
         d(1) = distance
c
         if ((channel .ge. 2 .and. channel .le. 6) .or.
     &      (channel .ge. 200 .and. channel .le. 300)) then
            if (curve .eq. 0 .or.
     &         (curve .eq. 1 .and. distance .lt. 15.0)) then
               call itplbv(id50, ih50, d50, h50, f55lv, 1, d, h, f)
            else if (curve .eq. 1) then
               call itplbv(id10, ih10, d10, h10, f51lv, 1, d, h, f)
            else if (curve .eq. 2) then
               call itplbv(id50, ih50, d50, h50, f59lv, 1, d, h, f)
            end if
         else if (channel .ge. 7 .and. channel .le. 13) then
            if (curve .eq. 0 .or.
     &         (curve .eq. 1 .and. distance .lt. 15.0)) then
               call itplbv(id50, ih50, d50, h50, f55hv, 1, d, h, f)
            else if (curve .eq. 1) then
               call itplbv(id10, ih10, d10, h10, f51hv, 1, d, h, f)
            else if (curve .eq. 2) then
               call itplbv(id50, ih50, d50, h50, f59hv, 1, d, h, f)
            end if
         else if (channel .ge. 14 .and. channel .le. 69) then
            if (curve .eq. 0 .or.
     &         (curve .eq. 1 .and. distance .lt. 15.0)) then
               call itplbv(id50, ih50, d50, h50, f55u, 1, d, h, f)
            else if (curve .eq. 1) then
               call itplbv(id10, ih10, d10, h10, f51u, 1, d, h, f)
            else if (curve .eq. 2) then
               call itplbv(id50, ih50, d50, h50, f59u, 1, d, h, f)
            end if
         end if
c
         field = f(1) + erp_db
c
c  Vertical pattern adjustment
c
         if (iant) then
            ichan = channel
            if (ichan .gt. 69) ichan = 2
            call get_band(ichan, iband)
            call gt_vert_rad_fac(max(30.0, ahaat), rec_ant_hgt,
     &         distance, vpat_bias, instance, iband, n_a, factor)
            if (factor .lt. 1.0) then
               db_factor = 20.0 * flog10(factor)
               field = field + db_factor
            end if
         end if
         goto 500
c
c  For switch = 2, the distance for a specified field is found by
c  computing a field vs. distance curve at increments of 0.5 kilometer
c  and then interpolating between the field points.
c
      else if (switch .eq. 2) then
         curve_2 = curve
         do 40 i=1, n_points, 1
            h(i) = height
  40     continue
c
c  Computations will be repeated with F(50,50) in place of F(50,90) in
c  case the requested field is too high.
c
 100     continue
         if (curve_2 .eq. 0 .or. curve_2 .eq. 2) then
            d_first =   1.5
            d_last  = 300.0
         else if (curve_2 .eq. 1) then
            d_first =  15.0
            d_last  = 500.0
         end if
c
         do 105 i=1, n_points, 1
            d(i) = d_first + (i - 1) * delta
 105     continue
c
c  Computations will be repeated with an extended range of points if
c  necessary.
c
 110     continue
         if ((channel .ge. 2 .and. channel .le. 6) .or.
     &      (channel .ge. 200 .and. channel .le. 300)) then
            if (curve_2 .eq. 0) then
               call itplbv(id50, ih50, d50, h50, f55lv, n_points,
     &            d, h, f)
            else if (curve_2 .eq. 1) then
               call itplbv(id10, ih10, d10, h10, f51lv, n_points,
     &            d, h, f)
            else if (curve_2 .eq. 2) then
               call itplbv(id50, ih50, d50, h50, f59lv, n_points,
     &            d, h, f)
               end if
            else if (channel .ge. 7 .and. channel .le. 13) then
               if (curve_2 .eq. 0) then
                  call itplbv(id50, ih50, d50, h50, f55hv, n_points,
     &               d, h, f)
               else if (curve_2 .eq. 1) then
                  call itplbv(id10, ih10, d10, h10, f51hv, n_points,
     &               d, h, f)
               else if (curve_2 .eq. 2) then
                  call itplbv(id50, ih50, d50, h50, f59hv, n_points,
     &               d, h, f)
               end if
            else if (channel .ge. 14 .and. channel .le. 69) then
               if (curve_2 .eq. 0) then
                  call itplbv(id50, ih50, d50, h50, f55u, n_points,
     &               d, h, f)
               else if (curve_2 .eq. 1) then
                  call itplbv(id10, ih10, d10, h10, f51u, n_points,
     &               d, h, f)
               else if (curve_2 .eq. 2) then
                  call itplbv(id50, ih50, d50, h50, f59u, n_points,
     &               d, h, f)
               end if
            end if
c
c  Get band in preparation for vertical pattern adjustments
c
            if (iant) then
               ichan = channel
               if (ichan .gt. 69) ichan = 2	!FM
               call get_band(ichan, iband)
            end if
c
c  Increase field by ERP and adjust for vertical transmitting antenna
c  pattern.  No further vertical pattern adjustments are needed when
c  factor at the i-th distance reaches unity.
c
            need_fac = .true.
            do 130 i=1, n_points, 1
               f(i) = f(i) + erp_db
               if (iant .and. need_fac) then
                  call gt_vert_rad_fac(max(30.0, ahaat), rec_ant_hgt,
     &               d(i), vpat_bias, instance, iband, n_a, factor)
                  if (factor .lt. 1.0) then
                     db_factor = 20.0 * flog10(factor)
                     f(i) = f(i) + db_factor
                  else
                     need_fac = .false.
                  end if
               end if
 130        continue
c
c  If the curve is F(50,10) and the requested field is higher than the
c  curve, change to F(50,50) and recalculate because the distance should
c  be less than 15 kilometers.
c
c  If the curve of interest is F(50,50) and the requested field is higher
c  than the curve, use free space.
c
            if (field .gt. f(1)) then
               if (curve_2 .eq. 1) then
                  curve_2 = 0
                  goto 100
               else
                  flag = 'A1'
                  if (.not. iant) then
                     e_volts_meter = 1.0e-6 * 10.0**(field / 20.0)
                     distance = 7.014271e-3 * sqrt(erpc * 1000.) /
     &                  e_volts_meter
                  else
                     call itp_fs(field, erpc, ahaat, rec_ant_hgt,
     &                  vpat_bias, iband, n_a, distance)
                  end if
                  goto 500
               end if
c
c  If field is less than last point on curve, recalculate curve with next
c  range of distances.
c
         else if (field .lt. f(n_points)) then
            do 140 i=1, n_points, 1
               d(i) = d(i) + range
  140       continue
            if (d(1) .lt. d_last) then
               goto 110
            else
               flag = 'A2'
               distance = 300.0
               if (curve .eq. 1) distance = 500.0
               goto 500
            end if
c
c  Field value lies on the curve.  Find the corresponding distance by
c  interpolation.
c
         else
            do 150 i=2, n_points, 1
               if (field .le. f(i-1) .and. field .gt. f(i)) then
                  distance = (field - f(i-1)) / (f(i) - f(i-1)) *
     &               (d(i) - d(i-1)) + d(i-1)
                  if (distance .gt. d_last) then
                     flag = 'A2'
                     distance = 300.0
                     if (curve .eq. 1) distance = 500.0
                  end if
                  goto 500
               end if
 150        continue
         end if
      else
         write(0, '(a)') 'Bad switch setting in call to tvfmfs'
      end if
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                            xmit_ant_fcc
c
c  This routine computes the vertical radiation factor based on the FCC
c  five parameter model.  The inputs to the routine are the declination
c  angle (ang) in degrees and the band.  The output is the factor
c  (relative gain).
c
c  iband = 1 > Low VHF
c  iband = 2 > High VHF
c  iband = 3 > UHF
c
c  The gain is unity when ang = tilt otherwise it is approximated
c  by the following formulae:
c
c  y = 1 - aa(x-b)**2.0  for ang between tilt and theta_1
c  y = aa/(x-bb)  for ang greater then theta_1
c
c  y = gain  x = tangent of declination  angle
c
c  The three values for tilt, theta_1, theta_2, volts_1, and volts_2
c  are for the three bands low vhf, high vhf and uhf.
c
c  The values for volts_1 and volts_2 are the gain when the declination
c  angle is equal to theta_1 and theta_2
c
c***********************************************************************
c
      subroutine xmit_ant_fcc(ang, iband, factor)
      include 'global.inc'
c
      character saved_routine_name*32
      data abset /.false./
c
c  Equate routine_name for error reporting
c
      saved_routine_name = routine_name
      routine_name = 'xmit_ant_fcc'
c
c  Initialize gain to 1.0
c
      factor = 1.0
      if (ang .eq. tilt(iband)) goto 500
      if (ang .eq. theta_1(iband)) then
         factor = volts_1(iband)
         goto 500
      end if
      if (ang .eq. theta_2(iband)) then
         factor = volts_2(iband)
         goto 500
      end if
c
c  Compute value of aa and bb if this is first time thru routine
c
      if (.not. abset) then
         abset = .true.
c
c  Get aa and bb for first formula.  The gain is 1.0 at 0 degrees for VHF,
c  and therefore bb must equal 0.0
c
         bb(1, 1) = 0.0
         bb(2, 1) = 0.0
c
         do i=1, 2
            x = theta_1(i) * DEG_TO_RAD
            aa(i, 1) = (1 - volts_1(i)) / ((tan(x))**2.0)
         end do
c
c  The gain is 1.0 at 0.5 degrees for UHF, and therefore bb must equal
c  the tangent of 0.5 degrees
c
         bb(3, 1) = tan(0.5 * DEG_TO_RAD)
         x = theta_1(3) * DEG_TO_RAD
         aa(3, 1) = (1 - volts_1(3)) / ((tan(x) - bb(3, 1))**2.0)
c
c  Get aa and bb for second formula
c
         do i=1, 3
            r = volts_1(i) / volts_2(i)
            x_1 = theta_1(i) * DEG_TO_RAD
            x_2 = theta_2(i) * DEG_TO_RAD
            bb(i, 2) = ((r * tan(x_1)) - tan(x_2)) / (r - 1.0)
            aa(i, 2) = volts_1(i) * (tan(x_1) - bb(i, 2))
         end do
      end if
c
c  Compute gain
c
      x = ang * DEG_TO_RAD
      if (ang .lt. theta_1(iband)) then
         factor = 1 - (aa(iband, 1) * ((tan(x) - bb(iband, 1))**2.0))
      else
         factor = aa(iband, 2) / (tan(x) - bb(iband, 2))
      end if
c
  500 continue
      routine_name = saved_routine_name
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                        zero_outside_cells
c
c  This routine sets the flag bit in array outside_cells to 0 for the
c  cell specified by the argument n_cell.  This indicates that the cell
c  in within the noise limited contour of at least one of the desired
c  stations.  Array outside_cells is maintained to identify cells of the
c  problem area grid that are outside the noise limited contour of all
c  stations in the desired list.
c
c***********************************************************************
c
      subroutine zero_outside_cells(n_cell)
      include 'global.inc'
c
c  Identify the correct array element
c
      n_el = int(n_cell / 8) + 1
      if (mod(n_cell, 8) .eq. 0) n_el = n_el - 1
c
c  Determine which bit to clear.  Remember the bits are numbered 0 thru 7.
c
      n_bit = mod(n_cell, 8) - 1
      if (n_bit .eq. -1) n_bit = 7
c
c  Clear bit
c     
      if (n_el .eq. 0) then
         write (0, '(2a)')
     &      'Incorrectly identified array element in subroutine',
     &      ' zero_outside_cells.'
         write(0, '(a,i10)') 'grid cell', n_cell
         stop
      end if
      outside_cells(n_el) = ibclr(outside_cells(n_el), n_bit) 
c
      return
      end
c  %W% %G%
c
c***********************************************************************
c
c                          zero_svc_bit
c
c  This routine sets the service flag bit to 0 (no service) for the
c  given station (n_sta) at the given problem area grid cell (n_cell)
c
c  See routine init_svc_flag for an explanation of how the arrays
c  are setup.
c
c***********************************************************************
c
      subroutine zero_svc_bit(n_cell, n_sta)
      include 'global.inc'
c
c  Get correct array element.  There is one element for each 8 stations
c  at each cell.
c
      n_per_cell = int(n_des_sta/8) + 1
      if (mod(n_des_sta, 8) .eq. 0) n_per_cell = n_per_cell - 1
c
      n_group = int(n_sta/8) + 1
      if (mod(n_sta, 8) .eq. 0) n_group = n_group - 1
c
      n_el = (n_per_cell * (n_cell - 1)) + n_group
c
c  Determine which bit to clear.  Remember bits are numbered 0 thru 7.
c
      n_bit = (n_sta - ((n_group - 1) * 8)) - 1
c
c  Clear bit
c     
      if (n_el .eq. 0) then
         write(0, 900)
         write(0, *) n_cell, n_sta, n_des_sta
      end if
      icell_service(n_el) = ibclr(icell_service(n_el), n_bit) 
      return
c
  900 format('No elements to clear in zero_svc_bit')
      end
